/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        


#include "constants.h"
#include "callbacks.h"


void
on_group_name_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{

}


void
on_add_group_button_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    extern  GtkWidget *group_receiver_clist;
    extern  GtkWidget *group_clist;

    gint              i = 1;
    gint              receiver_ids[MAX_NUM_GROUPS];
    gchar             *user_id;
    GList             *selected;

    selected = GTK_CLIST(group_receiver_clist)->selection;
    if ( selected == NULL )
    {
	generic_error("\n  You have not selected any receivers to add  \n", "G-Page Error");
	return;
    }
    while (selected != NULL)
    {
	if ( i == MAX_NUM_GROUPS )
	{
	    update_status_line("Error: Maximum group size exceeded");
	    return;
	}
      gtk_clist_get_text(GTK_CLIST(group_receiver_clist), GPOINTER_TO_INT(selected->data), 0, &user_id);
      selected = selected->next;
      receiver_ids[i++] = atoi(user_id);
    }
    receiver_ids[i] = -1;
    gtk_clist_unselect_all(GTK_CLIST(group_receiver_clist));

    for ( i = 1; receiver_ids[i] != -1; i++)
	load_group_list(receiver_ids[i], GTK_CLIST(group_clist));


}


void
on_remove_group_button_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    extern  GtkWidget *group_clist;
    gint              row_selected = -1;

    if ( (GTK_CLIST(group_clist)->selection) != NULL )
    {
	row_selected = GPOINTER_TO_INT(GTK_CLIST(group_clist)->selection->data);
	if ( row_selected != -1 )
	    gtk_clist_remove( (GtkCList*)group_clist, row_selected);
    }
    else
    {
	generic_error("\n  You have not selected any receivers to remove  \n", "G-Page Error");
	return;
    }
	
}


void
on_save_group_button_clicked           (GtkButton       *button,
                                        add_group_data  *user_data)
{
    extern GtkWidget *group_tree;

    GtkCTreeNode   *node = NULL;
    gchar          *name;
    gint           i = 0;
    gint           x = 0;
    gint           y = 0;
    gchar          *group_id;
    gchar          *rec_id;
    gchar          *group_data_line;
    gchar          *temp_id, *rec_line, *rname, *rid;
    gchar          group_id_node[MAX_FIELD_LENGTH];
    gint           group_array[MAX_NUM_GROUPS];
    gchar          *temp;
    gint           already_in_group = 0;

    temp = g_malloc(MAX_FIELD_LENGTH);

    name = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->group_name)));
    if ( strlen(name) == 0 )
    {
	generic_error("\n  You must enter a name for the group \n", "G-Page Error");
	g_free(name);
	return;
    }
    name = clean_string(name);

/*
 *   We store all of the receiver id's selected into an array
 */

    while ( gtk_clist_get_selectable(GTK_CLIST(group_clist), i) )
    {
	gtk_clist_get_text( GTK_CLIST(group_clist), i, 0, &rec_id);
	for ( x = 0; x < y; x++ )
	    if ( atoi(rec_id) == group_array[x] ) /* We check to see if this rec is already in the group */
		already_in_group = 1;

	if ( !already_in_group )
	    group_array[y++] = atoi(rec_id);

	i++;
	already_in_group = 0;
    }
    y--;

    if ( y < 0 )
    {
	generic_error("\n  You have not selected any members for the group \n", "G-Page Error");
	return;
    }

    if ( y == 0 )
    {
	generic_error("\n  A group must have at least two members \n", "G-Page Error");
	return;
    }

/*
 *   Get the id for this group 
 */
    if (user_data->edit_flag)
	group_id = g_strdup(user_data->group_id);
    else
    {
	memset(temp, '\0', MAX_FIELD_LENGTH);
	strcpy(temp, "G");
	group_id = g_strdup_printf("%d", (get_id("GROUP")) );
	strcat(temp, group_id);
	g_free(group_id);
	group_id = g_strdup(temp);
	//strcpy(group_id, temp);
    }

/*
 *   Load all of the data for this group into a single string
 */

    group_data_line = g_malloc( (strlen(name) + 16 + (16 * y) + y) );

    strcpy(group_data_line, group_id);
    strcat(group_data_line, ",");
    strcat(group_data_line, name);
    strcat(group_data_line, ",");



    if ( user_data->edit_flag )
    {
	delete_rec_by_id(group_id);	/* Take this entry out of the main receiver list */
	if ( !user_data->popup_flag )
	    gtk_ctree_remove_node( (GtkCTree *)group_tree, GTK_CLIST(group_tree)->selection->data);
    }	

    if ( !user_data->popup_flag )
    {
	if ( user_data->refresh_flag )
	{
	    strcat(group_id_node, group_id);
	    node = make_node(group_tree, NULL, NULL, group_id, name); 
	}
	if (user_data->edit_flag)
	{
	    node = make_node(group_tree, NULL, NULL, group_id, name); 
	}
    }

    while ( y >= 0 )
    {
	temp_id = g_strdup_printf("%d", group_array[y]);
	strcat(group_data_line, temp_id);
	if ( y != 0 )
	    strcat(group_data_line, ",");
	if ( !user_data->popup_flag )
	{
	    if ( (user_data->refresh_flag) || (user_data->edit_flag))
	    {
		rid = g_malloc(16);
		rname = g_malloc(MAX_FIELD_LENGTH);
		rec_line = get_uid_line("RECEIVER.dat", temp_id, RECEIVER_FIELDS, 0, MAX_LENGTH);
		get_name_id(rid, rname, rec_line);
		make_node(group_tree, node, NULL, "", rname);
		g_free(rec_line);
		g_free(rid);
		g_free(rname);
	    }
	}
	g_free(temp_id);
	y--;
    }
/*
 *   Add the data to the group database 
 */

    if (user_data->edit_flag)
    {
	strcat(group_data_line, "\n");
	edit_item(group_id, "GROUP.dat", 3, 1, group_data_line, EDIT, ',');
    }
    else
	add_data(group_data_line, "GROUP.dat");

/*
 *   Insert the group into the clist in the main interface
 */
    
    gtk_clist_freeze(GTK_CLIST(receiver_clist));
    gtk_clist_clear(GTK_CLIST(receiver_clist));
    load_receiver_list("RECEIVER.dat", GTK_CLIST(receiver_clist));
    load_receiver_list("GROUP.dat", GTK_CLIST(receiver_clist));
    load_main_list_with_pixmaps();
    gtk_clist_thaw(GTK_CLIST(receiver_clist));

    g_free(group_id);
    g_free(group_data_line);
    g_free(name);
    gtk_widget_destroy(user_data->groups);
    if ( user_data->edit_flag )
    {
	g_free(user_data->group_id);
	g_free(user_data->group_name_text);
    }
    g_free(user_data);
    g_free(temp);
}


void
on_cancel_group_button_clicked         (GtkButton       *button,
                                        add_group_data         *user_data)
{
    gtk_widget_destroy(user_data->groups);
    if ( user_data->edit_flag || user_data->popup_flag)
    {
	g_free(user_data->group_id);
	g_free(user_data->group_name_text);
    }
    g_free(user_data);
}



