/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"


/**************************************************************/
/***                                                        ***/
/***    This function is called by the menu items only      ***/
/***                                                        ***/
/**************************************************************/
 
void receiver_call(void)
{
    add_receiver(NULL, NULL);
}

/*
 *    This function is called by the add receiver button in the
 *    list receiver dialog box
 */

void list_receiver_call(void)
{
    extern  GtkWidget *service_clist;

    receiver *receiver_data;
    GtkWidget *add_receiver_dialog;

    receiver_data = g_malloc(sizeof(*receiver_data));
    receiver_data->edit_flag = 0;
    receiver_data->popup_flag = 0;
    receiver_data->refresh_list_flag = 1;

    add_receiver_dialog = create_add_receiver(receiver_data, NULL);
    load_generic_list("SERVICE.dat", GTK_CLIST(service_clist));
    gtk_widget_show(add_receiver_dialog);
}

/*
 *    This function is called by the add receiver button on the toolbar
 */

void add_receiver(GtkWidget *w, gchar *my_data)
{
    extern  GtkWidget *service_clist;

    receiver   *receiver_data;
    GtkWidget  *add_receiver_dialog;

    receiver_data = g_malloc(sizeof(*receiver_data));
    receiver_data->edit_flag = 0;
    receiver_data->popup_flag = 0;
    receiver_data->refresh_list_flag = 0;

    add_receiver_dialog = create_add_receiver(receiver_data, NULL);
    load_generic_list("SERVICE.dat", GTK_CLIST(service_clist));
    gtk_widget_show(add_receiver_dialog);
}

void edit_receiver_popup(GtkWidget *w, gchar *rec_id)
{
    receiver_text *receiver_data;

    gint field, i;
    gchar *line;
    gchar *list_data[RECEIVER_FIELDS];
    gint  ret_value = 0;

    if ( rec_id[0] == 'G' )
    {
	add_group_edit_popup(rec_id);
	return;
    }

    line = get_uid_line("RECEIVER.dat", rec_id, RECEIVER_FIELDS, 0, MAX_LENGTH);

    for (field = 0; field < RECEIVER_FIELDS; field++)
    {
	list_data[field] = g_malloc(MAX_FIELD_LENGTH);
	memset(list_data[field], '\0', sizeof(list_data[field]));
    }
    ret_value = get_fields_from_string(list_data, line, RECEIVER_FIELDS, ',');
    if ( ret_value == 1 )
    {
	update_status_line("Error corrupt database: RECEIVER.dat");
	for (field = 0; field < RECEIVER_FIELDS; field++)
	    g_free(list_data[field]);
	return;
    }

    receiver_data = g_malloc(sizeof(*receiver_data));

    receiver_data->uid = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->uid, list_data[0]);

    receiver_data->receiver_name = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->receiver_name, list_data[1]);

    receiver_data->receiver_number = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->receiver_number, list_data[2]);

    receiver_data->sid = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->sid, list_data[3]);

    receiver_data->receiver_smtp = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->receiver_smtp, list_data[4]);

    receiver_data->autosplit = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->autosplit, list_data[5]);

    receiver_data->checkbutton_wctp = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->checkbutton_wctp, list_data[6]);

    receiver_data->checkbutton_snpp = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->checkbutton_snpp, list_data[7]);

    receiver_data->checkbutton_smtp = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->checkbutton_smtp, list_data[8]);

    receiver_data->radio_wctp_one_way = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->radio_wctp_one_way, list_data[9]);

    receiver_data->radio_snpp_one_way = g_malloc(MAX_FIELD_LENGTH);
    strcpy(receiver_data->radio_snpp_one_way, list_data[10]);

    for ( i = 0; i < RECEIVER_FIELDS; i++)
	g_free(list_data[i]);

    receiver_data->popup_flag = 1;
    receiver_data->edit_flag = 0;
    receiver_data->refresh_list_flag = 0;

    edit_receiver(receiver_data);

}

void edit_receiver(receiver_text *edit_receiver_data)
{
    extern  GtkWidget *service_clist;
    extern  GtkWidget *list_receiver_clist;

    receiver   *receiver_data;
    GtkWidget  *add_receiver_dialog;
    gchar      *service_data;
    gint       row_number, return_value;
    FILE       *fp;
    gchar      *buffer;
    gchar      edit_id[MAX_FIELD_LENGTH];
    gchar      *temp_receiver_id;
    gchar      *receiver_id;
    gchar      *rec_id;


    rec_id = g_malloc(MAX_FIELD_LENGTH);
    if ( !edit_receiver_data->popup_flag)
    {
	gtk_clist_get_text(GTK_CLIST(list_receiver_clist), GPOINTER_TO_INT(GTK_CLIST(list_receiver_clist)->selection->data), 4, &temp_receiver_id);
	strcpy(rec_id, temp_receiver_id);
    }
    else
    {
	strcpy(rec_id, edit_receiver_data->uid);
    }

    receiver_data = g_malloc(sizeof(*receiver_data));
    if ( edit_receiver_data->popup_flag )
    {
	receiver_data->edit_flag = 0;
	receiver_data->popup_flag = 1;
	receiver_data->refresh_list_flag = 0;
    }	
    else
    {
	receiver_data->edit_flag = 1;
	receiver_data->popup_flag = 0;
	receiver_data->refresh_list_flag = 1;
    }

    add_receiver_dialog = create_add_receiver(receiver_data, edit_receiver_data);
    load_generic_list("SERVICE.dat", GTK_CLIST(service_clist));

    if ( ( fp = get_file_pointer("RECEIVER.dat", "r") ) == NULL)
	return;
    else
    {
	while ( ((buffer = read_file_by_line(fp)) != NULL) )
	{
	    if ( ((return_value = match_in_string(buffer, rec_id, RECEIVER_FIELDS, 1, ','))) == 0)
	    {
		receiver_id = get_field(buffer, RECEIVER_FIELDS, 4);
		strcpy(edit_id, receiver_id);
		g_free(receiver_id);
		fclose(fp);
		break;
	    }
	    g_free(buffer);
	}
    }

    g_free(rec_id);
    
    row_number = 0;
	while (1)
	{
	    gtk_clist_get_text(GTK_CLIST(service_clist), row_number, 0, &service_data); 
	    if(service_data == NULL)
	    {
		fprintf(stderr, "Error corrupt database\n");
		break;
	    }
	    if( !strcmp(service_data, edit_id) )
		break;
	    else
		row_number++;
	}

	gtk_clist_select_row(GTK_CLIST(service_clist), row_number, 0);
	gtk_widget_show(add_receiver_dialog);
	gtk_clist_moveto(GTK_CLIST(service_clist), row_number, 0, 0., 0.);
}


GtkWidget*
create_add_receiver (receiver *receiver_data, receiver_text *edit_data)
{
    
  GtkWidget *add_receiver;
  GtkWidget *frame2;
  GtkWidget *vbox3;
  GtkWidget *vbox4;
  GtkWidget *table1;
  GtkWidget *label12;
  GtkWidget *label13;
  GtkWidget *hbox4;
  GtkWidget *frame4;
  GtkWidget *hbox6;
  GtkWidget *vbox5;
  GtkWidget *hbox7;
  //GtkWidget *label15;
  GtkWidget *frame5;
  GtkWidget *frame3;
  GtkWidget *frame9;
  GtkWidget *wctp_setup;
  GtkWidget *hbox5;
  GtkWidget *hbox10;
  GSList *autosplit = NULL;
  GtkWidget *hbox8;
  GtkWidget *scrolledwindow6;

  GtkWidget *label16;
  GtkWidget *label17;
  GtkWidget *vbox6;
  GtkWidget *frame6;
  GtkWidget *vbox7;
  GtkWidget *label18;
  GtkWidget *hbuttonbox3;
  GtkWidget *add_service_button;
  GtkWidget *save_receiver_button;
  GtkWidget *cancel_receiver_button;
  GtkWidget *add_service_label;

  GtkWidget *snpp_setup;
  GtkWidget *hbox_snpp;

  gchar     *service_flag;
  GSList    *wctp_group = NULL;
  GSList    *snpp_group = NULL;

  window_data *win_data;
  service_flag = g_malloc(16);
  strcpy(service_flag, "refresh");

  win_data = g_malloc(sizeof(*win_data));
  add_receiver = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_object_set_data (GTK_OBJECT (add_receiver), "add_receiver", add_receiver);
  gtk_widget_set_usize (add_receiver, 425, 550);

  if ( receiver_data->edit_flag || receiver_data->popup_flag )
      gtk_window_set_title (GTK_WINDOW (add_receiver), "Edit Receiver");
  else
      gtk_window_set_title (GTK_WINDOW (add_receiver), "Add Receiver");
  gtk_window_set_modal (GTK_WINDOW (add_receiver), TRUE);

  win_data->window = add_receiver;
  win_data->rec_data = receiver_data;

  frame2 = gtk_frame_new (" Add Receiver ");
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_container_add (GTK_CONTAINER (add_receiver), frame2);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox3);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "vbox3", vbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 2);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (frame2), vbox3);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox4);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "vbox4", vbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox4);
  gtk_box_pack_start (GTK_BOX (vbox3), vbox4, TRUE, TRUE, 0);

  table1 = gtk_table_new (2, 2, FALSE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox4), table1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 5);

  label12 = gtk_label_new (" Name: ");
  gtk_widget_ref (label12);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "label12", label12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label12);
  gtk_table_attach (GTK_TABLE (table1), label12, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label13 = gtk_label_new (" Number: ");
  gtk_widget_ref (label13);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "label13", label13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label13);
  gtk_table_attach (GTK_TABLE (table1), label13, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  receiver_data->receiver_name = gtk_entry_new_with_max_length (64);

  if ( receiver_data->edit_flag || receiver_data->popup_flag)
      gtk_entry_set_text( GTK_ENTRY(receiver_data->receiver_name), edit_data->receiver_name);

  gtk_widget_ref (receiver_data->receiver_name);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "receiver_name", receiver_data->receiver_name,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->receiver_name);
  gtk_table_attach (GTK_TABLE (table1), receiver_data->receiver_name, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  receiver_data->receiver_number = gtk_entry_new_with_max_length (64);

  if ( receiver_data->edit_flag || receiver_data->popup_flag)
      gtk_entry_set_text( GTK_ENTRY(receiver_data->receiver_number), edit_data->receiver_number);

  gtk_widget_ref (receiver_data->receiver_number);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "receiver_number", receiver_data->receiver_number,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->receiver_number);
  gtk_table_attach (GTK_TABLE (table1), receiver_data->receiver_number, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

/*
 *   Begin the frame and the select protocols checkboxes
 */

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox4, TRUE, TRUE, 0);

  frame4 = gtk_frame_new (" Select Protocols ");
  gtk_widget_ref (frame4);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "frame4", frame4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (hbox4), frame4, TRUE, TRUE, 0);
  gtk_widget_set_usize (frame4, 100, 25);
  gtk_container_set_border_width (GTK_CONTAINER (frame4), 3);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox6);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "hbox6", hbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox6);
  gtk_container_add (GTK_CONTAINER (frame4), hbox6);

  receiver_data->checkbutton_wctp = gtk_check_button_new_with_label ("WCTP");
  gtk_widget_ref (receiver_data->checkbutton_wctp);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "checkbutton_wctp", receiver_data->checkbutton_wctp,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->checkbutton_wctp);
  gtk_box_pack_start (GTK_BOX (hbox6), receiver_data->checkbutton_wctp, FALSE, FALSE, 2);

  if ( receiver_data->edit_flag || receiver_data->popup_flag)
  {
      if (!strcmp(edit_data->checkbutton_wctp, "1"))
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->checkbutton_wctp), TRUE);
      else
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->checkbutton_wctp), FALSE);
  }
  else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->checkbutton_wctp), FALSE);

  receiver_data->checkbutton_snpp = gtk_check_button_new_with_label ("SNPP");
  gtk_widget_ref (receiver_data->checkbutton_snpp);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "checkbutton_snpp", receiver_data->checkbutton_snpp,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->checkbutton_snpp);
  gtk_box_pack_start (GTK_BOX (hbox6), receiver_data->checkbutton_snpp, FALSE, FALSE, 0);

  if ( receiver_data->edit_flag || receiver_data->popup_flag)
  {
      if (!strcmp(edit_data->checkbutton_snpp, "1"))
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->checkbutton_snpp), TRUE);
      else
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->checkbutton_snpp), FALSE);
  }
  else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->checkbutton_snpp), FALSE);


  receiver_data->checkbutton_smtp = gtk_check_button_new_with_label ("SMTP");
  gtk_widget_ref (receiver_data->checkbutton_smtp);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "checkbutton_smtp", receiver_data->checkbutton_smtp,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->checkbutton_smtp);
  gtk_box_pack_start (GTK_BOX (hbox6), receiver_data->checkbutton_smtp, FALSE, FALSE, 0);

  if ( receiver_data->edit_flag || receiver_data->popup_flag)
  {
      if (!strcmp(edit_data->checkbutton_smtp, "1"))
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->checkbutton_smtp), TRUE);
      else
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->checkbutton_smtp), FALSE);
  }
  else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->checkbutton_smtp), FALSE);

/*
 *   Begin the frame and Email address text entry box
 */

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox5);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "vbox5", vbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox5);
  gtk_box_pack_start (GTK_BOX (vbox4), vbox5, TRUE, TRUE, 0);


  frame5 = gtk_frame_new (" Receiver Email Address: ");
  gtk_widget_ref (frame5);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "frame5", frame5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_set_border_width (GTK_CONTAINER (frame5), 2);
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox5), frame5, TRUE, TRUE, 0);


  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox7);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "hbox7", hbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox7);
  gtk_container_add (GTK_CONTAINER (frame5), hbox7);

  receiver_data->receiver_smtp = gtk_entry_new_with_max_length (128);

  if ( receiver_data->edit_flag || receiver_data->popup_flag)
      gtk_entry_set_text( GTK_ENTRY(receiver_data->receiver_smtp), edit_data->receiver_smtp);

  gtk_widget_ref (receiver_data->receiver_smtp);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "receiver_smtp", receiver_data->receiver_smtp,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->receiver_smtp);

  gtk_box_pack_start (GTK_BOX (hbox7), receiver_data->receiver_smtp, TRUE, TRUE, 2);

/*
 *   Begin the frame and snpp options radio buttons
 */

  snpp_setup = gtk_frame_new (" SNPP Options: ");
  gtk_widget_ref (snpp_setup);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "snpp_setup", snpp_setup,
                            (GtkDestroyNotify) gtk_widget_unref);
  //gtk_widget_show (snpp_setup);
  gtk_box_pack_start (GTK_BOX (vbox5), snpp_setup, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (snpp_setup), 2);

  hbox_snpp = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox_snpp);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "hbox_snpp", hbox_snpp,
                            (GtkDestroyNotify) gtk_widget_unref);
  //gtk_widget_show (hbox_snpp); 
  gtk_container_add (GTK_CONTAINER (snpp_setup), hbox_snpp);

  receiver_data->radio_snpp_one_way = gtk_radio_button_new_with_label (snpp_group, " 1.0 Paging");
  snpp_group = gtk_radio_button_group (GTK_RADIO_BUTTON (receiver_data->radio_snpp_one_way));
  gtk_widget_ref (receiver_data->radio_snpp_one_way);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "receiver_data->radio_snpp_one_way", receiver_data->radio_snpp_one_way,
                            (GtkDestroyNotify) gtk_widget_unref);
  //gtk_widget_show (receiver_data->radio_snpp_one_way);
  gtk_box_pack_start (GTK_BOX (hbox_snpp), receiver_data->radio_snpp_one_way, FALSE, FALSE, 0);

  receiver_data->radio_snpp_respond_read = gtk_radio_button_new_with_label (snpp_group, " 1.5 Paging");
  snpp_group = gtk_radio_button_group (GTK_RADIO_BUTTON (receiver_data->radio_snpp_respond_read));
  gtk_widget_ref (receiver_data->radio_snpp_respond_read);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "receiver_data->radio_snpp_respond_read", receiver_data->radio_snpp_respond_read,
                            (GtkDestroyNotify) gtk_widget_unref);
  //gtk_widget_show (receiver_data->radio_snpp_respond_read);
  gtk_box_pack_start (GTK_BOX (hbox_snpp), receiver_data->radio_snpp_respond_read, FALSE, FALSE, 0);
  
  receiver_data->radio_snpp_two_way = gtk_radio_button_new_with_label (snpp_group, " 2.0 Paging");
  snpp_group = gtk_radio_button_group (GTK_RADIO_BUTTON (receiver_data->radio_snpp_two_way));
  gtk_widget_ref (receiver_data->radio_snpp_two_way);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "receiver_data->radio_snpp_two_way", receiver_data->radio_snpp_two_way,
                            (GtkDestroyNotify) gtk_widget_unref);  
  //gtk_widget_show (receiver_data->radio_snpp_two_way);
  gtk_box_pack_start (GTK_BOX (hbox_snpp), receiver_data->radio_snpp_two_way, FALSE, FALSE, 0);


  if ( receiver_data->edit_flag || receiver_data->popup_flag )
  {
      if (!strcmp(edit_data->radio_snpp_one_way, "0"))
	  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(receiver_data->radio_snpp_one_way), TRUE);
      else if (!strcmp(edit_data->radio_snpp_one_way, "1"))
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->radio_snpp_respond_read), TRUE);
      else
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->radio_snpp_two_way), TRUE);
  }
  else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->radio_snpp_one_way), TRUE);

/*
 *   Begin the frame and wctp options radio buttons
 */

  wctp_setup = gtk_frame_new (" WCTP Options: ");
  gtk_widget_ref (wctp_setup);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "wctp_setup", wctp_setup,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (wctp_setup);
  gtk_box_pack_start (GTK_BOX (vbox5), wctp_setup, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (wctp_setup), 2);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox10);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "hbox10", hbox10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox10);
  gtk_container_add (GTK_CONTAINER (wctp_setup), hbox10);

  receiver_data->radio_wctp_one_way = gtk_radio_button_new_with_label (wctp_group, " 1.0 Paging");
  wctp_group = gtk_radio_button_group (GTK_RADIO_BUTTON (receiver_data->radio_wctp_one_way));
  gtk_widget_ref (receiver_data->radio_wctp_one_way);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "receiver_data->radio_wctp_one_way", receiver_data->radio_wctp_one_way,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->radio_wctp_one_way);
  gtk_box_pack_start (GTK_BOX (hbox10), receiver_data->radio_wctp_one_way, FALSE, FALSE, 0);

  receiver_data->radio_wctp_respond_read = gtk_radio_button_new_with_label (wctp_group, " 1.5 Paging");
  wctp_group = gtk_radio_button_group (GTK_RADIO_BUTTON (receiver_data->radio_wctp_respond_read));
  gtk_widget_ref (receiver_data->radio_wctp_respond_read);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "receiver_data->radio_wctp_respond_read", receiver_data->radio_wctp_respond_read,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->radio_wctp_respond_read);
  gtk_box_pack_start (GTK_BOX (hbox10), receiver_data->radio_wctp_respond_read, FALSE, FALSE, 0);

  receiver_data->radio_wctp_two_way = gtk_radio_button_new_with_label (wctp_group, " 2.0 Paging");
  wctp_group = gtk_radio_button_group (GTK_RADIO_BUTTON (receiver_data->radio_wctp_two_way));
  gtk_widget_ref (receiver_data->radio_wctp_two_way);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "receiver_data->radio_wctp_two_way", receiver_data->radio_wctp_two_way,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->radio_wctp_two_way);
  gtk_box_pack_start (GTK_BOX (hbox10), receiver_data->radio_wctp_two_way, FALSE, FALSE, 0);

  
  if ( receiver_data->edit_flag || receiver_data->popup_flag )
  {
      if (!strcmp(edit_data->radio_wctp_one_way, "0"))
	  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(receiver_data->radio_wctp_one_way), TRUE);
      else if (!strcmp(edit_data->radio_wctp_one_way, "1"))
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->radio_wctp_respond_read), TRUE);
      else
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->radio_wctp_two_way), TRUE);
  }
  else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (receiver_data->radio_wctp_one_way), TRUE);


/*
 *   Begin the frame and the autosplit radio boxes
 */

  frame3 = gtk_frame_new (" Autosplit long messages: ");
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "frame3", frame3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox5), frame3, TRUE, TRUE, 0);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_set_border_width (GTK_CONTAINER (frame3), 2);
  gtk_widget_show (hbox5);
  gtk_container_add (GTK_CONTAINER (frame3), hbox5);

  receiver_data->autosplit_yes_radio_button = gtk_radio_button_new_with_label (autosplit, " Yes");
  autosplit = gtk_radio_button_group (GTK_RADIO_BUTTON (receiver_data->autosplit_yes_radio_button));
  gtk_widget_ref (receiver_data->autosplit_yes_radio_button);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "autosplit_yes_radio_button", receiver_data->autosplit_yes_radio_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->autosplit_yes_radio_button);
  gtk_box_pack_start (GTK_BOX (hbox5), receiver_data->autosplit_yes_radio_button, FALSE, FALSE, 2);
  /* gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(receiver_data->autosplit_yes_radio_button), TRUE);  */

  receiver_data->autosplit_no_radio_button = gtk_radio_button_new_with_label (autosplit, " No");
  autosplit = gtk_radio_button_group (GTK_RADIO_BUTTON (receiver_data->autosplit_no_radio_button));
  gtk_widget_ref (receiver_data->autosplit_no_radio_button);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "autosplit_no_radio_button", receiver_data->autosplit_no_radio_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_data->autosplit_no_radio_button);
  gtk_box_pack_start (GTK_BOX (hbox5), receiver_data->autosplit_no_radio_button, FALSE, FALSE, 0);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(receiver_data->autosplit_no_radio_button), TRUE);  

  if ( receiver_data->edit_flag || receiver_data->popup_flag)
  {
      if ( !strcmp(edit_data->autosplit, "1") )
	  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(receiver_data->autosplit_yes_radio_button), TRUE);
      else
	  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(receiver_data->autosplit_yes_radio_button), FALSE);
  }


  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox8);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "hbox8", hbox8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox8, TRUE, TRUE, 0);

  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow6);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "scrolledwindow6", scrolledwindow6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow6);
  gtk_box_pack_start (GTK_BOX (hbox8), scrolledwindow6, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  service_clist = gtk_clist_new (2);
  gtk_clist_set_column_visibility (GTK_CLIST(service_clist), 0, FALSE);
  gtk_clist_set_sort_column (GTK_CLIST(service_clist), 1);
  gtk_clist_set_auto_sort(GTK_CLIST(service_clist), TRUE);

  gtk_widget_ref (service_clist);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "service_clist", service_clist,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (service_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), service_clist);
  gtk_widget_set_usize (service_clist, 220, 175);
  gtk_container_set_border_width (GTK_CONTAINER (service_clist), 5);
  gtk_clist_set_column_width (GTK_CLIST (service_clist), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (service_clist), 1, 80);
  gtk_clist_column_titles_show (GTK_CLIST (service_clist));

  label16 = gtk_label_new ("");
  gtk_widget_ref (label16);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "label16", label16,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label16);
  gtk_clist_set_column_widget (GTK_CLIST (service_clist), 0, label16);

  label17 = gtk_label_new (" Select Service: ");
  gtk_widget_ref (label17);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "label17", label17,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label17);
  gtk_clist_set_column_widget (GTK_CLIST (service_clist), 1, label17);

  vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox6);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "vbox6", vbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox6);
  gtk_box_pack_start (GTK_BOX (hbox8), vbox6, FALSE, TRUE, 0);

  frame6 = gtk_frame_new ("");
  gtk_widget_ref (frame6);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "frame6", frame6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_set_border_width (GTK_CONTAINER (frame6), 2);
  gtk_widget_show (frame6);
  gtk_box_pack_start (GTK_BOX (vbox6), frame6, TRUE, TRUE, 0);

  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox7);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "vbox7", vbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (frame6), vbox7);

  label18 = gtk_label_new ("\nSelect a service\nor add a new one\n");
  gtk_widget_ref (label18);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "label18", label18,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label18); 
  gtk_box_pack_start (GTK_BOX (vbox7), label18, FALSE, FALSE, 0);

/*******************************************************************/


  frame9 = gtk_frame_new (" Add new service ");
  gtk_widget_ref (frame9);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "frame9", frame9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame9);
  gtk_box_pack_start (GTK_BOX (vbox6), frame9, TRUE, TRUE, 0);

  gtk_container_set_border_width (GTK_CONTAINER (frame9), 2);

  add_service_label = gtk_hbutton_box_new ();
  gtk_widget_ref (add_service_label);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "add_service_label", add_service_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (add_service_label);
  gtk_container_add (GTK_CONTAINER (frame9), add_service_label);

  add_service_button = gtk_button_new_with_label ("Add");
  gtk_widget_ref (add_service_button);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "add_service_button", add_service_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (add_service_button);
  gtk_container_add (GTK_CONTAINER (add_service_label), add_service_button);
  GTK_WIDGET_SET_FLAGS (add_service_button, GTK_CAN_DEFAULT);


/*****************************************************/
  hbuttonbox3 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox3);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "hbuttonbox3", hbuttonbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox3);
  gtk_box_pack_start (GTK_BOX (vbox6), hbuttonbox3, TRUE, TRUE, 0);

  save_receiver_button = gtk_button_new_with_label ("Save");
  gtk_widget_ref (save_receiver_button);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "save_receiver_button", save_receiver_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (save_receiver_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox3), save_receiver_button);
  GTK_WIDGET_SET_FLAGS (save_receiver_button, GTK_CAN_DEFAULT);

  cancel_receiver_button = gtk_button_new_with_label ("Cancel");
  gtk_widget_ref (cancel_receiver_button);
  gtk_object_set_data_full (GTK_OBJECT (add_receiver), "cancel_receiver_button", cancel_receiver_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cancel_receiver_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox3), cancel_receiver_button);
  GTK_WIDGET_SET_FLAGS (cancel_receiver_button, GTK_CAN_DEFAULT);

  if ( receiver_data->edit_flag || receiver_data->popup_flag)
  {
      receiver_data->uid = g_malloc(sizeof(edit_data->uid));
      strcpy(receiver_data->uid, edit_data->uid);
  }

  gtk_signal_connect (GTK_OBJECT (add_service_button), "clicked",
			     GTK_SIGNAL_FUNC (add_service_function),
			     (gpointer) "refresh");


  gtk_signal_connect (GTK_OBJECT (save_receiver_button), "clicked",
		      GTK_SIGNAL_FUNC (on_save_receiver_button_clicked),
		      win_data);


  if ( receiver_data->refresh_list_flag )
  {
      gtk_signal_connect (GTK_OBJECT (cancel_receiver_button), "clicked",
			  GTK_SIGNAL_FUNC (on_cancel_receiver_unselect),
			  win_data);
  }
  else
  {
      gtk_signal_connect (GTK_OBJECT (cancel_receiver_button), "clicked",
			  GTK_SIGNAL_FUNC (on_cancel_receiver_button_clicked),
			  win_data);
  }

  if ( receiver_data->edit_flag || receiver_data->popup_flag)
  {
      g_free(edit_data->uid);
      g_free(edit_data->receiver_name);
      g_free(edit_data->receiver_number);
      g_free(edit_data->sid);
      g_free(edit_data->receiver_smtp);
      g_free(edit_data->autosplit);
      g_free(edit_data->checkbutton_wctp);
      g_free(edit_data->checkbutton_snpp);
      g_free(edit_data->checkbutton_smtp);
      g_free(edit_data);
  }

  gtk_widget_grab_focus (cancel_receiver_button);
  return add_receiver;
}

