/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

void
on_save_receiver_button_clicked        (GtkButton       *button,
                                        window_data      *win_data)
{
    extern GtkWidget *list_receiver_clist;

    gchar *service_error = "\n  You must select a service  \n  for this receiver  \n";
    gchar *name_error = "\n  You must enter a name for this receiver  \n";
    gchar *number_error ="\n  You must enter a number for this receiver  \n";
    gchar *email_error = "\n  You must enter an email address for this receiver  \n";
    gchar *protocol_error = "\n  You must selecet at least one protocol for this receiver  \n";

    gint  id, row_selected, i;
    gchar *name;
    gchar *number;
    gchar *smtp_field;
    gchar uid[MAX_FIELD_LENGTH];
    gchar wctp[2];
    gchar snpp[2];
    gchar smtp[2];
    gchar wctp_options[2];
    gchar snpp_options[2];
    gchar autosplit[2];
    gchar *database_entry;
    gchar *r_data[5];
    gchar *main_gui_data[3];
    gchar *sid;
    gchar *service_line;
    gchar *service_name;
    extern GtkWidget *receiver_clist;
    extern GtkWidget *service_clist;

    if (GTK_CLIST(service_clist)->selection == NULL)
    {
	generic_error(service_error, "G-Page Error");
	return;
    }
    
    database_entry = g_malloc(MAX_LENGTH);


/*
 *    First we must gather all the data entered by the user from the entry boxes and check boxes
 */


    name = g_strdup(gtk_entry_get_text(GTK_ENTRY(win_data->rec_data->receiver_name)));
    if (is_empty_string(name, name_error))
    {
	g_free(database_entry);
	g_free(name);
	return;
    }
    name = clean_string(name); 

    number = g_strdup(gtk_entry_get_text(GTK_ENTRY(win_data->rec_data->receiver_number)));
    if (is_empty_string(number, number_error))
    {
	g_free(database_entry);
	g_free(name);
	g_free(number);
	return;
    }
    number = clean_string(number);

    if (GTK_TOGGLE_BUTTON (win_data->rec_data->checkbutton_wctp)->active)
	strcpy(wctp, "1\0");
    else
	strcpy(wctp,"0\0");

    if (GTK_TOGGLE_BUTTON (win_data->rec_data->checkbutton_snpp)->active)
	strcpy(snpp, "1\0");
    else
	strcpy(snpp,"0\0");

    smtp_field = g_strdup(gtk_entry_get_text(GTK_ENTRY(win_data->rec_data->receiver_smtp)));

    if (GTK_TOGGLE_BUTTON (win_data->rec_data->checkbutton_smtp)->active)
    {
	strcpy(smtp, "1\0");
	if (is_empty_string(smtp_field, email_error))
	{
	    g_free(database_entry);
	    g_free(name);
	    g_free(number);
	    return;
	}
    }
    else
    {
	strcpy(smtp,"0\0");
	if ( (strlen(smtp_field)) > 0)
	    smtp_field = clean_string(smtp_field);
	else 
	    strcpy(smtp_field, "none");
    }

    if ( (!strcmp(wctp, "0")) && (!strcmp(snpp, "0")) && (!strcmp(smtp, "0")) )
    {
	generic_error(protocol_error, "G-Page Error");
	g_free(name);
	g_free(database_entry);
	g_free(number);
	return;
    }

    if (GTK_TOGGLE_BUTTON (win_data->rec_data->autosplit_yes_radio_button)->active)
	strcpy(autosplit, "1\0");
    else
	strcpy(autosplit, "0\0");


    if (GTK_TOGGLE_BUTTON (win_data->rec_data->radio_snpp_one_way)->active)
	strcpy(snpp_options, "0\0");
    else if (GTK_TOGGLE_BUTTON (win_data->rec_data->radio_snpp_respond_read)->active)
	strcpy(snpp_options, "1\0");
    else
	strcpy(snpp_options, "2\0");


    if (GTK_TOGGLE_BUTTON (win_data->rec_data->radio_wctp_one_way)->active)
	strcpy(wctp_options, "0\0");
    else if (GTK_TOGGLE_BUTTON (win_data->rec_data->radio_wctp_respond_read)->active)
	strcpy(wctp_options, "1\0");
    else
	strcpy(wctp_options, "2\0");


    if (win_data->rec_data->edit_flag || win_data->rec_data->popup_flag)
    {
	strcpy(uid, win_data->rec_data->uid);
    }
    else
    {
	id = get_id("RECEIVER");
	g_snprintf(uid, MAX_FIELD_LENGTH, "%d", id);
    }

/*
 *   Now that we have all the data we need to load up a string for the database
 */


    strcpy(database_entry, uid);
    strcat(database_entry, ",");
    strcat(database_entry, name);
    strcat(database_entry, ",");
    strcat(database_entry, number);
    strcat(database_entry, ",");
    gtk_clist_get_text( (GtkCList *)service_clist, GPOINTER_TO_INT(GTK_CLIST(service_clist)->selection->data), 0, &sid);
    strcat(database_entry, sid);
    strcat(database_entry, ",");
    strcat(database_entry, smtp_field);
    strcat(database_entry, ",");
    strcat(database_entry, autosplit);
    strcat(database_entry, ",");
    strcat(database_entry, wctp);
    strcat(database_entry, ",");
    strcat(database_entry, snpp);
    strcat(database_entry, ",");
    strcat(database_entry, smtp);
    strcat(database_entry, ",");
    strcat(database_entry, wctp_options);
    strcat(database_entry, ",");
    strcat(database_entry, snpp_options);
    strcat(database_entry, ",0,0,0,0");




/*
 *    Now we need to refresh the clists depending on where we were called from
 *
 *    If we have the rec_data_edit_flag we were called from the lsit_receivers
 *    dialog box so we have to refresh that with the changes as well as the main
 *    clist box on the main screen
 */

    //service_line = g_malloc(MAX_LENGTH);
    service_line = get_uid_line("SERVICE.dat", sid, SERVICE_FIELDS, 0, MAX_LENGTH);
    service_name = get_field(service_line, SERVICE_FIELDS, 2);
    g_free(service_line);

    if (win_data->rec_data->edit_flag || win_data->rec_data->popup_flag)
    {
	strcat(database_entry, "\n");
	edit_item(win_data->rec_data->uid, "RECEIVER.dat", RECEIVER_FIELDS, 1, database_entry, EDIT, ',');

/*
 *   This section of code is not needed when we were called from the popup menu on the main screen 
 */

	if ( win_data->rec_data->edit_flag )
	{
	    row_selected = GPOINTER_TO_INT(GTK_CLIST(list_receiver_clist)->selection->data);
	
	    r_data[0] = g_strdup(name);
	    r_data[1] = g_strdup(number);
	    r_data[2] = g_strdup(smtp_field);
	    r_data[3] = g_strdup(service_name);
	    r_data[4] = g_strdup(uid);
	
	    gtk_clist_unselect_all(GTK_CLIST(list_receiver_clist)); 
	    gtk_clist_remove(GTK_CLIST(list_receiver_clist), row_selected);
	    gtk_clist_append( (GtkCList *)list_receiver_clist, r_data);

	    for ( i = 0; i < 5; i++)
		g_free(r_data[i]);

	}


	main_gui_data[0] = g_strdup(uid);
	main_gui_data[1] = g_strdup(" ");
	main_gui_data[2] = g_strdup(name);
	
	delete_rec_by_id(uid);
	gtk_clist_append( (GtkCList *)receiver_clist, main_gui_data);

	g_free(main_gui_data[0]);
	g_free(main_gui_data[1]);
	g_free(main_gui_data[2]);

	g_free(service_name);
    }
    else
/*
 *    If we get here we are adding a new receiver and we were called from the
 *    main menu or the toolbar so we just have to refresh the main clist on the
 *    main screen
 */
    {
	add_data(database_entry, "RECEIVER.dat");

	main_gui_data[0] = g_strdup(uid);
	main_gui_data[1] = g_strdup(" ");
	main_gui_data[2] = g_strdup(name);
    
	gtk_clist_append( (GtkCList *)receiver_clist, main_gui_data);
	g_free(main_gui_data[0]);
	g_free(main_gui_data[1]);
	g_free(main_gui_data[2]);
    }


/*
 *    If we have the rec_data_refresh_flag we are adding a new record from the
 *    list_receiver dialog box. We need to add the new data to that clist box,
 *    the main clist box has already had the new data added to it
 */


    if ( win_data->rec_data->refresh_list_flag  && !win_data->rec_data->edit_flag )
    {
	r_data[0] = g_strdup(name);
	r_data[1] = g_strdup(number);
	r_data[2] = g_strdup(smtp_field);
	r_data[3] = g_strdup(service_name);
	r_data[4] = g_strdup(uid);

	gtk_clist_append( (GtkCList *)list_receiver_clist, r_data);

	for ( i = 0; i < 5; i++)
	    g_free(r_data[i]);
	g_free(service_name);

    }

    g_free(database_entry);
    g_free(win_data->rec_data);
    g_free(name);
    g_free(number);
    g_free(smtp_field);

    gtk_widget_destroy(win_data->window);
    g_free(win_data);

}


void on_cancel_receiver_button_clicked  (GtkWidget *window, window_data  *user_data)
{
    gtk_widget_destroy(user_data->window);
    g_free(user_data->rec_data);
    g_free(user_data);
}

void on_cancel_receiver_unselect (GtkWidget    *window, window_data *user_data)
{
    extern GtkWidget *list_receiver_clist;

    gtk_clist_unselect_all(GTK_CLIST(list_receiver_clist)); 
    gtk_widget_destroy(user_data->window);
    g_free(user_data->rec_data);
    g_free(user_data);

}










