/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"
#include <ctype.h>

void
on_save_button_clicked                 (GtkButton          *button,
                                       add_service_window  *s_data)
{
    gchar *name_error = "\n  You must enter a name for this service  \n";
    gchar *max_msg_error = "\n  You must enter the maximum length for a message  \n   for this service  \n";
    gchar *snpp_adr_error = "\n  You must enter an snpp address  \n";
    gchar *snpp_port_error = "\n  You must enter an snpp port number  \n";

    gint  id;
    gint  row_selected = -1;
    gint  i, ii;
    gchar *name;
    gchar *wctp_url;
    gchar *max_msg;
    gchar sid[MAX_FIELD_LENGTH];
    gchar wctp[2];
    gchar snpp[2];
    gchar smtp[2];
    gchar password[2];
    gchar *snpp_url;
    gchar *snpp_port = "";    
    gchar *snpp_password;
    gchar *database_entry;
    gchar *list_service_id;
    gchar *data[2];
    gchar *slist_data[6];
    extern GtkWidget *list_service_clist;
    extern GtkWidget *service_clist;

    database_entry = g_malloc(MAX_LENGTH);

/*
 *    The first part of this function we go through all the entry boxes and check buttons, 
 *    getting all the values entered and checking for errors. An error is going to be
 *    an entry box left blank that requires a value. 
 */

    name = g_strdup(gtk_entry_get_text(GTK_ENTRY(s_data->serv_data->service_name)));
    if (is_empty_string(name, name_error))
	return;
    name = clean_string(name);

/*
 *    The number field is now the WCTP url field. First get it working then change the variables
 */

    wctp_url = g_strdup(gtk_entry_get_text(GTK_ENTRY(s_data->serv_data->service_wctp)));
    if ( (strlen(wctp_url)) == 0 )
	strcpy(wctp_url, "none");
    wctp_url = clean_string(wctp_url);

    max_msg = g_strdup(gtk_entry_get_text(GTK_ENTRY(s_data->serv_data->max_msg)));
    if (is_empty_string(max_msg, max_msg_error))
	return;
    max_msg = clean_string(max_msg);

    /*  This is now the WCTP checkbutton */
    if (GTK_TOGGLE_BUTTON (s_data->serv_data->tap_checkbutton)->active)
	strcpy(wctp, "1");
    else
	strcpy(wctp,"0");

    if (GTK_TOGGLE_BUTTON (s_data->serv_data->snpp_checkbutton)->active)
	strcpy(snpp, "1");
    else
	strcpy(snpp,"0");

    if (GTK_TOGGLE_BUTTON (s_data->serv_data->smtp_checkbutton)->active)
	strcpy(smtp, "1");
    else
	strcpy(smtp,"0");

    if (GTK_TOGGLE_BUTTON (s_data->serv_data->password_checkbutton_yes)->active)
	strcpy(password, "1");
    else
	strcpy(password,"0");

    if ( wctp[0] == '1' )
	if ( !strcmp(wctp_url, "none")  )
	{
	    generic_error("\n  You must enter the full Url for the WCTP address  \n", "G-Page Error");
	    return;
	}

    snpp_url = g_strdup(gtk_entry_get_text(GTK_ENTRY(s_data->serv_data->snpp_url)));
    snpp_port = g_strdup(gtk_entry_get_text(GTK_ENTRY(s_data->serv_data->snpp_port)));
    snpp_password = g_strdup(gtk_entry_get_text(GTK_ENTRY(s_data->serv_data->snpp_password)));

    if (GTK_TOGGLE_BUTTON (s_data->serv_data->snpp_checkbutton)->active)
    {
	if (is_empty_string(snpp_url, snpp_adr_error))
	    return;
	snpp_url = clean_string(snpp_url);

/*  We need to keep snpp_url lowercase here because we sort the paging services by the snpp address
    if we don't keep it lowercase it will not sort properly */

	for (ii = 0; *(snpp_url + ii); ii++) 
	    *(snpp_url + ii) = tolower(*(snpp_url + ii));

	if (is_empty_string(snpp_port, snpp_port_error))
	    return;
	snpp_port = clean_string(snpp_port);

	if ( strlen(snpp_password) != 0)
	    snpp_password = clean_string(snpp_password);
	else
	    strcpy(snpp_password, "none");

    }
    else			/* This service does not support snpp so we don't care if the 
				   user has filled in any values. */
    {
	if ( (strlen(snpp_url)) == 0 )
	    strcpy(snpp_url, "none");
	if ( (strlen(snpp_port)) == 0 )
	    strcpy(snpp_port, "none");
	if ( (strlen(snpp_password)) == 0 )
	    strcpy(snpp_password, "none");
    }	


/*
 *   At this point we have all the data the user has entered. Now we need to save the data to the proper
 *   database file and refresh the proper clist boxes.
 *
 *   NOTE: The string that is passed to the add_data() or edit_item() functions should not be terminated
 *         with a newline char. The newline char gets added in those functions.
 *
 */

    if (s_data->serv_data->edit_flag)
    {
	row_selected = GPOINTER_TO_INT(GTK_CLIST(list_service_clist)->selection->data);

	gtk_clist_get_text(GTK_CLIST(list_service_clist), row_selected, 5, &list_service_id);
	strcpy(sid, list_service_id);
    }
    else
    {
	id = get_id("TERMINAL");
	g_snprintf(sid, MAX_FIELD_LENGTH, "%d", id);
    }

    strcpy(database_entry, sid);
    strcat(database_entry, ",");
    strcat(database_entry, name);
    strcat(database_entry, ",");
    strcat(database_entry, wctp_url);
    strcat(database_entry, ",");
    strcat(database_entry, max_msg);
    strcat(database_entry, ",");
    strcat(database_entry, wctp);
    strcat(database_entry, ",");
    strcat(database_entry, snpp);
    strcat(database_entry, ",");
    strcat(database_entry, smtp);
    strcat(database_entry, ",");
    strcat(database_entry, snpp_url);
    strcat(database_entry, ",");
    strcat(database_entry, snpp_port);
    strcat(database_entry, ",");
    strcat(database_entry, snpp_password);
    strcat(database_entry, ",");
    strcat(database_entry, password);
    strcat(database_entry, ",0,0,0,0"); /* These are empty fields */


/*
 *    If we have the edit flag we are editing an entry and we have been called by the list_service
 *    dialog box. In that case we want to edit the database with the altered entry. Then we remove
 *    the edited row from the clist box and add the new one.
 */


    if (s_data->serv_data->edit_flag)
    {
	strcat(database_entry, "\n");
	edit_item(list_service_id, "SERVICE.dat", SERVICE_FIELDS, 1, database_entry, EDIT, ',');

	slist_data[0] = g_strdup(name);
	slist_data[1] = g_strdup(wctp_url);
	slist_data[2] = g_strdup(snpp_url);
	slist_data[3] = g_strdup(max_msg);
	slist_data[4] = g_strdup(max_msg); 
	slist_data[5] = g_strdup(list_service_id);

	gtk_clist_remove( GTK_CLIST(list_service_clist), row_selected);
	gtk_clist_append( (GtkCList *)list_service_clist, slist_data);

	for ( i = 0; i < 6; i++)
	    g_free(slist_data[i]);

	gtk_clist_unselect_all( (GtkCList *)list_service_clist);
    }
	else
	    add_data(database_entry, "SERVICE.dat");

/*
 *    If we have the refresh flag we are adding a new entry from the list_service
 *    dialog box. In this case we simply add the new entry to the list box.
 */


    if(s_data->serv_data->refresh_flag) 
    {
	slist_data[0] = g_strdup(name);
	slist_data[1] = g_strdup(wctp_url);
	slist_data[2] = g_strdup(snpp_url);
	slist_data[3] = g_strdup(max_msg);
	slist_data[4] = g_strdup(max_msg); 
	slist_data[5] = g_strdup(sid);

	gtk_clist_append( (GtkCList *)list_service_clist, slist_data);
	for ( i = 0; i < 6; i++)
	    g_free(slist_data[i]);
    }


/*
 *    If we have the add_rec_refresh_flag we were called from the add_receiver dialog box
 *    We need to add the new entry into that clist and we are done !
 */


    if(s_data->serv_data->add_rec_refresh_flag)
    {
	data[0] = g_strdup(sid);
	data[1] = g_strdup(name);
	
	gtk_clist_append( (GtkCList *)service_clist, data);

	g_free(data[0]);
	g_free(data[1]);
    }


    gtk_widget_destroy(s_data->window);

    g_free(database_entry);
    g_free(name);
    g_free(wctp_url);
    g_free(max_msg);
    g_free(snpp_url);
    g_free(snpp_password);
    g_free(snpp_port);
    g_free(s_data->serv_data);
    g_free(s_data);

}


void
on_cancel_button_clicked               (GtkWidget       *window,
                                        service        *user_data)
{

    gtk_widget_destroy(window);
    if (user_data->edit_flag)
	 gtk_clist_unselect_all(GTK_CLIST(list_service_clist)); 

}

void on_cancel_button_clicked_unselect (GtkWidget       *window, service        *user_data)
{
	 gtk_clist_unselect_all(GTK_CLIST(list_service_clist)); 
}

int is_empty_string(gchar *string, gchar *error_msg)
{
    if ( (strlen(string)) == 0 )
    {
	generic_error(error_msg, "G-Page Error");
	return (TRUE);
    }
    return (FALSE);
}

