/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

static void connect_alarm();

gint server_connection(gint mode)
{
    enum {
	CONNECT_TIMEOUT = 10
    };

  struct  sockaddr_in    address;
  struct  in_addr        inaddr;
  struct  hostent        *host;
  gint                   sock;
  gchar                  *port_data, *domain;
  gchar                  *buffer;
  Sigfunc                *sigfunc;


/*
 *        Get the ip address of the paging server
 */

  if ( (domain = get_resource_data("CONFIG.dat", "ip_address", MAX_FIELD_LENGTH)) == NULL)
  {
      if ( mode == NORMAL )
	  update_status_line("Error: corrupt file: CONFIG.dat");
      return (-1);
  }

/*
 *       Get the port number of the paging server
 */

  if ( (port_data = get_resource_data("CONFIG.dat", "port", MAX_FIELD_LENGTH)) == NULL)
  {
      if ( mode == NORMAL )
	  update_status_line("Error: unable to read port from CONFIG.dat");
      return (-1);
  }


   if (inet_aton(domain, &inaddr))
      host = gethostbyaddr((char *) &inaddr, sizeof(inaddr), AF_INET);
   else
      host = gethostbyname(domain);

   g_free(domain);
   
  if (!host)
  {
      if ( mode == NORMAL )
	  update_status_line("Error looking up host (server) ");
      perror("error looking up host");
      g_free(port_data);
      return(-1);
  }

  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
  {
      if ( mode == NORMAL )
	  update_status_line("Error creating socket ");
      g_free(port_data);
      perror("socket");
      return(-1);
  }

  address.sin_family = AF_INET;

  address.sin_port = htons(atoi(port_data));
  g_free(port_data);
 
  memcpy(&address.sin_addr, host->h_addr_list[0], sizeof(address.sin_addr));

  sigfunc = Signal(SIGALRM, connect_alarm );

  alarm(CONNECT_TIMEOUT);

  if (connect(sock, (struct sockaddr *) &address, sizeof(address)))
  {
      if ( errno == EINTR )
      {
	  if ( mode == NORMAL )
	      update_status_line("Timeout connecting with g-page server");
      }
      else
				/* do nothing */

      alarm(0);
      Signal(SIGALRM, sigfunc);

      return (-1);		/* No connection */
  }
  else
  {
      alarm(0);
      Signal(SIGALRM, sigfunc);

      buffer = g_malloc(MAX_LENGTH);
      read_a_line(sock, buffer, MAX_LENGTH);
      if ( !strcmp( (buffer + 2), "denied\n") )
      {
	  g_free(buffer);
	  close(sock);
	  return (-2); 
      }
      else
      {
	  g_free(buffer);
	  return (sock);		/* We have a connection here */
      }
  }

  return (-1);		/* Never get here */

}
 
void free_the_mem(gchar *one, gchar *two, gchar *three)
{
    g_free(one);
    g_free(two);
    g_free(three);
}

static void connect_alarm()
{
    return;
}
