/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

typedef struct {
    gint mode;
    gint file_handle;
}status_data;

#define max(a,b)        ((a) > (b) ? (a) : (b))

static void *do_page(pg_data  *data);
static gint get_status_line(status_data *user_data);

/*********************************************************************/
/****                                                             ****/
/**** get_status_line will check to see if there is any data      ****/
/**** in the FIFO. If there is some it reads chars until it       ****/
/**** hits a newline, then it sends that to the status line on    ****/
/**** on the gui. If there is no data wating in the pipe we       ****/
/**** return. A return of FALSE turns the timer off, a TRUE       ****/
/**** will reset the timer and leave it running                   ****/
/****                                                             ****/
/*********************************************************************/

static gint get_status_line(status_data *user_data)
{
    extern GtkWidget *text_entry;

    gint   file_handle;
    gint   return_value;
    gint   txt_length;

    gchar  *buffer = "";
    gchar  *clear_mode = "";

    buffer = g_malloc(MAX_LENGTH);
    file_handle = user_data->file_handle;
    
    return_value = read_cached_line(file_handle, buffer, MAX_LENGTH);
    if ( !strcmp(buffer, "done") )
    {
	close(file_handle);
	g_free(buffer);
	g_free(user_data);
	return(FALSE);
    }
    switch (return_value)
    {
    case -1:
	g_free(buffer);
	if ( user_data->mode == NORMAL )
	    update_status_line("Error reading staus: (select)");
	g_free(user_data);
	return (FALSE);
	break;

    case 0:
	g_free(buffer);
	return(TRUE);
	break;

    case 1:	    
	if ( !strcmp(buffer, "sent") )
	{
	    if ( user_data->mode == NORMAL )
	    {
		if ( (clear_mode = get_resource_data("CONFIG.dat", "clear_msg_box", MAX_FIELD_LENGTH)) )
		{
		    if ( !strcmp(clear_mode, "success") )
		    {
			txt_length = gtk_text_get_length( GTK_TEXT(text_entry) );
			gtk_text_freeze( GTK_TEXT(text_entry) );
			gtk_text_set_point(GTK_TEXT(text_entry), 0);
			gtk_text_forward_delete(GTK_TEXT(text_entry), txt_length);
			gtk_text_thaw( GTK_TEXT(text_entry) );
		    }
		    g_free(clear_mode);
		}
		else
		    update_status_line("Error reading CONFIG.dat");
	    }
	}
	else if ( !strcmp(buffer, "failed") )
	{
	       /* Do nothing */
	}
	else if ( !strstr(buffer, "16Elapsed") ) /* Kludge  */
	{
	    if ( user_data->mode == NORMAL )
	    {
		update_status_line(buffer);
		update_status_line_small(" ");
	    }
	}
	
	else
	    if ( user_data->mode == NORMAL )
		update_status_line_small( (buffer + 2) );

	g_free(buffer);
	return(TRUE);
	break;
    }
    g_free(buffer);
    g_free(user_data);
    return(FALSE);		/* Should never get here */
}
    
/*********************************************************************/
/****                                                             ****/
/****         Write the status to a named pipe for the client     ****/
/****              to display on the status line                  ****/          
/****                                                             ****/
/*********************************************************************/
/*                                                                   */
/*         named_pipe is a global variable                           */

void write_to_status_line(char *status)
{
    extern char       named_pipe[32];
    extern gint       daemon_flag;

    gint num_written, handle;


    if ( daemon_flag )
	return;

    if ( (handle = open(named_pipe, O_WRONLY)) == -1 )
    {
	perror("Error opening named pipe");
	return;
    }
    num_written = write(handle, status, strlen(status)); 
    num_written = write(handle, "\n", 1); 

    close(handle);

    return;
}

/*********************************************************************/
/****                                                             ****/
/****         Catch any child processes when they die             ****/
/****                                                             ****/
/*********************************************************************/

void sig_child(int signo)
{
    pid_t     pid;
    gint       stat;

    while( (pid = waitpid(-1, &stat, WNOHANG)) > 0)
	debug_data("Gui child %d has terminated\n",(int ) pid);
    return;
}


/*********************************************************************/
/****                                                             ****/
/****      connect_to_server function  send the page data         ****/
/****                                                             ****/
/*********************************************************************/

void connect_to_server(uid_list *active_list, gchar *msg, gchar *sched_msg, sched_data *sched_obj, int mode, gchar *tracking_number, gint file_desc)
{
  extern GtkWidget *text_entry;
  extern gint      daemon_flag;
  extern gint      command_line_flag;

  gint         txt_length = 0;
  gint         quick;
  gint         sock = 0;
  gint         thread_status;
  gchar        *status_mode = "";
  gchar        *clear_mode = "";
  pg_data      *page_data;
  pthread_t    page_thread;
  status_data  *mpointer;
  void         *thread_result;
  

  if ( mode == NORMAL )
      update_status_line("Attempting to resolve server address");

  sock = server_connection(mode);

  if ( sock == -1 )
  {
      if ( mode == NORMAL )
	  update_status_line("Unable to connect with server");
      if ( file_desc != -1 )
	  close(file_desc);
      return;
  }
  if ( sock == -2 )
  {
      if ( mode == NORMAL )
	  update_status_line("Permision denied");
      if ( file_desc != -1 )
	  close(file_desc);
      return;
  }
  else
      if ( mode == NORMAL )
	  update_status_line("Connect with server...");


  if ( (status_mode = get_resource_data("CONFIG.dat", "status_mode", MAX_FIELD_LENGTH)) )
  {
      if (!strcmp(status_mode, "no"))
	  quick = 1;
      else 
	  quick = 0;
      
      g_free(status_mode);
  }
  else
  {
      if ( mode == NORMAL )
	  update_status_line("Error: corrupt file: CONFIG.dat");
      close(sock);
      return;
  }



  if ( (mode != TRACK_WCTP) && (mode != TRACK_SNPP) )
  {
      tracking_number = g_malloc(MAX_FIELD_LENGTH);
      tracking_number[0] = '\0';
  }

  page_data = g_malloc(sizeof(*page_data));

  page_data->sock = sock;
  page_data->active_uids = active_list;
  page_data->msg = msg;
  page_data->tracking_number = tracking_number;
  page_data->sched_msg = sched_msg;
  page_data->sched_obj = sched_obj;
  page_data->mode = mode;
  page_data->fd = file_desc;

  thread_status = pthread_create(&page_thread, NULL, (void *)do_page, page_data);

  if ( mode == NORMAL )
  {
      if ( (clear_mode = get_resource_data("CONFIG.dat", "clear_msg_box", MAX_FIELD_LENGTH)) )
      {
	  if ( !strcmp(clear_mode, "always") )
	  {
	      txt_length = gtk_text_get_length( GTK_TEXT(text_entry) );
	      gtk_text_freeze( GTK_TEXT(text_entry) );
	      gtk_text_set_point(GTK_TEXT(text_entry), 0);
	      gtk_text_forward_delete(GTK_TEXT(text_entry), txt_length);
	      gtk_text_thaw( GTK_TEXT(text_entry) );
	  }
	  g_free(clear_mode);
      }
      else
	  update_status_line("Error: corrupt file: CONFIG.dat ");
  }


  if ( !daemon_flag )
  {
      mpointer = g_malloc(sizeof(*mpointer));
      mpointer->mode = mode;

      if ( (mpointer->file_handle = open(named_pipe, O_RDONLY)) < 0 )
      {
	  info_msg("Error opening named pipe in new thread: %s ", named_pipe);
	  if ( mode == NORMAL )
	      update_status_line("Error opening pipe - Unable to connect with server");
	  return;
      }

/*
 *  Set a timer to check for data to send to the status line
 */
      if (quick)
	  gtk_timeout_add(QUARTER_SECOND, (GtkFunction)get_status_line, (gpointer) mpointer);
      else
	  gtk_timeout_add(THREE_QUARTER_SECOND, (GtkFunction)get_status_line, (gpointer) mpointer);
  }


  if ( command_line_flag )
  {
      pthread_join(page_thread, &thread_result);
  }

}



static void *do_page(pg_data  *data)
{
    extern  gint             live_page;
    extern  pthread_mutex_t  update_live_page_mutex;
    extern  gint             command_line_flag;

    gint                     page_result;
    gint                     mutex_status;
    gint                     status;


    if ( !command_line_flag )	/* We are waiting for the thread to exit when called from the command line */
    {
	status = pthread_detach(pthread_self());
	if ( status != 0 )
	    debug_data("Error detaching thread in do_page\n");
    }
    
    if ( (mutex_status = pthread_mutex_lock(&update_live_page_mutex)) != 0 )
	debug_data("Error locking mutex - possible data corruption");
    //perror("Error locking mutex - possible data corruption");
    live_page++;
    if ( (mutex_status = pthread_mutex_unlock(&update_live_page_mutex)) != 0 )
	debug_data("Error unlocking mutex - possible data corruption");
    //perror("Error unlocking mutex - possible data corruption");
    
    debug_data("I'm sending a page, live page = %d\n", live_page);

    page_result = send_page(data->sock, data->active_uids, data->msg, data->sched_msg, data->sched_obj, data->mode, data->tracking_number, 
			    data->fd);
    debug_data("I got back from send_page in the do page routine\n");

    if (page_result == -1)
	write_to_status_line("Permission denied");

    if ( (mutex_status = pthread_mutex_lock(&update_live_page_mutex)) != 0 )
	debug_data("Error locking mutex - possible data corruption");
	//perror("Error locking mutex - possible data corruption");
    live_page--;
    if ( (mutex_status = pthread_mutex_unlock(&update_live_page_mutex)) != 0 )
	debug_data("Error unlocking mutex - possible data corruption");
    //perror("Error unlocking mutex - possible data corruption");

    close(data->sock);
    
    /* Check this, it might not always be allocated */
    g_free(data->tracking_number);

    g_free(data->msg);
    g_free(data);
    debug_data("All done if the do_page routine - I'm going to call pthread_exit\n");
    pthread_exit(NULL);

    return(NULL);
}



