/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"


#define NUMBER_OF_ARGUMENTS 25
#define MAX_RESPONSE_LENGTH 1000

static gint check_for_response(gchar two_way_response[5][MAX_RESPONSE_LENGTH]);

void die(char *message)
{
  perror(message);
  _exit(EXIT_FAILURE);
}

int send_page(int socket_fd, uid_list *active_list, char *message, gchar *sched_msg, sched_data *sched_obj, gint pmode, gchar *tracking_number, gint fd)
{
 gchar   *in_buff;
 gint    index =0;
 gint    num_pages = 0;
 gint    n_pages = 0;
 gint    page_count = 0;
 gchar   *receiver_line = NULL;
 gchar   *service_line = NULL;
 gchar   *service_list_data[SERVICE_FIELDS], *receiver_list_data[RECEIVER_FIELDS];
 gint    return_value = 0;
 gint    i;
 gint    default_server = 0;
 gint    responses = 0;
 gint    rollover_no_connect = 0;
 gchar   *isp_domain;
 gchar   *isp_email_domain;
 gchar   *email_adr;
 gchar   *final_msg;
 gchar   *mode = "0\n";		/* Note this is the number zero not the letter oh */
 gchar   *serial_mode;
 gchar   *sched_data;
 gchar   protocols[MAX_LENGTH];
 gchar   *send_order;
 gchar   *cc_email = NULL;
 gchar   *qpage_mode;
 gchar   *qpage_adr = NULL;
 gchar   *qpage_port = NULL;
 gchar   *auto_rollover = NULL;
 gchar   *auto_dial = NULL;
 gchar   *time_stamp;
 gchar   two_way_response[5][MAX_RESPONSE_LENGTH];
 uid_list *head; 

 struct {
     gchar *sig;
     gchar *sig_yes_no;
     gchar *sig_place;
 }prefs;

/*********************************************************************/

 if ( tracking_number == NULL )
     tracking_number[0] = '\0';

 responses = check_for_response(two_way_response);

 i = 0;

 if ( (prefs.sig = get_resource_data("CONFIG.dat", "sig", MAX_FIELD_LENGTH)) == NULL )
     prefs.sig = g_strdup(" ");

 if ( (prefs.sig_yes_no = get_resource_data("CONFIG.dat", "msg_sig", MAX_FIELD_LENGTH)) == NULL )
     prefs.sig_yes_no = g_strdup(" ");

 if ( (prefs.sig_place = get_resource_data("CONFIG.dat", "sig_placement", MAX_FIELD_LENGTH)) == NULL )
     prefs.sig_place = g_strdup("after");

 if ( (cc_email = get_resource_data("CONFIG.dat", "cc_email", MAX_FIELD_LENGTH)) == NULL)
     cc_email = g_strdup("no");

 if ( (send_order = get_resource_data("CONFIG.dat", "order_of_protocols", MAX_FIELD_LENGTH)) == NULL )
     send_order = g_strdup("123");

 if ( (qpage_mode = get_resource_data("CONFIG.dat", "qpage_active", MAX_FIELD_LENGTH)) == NULL )
     qpage_mode = g_strdup("no");

 if ( (serial_mode = get_resource_data("CONFIG.dat", "serial_mode", MAX_FIELD_LENGTH)) == NULL )
     serial_mode = g_strdup("yes");

 if ( (auto_rollover = get_resource_data("CONFIG.dat", "rollover_no_connect", MAX_FIELD_LENGTH)) == NULL)
     auto_rollover = g_strdup("no");

 if ( (auto_dial = get_resource_data("CONFIG.dat", "dial_no_connect", MAX_FIELD_LENGTH)) == NULL )
     auto_dial = g_strdup("no");

 if ( !strcmp(serial_mode, "yes") )
     strcpy(serial_mode, "0");	/* This is a zero */
 else
     strcpy(serial_mode, "1");

 if ( (!strcmp(qpage_mode, "yes")) || (!strcmp(auto_rollover,"yes")) )
 {    
     if ( (qpage_adr = get_resource_data("CONFIG.dat", "default_address", MAX_FIELD_LENGTH)) == NULL )
	 qpage_adr = g_strdup("localhost");

     if ( (qpage_port = get_resource_data("CONFIG.dat", "default_port", MAX_FIELD_LENGTH)) == NULL )
	 qpage_port = g_strdup("444");
 }

 if ( !strcmp(auto_rollover,"yes")  )
     rollover_no_connect = 1;
 else
     rollover_no_connect = 0;

 if ( !strcmp(qpage_mode,"yes")  )
     default_server = 1;
 else
     default_server = 0;

 g_free(qpage_mode);
 g_free(auto_rollover);
 
 if ( !strcmp(prefs.sig_yes_no, "yes") )
 {
     if ( !strcmp(prefs.sig_place, "before") )
	 final_msg = g_strdup_printf("%s%s", prefs.sig, message);
     else
	 final_msg = g_strdup_printf("%s%s",message, prefs.sig);
 }
 else
     final_msg = g_strdup(message);
 
 if ( (time_stamp = get_resource_data("CONFIG.dat", "time_stamp", MAX_FIELD_LENGTH)) )
 {
     if ( !strcmp(time_stamp, "yes") )
	 strcpy(time_stamp, "1");
     else
	 strcpy(time_stamp, "0");
 }
 else
     time_stamp = g_strdup("0");

 g_free(prefs.sig);
 g_free(prefs.sig_yes_no);
 g_free(prefs.sig_place);

 in_buff = g_malloc(MAX_LENGTH); /* freed */
 sched_data = g_malloc(MAX_LENGTH); /* freed */

/*
 *   Send the mode to the server
 */

#ifdef DEBUG
      fprintf(stderr, "Sending the mode to the server\n");
#endif

 if ( ! ( commit_transaction(socket_fd, mode) ) )
     return (-1); 

#ifdef DEBUG
      fprintf(stderr, "Sent the mode to the server\n");
#endif

/*
 *  Get the isp domain, email domain, and email address from the resource file 
 *  and write send them to the server
 */

 if ( (isp_domain = get_resource_data("CONFIG.dat", "isp_domain", MAX_FIELD_LENGTH)) == NULL )
 {
     if ( pmode == NORMAL )
     {
	 update_status_line("Error: corrupt file: CONFIG.dat");
	 update_status_line("done");
      }	 
	 return(0);
 }
 isp_domain = g_realloc(isp_domain, (strlen(isp_domain) + 2));
 strcat(isp_domain, "\n");

#ifdef DEBUG
      fprintf(stderr, "Sending the isp to the server\n");
#endif

 if ( ! ( commit_transaction(socket_fd, isp_domain) ) )
     return (0);

  if ( (isp_email_domain = get_resource_data("CONFIG.dat", "isp_email_domain", MAX_FIELD_LENGTH)) == NULL )
  {
      if ( pmode == NORMAL )
      {
	  update_status_line("Error: corrupt file: CONFIG.dat");
	  update_status_line("done");
      }
      return(0);
  }
  isp_email_domain = g_realloc(isp_email_domain, (strlen(isp_email_domain) + 2));
  strcat(isp_email_domain, "\n");

#ifdef DEBUG
      fprintf(stderr, "Sending the email domain to the server\n");
#endif

 if ( ! ( commit_transaction(socket_fd, isp_email_domain) ) )
     return (0);

#ifdef DEBUG
      fprintf(stderr, "Sent the email domain to the server\n");
#endif

  if ( (email_adr = get_resource_data("CONFIG.dat", "email_adr", MAX_FIELD_LENGTH)) == NULL)
  {
      if ( pmode == NORMAL )
      {
	  update_status_line("Error: corrupt file: CONFIG.dat");
	  update_status_line("done");
      }
      return(0);
  }
  email_adr = g_realloc(email_adr, (strlen(email_adr) + 2));
  strcat(email_adr, "\n");

#ifdef DEBUG
      fprintf(stderr, "Sending the email address to the server\n");
#endif

 if ( ! ( commit_transaction(socket_fd, email_adr) ) )
     return (0);

#ifdef DEBUG
      fprintf(stderr, "g_free the mallocs\n");
#endif


 g_free(isp_domain);
 g_free(isp_email_domain);
 g_free(email_adr);


/*
 *   Tell the server how many pages we are sending
 */
#ifdef DEBUG
      fprintf(stderr, "Sending the number of pages to the server\n");
#endif


 head = active_list;

 while (active_list->next)
 {
     num_pages++;
     active_list = active_list->next;
 }
 n_pages = num_pages;
 active_list = head;

 memset(in_buff, '\0', MAX_LENGTH);
 g_snprintf(in_buff, MAX_LENGTH, "%d\n", num_pages);

 if ( ! ( commit_transaction(socket_fd, in_buff) ) )
     return (0);

/*
 *   This is the main loop for sending the pages to the server
 */

 while(num_pages > 0)
 {

/*
 *   First we must get the data from RECEIVER.dat for the user id we are going to send a message to
 */

     receiver_line = get_uid_line("RECEIVER.dat", active_list->uid, RECEIVER_FIELDS, 0, MAX_LENGTH);

     if ( receiver_line )
     {
	 for(i = 0; i < RECEIVER_FIELDS; i++)
	     receiver_list_data[i] = g_malloc(MAX_LENGTH);
     }
     else
     {
	 if ( pmode == TRACK_WCTP )
	     generic_error("\n Unable to find Receiver in database \n  Have you changed database locations? \n", " G-Page Error ");
	 else
	     write_to_status_line("Error: Corrupt database - RECEIVER.dat");

	 write_to_status_line("done");
	 g_free(receiver_line);
	 g_free(service_line);
	 close(socket_fd);
	 return(0);
     }

/*
 *   Now we load each field from the users data into a seperate array 
 */

     return_value = get_fields_from_string(receiver_list_data, receiver_line, RECEIVER_FIELDS, ',');
     if (return_value == 1)
     {
	 if ( pmode == TRACK_WCTP )
	     generic_error("\n Unable to find Receiver in database \n  Have you changed database locations? \n", " G-Page Error ");
	 else
	     write_to_status_line("Error: Corrupt database - RECEIVER.dat");

	 write_to_status_line("done");
	 g_free(receiver_line);
	 g_free(service_line);
	 for(i = 0; i < RECEIVER_FIELDS; i++)
	     g_free(receiver_list_data[i]);
	 close(socket_fd);
	 return(0);

     }

/*
 *   We take the id for the service of this pager and get the data for it from SERVICE.dat
 */

	 service_line = get_uid_line("SERVICE.dat", receiver_list_data[3], SERVICE_FIELDS, 0, MAX_LENGTH);
	 
	 for(i = 0; i < SERVICE_FIELDS; i++)
	     service_list_data[i] = g_malloc(MAX_LENGTH);

/*
 *   Load each field from the service data into a seperate array
 */
	 return_value = get_fields_from_string(service_list_data, service_line, SERVICE_FIELDS, ',');
	 if (return_value == 1)
	 {
	     if ( pmode == TRACK_WCTP )
		 generic_error("\n Unable to find paging service in database \n  Corrupt database? \n", " G-Page Error ");
	     else
		 write_to_status_line("Error: Corrupt database - SERVICE.dat");

	     write_to_status_line("done");
	     g_free(receiver_line);
	     g_free(service_line);
	     for(i = 0; i < RECEIVER_FIELDS; i++)
		 g_free(receiver_list_data[i]);
	     for(i = 0; i < SERVICE_FIELDS; i++)
		 g_free(service_list_data[i]);
	     
	     return (0);
	 }
     

/*
 *   Send the data to the server for each receiver
 */

     for( index = 0; index < NUMBER_OF_ARGUMENTS; index++)
     {
	 switch (index)
	 {
	 case 0:
	     /* user id */
	     load_string(in_buff, active_list->uid);
	     break;

	 case 1:
	     /* receiver_number */
	     load_string(in_buff, receiver_list_data[REC_NUMBER]);
	     break;

	 case 2:
	     /* The wctp url for this service */
	     load_string(in_buff, service_list_data[SERVICE_WCTP_URL]);
	     break;

	 case 3:
	     /* smtp address */
	     load_string(in_buff, receiver_list_data[REC_EMAIL_ADR]);
	     break;

	 case 4:
	     /* snpp url */
	     if ( default_server == 0 )
		 load_string(in_buff, service_list_data[SERVICE_SNPP_URL]); 
	     else
		 load_string(in_buff, qpage_adr); 
	     break;
	 case 5:
	     /* snpp port */
	     if ( default_server == 0 )
		 load_string(in_buff, service_list_data[SERVICE_SNPP_PORT]);
	     else
		 load_string(in_buff, qpage_port);
	     break;

	 case 6:
	     /* snpp password */
	     load_string(in_buff, service_list_data[SERVICE_SNPP_PASS]);
	     break;

	 case 7:
	     /* max_msg_length */
	     load_string(in_buff, service_list_data[SERVICE_MAX_MSG]);
	     break;

	 case 8:
	     /* auto split flag */
	     load_string(in_buff, receiver_list_data[REC_AUTOSPLIT]);
	     break;

	 case 9:
	     /* receiver_name */
	     load_string(in_buff, receiver_list_data[REC_NAME]);
	     break;

	 case 10:
	     /* page message */
	     load_string(in_buff, final_msg);
	     break;

	 case 11:
	     /* Is there a password? */
	     load_string(in_buff,service_list_data[SERVICE_PASS_YES_NO]);
	     break;

	 case 12:
	     /* sscheduled message data */
	     load_string(in_buff, sched_msg);
	     break;

	 case 13:
	     /* Protocols this receiver supports */
	     if ( default_server == 0 )
	     {
		 g_snprintf(protocols, MAX_LENGTH, "%s%s%s%s%s", receiver_list_data[REC_WCTP], receiver_list_data[REC_SNPP], 
			    receiver_list_data[REC_SMTP], receiver_list_data[REC_WCTP_OPTIONS],receiver_list_data[REC_SNPP_OPTIONS]);
		 load_string(in_buff, protocols);
	     }
	     else
		 load_string(in_buff, "0100"); /* This just activates the snpp protocol */
	     break;

	 case 14:		/* The order , by protocol, that the pages will be attempted */
	     load_string(in_buff, send_order);
	     break;

	 case 15:
	     load_string(in_buff, cc_email);
	     break;
	     
	 case 16:
	     load_string(in_buff, serial_mode);	/* This is zero for serial and one for asynch mode */
	     break;

	 case 17:
	     load_string(in_buff, time_stamp); /* A 1 means we have a time stamp, a zero means no stamp */
	     break;

	 case 18:
	     load_string(in_buff, two_way_response[0]);
	     break;

	 case 19:
	     load_string(in_buff, two_way_response[1]);
	     break;

	 case 20:
	     load_string(in_buff, two_way_response[2]);
	     break;

	 case 21:
	     load_string(in_buff, tracking_number);
	     break;

	 case 22:
	     if (rollover_no_connect)
		 load_string(in_buff, qpage_adr);
	     else
		 load_string(in_buff, "NULL");
	     break;

	 case 23:
	     if (rollover_no_connect)
		 load_string(in_buff, qpage_port);
	     else
		 load_string(in_buff, "NULL");
	     break;

	 case 24:		/* This is a "yes" or "no" */
	     load_string(in_buff, auto_dial);
	     break;

	 } /* End of the switch statement */
/* 
 *   Write the data to the socket, read it back then go on with the loop
 */

#ifdef DEBUG
      fprintf(stderr, "Sending the data to the server\n");
#endif

	 if ( ! ( commit_transaction(socket_fd, in_buff) ) )
	     return (0);

     } /* This is the end of the for loop */
/* 
 *  Free the mallocs !!
 */
     g_free(receiver_line);
     g_free(service_line);
     for(i = 0; i < RECEIVER_FIELDS; i++)
	 g_free(receiver_list_data[i]);
     for(i = 0; i < SERVICE_FIELDS; i++)
	 g_free(service_list_data[i]);
     
     if ( strcmp(sched_msg, "none") )
     {
	 read_a_line(socket_fd, sched_data, MAX_LENGTH);
	 log_schedule_data(sched_data, sched_obj->sch_time, sched_obj->sch_msg, active_list->uid);
     }

     num_pages--;
     page_count++;

     head = active_list;
     active_list = active_list->next;
     g_free(head->uid);
     g_free(head);
     debug_data("I'm at the end of the main while loop in client_send\n");
 } /* End of the main while loop */
 
 debug_data("I'm out of the main while loop in client_send\n");
 g_free(active_list->uid);
 g_free(active_list);

 g_free(serial_mode);
 g_free(qpage_adr);
 g_free(qpage_port);
 g_free(sched_data);
 g_free(time_stamp);
 g_free(auto_dial);
 g_free(cc_email);
 g_free(send_order);

 g_free(in_buff);

/*
 *   Tell the user we sent everything okay then go wait for the server
 *   to send us status updates while it sends the messages
 */
 debug_data("I'm going to write to status line - right before i call monitor message transactions\n");
 write_to_status_line("Message sent to server for processing... ");
 debug_data("I just got back from write to status line - right before i call monitor message transactions\n");

 debug_data("I'm going to call monitor_message_transactions in client_send\n");
 monitor_message_transactions(socket_fd, final_msg, fd, pmode);
 debug_data("I'm back from monitor_message_transactions in client_send\n");

 return (1);
}

int commit_transaction(int socket_fp, char *buffer)
{
    int send_num = 0;
    int read_num = 0;
    char *read_buffer;

    read_buffer = g_malloc(MAX_LENGTH);
    send_num = write_num(socket_fp, buffer, strlen(buffer));
    if ( send_num != strlen(buffer) )
    {
        info_msg("Error writing to server: commit_transaction\n");
	//update_status_line("Error communicating with server");
	g_free(read_buffer);
	return 0;
    } 
    read_num = read_a_line(socket_fp, read_buffer, send_num);
    if ( !strcmp(read_buffer, "denied") )
	return 0;

    if (read_num < 0)
    {
        info_msg("Error reading from server: commit_transaction\n");
	//update_status_line("Error communicating with server");
	g_free(read_buffer);
	return 0;
    }
    g_free(read_buffer);
    return 1;
}

void log_for_debug(char *buffer)
{
    FILE *fp;

    if ( (fp = get_file_pointer("debug.out", "a")) == NULL)
	return;

    fputs(buffer, fp);
    fclose(fp);
}

void load_string(char *buffer, char data_to_copy[MAX_LENGTH])
{
    memset (buffer, '\0', MAX_LENGTH);
    strcpy(buffer, data_to_copy);
    strcat(buffer, "\n");
return;
}

void log_schedule_data(gchar *time_data, gchar *sch_id, gchar *sch_msg, gchar *receiver)
{
    FILE   *fp;
    gchar  *buffer;

    if  ( ( fp = get_file_pointer("SCHED_DB.dat", "a+")) == NULL)
    {
	info_msg("Error opening SCHED_DB.dat log_schedule_data client_send.c\n");
	return;
    }

    time_data[strlen(time_data) -1] = '\0';
    buffer = g_strdup_printf("%s %s %s %s %s\n", receiver, (time_data + 4), sch_id, "0", sch_msg);
    fputs(buffer, fp);
    fclose(fp);

    g_free(buffer);


}

static gint check_for_response(gchar two_way_response[5][MAX_RESPONSE_LENGTH])
{
    extern gchar *response_one;
    extern gchar *response_two;
    extern gchar *response_three;

    strcpy(two_way_response[0], "");
    strcpy(two_way_response[1], "");
    strcpy(two_way_response[2], "");

    if ( response_one == NULL  )
	return 0;
    else
	strcpy(two_way_response[0], response_one);

    if ( response_two == NULL  )
	return 1;
    else
	strcpy(two_way_response[1], response_two);

    if ( response_three ==  NULL  )
	return 2;
    else
	strcpy(two_way_response[2], response_three);

    return 3;

}

