/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"
#include "logo.xpm" 

static void catch_click (GtkWidget *widget, gint row, gint col, GdkEvent *event, gpointer data);

void generic_error(gchar *error_string, gchar *window_title)
{
  GtkWidget *window;
  GtkWidget *ok_button;
  GtkWidget *label;

  window = gtk_dialog_new();
  gtk_window_set_title (GTK_WINDOW (window), window_title);
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
  label = gtk_label_new(error_string);
  gtk_box_pack_start( GTK_BOX(GTK_DIALOG (window)->vbox), label, TRUE, TRUE, 0);
  ok_button = gtk_button_new_with_label("   OK   ");
  gtk_signal_connect_object( GTK_OBJECT(ok_button), "clicked",
		      GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(window));
  gtk_box_pack_start( GTK_BOX(GTK_DIALOG (window)->action_area), ok_button, TRUE, FALSE, 0);
  gtk_widget_show(ok_button);
  gtk_widget_show(label);
  gtk_widget_grab_focus(ok_button);
  gtk_window_set_modal(GTK_WINDOW(window), TRUE);
  gtk_widget_show(window);
}


void notify_user()
{
  GtkWidget *window;
  GtkWidget *ok_button;
  GtkWidget *label;
  gchar     *notify = "\n Welcome to g-page! \n\n g-page has created a new directory in your home directory called .gpage \n All your database files will be stored here. If you are upgrading and wish to use \n a database in a different directory you can do so by clicking on the Pref button \n on the main toolbar and selecting a new directory \n";

  window = gtk_dialog_new();
  gtk_window_set_title (GTK_WINDOW (window), "Attention");
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
  label = gtk_label_new(notify);
  gtk_box_pack_start( GTK_BOX(GTK_DIALOG (window)->vbox), label, TRUE, TRUE, 0);
  ok_button = gtk_button_new_with_label("   OK   ");
  gtk_signal_connect_object( GTK_OBJECT(ok_button), "clicked",
		      GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(window));
  gtk_box_pack_start( GTK_BOX(GTK_DIALOG (window)->action_area), ok_button, TRUE, FALSE, 0);
  gtk_widget_show(ok_button);
  gtk_widget_show(label);
  gtk_widget_grab_focus(ok_button);
  gtk_window_set_modal(GTK_WINDOW(window), TRUE);
  gtk_widget_show(window);
}

void about_box()
{
  GtkWidget *window;
  GtkWidget *ok_button;
  GtkWidget *label1, *label2, *label3;
  GdkPixmap *about_pix;
  GdkBitmap *mask;
  GtkStyle  *style;
  GtkWidget *about_pic;


  window = gtk_dialog_new();
  gtk_widget_realize (window);
  style = gtk_widget_get_style(window);

  gtk_window_set_title(GTK_WINDOW (window), "About g-page");

 about_pix = gdk_pixmap_create_from_xpm_d( window->window,  &mask,
                                        &style->bg[GTK_STATE_NORMAL],
                                        (gchar **)logo_xpm);
 about_pic = gtk_pixmap_new ( about_pix, mask );


 label1 = gtk_label_new(" ");
 label2 = gtk_label_new(" ");
 label3 = gtk_label_new("  G-Page Version 2.0 \n  Author: Hiro Protagonist\n Copyright (C) 2001 \n  Bugs: H_Protagonist@users.sourceforge.net \n 
 http://g-page.sourceforge.net \n
Released under the GNU Public License");

  gtk_box_pack_start( GTK_BOX (GTK_DIALOG (window)->vbox), label2, TRUE, TRUE, 0);
  gtk_box_pack_start( GTK_BOX (GTK_DIALOG (window)->vbox), about_pic, TRUE, TRUE, 0);
  gtk_box_pack_start( GTK_BOX (GTK_DIALOG (window)->vbox), label1, TRUE, TRUE, 0);
  gtk_box_pack_start( GTK_BOX (GTK_DIALOG (window)->vbox), label3, TRUE, TRUE, 0);

  ok_button = gtk_button_new_with_label("   OK   ");
  gtk_signal_connect_object( GTK_OBJECT(ok_button), "clicked",
                      GTK_SIGNAL_FUNC(gtk_widget_destroy),GTK_OBJECT(window));
  gtk_box_pack_start( GTK_BOX (GTK_DIALOG (window)->action_area), ok_button, TRUE, FALSE, 0);
  gtk_widget_grab_focus(ok_button);
  gtk_widget_show_all(window);
}                                          


GtkWidget*
create_ok_delete (void)
{
    GtkWidget *ok_delete;
    GtkWidget *vbox14;
    GtkWidget *label35;
    GtkWidget *hseparator2;
    GtkWidget *hbuttonbox9;
    GtkWidget *confirm_okay;
    GtkWidget *confirm_cancel;
    extern    GtkWidget *list_service_clist;

/*
 *   Make sure the user has selected a service to delete
 */

    if (GTK_CLIST(list_service_clist)->selection == NULL)
    {
	generic_error("\n  You must select a service to delete \n", "G-Page Error");
	return NULL;
    }

    ok_delete = gtk_window_new (GTK_WINDOW_DIALOG);
    gtk_object_set_data (GTK_OBJECT (ok_delete), "ok_delete", ok_delete);
    gtk_window_set_title (GTK_WINDOW (ok_delete), "Confirm Delete");
    gtk_window_set_modal (GTK_WINDOW (ok_delete), TRUE);
    gtk_window_set_policy (GTK_WINDOW (ok_delete), FALSE, FALSE, FALSE);
    
    vbox14 = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox14);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "vbox14", vbox14,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox14);
    gtk_container_add (GTK_CONTAINER (ok_delete), vbox14);
    
    label35 = gtk_label_new ("\n   Deleting this service will also delete  \n  all receivers associated with it.  \n  Do you wish to continue ?  \n");
    gtk_widget_ref (label35);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "label35", label35,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (label35);
    gtk_box_pack_start (GTK_BOX (vbox14), label35, FALSE, FALSE, 0);
    
    hseparator2 = gtk_hseparator_new ();
    gtk_widget_ref (hseparator2);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hseparator2", hseparator2,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hseparator2);
    gtk_box_pack_start (GTK_BOX (vbox14), hseparator2, TRUE, TRUE, 0);
    
    hbuttonbox9 = gtk_hbutton_box_new ();
    gtk_widget_ref (hbuttonbox9);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hbuttonbox9", hbuttonbox9,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hbuttonbox9);
    gtk_box_pack_start (GTK_BOX (vbox14), hbuttonbox9, TRUE, TRUE, 0);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox9), GTK_BUTTONBOX_SPREAD);
    
    confirm_okay = gtk_button_new_with_label ("OK");
    gtk_widget_ref (confirm_okay);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_okay", confirm_okay,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_okay);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_okay);
    GTK_WIDGET_SET_FLAGS (confirm_okay, GTK_CAN_DEFAULT);
    
    confirm_cancel = gtk_button_new_with_label ("Cancel");
    gtk_widget_ref (confirm_cancel);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_cancel", confirm_cancel,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_cancel);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_cancel);
    GTK_WIDGET_SET_FLAGS (confirm_cancel, GTK_CAN_DEFAULT);
    
    gtk_signal_connect_object (GTK_OBJECT (confirm_okay), "clicked",
			       GTK_SIGNAL_FUNC (on_list_delete_service_clicked),
			       GTK_OBJECT(ok_delete));

    gtk_signal_connect_object( GTK_OBJECT(confirm_cancel), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_widget_grab_focus(confirm_cancel);
    return ok_delete;
}



GtkWidget *generic_confirm_delete (GtkWidget *selection_clist, gchar *delete_function, gchar *id)
{
    GtkWidget *ok_delete;
    GtkWidget *vbox14;
    GtkWidget *label35;
    GtkWidget *hseparator2;
    GtkWidget *hbuttonbox9;
    GtkWidget *confirm_okay;
    GtkWidget *confirm_cancel;

/*
 *   Make sure the user has selected something to delete
 */
    if ( id == NULL )
	if (GTK_CLIST(selection_clist)->selection == NULL)
	{
	    generic_error("\n  You have not selected anything to delete \n", "G-Page Error");
	    return NULL;
	}

    ok_delete = gtk_window_new (GTK_WINDOW_DIALOG);
    gtk_object_set_data (GTK_OBJECT (ok_delete), "ok_delete", ok_delete);
    gtk_window_set_title (GTK_WINDOW (ok_delete), "Confirm Delete");
    gtk_window_set_modal (GTK_WINDOW (ok_delete), TRUE);
    gtk_window_set_policy (GTK_WINDOW (ok_delete), FALSE, FALSE, FALSE);
    
    vbox14 = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox14);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "vbox14", vbox14,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox14);
    gtk_container_add (GTK_CONTAINER (ok_delete), vbox14);
    
    label35 = gtk_label_new ("\n   Are you sure you wish to delete this entry?  \n");
    gtk_widget_ref (label35);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "label35", label35,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (label35);
    gtk_box_pack_start (GTK_BOX (vbox14), label35, FALSE, FALSE, 0);
    
    hseparator2 = gtk_hseparator_new ();
    gtk_widget_ref (hseparator2);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hseparator2", hseparator2,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hseparator2);
    gtk_box_pack_start (GTK_BOX (vbox14), hseparator2, TRUE, TRUE, 0);
    
    hbuttonbox9 = gtk_hbutton_box_new ();
    gtk_widget_ref (hbuttonbox9);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hbuttonbox9", hbuttonbox9,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hbuttonbox9);
    gtk_box_pack_start (GTK_BOX (vbox14), hbuttonbox9, TRUE, TRUE, 0);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox9), GTK_BUTTONBOX_SPREAD);
    
    confirm_okay = gtk_button_new_with_label ("OK");
    gtk_widget_ref (confirm_okay);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_okay", confirm_okay,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_okay);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_okay);
    GTK_WIDGET_SET_FLAGS (confirm_okay, GTK_CAN_DEFAULT);
    
    confirm_cancel = gtk_button_new_with_label ("Cancel");
    gtk_widget_ref (confirm_cancel);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_cancel", confirm_cancel,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_cancel);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_cancel);
    GTK_WIDGET_SET_FLAGS (confirm_cancel, GTK_CAN_DEFAULT);


	
    if ( !(strcmp(delete_function, "receiver")) )
    {
	if ( id != NULL )
	{
	    gtk_signal_connect (GTK_OBJECT (confirm_okay), "clicked",
				GTK_SIGNAL_FUNC (delete_receiver_popup),
				id);
	    gtk_signal_connect_object( GTK_OBJECT(confirm_okay), "clicked",
				       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));
	}
	else
	    gtk_signal_connect_object (GTK_OBJECT (confirm_okay), "clicked",
				       GTK_SIGNAL_FUNC (on_delete_button_clicked),
				       GTK_OBJECT(ok_delete));
    }

    else if ( !(strcmp(delete_function, "group")) )
    {
	if ( id != NULL )
	{
	    gtk_signal_connect (GTK_OBJECT (confirm_okay), "clicked",
				GTK_SIGNAL_FUNC (delete_group_popup),
				id);
	    gtk_signal_connect_object( GTK_OBJECT(confirm_okay), "clicked",
				       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));
	}
	else
	    gtk_signal_connect_object (GTK_OBJECT (confirm_okay), "clicked",
				       GTK_SIGNAL_FUNC (on_list_group_delete_button_clicked),
				       GTK_OBJECT(ok_delete));
    }

    gtk_signal_connect_object( GTK_OBJECT(confirm_cancel), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_widget_grab_focus(confirm_cancel);
    return ok_delete;
}

void show_pre_defined_list()
{
  GtkWidget    *pre_defined;
  GtkWidget    *vbox30;
  GtkWidget    *scrolledwindow14;
  GtkWidget    *pre_def_clist;
  GtkWidget    *def_label;
  stored_msg   *pre_def_msg_data;

  pre_def_msg_data = g_malloc(sizeof(*pre_def_msg_data));
  
  pre_defined = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_object_set_data (GTK_OBJECT (pre_defined), "pre_defined",pre_defined);
  gtk_window_set_title (GTK_WINDOW (pre_defined), "Pre Defined Messages");
  gtk_window_set_modal (GTK_WINDOW (pre_defined), TRUE);

  vbox30 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox30);
  gtk_object_set_data_full (GTK_OBJECT (pre_defined), "vbox30", vbox30,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox30);
  gtk_container_add (GTK_CONTAINER (pre_defined), vbox30);
  gtk_widget_set_usize (vbox30, 300, 300);

  scrolledwindow14 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow14);
  gtk_object_set_data_full (GTK_OBJECT (pre_defined), "scrolledwindow14", scrolledwindow14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow14);
  gtk_box_pack_start (GTK_BOX (vbox30), scrolledwindow14, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow14), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  pre_def_clist = gtk_clist_new (2);
  gtk_widget_ref (pre_def_clist);
  gtk_object_set_data_full (GTK_OBJECT (pre_defined), "pre_def_clist", pre_def_clist,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pre_def_clist);
  gtk_clist_set_column_visibility (GTK_CLIST(pre_def_clist), 0, FALSE); 
  gtk_container_add (GTK_CONTAINER (scrolledwindow14), pre_def_clist);
  gtk_clist_set_column_width (GTK_CLIST (pre_def_clist), 0, 5);
  gtk_clist_set_column_width (GTK_CLIST (pre_def_clist), 1, 40);
  gtk_clist_column_titles_show (GTK_CLIST (pre_def_clist));

/*
 *  Here is where we load the message list
 */

  load_pre_def_list(pre_def_clist, "MSG.dat");

  def_label = gtk_label_new ("Click on a message to select it");
  gtk_widget_ref (def_label);
  gtk_object_set_data_full (GTK_OBJECT (pre_defined), "def_label", def_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (def_label);
  gtk_clist_set_column_widget (GTK_CLIST (pre_def_clist), 1, def_label);
  gtk_label_set_justify (GTK_LABEL (def_label), GTK_JUSTIFY_LEFT);

  pre_def_msg_data->main_window = pre_defined;
  pre_def_msg_data->msg_list =   pre_def_clist;
  pre_def_msg_data->msg_text_box = NULL;


  gtk_signal_connect (GTK_OBJECT (pre_def_clist), "select_row",
                      GTK_SIGNAL_FUNC (catch_click),
                      pre_def_msg_data);

  gtk_widget_show(pre_defined);
}

/* 
 *  When the user has selected a row from the predefined message list (in the popup menu)
 *  we need to call on_ok_pref_clicked which loads the string into the text box and
 *  kills the list
 */

static void catch_click (GtkWidget *widget, gint row, gint col, GdkEvent *event, gpointer data)
{
    on_ok_pref_clicked(widget, data);
    
}


GtkWidget *confirm_log_delete (log_window_data  *user_data)
{
    GtkWidget *ok_delete;
    GtkWidget *vbox14;
    GtkWidget *label35;
    GtkWidget *hseparator2;
    GtkWidget *hbuttonbox9;
    GtkWidget *confirm_okay;
    GtkWidget *confirm_cancel;

    ok_delete = gtk_window_new (GTK_WINDOW_DIALOG);
    gtk_object_set_data (GTK_OBJECT (ok_delete), "ok_delete", ok_delete);
    gtk_window_set_title (GTK_WINDOW (ok_delete), "Confirm Delete");
    gtk_window_set_modal (GTK_WINDOW (ok_delete), TRUE);
    gtk_window_set_policy (GTK_WINDOW (ok_delete), FALSE, FALSE, FALSE);
    
    vbox14 = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox14);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "vbox14", vbox14,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox14);
    gtk_container_add (GTK_CONTAINER (ok_delete), vbox14);
    
    label35 = gtk_label_new ("\n   Deleting logs  \n  Do you wish to continue ?  \n");
    gtk_widget_ref (label35);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "label35", label35,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (label35);
    gtk_box_pack_start (GTK_BOX (vbox14), label35, FALSE, FALSE, 0);
    
    hseparator2 = gtk_hseparator_new ();
    gtk_widget_ref (hseparator2);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hseparator2", hseparator2,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hseparator2);
    gtk_box_pack_start (GTK_BOX (vbox14), hseparator2, TRUE, TRUE, 0);
    
    hbuttonbox9 = gtk_hbutton_box_new ();
    gtk_widget_ref (hbuttonbox9);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hbuttonbox9", hbuttonbox9,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hbuttonbox9);
    gtk_box_pack_start (GTK_BOX (vbox14), hbuttonbox9, TRUE, TRUE, 0);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox9), GTK_BUTTONBOX_SPREAD);
    
    confirm_okay = gtk_button_new_with_label ("OK");
    gtk_widget_ref (confirm_okay);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_okay", confirm_okay,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_okay);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_okay);
    GTK_WIDGET_SET_FLAGS (confirm_okay, GTK_CAN_DEFAULT);
    
    confirm_cancel = gtk_button_new_with_label ("Cancel");
    gtk_widget_ref (confirm_cancel);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_cancel", confirm_cancel,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_cancel);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_cancel);
    GTK_WIDGET_SET_FLAGS (confirm_cancel, GTK_CAN_DEFAULT);
    

    gtk_signal_connect (GTK_OBJECT (confirm_okay), "clicked",
			GTK_SIGNAL_FUNC (on_delete_log_button_clicked),
			user_data);

    gtk_signal_connect_object( GTK_OBJECT(confirm_okay), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_signal_connect_object( GTK_OBJECT(confirm_cancel), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_widget_grab_focus(confirm_cancel);
    return ok_delete;
}

GtkWidget *exit_anyway()
{
    GtkWidget *ok_delete;
    GtkWidget *vbox14;
    GtkWidget *label35;
    GtkWidget *hseparator2;
    GtkWidget *hbuttonbox9;
    GtkWidget *confirm_okay;
    GtkWidget *confirm_cancel;

    ok_delete = gtk_window_new (GTK_WINDOW_DIALOG);
    gtk_object_set_data (GTK_OBJECT (ok_delete), "ok_delete", ok_delete);
    gtk_window_set_title (GTK_WINDOW (ok_delete), "Confirm Exit");
    gtk_window_set_modal (GTK_WINDOW (ok_delete), TRUE);
    gtk_window_set_policy (GTK_WINDOW (ok_delete), FALSE, FALSE, FALSE);
    
    vbox14 = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox14);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "vbox14", vbox14,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox14);
    gtk_container_add (GTK_CONTAINER (ok_delete), vbox14);
    
    label35 = gtk_label_new ("\n You currently have autopage running  \n Exiting will kill autopage \n Do you really wish to exit? \n");
    gtk_widget_ref (label35);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "label35", label35,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (label35);
    gtk_box_pack_start (GTK_BOX (vbox14), label35, FALSE, FALSE, 0);
    
    hseparator2 = gtk_hseparator_new ();
    gtk_widget_ref (hseparator2);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hseparator2", hseparator2,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hseparator2);
    gtk_box_pack_start (GTK_BOX (vbox14), hseparator2, TRUE, TRUE, 0);
    
    hbuttonbox9 = gtk_hbutton_box_new ();
    gtk_widget_ref (hbuttonbox9);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hbuttonbox9", hbuttonbox9,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hbuttonbox9);
    gtk_box_pack_start (GTK_BOX (vbox14), hbuttonbox9, TRUE, TRUE, 0);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox9), GTK_BUTTONBOX_SPREAD);
    
    confirm_okay = gtk_button_new_with_label ("OK");
    gtk_widget_ref (confirm_okay);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_okay", confirm_okay,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_okay);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_okay);
    GTK_WIDGET_SET_FLAGS (confirm_okay, GTK_CAN_DEFAULT);
    
    confirm_cancel = gtk_button_new_with_label ("Cancel");
    gtk_widget_ref (confirm_cancel);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_cancel", confirm_cancel,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_cancel);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_cancel);
    GTK_WIDGET_SET_FLAGS (confirm_cancel, GTK_CAN_DEFAULT);
    

    gtk_signal_connect (GTK_OBJECT (confirm_okay), "clicked",
			GTK_SIGNAL_FUNC (exit_button_clicked),
			NULL);

    gtk_signal_connect_object( GTK_OBJECT(confirm_okay), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_signal_connect_object( GTK_OBJECT(confirm_cancel), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_widget_grab_focus(confirm_cancel);
    return ok_delete;
}

GtkWidget *confirm_convert(void)
{
    GtkWidget *ok_delete;
    GtkWidget *vbox14;
    GtkWidget *label35;
    GtkWidget *hseparator2;
    GtkWidget *hbuttonbox9;
    GtkWidget *confirm_okay;
    GtkWidget *confirm_cancel;

    ok_delete = gtk_window_new (GTK_WINDOW_DIALOG);
    gtk_object_set_data (GTK_OBJECT (ok_delete), "ok_delete", ok_delete);
    gtk_window_set_title (GTK_WINDOW (ok_delete), "Confirm Convert");
    gtk_window_set_modal (GTK_WINDOW (ok_delete), TRUE);
    gtk_window_set_policy (GTK_WINDOW (ok_delete), FALSE, FALSE, FALSE);
    
    vbox14 = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox14);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "vbox14", vbox14,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox14);
    gtk_container_add (GTK_CONTAINER (ok_delete), vbox14);
    
    label35 = gtk_label_new ("\n You local database files will be  \n  overwritten by your network files. \n There will be no way to recover them. \n Do you wish to Continue?\n ");
    gtk_widget_ref (label35);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "label35", label35,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (label35);
    gtk_box_pack_start (GTK_BOX (vbox14), label35, FALSE, FALSE, 0);
    
    hseparator2 = gtk_hseparator_new ();
    gtk_widget_ref (hseparator2);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hseparator2", hseparator2,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hseparator2);
    gtk_box_pack_start (GTK_BOX (vbox14), hseparator2, TRUE, TRUE, 0);
    
    hbuttonbox9 = gtk_hbutton_box_new ();
    gtk_widget_ref (hbuttonbox9);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hbuttonbox9", hbuttonbox9,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hbuttonbox9);
    gtk_box_pack_start (GTK_BOX (vbox14), hbuttonbox9, TRUE, TRUE, 0);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox9), GTK_BUTTONBOX_SPREAD);
    
    confirm_okay = gtk_button_new_with_label ("OK");
    gtk_widget_ref (confirm_okay);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_okay", confirm_okay,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_okay);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_okay);
    GTK_WIDGET_SET_FLAGS (confirm_okay, GTK_CAN_DEFAULT);
    
    confirm_cancel = gtk_button_new_with_label ("Cancel");
    gtk_widget_ref (confirm_cancel);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_cancel", confirm_cancel,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_cancel);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_cancel);
    GTK_WIDGET_SET_FLAGS (confirm_cancel, GTK_CAN_DEFAULT);
    

    gtk_signal_connect (GTK_OBJECT (confirm_okay), "clicked",
			GTK_SIGNAL_FUNC (convert_database_okay),
			NULL);

    gtk_signal_connect_object( GTK_OBJECT(confirm_okay), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_signal_connect_object( GTK_OBJECT(confirm_cancel), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_widget_grab_focus(confirm_cancel);
    return ok_delete;
}

GtkWidget *confirm_restore(void)
{
    GtkWidget *ok_delete;
    GtkWidget *vbox14;
    GtkWidget *label35;
    GtkWidget *hseparator2;
    GtkWidget *hbuttonbox9;
    GtkWidget *confirm_okay;
    GtkWidget *confirm_cancel;

    ok_delete = gtk_window_new (GTK_WINDOW_DIALOG);
    gtk_object_set_data (GTK_OBJECT (ok_delete), "ok_delete", ok_delete);
    gtk_window_set_title (GTK_WINDOW (ok_delete), "Confirm Restore");
    gtk_window_set_modal (GTK_WINDOW (ok_delete), TRUE);
    gtk_window_set_policy (GTK_WINDOW (ok_delete), FALSE, FALSE, FALSE);
    
    vbox14 = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox14);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "vbox14", vbox14,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox14);
    gtk_container_add (GTK_CONTAINER (ok_delete), vbox14);
    
    label35 = gtk_label_new ("\n You local database files will be  \n  overwritten by your backup files. \n There will be no way to recover them. \n Do you wish to Continue?\n ");
    gtk_widget_ref (label35);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "label35", label35,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (label35);
    gtk_box_pack_start (GTK_BOX (vbox14), label35, FALSE, FALSE, 0);
    
    hseparator2 = gtk_hseparator_new ();
    gtk_widget_ref (hseparator2);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hseparator2", hseparator2,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hseparator2);
    gtk_box_pack_start (GTK_BOX (vbox14), hseparator2, TRUE, TRUE, 0);
    
    hbuttonbox9 = gtk_hbutton_box_new ();
    gtk_widget_ref (hbuttonbox9);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hbuttonbox9", hbuttonbox9,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hbuttonbox9);
    gtk_box_pack_start (GTK_BOX (vbox14), hbuttonbox9, TRUE, TRUE, 0);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox9), GTK_BUTTONBOX_SPREAD);
    
    confirm_okay = gtk_button_new_with_label ("OK");
    gtk_widget_ref (confirm_okay);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_okay", confirm_okay,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_okay);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_okay);
    GTK_WIDGET_SET_FLAGS (confirm_okay, GTK_CAN_DEFAULT);
    
    confirm_cancel = gtk_button_new_with_label ("Cancel");
    gtk_widget_ref (confirm_cancel);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_cancel", confirm_cancel,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_cancel);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_cancel);
    GTK_WIDGET_SET_FLAGS (confirm_cancel, GTK_CAN_DEFAULT);
    

    gtk_signal_connect (GTK_OBJECT (confirm_okay), "clicked",
			GTK_SIGNAL_FUNC (convert_database_okay),
			"restore");

    gtk_signal_connect_object( GTK_OBJECT(confirm_okay), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_signal_connect_object( GTK_OBJECT(confirm_cancel), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_widget_grab_focus(confirm_cancel);
    return ok_delete;
}

GtkWidget *profile_confirm_delete (GtkWidget *combo_box)
{
    GtkWidget *ok_delete;
    GtkWidget *vbox14;
    GtkWidget *label35;
    GtkWidget *hseparator2;
    GtkWidget *hbuttonbox9;
    GtkWidget *confirm_okay;
    GtkWidget *confirm_cancel;

    ok_delete = gtk_window_new (GTK_WINDOW_DIALOG);
    gtk_object_set_data (GTK_OBJECT (ok_delete), "ok_delete", ok_delete);
    gtk_window_set_title (GTK_WINDOW (ok_delete), "Confirm Delete");
    gtk_window_set_modal (GTK_WINDOW (ok_delete), TRUE);
    gtk_window_set_policy (GTK_WINDOW (ok_delete), FALSE, FALSE, FALSE);
    
    vbox14 = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox14);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "vbox14", vbox14,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox14);
    gtk_container_add (GTK_CONTAINER (ok_delete), vbox14);
    
    label35 = gtk_label_new ("\n   Are you sure you wish to delete this profile?  \n");
    gtk_widget_ref (label35);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "label35", label35,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (label35);
    gtk_box_pack_start (GTK_BOX (vbox14), label35, FALSE, FALSE, 0);
    
    hseparator2 = gtk_hseparator_new ();
    gtk_widget_ref (hseparator2);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hseparator2", hseparator2,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hseparator2);
    gtk_box_pack_start (GTK_BOX (vbox14), hseparator2, TRUE, TRUE, 0);
    
    hbuttonbox9 = gtk_hbutton_box_new ();
    gtk_widget_ref (hbuttonbox9);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "hbuttonbox9", hbuttonbox9,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (hbuttonbox9);
    gtk_box_pack_start (GTK_BOX (vbox14), hbuttonbox9, TRUE, TRUE, 0);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox9), GTK_BUTTONBOX_SPREAD);
    
    confirm_okay = gtk_button_new_with_label ("OK");
    gtk_widget_ref (confirm_okay);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_okay", confirm_okay,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_okay);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_okay);
    GTK_WIDGET_SET_FLAGS (confirm_okay, GTK_CAN_DEFAULT);
    
    confirm_cancel = gtk_button_new_with_label ("Cancel");
    gtk_widget_ref (confirm_cancel);
    gtk_object_set_data_full (GTK_OBJECT (ok_delete), "confirm_cancel", confirm_cancel,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (confirm_cancel);
    gtk_container_add (GTK_CONTAINER (hbuttonbox9), confirm_cancel);
    GTK_WIDGET_SET_FLAGS (confirm_cancel, GTK_CAN_DEFAULT);

	
    gtk_signal_connect (GTK_OBJECT (confirm_okay), "clicked",
			GTK_SIGNAL_FUNC (unlink_profile),
			combo_box);

    gtk_signal_connect_object( GTK_OBJECT(confirm_okay), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_signal_connect_object( GTK_OBJECT(confirm_cancel), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(ok_delete));

    gtk_widget_grab_focus(confirm_cancel);
    return ok_delete;
}
