/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

typedef struct {
    GtkWidget  *right;
    GtkWidget  *left;
}bars;

void button_selection_made_text(GtkEntry *w,GdkEventButton *event, gpointer data);
void popup_menu(gchar *uid, GtkCList *clist, GdkEventButton *event);
void button_selection_made(GtkCList *list, GdkEventButton *event, gpointer data);
void insert_last_message(GtkWidget *w, gpointer data);
void popup_menu_text(GtkEntry *e, GdkEventButton *event);
void load_last_msg();
void backup_from_menu(void);
void restore_from_menu(void);


static gint change_focus (GtkWidget *widget, GdkEventKey *event, GtkWidget *next);
static GtkWidget *build_toolbar(GtkWidget *window);

/*
 *  These are the icons that go on the toolbar
 */

#include "icon_add.xpm"
#include "icon_config.xpm"
#include "log.xpm"
#include "xearth.xpm"
#include "tb_index.xpm"
#include "tb_clock_run.xpm"
#include "icon_script.xpm"

/* 
 * Global Variables
 */

extern GtkWidget *receiver_clist;
extern GtkWidget *text_entry;
extern GtkWidget *statusbar1;
extern GtkWidget *statusbar_small;
GtkWidget        *qp_hbox;


/*
 *   Main menu
 */

static GtkItemFactoryEntry menu_items[] = {
        { "/_File",                  NULL,         NULL, 0, "<Branch>" },
        { "/File/sep1",              NULL,         NULL, 0, "<Separator>" },
        { "/File/Backup DB",             NULL, backup_from_menu, 0, NULL },
        { "/File/Restore DB",             NULL, restore_from_menu, 0, NULL },
        { "/File/_Quit",             "<control>Q", exit_from_menu, 0, NULL },
        { "/_Maintenance",          NULL,         NULL, 0, "<Branch>" },
        { "/Maintenance/sep1",              NULL,         NULL, 0, "<Separator>" },
        { "/Maintenance/Receivers", "<control>H",        list_receivers, 0, NULL },
        { "/Maintenance/Services",  "<control>D",        list_services , 0, NULL },
        { "/Maintenance/Groups",  "<control>M",        list_groups , 0, NULL },
        { "/Maintenance/Settings ",  "<control>N",        config_client , 0, NULL },
        { "/Maintenance/Preferences",   "<control>P",        preferences , 0, NULL },
        { "/Maintenance/Stored Msgs",   "<control>B",        pre_defined_msg , 0, NULL },
        { "/Maintenance/Scheduled Msgs",   "<control>X",        list_scheduled , 0, NULL },
        { "/Maintenance/Msg Replies",   "<control>F",        show_reply_window , 0, NULL },
        { "/_Options",      NULL,         NULL, 0, "<Branch>" },
        { "/Options/sep1",              NULL,         NULL, 0, "<Separator>" },
        { "/Options/Add Receiver",  "<control>R",         receiver_call, 0, NULL },
        { "/Options/Add Service",   "<control>T",        service_call , 0, NULL },
        { "/Options/Add Group",   "<control>G",        add_group , 0, NULL },
        { "/Options/View Log",   "<control>V",        view_log_main , 0, NULL },
        { "/_Utilities",      NULL,         NULL, 0, "<Branch>" },
        { "/Utilities/sep1",              NULL,         NULL, 0, "<Separator>" },
        { "/Utilities/Send page",     "<control>S", send_text, 0, NULL },
        { "/Utilities/Clear screen",     "<control>C", clear_text, 0, NULL },
        { "/Utilities/Schedule page",     "<control>Y", schedule_page, 0, NULL },
        { "/Utilities/Insert last msg",     "<control>L", load_last_msg, 0, NULL },
        { "/Utilities/Check server status",     "<control>E", check_server, 0, NULL },
        { "/Utilities/Script page",     "<control>A", show_script_page, 0, NULL },
        { "/Utilities/2 way response options",     "<control>O", show_response_window, 0, NULL },
        { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
        { "/_Help/About",   NULL,         about_box, 0, NULL },
      };


void get_main_menu( GtkWidget  *window,
		    GtkWidget **menubar )
{
    GtkItemFactory *item_factory;
    GtkAccelGroup *accel_group;
    gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
    
    accel_group = gtk_accel_group_new ();
    item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
					 accel_group);
    gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
    gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
    
    if (menubar)
	*menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}

/*
 *   We create the main window and return the widget to the calling function
 */

GtkWidget*
create_window_main (void)
{
  extern GtkWidget  *qp_hbox;

  GtkWidget         *window_main;
  GtkWidget         *toolbar;
  GtkWidget         *handle;

  GtkWidget         *vbox1;
  GtkWidget         *hbox1;
  GtkWidget         *hbox_status;

  GtkWidget         *list_scrolled_window;
  GtkWidget         *msg_scrolled_window;

  GtkWidget         *select_receivers_label;
  GtkWidget         *message_entry_label;

  GtkWidget         *notebook1;

  GtkWidget         *hbuttonbox1;
  GtkWidget         *exit_button;
  GtkWidget         *send_button;
  GtkWidget         *clear_button;

  GtkWidget         *menubar;

  GSList            *qpage_group = NULL;

  GtkWidget         *qpage_on;
  GtkWidget         *qpage_off;
  GtkWidget         *qpage_label;

  GtkWidget         *hseparator1;
  GtkWidget         *hseparator01;
  GtkWidget         *fixed_left;

  GtkWidget         *statusbar_table;

  gchar             *qpage_options;
  gchar             *show_widget_options;

/*
 *  These set the dimensions for the list of recievers/groups
 */

  gint              column_zero_width = 0; /* This one is holds the id number and is not visible */
  gint              column_one_width = 14;
  gint              column_two_width = 100;

  GdkPixmap  *stat_pic_earth;
  GdkBitmap  *mask1;
  GtkWidget  *pixwidget;

  bars       *sbars;

  sbars = g_malloc(sizeof(*sbars));

  window_main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  
  gtk_widget_set_name (window_main, "main window");

  gtk_object_set_data (GTK_OBJECT (window_main), "window_main", window_main);
  gtk_widget_set_usize (window_main, 540, 320);
  gtk_window_set_title (GTK_WINDOW (window_main), "g-page");

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window_main), vbox1);

  gtk_container_border_width (GTK_CONTAINER (vbox1), 1);        
  get_main_menu (window_main, &menubar);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar, FALSE, TRUE, 0);
  gtk_widget_show (menubar);

  gtk_widget_realize ( window_main );


/*
 * Here is the horizontal separator which is drawn under the menubar
 */


  hseparator01 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator01);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "hseparator01", hseparator01,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator01);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator01, FALSE, TRUE, 0);


/*
 *  Here is the handlebox where that toolbar will reside
 */


  handle = gtk_handle_box_new();
  gtk_handle_box_set_shadow_type((GtkHandleBox *)(handle), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show(handle);


/*
 *  Build the toolbar here
 */


  toolbar = build_toolbar(window_main);

  gtk_widget_ref (toolbar);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "toolbar", toolbar,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (toolbar);
  gtk_container_add(GTK_CONTAINER(handle), toolbar);
  gtk_box_pack_start (GTK_BOX (vbox1), handle, FALSE, FALSE, 0);


/*
 *  Here is the horizontal separator line that is drawn under the toolbar
 */


  hseparator1 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator1);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "hseparator1", hseparator1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, FALSE, TRUE, 0);


/*
 *   Now we build the receiver clist. It goes in a scrolled window which resides 
 *   in an hbox
 */


  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);
  
  list_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (list_scrolled_window);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "list_scrolled_window", list_scrolled_window,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (list_scrolled_window);
  gtk_box_pack_start (GTK_BOX (hbox1), list_scrolled_window, TRUE, TRUE, 0);

  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (list_scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC); 

  receiver_clist = gtk_clist_new (3);
  gtk_clist_set_column_visibility (GTK_CLIST(receiver_clist), 0, FALSE);

  /* Uncomment this to see the hiden column in the receiver clist - this lets you 
     look at the user or group id numbers to make sure all is as it should be */

  /* gtk_clist_set_column_visibility (GTK_CLIST(receiver_clist), 0, TRUE); */

  gtk_clist_set_sort_column (GTK_CLIST(receiver_clist), 2);
  gtk_clist_set_auto_sort(GTK_CLIST(receiver_clist), TRUE);

  gtk_widget_ref (receiver_clist);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "receiver_clist", receiver_clist,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_clist);
  gtk_container_add (GTK_CONTAINER (list_scrolled_window), receiver_clist);

  gtk_clist_set_column_width (GTK_CLIST (receiver_clist), 0, column_zero_width);
  gtk_clist_set_column_width (GTK_CLIST (receiver_clist), 1, column_one_width);
  gtk_clist_set_column_width (GTK_CLIST (receiver_clist), 2, column_two_width);
  gtk_clist_set_selection_mode (GTK_CLIST (receiver_clist), GTK_SELECTION_MULTIPLE);
  gtk_clist_column_titles_show (GTK_CLIST (receiver_clist));
  
  select_receivers_label = gtk_label_new ("Select Receiver(s):");
  gtk_widget_ref (select_receivers_label);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "select_receivers_label;", select_receivers_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (select_receivers_label);
  gtk_clist_set_column_widget (GTK_CLIST (receiver_clist), 1, select_receivers_label);

  
/*
 *   Here is the notebook where the message entry box resides. 
 */


  notebook1 = gtk_notebook_new ();
  gtk_widget_ref (notebook1);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "notebook1", notebook1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (hbox1), notebook1, TRUE, TRUE, 0);

  msg_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (msg_scrolled_window);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "msg_scrolled_window", msg_scrolled_window,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (msg_scrolled_window);
  gtk_container_add (GTK_CONTAINER (notebook1), msg_scrolled_window);


/*
 *   Here is the text entry box
 */


  text_entry = gtk_text_new (NULL, NULL);
  gtk_signal_connect (GTK_OBJECT (text_entry), "changed",
                      GTK_SIGNAL_FUNC (NULL),
                      text_entry);
  gtk_widget_ref (text_entry);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "text_entry", text_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  GTK_WIDGET_SET_FLAGS (text_entry, GTK_CAN_DEFAULT);
  gtk_widget_show (text_entry);
  gtk_container_add (GTK_CONTAINER (msg_scrolled_window), text_entry);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (msg_scrolled_window), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC); 
  gtk_widget_set_usize (text_entry, 200, 136);
  gtk_text_set_editable (GTK_TEXT (text_entry), TRUE);

  message_entry_label = gtk_label_new (" Msg Entry ");
  gtk_widget_ref (message_entry_label);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "message_entry_label", message_entry_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (message_entry_label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), message_entry_label);


/* 
 *   Here is the buttonbox where the send, clear and exit buttons reside
 */


  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, TRUE, 0);

  exit_button = gtk_button_new_with_label ("Exit");
  gtk_widget_ref (exit_button);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "exit_button", exit_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (exit_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), exit_button);
  GTK_WIDGET_SET_FLAGS (exit_button, GTK_CAN_DEFAULT);

  send_button = gtk_button_new_with_label ("Send");
  gtk_widget_ref (send_button);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "send_button", send_button,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_show (send_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), send_button);
  GTK_WIDGET_SET_FLAGS (send_button, GTK_CAN_DEFAULT);

  clear_button = gtk_button_new_with_label ("Clear");
  gtk_widget_ref (clear_button);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "clear_button", clear_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (clear_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), clear_button);
  GTK_WIDGET_SET_FLAGS (clear_button, GTK_CAN_DEFAULT);


/*
 *   Here is where the QuickPage options - radio buttons - get built
 *   They are turned off by default.
 */


  qp_hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (qp_hbox);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "qp_hbox", qp_hbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (vbox1), qp_hbox, FALSE, FALSE, 0);

  qpage_label = gtk_label_new (" Activate qpage mode ");
  gtk_widget_ref (qpage_label);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "qpage_label", qpage_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (qpage_label);
  gtk_box_pack_start (GTK_BOX (qp_hbox), qpage_label, FALSE, FALSE, 0);


  qpage_on = gtk_radio_button_new_with_label (qpage_group, " Yes");
  qpage_group = gtk_radio_button_group (GTK_RADIO_BUTTON (qpage_on));
  gtk_widget_ref (qpage_on);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "qpage_on", qpage_on,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (qpage_on);
  gtk_box_pack_start (GTK_BOX (qp_hbox), qpage_on, FALSE, FALSE, 2);


  qpage_off = gtk_radio_button_new_with_label (qpage_group, " No ");
  qpage_group = gtk_radio_button_group (GTK_RADIO_BUTTON (qpage_off));
  gtk_widget_ref (qpage_off);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "qpage_off", qpage_off,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (qpage_off);
  gtk_box_pack_start (GTK_BOX (qp_hbox), qpage_off, FALSE, FALSE, 2);


/*
 *     Set the QuickPage yes/no radio button here
 */

  if ( (qpage_options = get_resource_data("CONFIG.dat", "qpage_active", MAX_FIELD_LENGTH)) == NULL )
      qpage_options = g_strdup("no");
  else
  {
      if ( !strcmp(qpage_options, "no") )
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (qpage_off), TRUE);
      else
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (qpage_on), TRUE);
  }


  if ( (show_widget_options = get_resource_data("CONFIG.dat", "show_advanced_options", MAX_FIELD_LENGTH)) == NULL )
      show_widget_options = g_strdup("no");
  else
  {
      if ( !strcmp(show_widget_options, "yes") )
	  gtk_widget_show (qp_hbox);
  }

  g_free(show_widget_options);
  g_free(qpage_options);


/*
 *   Here is the status bar
 */


  hbox_status = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox_status);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "hbox_status", hbox_status,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox_status);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox_status, FALSE, FALSE, 0);


  stat_pic_earth = gdk_pixmap_create_from_xpm_d ( window_main->window, &mask1,
					&window_main->style->white, xearth_xpm );

  pixwidget = gtk_pixmap_new(stat_pic_earth, mask1);

  fixed_left = gtk_fixed_new();
  gtk_widget_show(fixed_left);


  gtk_widget_show(pixwidget);

  statusbar_table = gtk_table_new (1, 3, TRUE);
  gtk_widget_ref (statusbar_table);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "statusbar_table", statusbar_table,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (statusbar_table);
  gtk_box_pack_start (GTK_BOX (hbox_status), statusbar_table, TRUE, TRUE, 0);

  statusbar1 = gtk_statusbar_new ();
  gtk_widget_ref (statusbar1);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "statusbar1", statusbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (statusbar1);
  gtk_table_attach (GTK_TABLE (statusbar_table), statusbar1, 0, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  statusbar_small = gtk_statusbar_new ();
  gtk_widget_ref (statusbar_small);
  gtk_object_set_data_full (GTK_OBJECT (window_main), "statusbar_small", statusbar_small,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (statusbar_small);
  gtk_table_attach (GTK_TABLE (statusbar_table), statusbar_small, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

/*
 *    Finally we connect all the signals up
 */

  sbars->right = statusbar_small;
  sbars->left = statusbar1;

  gtk_signal_connect(GTK_OBJECT(receiver_clist), "button_press_event",
		    GTK_SIGNAL_FUNC(button_selection_made), NULL);

/*
 *    Here is where we catch the send button to initiate the 
 *    sending of the message
 */

  gtk_signal_connect (GTK_OBJECT (send_button), "clicked",
		      GTK_SIGNAL_FUNC (get_text),
                      GTK_EDITABLE(text_entry));

  gtk_signal_connect(GTK_OBJECT(text_entry), "button_press_event",
		     GTK_SIGNAL_FUNC(button_selection_made_text), NULL);
  
  gtk_signal_connect (GTK_OBJECT (text_entry), "changed",
                      GTK_SIGNAL_FUNC (text_entry_changed),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (text_entry), "key_press_event",
		      GTK_SIGNAL_FUNC (change_focus),
		      send_button);

  gtk_signal_connect (GTK_OBJECT (receiver_clist), "key_press_event",
		      GTK_SIGNAL_FUNC (change_focus),
		      text_entry);

  gtk_signal_connect (GTK_OBJECT (send_button), "key_press_event",
		      GTK_SIGNAL_FUNC (change_focus),
		      receiver_clist);
  
  gtk_signal_connect (GTK_OBJECT (exit_button), "clicked",
                      GTK_SIGNAL_FUNC (check_for_exit),
                      window_main);

  gtk_signal_connect (GTK_OBJECT (clear_button), "clicked",
                      GTK_SIGNAL_FUNC (clear_button_clicked),
                      text_entry);

  gtk_signal_connect (GTK_OBJECT (window_main), "delete_event", 
		      GTK_SIGNAL_FUNC(check_for_exit), NULL);

  gtk_signal_connect (GTK_OBJECT (window_main), "delete_event", 
		      GTK_SIGNAL_FUNC(destroy_event), NULL);

  gtk_signal_connect (GTK_OBJECT (qpage_on), "toggled",
                      GTK_SIGNAL_FUNC (qpage_active),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (qpage_off), "toggled",
                      GTK_SIGNAL_FUNC (qpage_disabled),
                      NULL);
/*
  gtk_signal_connect(GTK_OBJECT(window_main), "size_allocate", 
                     GTK_SIGNAL_FUNC(size_allocate), sbars);
*/
  gtk_signal_connect (GTK_OBJECT (receiver_clist), "select_row",
                      GTK_SIGNAL_FUNC (count_receivers),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (receiver_clist), "unselect_row",
                      GTK_SIGNAL_FUNC (count_receivers),
                      NULL);


  gtk_widget_grab_focus(receiver_clist);

  return window_main;
}


/*
 *  We build the toolbar and return it to the calling function
 */


static GtkWidget *build_toolbar(GtkWidget *main_window)
{
    GtkWidget  *bar;

    GtkWidget  *receiver_icon;
    GtkWidget  *service_icon;
    GtkWidget  *group_icon;
    GtkWidget  *config_icon;
    GtkWidget  *log_icon;
    GtkWidget  *pref_icon;
    GtkWidget  *sched_icon;
    GtkWidget  *script_icon;

    GtkWidget  *configure_button;
    GtkWidget  *receiver_button;
    GtkWidget  *receiver1_button;
    GtkWidget  *terminal_button;
    GtkWidget  *group_button;
    GtkWidget  *pref_button;
    GtkWidget  *schedule_button;
    GtkWidget  *script_button;

    GdkPixmap  *icon;
    GdkPixmap  *icon_config;
    GdkPixmap  *icon_log;
    GdkPixmap  *icon_pref;
    GdkPixmap  *icon_sched;
    GdkPixmap  *icon_script;
    GdkBitmap  *mask;

/*
 *  This icon is used for the receiver, group and service buttons
 */

    icon = gdk_pixmap_create_from_xpm_d ( main_window->window, &mask,
					  &main_window->style->white, add_xpm );


    bar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
    gtk_toolbar_set_button_relief(GTK_TOOLBAR(bar), GTK_RELIEF_NONE);
    gtk_toolbar_set_space_size(GTK_TOOLBAR(bar), 1);
    gtk_toolbar_set_space_style(GTK_TOOLBAR(bar), GTK_TOOLBAR_SPACE_LINE);


/*
 *  Create the receiver button
 */

  receiver_icon = gtk_pixmap_new ( icon, mask );
  receiver_button = gtk_toolbar_append_element (GTK_TOOLBAR (bar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                " Receiver ",
                                NULL, NULL,
                                receiver_icon, NULL, NULL);
//  gtk_button_set_relief(GTK_BUTTON(receiver_button), GTK_RELIEF_NONE);
  gtk_widget_ref (receiver_button);
  gtk_object_set_data_full (GTK_OBJECT (main_window), "receiver_button", receiver_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver_button);


/*
 * Create the service button
 */

  service_icon = gtk_pixmap_new ( icon, mask );
  terminal_button = gtk_toolbar_append_element (GTK_TOOLBAR (bar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Service",
                                NULL, NULL,
                                service_icon, NULL, NULL);
  gtk_widget_ref (terminal_button);
  gtk_object_set_data_full (GTK_OBJECT (main_window), "terminal_button", terminal_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (terminal_button);


/*
 *  Create the group button
 */

  group_icon = gtk_pixmap_new ( icon, mask );
  group_button = gtk_toolbar_append_element (GTK_TOOLBAR (bar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Group",
                                NULL, NULL,
                                group_icon, NULL, NULL);
  gtk_widget_ref (group_button);
  gtk_object_set_data_full (GTK_OBJECT (main_window), "group_button", group_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (group_button);
  gtk_toolbar_append_space((GtkToolbar *)bar);


/*
 *  Create the config button
 */

  icon_config = gdk_pixmap_create_from_xpm_d ( main_window->window, &mask,
					&main_window->style->white, icon_config_xpm );
  config_icon = gtk_pixmap_new ( icon_config, mask );
  configure_button = gtk_toolbar_append_element (GTK_TOOLBAR (bar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Config",
                                NULL, NULL,
                                config_icon, NULL, NULL);
  gtk_widget_ref (configure_button);
  gtk_object_set_data_full (GTK_OBJECT (main_window), "configure_button", configure_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (configure_button);


/*
 *  Create the log button
 */

  icon_log = gdk_pixmap_create_from_xpm_d ( main_window->window, &mask,
					&main_window->style->white, log_xpm );
  log_icon = gtk_pixmap_new ( icon_log, mask );
  receiver1_button = gtk_toolbar_append_element (GTK_TOOLBAR (bar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Log",
                                NULL, NULL,
                                log_icon, NULL, NULL);
  gtk_widget_ref (receiver1_button);
  gtk_object_set_data_full (GTK_OBJECT (main_window), "receiver1_button", receiver1_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (receiver1_button);


/*
 *  Create the preferences button
 */

  icon_pref = gdk_pixmap_create_from_xpm_d ( main_window->window, &mask,
					&main_window->style->white, tb_index_xpm );
  pref_icon = gtk_pixmap_new ( icon_pref, mask );
  pref_button = gtk_toolbar_append_element (GTK_TOOLBAR (bar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Pref",
                                NULL, NULL,
                                pref_icon, NULL, NULL);
  gtk_widget_ref (pref_button);
  gtk_object_set_data_full (GTK_OBJECT (main_window), "pref_button", pref_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pref_button);

/*
 *  Create the scheduled button
 */

  icon_sched = gdk_pixmap_create_from_xpm_d ( main_window->window, &mask,
					&main_window->style->white, tb_clock_run_xpm );
  sched_icon = gtk_pixmap_new ( icon_sched, mask );
  schedule_button = gtk_toolbar_append_element (GTK_TOOLBAR (bar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Schedule",
                                NULL, NULL,
                                sched_icon, NULL, NULL);
  gtk_widget_ref (schedule_button);
  gtk_object_set_data_full (GTK_OBJECT (main_window), "schedule_button", schedule_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (schedule_button);

/*
 *  Create the script button
 */

  icon_script = gdk_pixmap_create_from_xpm_d ( main_window->window, &mask,
					&main_window->style->white, icon_script_xpm );
  script_icon = gtk_pixmap_new ( icon_script, mask );
  script_button = gtk_toolbar_append_element (GTK_TOOLBAR (bar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                " Auto ",
                                NULL, NULL,
                                script_icon, NULL, NULL);
  gtk_widget_ref (script_button);
  gtk_object_set_data_full (GTK_OBJECT (main_window), "script_button", script_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (script_button);



  gtk_signal_connect (GTK_OBJECT (group_button), "clicked",
                      GTK_SIGNAL_FUNC (list_groups),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (receiver_button), "clicked",
                      GTK_SIGNAL_FUNC (list_receivers),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (configure_button), "clicked",
                      GTK_SIGNAL_FUNC (config_client),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (receiver1_button), "clicked",
                      GTK_SIGNAL_FUNC (view_log_main),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (pref_button), "clicked",
                      GTK_SIGNAL_FUNC (preferences),
                      qp_hbox);

  gtk_signal_connect (GTK_OBJECT (schedule_button), "clicked",
                      GTK_SIGNAL_FUNC (schedule_page),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (terminal_button), "clicked",
                      GTK_SIGNAL_FUNC (list_services),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (script_button), "clicked",
                      GTK_SIGNAL_FUNC (show_script_page),
                      NULL);

  return (bar);
}


/*
 *   This is used to cycle from the receiver_clist to the text box to the send box
 *   by hitting the tab key
 */

static gint change_focus (GtkWidget *widget, GdkEventKey *event, GtkWidget *next)
{
    g_return_val_if_fail (widget != NULL, FALSE);
    g_return_val_if_fail (event != NULL, FALSE);

    if (event->keyval == GDK_Tab) 
    {
	gtk_widget_grab_focus (GTK_WIDGET (next));
	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
	return TRUE;
    }
    
  return FALSE;
}

/*
 * If we have a right mouse click we call the popup menu function, otherwise return
 */

void button_selection_made_text(GtkEntry *w, GdkEventButton *event, gpointer data)
{

 if (event->button == 3)        /* We have a right mouse click here */
    popup_menu_text(w, event);
   return;

}       

/*
 * This is the popup menu for right mouse clicks in the message text box
 */

void popup_menu_text(GtkEntry *e, GdkEventButton *event)
{

  GtkWidget *pop_menu;
  GtkWidget *last_msg;
  GtkWidget *pre_defined;
  GtkWidget *two_way_response;

  pop_menu = gtk_menu_new();

  last_msg = gtk_menu_item_new_with_label("Insert last message");
  gtk_signal_connect( GTK_OBJECT(last_msg), "activate",
		      GTK_SIGNAL_FUNC( insert_last_message ), NULL);

  pre_defined = gtk_menu_item_new_with_label("Insert predefined message");
  gtk_signal_connect( GTK_OBJECT(pre_defined), "activate",
		      GTK_SIGNAL_FUNC( show_pre_defined_list ), NULL);

  two_way_response = gtk_menu_item_new_with_label("Select 2-way response");
  gtk_signal_connect( GTK_OBJECT(two_way_response), "activate",
		      GTK_SIGNAL_FUNC( show_response_window ), NULL);

  gtk_menu_append( GTK_MENU (pop_menu), last_msg);
  gtk_menu_append( GTK_MENU (pop_menu), pre_defined);
  gtk_menu_append( GTK_MENU (pop_menu), two_way_response);

  gtk_widget_show(last_msg);
  gtk_widget_show(pre_defined);
  gtk_widget_show(two_way_response);

  gtk_menu_popup(GTK_MENU(pop_menu), NULL, NULL, NULL, NULL, event->button, event->time);
  return;

}             

/* 
 *   Catch a right click made in the receiver list box
 */

void button_selection_made(GtkCList *list, GdkEventButton *event, gpointer data)
{
  gint row, col;
  gchar *id;

  gtk_clist_get_selection_info(list, event->x, event->y, &row, &col);

  gtk_clist_get_text(GTK_CLIST(list), row, 0, &id);

 if (event->button == 3)        /* We have a right mouse click here */
    popup_menu(id,list, event);
   return;

}       

/*
 *   This is the pop up menu from right clicks in the reciever clist
 */

void popup_menu(gchar *uid, GtkCList *clist, GdkEventButton *event)
{

  GtkWidget *pop_menu;
  GtkWidget *edit_item ;
  GtkWidget *delete_item ;

  pop_menu = gtk_menu_new();


  edit_item = gtk_menu_item_new_with_label("Edit item");
  gtk_signal_connect( GTK_OBJECT(edit_item), "activate",
                      GTK_SIGNAL_FUNC( edit_receiver_popup ), uid);

  delete_item = gtk_menu_item_new_with_label("Delete item");
  gtk_signal_connect( GTK_OBJECT(delete_item), "activate",
                      GTK_SIGNAL_FUNC(delete_item_popup), uid);

  gtk_menu_append( GTK_MENU (pop_menu), edit_item);
  gtk_menu_append( GTK_MENU (pop_menu), delete_item);

  gtk_widget_show(edit_item);
  gtk_widget_show(delete_item);

  gtk_menu_popup(GTK_MENU(pop_menu), NULL, NULL, NULL, NULL, event->button, event->time);

  return;

}
             
void load_last_msg()
{
    insert_last_message(NULL, NULL);
}

void insert_last_message(GtkWidget *w, gpointer data)
{
    extern GtkWidget *text_entry;

    gchar    *last_msg;
    GdkFont  *current_font;

    if ( (last_msg = get_resource_data("CONFIG.dat", "last", MAX_LENGTH)) == NULL)
	last_msg = g_strdup("Error reading last message...");
    
    gtk_text_set_point(GTK_TEXT(text_entry), 0);
    current_font = gdk_font_load(last_msg);
    gtk_text_insert(GTK_TEXT(text_entry), current_font, NULL, NULL, last_msg, strlen(last_msg));

    text_entry_changed(NULL, NULL);

    g_free(last_msg);


}

void backup_from_menu(void)
{
    convert_database_okay(NULL, "backup");
    generic_error("\n Database backup complete \n", "G-page Info");
}

void restore_from_menu(void)
{
    GtkWidget  *dlg_box;

    dlg_box = confirm_restore();
    gtk_widget_show(dlg_box);

}

void count_receivers(GtkWidget *rec_list, gint row, gint column, GdkEventButton *event, gpointer data)
{
    GList            *selected;
    gint             active_number;
    gchar            *buffer;

    selected = GTK_CLIST(rec_list)->selection;
    active_number = 0;

    while (selected != NULL)
    {
	active_number++;
	selected = selected->next;
    }
    buffer = g_strdup_printf("Selected receivers: %d", active_number);
    update_status_line_small(buffer);
    g_free(buffer);

    return;
}
