/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"


/*
 *     We display the full message from the row selected into the text box. A short message prieview is all the
 *     user can see otherwise
 */

void
on_view_button_clicked             (GtkButton        *button,
                                    sched_win_data   *user_data)
{
    gchar         *msg = "";
    gchar         *tmsg = "";

    GdkFont       *current_font;


    if (GTK_CLIST(user_data->clist)->selection == NULL)
    {
        generic_error("\n  You must select the message you wish to view \n", "G-Page Error");
        return;
    }

    msg = g_malloc(MAX_LENGTH);

    delete_text_in_text_box(user_data->text_box);

    gtk_clist_get_text( (GtkCList *)user_data->clist, GPOINTER_TO_INT(GTK_CLIST(user_data->clist)->selection->data), 3, &tmsg);
    strcpy(msg, tmsg);

    current_font = gdk_font_load(msg);
    gtk_text_insert(GTK_TEXT(user_data->text_box), current_font, NULL, NULL, msg, strlen(msg));
    gtk_text_set_point( GTK_TEXT( user_data->text_box) , 0);

    g_free(msg);
}

/*
 *     We delete the data for the selected scheduled page. We do this by sending the server the id of the 
 *     message so it can delete all the data for that message on it's end,  then we delete the data for the 
 *     client. 
 *
 *     The first thing we send to the server is the mode, which is 4 for deleting a scheduled message.
 */

void
on_delete_schedule_clicked             (GtkButton        *button,
                                        sched_win_data   *user_data)
{
    gchar         *id, *msg_id;
    gchar         *mode = "4\n";
    gchar         *query;
    gint          sock = 0;

    if (GTK_CLIST(user_data->clist)->selection == NULL)
    {
        generic_error("\n  You must select the message you wish delete \n", "G-Page Error");
        return;
    }

/*
 *   Delete any text in the text box first to save a little confusion
 */

    delete_text_in_text_box(user_data->text_box);
    gtk_text_set_point(GTK_TEXT(user_data->text_box), 0);

    id = g_malloc(MAX_LENGTH);
    query = g_malloc(MAX_LENGTH);

    gtk_clist_get_text( (GtkCList *)user_data->clist, GPOINTER_TO_INT(GTK_CLIST(user_data->clist)->selection->data), 4, &msg_id);
    strcpy(id, msg_id);

    sock = server_connection(NORMAL);

    if ( sock < 0 )
    {
	if ( sock == -1 )
	    generic_error("\n Error connecting with server \n  Bad address or server is not running  \n", "G-Page Error");
	else if ( sock == -2 )
	    generic_error("\n Error connecting with server \n  Permission denied  \n", "G-Page Error");
	else
	    generic_error("\n Error connecting with server \n  Unkown Error  \n", "G-Page Error");

	g_free(id);
	g_free(query);
	return;
    }	

    commit_transaction(sock, mode);
    strcat(id, "\n");
    commit_transaction(sock, id);

    write_num(sock, "delete\n", 7);
    read_a_line(sock, query, MAX_LENGTH);

/*
 *     For some reason I dont' get the full result (query) back sometimes, so I look at the first character which is 
 *     a t (true) or an f (false) as a workaround.
 *     This is actually a pretty nasty hack but I can't reliably reproduce the bug so this will have to do for now
 */

    if ( query[0] == 't' )
    {
	id[strlen(id) -1] = '\0';
	delete_msg_id(id, user_data->clist);
    }

    g_free(id);
    g_free(query);
}

void
on_ok_schedule_clicked               (GtkButton       *button,
                                     sched_win_data   *user_data)
{
    gtk_widget_destroy(user_data->main_win);
    g_free(user_data);
}

/*
 *     We poll the server for the status of any scheduled pages. We are going to make sure the
 *     user has selected a message, form a connection with the server, and let the obtain_schedule_status
 *     function do all the communicating with the server
 */

void
on_status_schedule_clicked               (GtkButton        *button,
                                          sched_win_data   *user_data)
{
    gchar           *msg_id;
    gchar           *id;
    gchar           *msg_time, *msg_ttime;
    gchar           *count, *cnt;
    gint            sock;

    sched_update    *msg_obj;

    if ( GTK_CLIST(user_data->clist)->selection == NULL)
    {
        generic_error("\n  You must select the message you wish \n to obtain the status of. \n", "G-Page Error");
        return;
    }

    id = g_malloc(MAX_LENGTH);
    msg_time = g_malloc(MAX_LENGTH);
    count = g_malloc(MAX_LENGTH);

    gtk_clist_get_text( (GtkCList *)user_data->clist, GPOINTER_TO_INT(GTK_CLIST(user_data->clist)->selection->data), 4, &msg_id);
    strcpy(id, msg_id);

    gtk_clist_get_text( (GtkCList *)user_data->clist, GPOINTER_TO_INT(GTK_CLIST(user_data->clist)->selection->data), 1, &msg_ttime);
    strcpy(msg_time, msg_ttime);

    gtk_clist_get_text( (GtkCList *)user_data->clist, GPOINTER_TO_INT(GTK_CLIST(user_data->clist)->selection->data), 5, &cnt);
    strcpy(count, cnt);
    
    gtk_label_set_text(GTK_LABEL(user_data->count_label), count);
    gtk_widget_show(user_data->count_label);

    sock = server_connection(NORMAL);

    if ( sock <= 0 )
    {
	if ( sock == -1 )  
	    generic_error("\n  Unable to connect with server: Connection refused \n  Is the server running? ", "G-Page Error");
	else if  ( sock == -2 )
	    generic_error("\n  Unable to connect with server: Permission denied \n", "G-Page Error");
	else if ( sock == 0 )
	    generic_error("\n  A fatal error occured while trying to connect to the server \n Check your configuration files \n", "G-Page Error");

    }
    else
    {
	msg_obj = obtain_schedule_status(sock, id, user_data->text_box, user_data->clist, msg_time, user_data, count);
	close(sock);

	g_free(msg_obj);
    }

    g_free(id);
    g_free(msg_time);
    g_free(count);

    return;
}

/*
 *     We query the server for the status of a scheduled page. The transaction looks like this:
 *     
 *     1) First we send a 3 to the server to tell it what mode it needs to be in
 *     2) We send the server the message Id that we want to query
 *     3) We will then read data from the server until we get the string "done" 
 *     4) We parse each line we get back from the server, which is actually logging information for 
 *        any messages that were sent with the ID we are running the query on
 *     5) The data we receive looks like this:
 *       
 *        msg id, epoch time, msg status, time message was sent, receiver message was sent to, the message\n
 *
 *        A sample message could look like this:
 *        8671r 971401500, 020, Thu Oct 12 21:45:00 2000, 0 Pagemart snpp, I am a scheduled message\n
 *
 *     6) We check each line of data to see if it is the string "time\n", when we get that string we know
 *        the next line of data will be the next scheduled time(in a human readable format) for this message to be sent.
 *
 *     Note - If we query the server for status on a message that has yet to be sent we will only get the "time\n" string, 
 *            and the time data, followed by the "done\n" string.
 *
 *     NEED TO DO: I need to report status back to the user when we send an ID that the server does not have, this could
 *     happen if an error occurs and the server loses data, or if someone munges a file by editing it.
 */

sched_update  *obtain_schedule_status(int sock, gchar *id, GtkWidget *text, GtkWidget *list, char *msg_time, sched_win_data *user_data, gchar *count)
{
    FILE          *fp;
    gchar         *buffer, *buffer1;
    gchar         *mode = "3\n";
    gchar         *scode, *sdata, *sreceiver, *smsg, *e_msg, *track, *uid;
    gchar         *m_epoch;
    gchar         *line, *two_way_log;
    gchar         *dash = "---------------------------------------\n";
    gint          flag = 0;
    gint          length = 0;
    gint          id_len = 0;
    gint          rflag = 1;	/* This is a flag that is set when we have a recurring page */
    gint          lflag = 0;	/* This is a flag that is set if we got logging info about a scheduled page */
    gint          row = 0;

    GdkFont       *current_font;
    sched_update  *data;

    row = GPOINTER_TO_INT(GTK_CLIST(user_data->clist)->selection->data);
    id_len = strlen(id);
    id_len--;			/* We use this to point to the last char in the id */

    if ( ( fp = get_file_pointer("log.gpage", "a+")) == NULL)
        return (NULL);

    scode = g_malloc(32);
    sdata = g_malloc(64);
    sreceiver = g_malloc(MAX_LENGTH);
    smsg = g_malloc(MAX_LENGTH);
    track = g_malloc(MAX_LENGTH);
    uid = g_malloc(MAX_LENGTH);

    data = g_malloc(sizeof(*data));
    buffer = g_malloc(MAX_LENGTH);
    buffer1 = g_malloc(MAX_LENGTH);
    m_epoch = g_malloc(MAX_LENGTH);

    commit_transaction(sock, mode);
    strcat(id, "\n");
    commit_transaction(sock, id);

    delete_text_in_text_box(text);

    gtk_text_set_point(GTK_TEXT(text), 0);
    current_font = gdk_font_load(buffer);
    id[strlen(id) -1] = '\0';
    flag = 0;
    rflag = 1;
    lflag = 0;

    do {
	read_a_line(sock, buffer, MAX_LENGTH);
	write_num(sock, buffer, strlen(buffer));

	remove_carrige_return(buffer);

	if ( (strcmp(buffer, "done\n")) && (strlen(buffer)) )
	{

/*
 *     Check to see if this is a recurring page here, if we have an 'r' at the end of the id it's recurring
 */

	    if ( *(id + id_len) != 'r' )
		rflag = 0;

	    if ( !strcmp(buffer, "time\n") )
	    {
		read_a_line(sock, buffer, MAX_LENGTH);
		write_num(sock, buffer, strlen(buffer));
		if ( strlen(buffer) < 1 )
		    continue;
/*
 *     This is where we capture the time of the next scheduled attempt to send this page, the time looks like this:
 *     hh:mm AM/PM MM/DD/YYYY 
 *     An example could look like this: 7:30 AM 10/5/2000
 */

		if ( buffer[strlen(buffer) -1] == '\n' )
		    buffer[strlen(buffer) -1] = '\0';

		gtk_label_set_text(GTK_LABEL(user_data->time_label), buffer);
		gtk_widget_show(user_data->time_label);
		continue;
	    }

/*
 *     We only delete the data if this page is not recurring
 */

	    if ( !rflag )  
		delete_msg_id(id, list);
	    
	    lflag = 1;
	    parse_status_data(buffer, m_epoch, scode, sdata, sreceiver, track, uid, smsg);
	    
/*
 *  If we sent a message to a two way pager we need to log the tracking info in the
 *  two way log file so the user can check for a reply
 */

	    if ( (scode[0] == '0') &&  (strcmp(track, "NULL"))  )
	    {
		two_way_log = g_strdup_printf("%s,%s,U,w,%s,%s,%s", sdata, uid,track,sreceiver,smsg);
		add_data(two_way_log, "2way_log.gpage");
		g_free(two_way_log);
	    }

	    //line = g_strdup_printf("%s,%s, %s, %s, %s\n", (m_epoch + 1), scode,sdata,sreceiver,smsg);
	    line = g_strdup_printf("%s,%s, %s, %s, %s\n", m_epoch, scode,sdata,sreceiver,smsg);
	    fputs(line, fp);
	    g_free(line);

/*
 *   Here is where we keep track of how many times the page has been sent. If the first characer in the message status is 
 *   is a zero then the page was good
 */

	    if ( scode[0] == '0' )
	    {
		sprintf(count, "%d", (atoi(count) + 1) );
		/*
		num = atoi(count) + 1;
		fprintf(stderr, "Here is the num: %d\n", num);
		g_free(count);
		count = g_strdup_printf("%d", (num + 1) );
		*/
	    }
	    gtk_label_set_text(GTK_LABEL(user_data->count_label), count);
	    gtk_widget_show(user_data->count_label);

/*
 *     This is where we load the data from the sent page into the text box, we tell the user if the page was sent 
 *     or the error messaege if it was not sent, the time it was sent, who it was sent to, and the message itself
 */	    

	    e_msg = get_error_string(scode);
	    line = g_strdup_printf("%s\nTIME: %s\nTO: %s\nMSG: %s\n", e_msg, sdata, sreceiver, smsg); 
	    g_free(e_msg);

	    length = gtk_text_get_length( GTK_TEXT(text) );
	    gtk_text_set_point( GTK_TEXT( text) , length );
	    gtk_text_insert(GTK_TEXT(text), current_font, NULL, NULL, line, strlen(line));

	    length = gtk_text_get_length( GTK_TEXT(text) );
	    gtk_text_set_point( GTK_TEXT( text) , length );
	    gtk_text_insert(GTK_TEXT(text), current_font, NULL, NULL, dash, strlen(dash));
	    gtk_text_set_point( GTK_TEXT( text) , 0);
	    flag = 1;
	    g_free(line);
	}
	else
	{
/*
 *     If we get here then the no attempt has been made to send the page yet, or the server has a corrupt file.
 */
	    if ( !flag && !rflag )
	    {
		strcpy(buffer1, "No logging data is available for this message");
		gtk_text_insert(GTK_TEXT(text), current_font, NULL, NULL, buffer1, strlen(buffer1));
		gtk_text_set_point( GTK_TEXT( text) , 0);
	    }
	}

    }while( strcmp(buffer, "done\n") );

/*
 *     This is a kludge to get the text box to scroll to the top. It seems to ignore the set point 
 *     statement unless I insert something.
 */

    gtk_text_set_point( GTK_TEXT( text) , 0);
    gtk_text_insert(GTK_TEXT(text), current_font, NULL, NULL, "\0", 1);
    gtk_text_set_point( GTK_TEXT( text) , 1);
    gtk_text_backward_delete ( GTK_TEXT(text), 1 );
    gtk_text_set_point( GTK_TEXT( text) , 0);

/*
 *     Another kludge to force the client to re-read the scheduled page data so we can make
 *     sure we have the correct count value after a page, or pages have been sent
 */

    if ( flag && rflag )
    {
	gtk_clist_freeze(GTK_CLIST(user_data->clist));
	updata_message_count(count, id);
	gtk_clist_clear(GTK_CLIST(user_data->clist));
	load_scheduled_pages(user_data->clist);
	gtk_clist_select_row(GTK_CLIST(user_data->clist), row, 0);
	gtk_clist_thaw(GTK_CLIST(user_data->clist));
    }

    g_free(scode);
    g_free(sdata);
    g_free(sreceiver);
    g_free(smsg);
    g_free(buffer);
    g_free(buffer1);
    g_free(m_epoch);
    g_free(track);
    g_free(uid);

    fclose(fp);
    return (data);

}

/*
 *     If we have sent a recurring page we need to update the counter field, we do this by writing the new
 *     data to a temp file then renaming the files
 */

void updata_message_count(gchar *count, gchar *id)
{

    FILE          *fp_sch, *fp_tmp;
    gchar         *sid;
    gchar         *s_time;
    gchar         *sch_id;
    gchar         *msg;
    gchar         *line;
    gchar         *name;
    gchar         *recur;
    gchar         *tcount;
    gchar         *buffer;
    gchar         *full_path_old;
    gchar         *full_path_new;

    sid = g_malloc(MAX_LENGTH);
    s_time = g_malloc(MAX_LENGTH);
    sch_id = g_malloc(MAX_LENGTH);
    msg = g_malloc(MAX_LENGTH);
    name = g_malloc(MAX_LENGTH);
    line = g_malloc(MAX_LENGTH + 256);
    recur = g_malloc(32);
    tcount = g_malloc(32);

    if ( ( fp_sch = get_file_pointer("SCHED_DB.dat", "r")) == NULL)
	return;
    if ( ( fp_tmp = get_file_pointer("t_sch_db.tmp", "w")) == NULL)
	return;
    
    while (fgets(line, MAX_LENGTH, fp_sch) )
    {
	if ( parse_line(line, sid, sch_id, s_time, msg, recur, tcount, 0) )
	    if ( !strcmp(sch_id, id) )
	    {
		buffer = g_strdup_printf("%s %s %s %s %s\n", sid, sch_id, s_time, count, msg);
		fputs(buffer, fp_tmp);
		g_free(buffer);
	    }
	    else
	    {
		fputs(line, fp_tmp);
	    }
	else
	{
	    fclose(fp_sch);
	    fclose(fp_tmp);
	    generic_error("\n Corrupt database file: SCHED_DB.dat \n", "G-page Error");
	    g_free(sid);
	    g_free(s_time);
	    g_free(sch_id);
	    g_free(msg);
	    g_free(name);
	    g_free(line);
	    g_free(recur);
	    g_free(tcount);
	    return;
	}
	    
    }
    fclose(fp_sch);
    fclose(fp_tmp);

    full_path_old = get_working_dir();
    strcat(full_path_old, "SCHED_DB.dat"); 
    full_path_new = get_working_dir();
    strcat(full_path_new, "t_sch_db.tmp"); 

    if (rename(full_path_new, full_path_old))
	perror("Error renaming temp file - update message count: ");
    
    g_free(sid);
    g_free(full_path_old);
    g_free(full_path_new);
    g_free(s_time);
    g_free(sch_id);
    g_free(msg);
    g_free(name);
    g_free(line);
    g_free(recur);
    g_free(tcount);

}

/*
 *     A non-recurring message has been sent or is being deleted. We have already told the server to delete it's data
 *     so this function only deletes the client data.
 */

void  delete_msg_id(gchar *id, GtkWidget *list)
{
    FILE    *fp, *fpt;
    gchar   *line, *msg_id;
    gchar   *temp_line;
    gint    i = 0;
    gint    x = 0;

    if ( (fp = get_file_pointer("SCHED_DB.dat", "r")) == NULL)
    {
	fprintf(stderr, "Error opening SCHED_DB.dat load_scheduled_pages my_functions.c\n");
	return;
    }

    line = g_malloc(MAX_LENGTH + 256);
    msg_id = g_malloc(MAX_LENGTH);

    fpt = get_temp_file();
    if ( fpt == NULL )
    {
	g_free(line);
	g_free(msg_id);
	return;
    }

    while ( fgets(line, MAX_LENGTH + 256, fp) )
    {
	while ( *(line + i++) != ' ' );
	while ( *(line + i) != ' ' )
	    *(msg_id + x++) = *(line + i++);
	*(msg_id + x) = '\0';
	x = i = 0;
	if ( !strcmp(id, msg_id) )
	    continue;
	else
	    fputs(line, fpt);
    }
    
    fclose(fp);
    rewind(fpt);

    if ( (fp = get_file_pointer("SCHED_DB.dat", "w")) == NULL)
	return;

    temp_line = g_malloc(MAX_LENGTH);

    while (fgets(temp_line, MAX_LENGTH, fpt))
      fputs(temp_line, fp);
    fclose(fp);
    fclose(fpt);
    g_free(temp_line);
    
    gtk_clist_freeze(GTK_CLIST(list));
    gtk_clist_clear( GTK_CLIST(list));
    load_scheduled_pages(list);
    gtk_clist_thaw(GTK_CLIST(list));

    g_free(line);
    g_free(msg_id);

    return;
}

/*
 *     Here we parse the log data sent to us from the server. We need to parse this so we can report 
 *     status to the user in a human readable format.
 */

void parse_status_data(gchar *line, gchar *epoch, gchar *code, gchar *sdata, gchar *sreceiver, gchar *track, gchar *uid, gchar *msg)
{
    gint   i, x;
    
    i = x = 0;

    while ( *(line + i) == ' ' ) /* Get rid of leading spaces */
	i++;

    while ( *(line + i) != ',' )
	*(epoch + x++) = *(line + i++);
    *(epoch + x) = '\0';
    x = 0;
    i += 2;			/* Skip the comma and the blank space */

    while ( *(line + i) != ',' )
	*(code + x++) = *(line + i++);
    *(code + x) = '\0';
    x = 0;
    i += 2;

    while ( *(line + i) != ',' )
	*(sdata + x++) = *(line + i++);
    *(sdata + x) = '\0';
    x = 0;
    i += 2;

    while ( *(line + i) != ',' )
	*(sreceiver + x++) = *(line + i++);
    *(sreceiver + x) = '\0';
    x = 0;
    i += 2;

    while ( *(line + i) != ',' )
	*(track + x++) = *(line + i++);
    *(track + x) = '\0';
    x = 0;
    i += 2;

    while ( *(line + i) != ',' )
	*(uid + x++) = *(line + i++);
    *(uid + x) = '\0';
    x = 0;
    i += 2;

    while ( *(line + i) != '\n' )
	*(msg + x++) = *(line + i++);
    *(msg + x) = '\0';

}

