/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#define MAXFD  64

#include "constants.h"
#include "callbacks.h"

/*
 *   Evil Global Variables
 */

script_text *active_script_data;
char        named_pipe[32];
gchar       *response_one, *response_two, *response_three;
gint        my_pid;
gint        sig_length;
gint        network_db;
gint        live_page;
gint        log_delete_active;
guint       log_poll_timer_id = -1;
guint       timeout_id;
int         daemon_flag;
gint        command_line_flag;
pthread_t   autopage_thread = -1;

static void create_empty_file( gchar  *filename);
static void create_pipe(void);
static void get_database_info();
static void init_for_daemon();
static gint is_valid_file(gchar *filename);

pthread_mutex_t  update_live_page_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t  update_reply_log_mutex = PTHREAD_MUTEX_INITIALIZER;

/*
 *   This gets called if the users kills the window by clicking on 
 *   the exit buton on the main title bar
 */

gint destroy_event( GtkWidget *widget, GdkEvent  *event, gpointer  data )
{
    extern  char named_pipe[32];

    if ( get_status_script_page() )
	return(TRUE);
    unlink(named_pipe);         /* Do some cleanup */
    gtk_main_quit();
    return(FALSE);
}        


int
main (int argc, char *argv[])
{
  extern int        my_pid;	         /* The process id of gpage */
  extern GtkWidget  *receiver_clist;     /* This is the main list of receivers and groups on the main screen */
  extern gint       sig_length;
  extern gint       network_db;
  extern guint      timeout_id;
  extern int        daemon_flag;
  extern gint       command_line_flag;

  gchar             *resource_field;
  gchar             *rc_location;
  gint              first_time = 0;      /* Is this the first time gpage has been run? */
  gint              i;

  GtkWidget         *window_main;

  Signal(SIGCHLD, sig_child);

  daemon_flag = 0;
  network_db = 0;
  command_line_flag = 0;

  my_pid = getpid();  

  if ( argc > 1 )
  {
      if (!strcmp(argv[1], "auto") )
      {
	  if ( is_valid_file(argv[2]) )
	      info_msg("running as a daemon in auto mode\n");
	  else
	  {
	      info_msg("Error: ~/.gpage/profiles/%s does not exist\n", argv[2]);
	      exit(1);
	  }
	  
	  daemon_flag = 1;

	  init_for_daemon();

	  openlog("gpage_client", LOG_PID, LOG_USER);

	  if ( ( fork() ) == 0 ) /* We are the child  */
	  {
	      if ( ( setsid() ) == -1 ) /* Become the session leader */
	      {
		  perror("Error Establishing new session... exiting");
		  exit(EXIT_FAILURE);
	      }
	      
	      Signal(SIGHUP, SIG_IGN);
	      
	      if ( (fork()) != 0 ) /* The first child proc terminates */
		  exit(EXIT_SUCCESS);
	      
	      if ( (chdir("/")) == -1 )
	      {
		  perror("Error changing working directory to root");
		  exit(EXIT_FAILURE);
	      }
	      umask(0);
	      
	      for ( i = 0; i < MAXFD; i++ )
		  close(i);

	      reload_previous_script_daemon(argv[2]);
	      exit(EXIT_SUCCESS);
	  }
	  else
	      exit(EXIT_SUCCESS);	/* Parent must exit */
      }
      else /* We are being called from the command line */
      {
	  gchar              *uid;
	  uid_list           *list;
	  gint               good_id = 0;
	  gchar              *message;

	  command_line_flag = 0;
	  get_database_info();

	  list = get_uid_node();

	  uid = get_id_from_receiver_name(argv[1]);

	  if ( strlen(uid) )	/* This is a empty string if we didn't match the name to an ID */
	  {
	      strcpy(list->uid, uid);
	      list->next = get_uid_node();
	      good_id = 1;
	      g_free(uid);
	  }
	  else   /* no match in the receiver database - look in the group database */
	  {
	      g_free(uid);
	      uid = get_id_from_group_name(argv[1]);
	      if ( strlen(uid) )
	      {
		  parse_group(uid, list);
		  good_id = 1;
	      }
	      g_free(uid);
	  }
	  
	  if ( good_id )
	  {
	      if ( strlen(argv[2]) )
	      {
		  command_line_flag = 1;
		  create_pipe();

		  message = g_strdup(argv[2]);
		  connect_to_server(list, message, "none", NULL, AUTO, NULL, -1);
		  unlink( named_pipe );
		  exit(EXIT_SUCCESS);
	      }
	      else		/* We have a zero length message - no good */
	      {
		  exit(EXIT_FAILURE);
	      }

	  }
	  else /* This is not a valid receiver or group name */
	      exit(EXIT_FAILURE);    /* I need to log an error here */

      }
  }



/*
 *   Here is the thread init stuff 
 */

  //g_thread_init(NULL);

  rc_location = g_strdup_printf("%s/.gpage/rc_gpage", getenv("HOME"));
  gtk_rc_parse(rc_location);
  g_free(rc_location);

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  first_time = initialize_main();
  create_pipe();

  window_main = create_window_main ();

  init_script_page_data();

  load_receiver_list("RECEIVER.dat", GTK_CLIST(receiver_clist));
  load_receiver_list("GROUP.dat", GTK_CLIST(receiver_clist));

  load_main_list_with_pixmaps();

  GTK_CLIST(receiver_clist)->focus_row = 0;
  gtk_clist_moveto(GTK_CLIST(receiver_clist), 0, 0, 0.0, 0.0);

  check_server();

  if (first_time)
  {
      gtk_widget_show (window_main);

/*
 *    This is the first time gpage has been run, or at least there were no
 *    data files found so we are going to assume this is the first time. We
 *    are going to display a dialog box telling them where the database files
 *    are going to be located
 */

      notify_user();
  }
  else
      gtk_widget_show (window_main);

  timeout_id = 0;

  if ( network_db )		/* network_db is set in the call to initialize_main() */
  {
      if ( (get_network_database()) ) /* This function returns a one if it cannot connect to the network database */
      {
	  edit_item("database", "CONFIG.dat", 2, 1, "database=local\n", EDIT, '=');
	  network_db = 0;
      }
      else
      {
	  if ( (resource_field = get_resource_data("CONFIG.dat", "update_interval", MAX_FIELD_LENGTH)) )
	  {
	      if ( strcmp(resource_field, "0") )
		  timeout_id = gtk_timeout_add( (atoi(resource_field) * 60) * ONE_SECOND, (GtkFunction)network_db_refresh, NULL);
	      g_free(resource_field);
	  }
      }
  }
       
  if ( (resource_field = get_resource_data("CONFIG.dat", "msg_sig", MAX_FIELD_LENGTH)) )
  {
      if ( !strcmp(resource_field, "no") )
      {
	  sig_length = 0;
	  g_free(resource_field);
      }
      else
      {
	  g_free(resource_field);
	  if ( (resource_field = get_resource_data("CONFIG.dat", "sig", MAX_FIELD_LENGTH)) )
	  {
	      sig_length = strlen(resource_field);
	      g_free(resource_field);
	  }
	  else
	      sig_length = 0;
      }
  }
  else
      sig_length = 0;		/* default to zero if we can't read the resource file */

  if ( (resource_field = get_resource_data("CONFIG.dat", "time_stamp", MAX_FIELD_LENGTH)) )
  {
      if ( !strcmp(resource_field, "yes") )
	  sig_length += 6;
      g_free(resource_field);
  }
  /* else we do nothing on error here, leave the sig length as it is */
  
/*
 *   Here is the last of the thread init stuff 
 */
    
  //gdk_threads_enter();
  gtk_main();
  //gdk_threads_leave();

  return 0;
}


/*
 *     Look for all the needed database files here. We create them if they don't exist. This 
 *     function can be called from anywhere in the program, for example if the user selects
 *     a new location for the database files.
 */

int  initialize_main()
{
    extern  gint        live_page;
    extern  gchar       *response_one, *response_two, *response_three;
    extern  gint        log_delete_active;
    extern  guint       log_poll_timer_id;

    FILE    *fp;
    struct  stat statbuf;
    mode_t  mode;
    gchar   *full_path;		/* This is the absolute path of the directory holding a data file */
    gchar   *home;		/* This is the absolute path of the users home directory */
    gchar   temp[MAX_LENGTH];
    gchar   *resource_field;
    gint    first = 0;
    gint    poll_interval;

    response_one = response_two = response_three = NULL;
/*
 *     We check to see if the ~/.gpage directory exists, if not then we assume this is 
 *     the first time gpage has been run and we create all the 
 *     database files. We will load default values in the data files
 *     where appropriate.
 */

    //log_delete_active = 0;
    live_page = 0;

    home = g_strdup_printf("%s/.gpage", getenv("HOME"));

    memset(&statbuf, 0, sizeof(statbuf));
    stat(home, &statbuf);
    mode = statbuf.st_mode;
    if (! S_ISDIR(mode))
    {
	first = 1;
	if ( (mkdir(home, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP
		         | S_IXGRP | S_IXOTH)) == -1)
	{
	    perror("Error creating .gpage directory: ");
	    exit(EXIT_FAILURE);
	}
    }
    else
	get_database_info();
/*
 *   Create default CONFIG.dat if it doesn't exist. The default values are pretty
 *   generic here.
 */

    memset(&statbuf, 0, sizeof(statbuf));
    full_path = g_strdup_printf("%s/CONFIG.dat", home);
    stat(full_path, &statbuf);
    mode = statbuf.st_mode;
    if (! S_ISREG(mode) )
    {
	if ( ( fp = fopen(full_path, "w+")) == NULL)
	{
	    perror("Error creating file CONFIG.dat - main.c");
	    _exit(EXIT_FAILURE);
	}
	else
	{
	    fputs("ip_address=localhost\n", fp);
	    fputs("port=1299\n", fp);
	    fputs("timeout=60\n", fp);
	    fputs("isp_domain=localhost\n", fp);
	    fputs("isp_email_domain=localhost\n", fp);
	    fputs("email_adr=localhost@localdomain\n", fp);
	    g_snprintf(temp, MAX_LENGTH, "db_dir=%s/\n", home);	  /* home has the absolute path of the users home directory */
	    fputs(temp, fp);
	    fputs("auto_delete=no\n", fp);
	    fputs("wait_days=1\n", fp);
	    fputs("msg_sig=no\n", fp);
	    fputs("sig_placement=after\n", fp);
	    fputs("sig_snpp=yes\n", fp);
	    fputs("sig_email=no\n", fp);
	    fputs("sig=\n", fp);
	    fputs("last=No previous message\n", fp);
	    fputs("cc_email=no\n", fp);
	    fputs("set_protocols=no\n", fp);
	    fputs("order_of_protocols=123\n", fp);
	    fputs("show_advanced_options=no\n", fp);
	    fputs("default_address=localhost\n", fp);
	    fputs("default_port=444\n", fp);
	    fputs("qpage_active=no\n", fp);
	    fputs("status_mode=yes\n", fp);
	    fputs("serial_mode=yes\n", fp);
	    fputs("time_stamp=no\n", fp);
	    fputs("polling_interval=10\n", fp);
	    fputs("script_delay=2\n", fp);
	    fputs("locate_retries=10\n", fp);
	    fputs("database=local\n", fp);
	    fputs("database_adr=localhost\n", fp);
	    fputs("database_port=1301\n", fp);
	    fputs("update_interval=0\n", fp);
	    fputs("clear_msg_box=always\n", fp);
	    fputs("log_delete_poll=0\n", fp);
	    fputs("rollover_no_connect=no\n", fp);
	    fputs("dial_no_connect=no\n", fp);
	    fputs("create_error_status_file=no\n", fp);
	    fclose(fp);
	}
    }

    g_free(full_path);

    memset(&statbuf, 0, sizeof(statbuf));
    g_free(home);
    home = get_working_dir();
    full_path = g_strdup_printf("%sprofiles", home);
    stat(full_path, &statbuf);
    mode = statbuf.st_mode;
    if (! S_ISDIR(mode) )
	mkdir(full_path, S_IRUSR | S_IWUSR | S_IXUSR);

    g_free(home);
    g_free(full_path);

/*
 *  Check to see if the database files exist
 */

    create_empty_file("RECEIVER.dat");

    create_empty_file("SERVICE.dat");

    create_empty_file("GROUP.dat");

    create_empty_file("MSG.dat");

    create_empty_file("SCHED_DB.dat");

    create_empty_file("profiles/AUTO_SCRIPT.dat");

    create_empty_file("TWO_WAY_RESPONSE.dat");

    create_empty_file("reply.gpage");

    create_empty_file("2way_log.gpage");

    full_path = get_working_dir();
    strcat(full_path, "IDS.dat");
    memset(&statbuf, 0, sizeof(statbuf));
    stat(full_path, &statbuf);
    mode = statbuf.st_mode;

    if (! S_ISREG(mode) )
    {
	if ( ( fp = get_file_pointer("IDS.dat", "w+")) == NULL)
	    _exit(EXIT_FAILURE);
	else
	{
	    fputs("-1\n", fp);
	    fputs("RECEIVER,2\n", fp);
	    fputs("-1\n", fp);
	    fputs("SERVICE,2\n", fp);
	    fputs("-1\n", fp);
	    fputs("GROUP,2\n", fp);
	    fclose(fp);
	}
    }
    g_free(full_path);


/*
 *  Create the log file and check for auto-delete logs. If the user has auto-log-delete
 *  turned on then we willl check to see if there are any logs which need to be deleted
 */

    full_path = get_working_dir();
    strcat(full_path, "log.gpage");
    memset(&statbuf, 0, sizeof(statbuf));
    stat(full_path, &statbuf);
    mode = statbuf.st_mode;
    if (! S_ISREG(mode) )
    {
	if ( ( fp = get_file_pointer("log.gpage", "w+")) == NULL)
	    _exit(EXIT_FAILURE);
	else
	    fclose(fp);
    }
    else if ( log_delete_active == 0 )
    {
	if ( (resource_field = get_resource_data("CONFIG.dat", "log_delete_poll", MAX_FIELD_LENGTH)) )
	{
	    poll_interval = atoi(resource_field);

	    if ( poll_interval != 0 )
	    {
		automated_check_for_log_delete();
		log_poll_timer_id = gtk_timeout_add( ( 60 * ONE_SECOND  * 60 * poll_interval ), (GtkFunction)automated_check_for_log_delete, NULL); 
	    }
	    else
		automated_check_for_log_delete();

	    g_free(resource_field);
	}
    }

    g_free(full_path);

return (first);
}


static void init_for_daemon()
{
    extern  gint        live_page;
    extern  gint        log_delete_active;
    extern  gint        network_db;
    extern  gint        sig_length;
    extern  gchar       *response_one, *response_two, *response_three;

    gchar   *resource_field;

    create_pipe();
    init_script_page_data();

    log_delete_active = 0;
    live_page = 0;
    network_db = 0;
    response_one = response_two = response_three = NULL;

    get_database_info();	/* This sets the global network_db flag */

    if ( (resource_field = get_resource_data("CONFIG.dat", "msg_sig", MAX_FIELD_LENGTH)) )
    {
	if ( !strcmp(resource_field, "no") )
	{
	    sig_length = 0;
	    g_free(resource_field);
	}
	else
	{
	    g_free(resource_field);
	    if ( (resource_field = get_resource_data("CONFIG.dat", "sig", MAX_FIELD_LENGTH)) )
	    {
		sig_length = strlen(resource_field);
		g_free(resource_field);
	    }
	    else
		sig_length = 0;
	}
	
    }
    else
	sig_length = 0;

    if ( (resource_field = get_resource_data("CONFIG.dat", "time_stamp", MAX_FIELD_LENGTH)) )
    {
	if ( !strcmp(resource_field, "yes") )
	    sig_length += 6;
	g_free(resource_field);
    }
    /* else - we do nothing with the sig length */


}


static void create_pipe(void)
{
    FILE          *fp;
    extern  char  named_pipe[32];
    extern  int   my_pid;
    extern  int   daemon_flag;

    gchar   *pid_str;
    gint    pipe_result;
    
    memset(named_pipe, '\0', sizeof(named_pipe));
    
    pid_str = g_strdup_printf("%d", my_pid);
    
    strcpy(named_pipe, "/tmp/pclient_");
    strcat(named_pipe, pid_str);

    pipe_result = mkfifo(named_pipe, 0777);
    if (pipe_result < 0)
    {
	perror("Error making FIFO");
	    update_status_line("Error creating pipe");
    }
    
    g_free(pid_str);
    
    if (daemon_flag)
    {
	fp = get_file_pointer("daemon_cleanup", "w");
	fputs(named_pipe, fp);
	fclose(fp);
    }
}

void create_empty_file(gchar  *filename)
{
    gchar          *full_path;
    struct  stat   statbuf;
    mode_t         mode;
    FILE           *fp;

    full_path = get_working_dir();
    strcat(full_path, filename);
    memset(&statbuf, 0, sizeof(statbuf));
    stat(full_path, &statbuf);
    mode = statbuf.st_mode;
    if (! S_ISREG(mode) )
    {
	if ( ( fp = get_file_pointer(filename, "w+")) == NULL)
	    _exit(EXIT_FAILURE);
	else
	    fclose(fp);
    }
    g_free(full_path);
}

Sigfunc *Signal(int signo, Sigfunc *func)
{

    struct sigaction  act, oact;

    act.sa_handler = func;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    if (  signo == SIGALRM )
    {
#ifdef SA_INTERRUPT
	act.sa_flags |= SA_INTERRUPT;
#endif
    }
    else
    {
#ifdef SA_RESTART
	act.sa_flags |= SA_RESTART;
#endif
    }
    if ( sigaction(signo, &act, &oact) < 0 )
	return(SIG_ERR);
    return (oact.sa_handler);

}

void check_server()
{
    gint sock;

    sock = server_connection(NORMAL);

    if ( sock == -1 )
	update_status_line("gserver status: not running");
    else
	if ( sock == -2 )
	    update_status_line("Permission to connect to server denied");
    else
    {
	update_status_line("gserver status: running");
	close(sock);
    }

}

void get_database_info()
{
    extern gint  network_db;
    gchar        *data;
    

    if ( (data = get_resource_data("CONFIG.dat", "database", MAX_FIELD_LENGTH)) )
    {
	if ( !strcmp(data, "local") )
	    network_db = 0;
	else
	    network_db = 1;
	g_free(data);
    }
    else
	network_db = 0;


}

gint automated_check_for_log_delete()
{
    extern pthread_mutex_t   update_live_page_mutex;

    gint                     mutex_status;
    gint                     active;

    active = 0;

    if ( (mutex_status = pthread_mutex_lock(&update_live_page_mutex)) != 0 )
	perror("Error locking mutex - possible data corruption");

    active = handle_auto_log_delete();

    if ( (mutex_status = pthread_mutex_unlock(&update_live_page_mutex)) != 0 )
	perror("Error unlocking mutex - possible data corruption");

    return (active);
}

/*
 *    This is used if we are running as a daemon to verify that the 
 *    profile given on the command line is an actual file
 */

static gint is_valid_file(gchar *filename)
{
    gchar          *full_path;
    struct  stat   statbuf;
    mode_t         mode;

    memset(&statbuf, 0, sizeof(statbuf));

    full_path = g_strdup_printf("%s/.gpage/profiles/%s", getenv("HOME"), filename);

    stat(full_path, &statbuf);
    mode = statbuf.st_mode;
    g_free(full_path);

    if (S_ISREG(mode) )
	return (1);
    else
	return (0);
}
