/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

static gint get_log_mode(gchar *buffer);
static void clean_up(gchar *one, gchar *two);


enum {
    ONE_WAY_LOG_DATA = 0,
    TWO_WAY_REPLY_DATA = 1,
    TWO_WAY_LOG_DATA = 2,
    CONNECTION_ERROR = 3,
    WRITE_TO_SMALL_STATUS_LINE = 4,
    ALWAYS_UPDATE = 5,
    UPDATE_WHEN_NO_LIGHT_MODE = 6,
    INVALID_MODE = 7
};

void  monitor_message_transactions(int socket_fd, char *pg_message, gint file_desc, gint gui_mode)
{
    gint             number_of_loops = 0;
    gint             not_done = 1;
    gint             select_result, maxfdp1;
    gint             light_mode;
    gint             log_mode;
    gint             timeout_value;
    gchar            *out_buff;
    gchar            *timeout_data;
    gchar            *status_mode;
    fd_set           rset;

    struct timeval   time_out;

    debug_data("We got into monitor_message_transactions\n");
    g_free(pg_message);


    if ( (timeout_data = get_resource_data("CONFIG.dat", "timeout", MAX_FIELD_LENGTH)) == NULL)
    {
	update_status_line("Error: corrupt file: CONFIG.dat");
	return;
    }

    if ( (status_mode = get_resource_data("CONFIG.dat", "status_mode", MAX_FIELD_LENGTH)) == NULL)
    {
	update_status_line("Error: corrupt file: CONFIG.dat");
	g_free(timeout_data);
	return;
    }

    if (!strcmp(status_mode, "no"))
	light_mode = 1;
    else
	light_mode = 0;
    g_free(status_mode);

    maxfdp1 = socket_fd + 1;

    timeout_value = atoi(timeout_data);
    g_free(timeout_data);

    while ( not_done )
    {
	number_of_loops++;

/*
 *   Set the timeout here for reading from the server. Otherwise we would block here
 *   forever is something evil happens to the server
 */

	time_out.tv_sec = timeout_value;
	time_out.tv_usec = 0;
	
	FD_ZERO(&rset);
	FD_SET(socket_fd, &rset);
	select_result = select(maxfdp1, &rset, NULL, NULL, &time_out);

	if ( select_result < 0 )
	{
	    debug_data("MSG TRANS: we got an error in select\n");
	    if ( errno == EINTR ) /* restart if we catch an interrupt signal */
		continue;
	    else
	    {
		write_to_status_line("Error reading from server...connection lost ");
		write_to_status_line("done");
		return;
	    }
	}
	if ( select_result == 0 )
	{
	    debug_data("MSG TRANS: we got a timeout in select\n");
	    write_to_status_line("Timeout reading from server...terminating connection ");
	    write_to_status_line("done");
	    return;
	}
	if ( select_result > 0 )
	{
	    debug_data("MSG TRANS: we got data on the socket in select\n");
	    out_buff = read_a_big_line(socket_fd);

	    
	    if ( out_buff == NULL )
	    {
		if (number_of_loops < 2 )
		    write_to_status_line("Server has terminated connection");
		write_to_status_line("done");
		return;
	    }
	    else
	    {
#ifdef DEBUG		
		log_all(out_buff);
#endif
		remove_carrige_return(out_buff);
		*(out_buff + (strlen(out_buff) - 1)) = '\0'; /* kill the pesky newline char */
		debug_data("MSG TRANS: We read this: %s\n, out_buff");

		if ( strlen(out_buff) > 1 )
		{

		    log_mode = get_log_mode(out_buff);

/*
 *   The first two characters in out_buff are numbers which tell us what do to with the data.
 *   When we send (out_buff + 2) to another function we are skipping those numbers
 */

		    switch (log_mode) {

		    case  UPDATE_WHEN_NO_LIGHT_MODE:
			if ( !light_mode )
			    write_to_status_line( (out_buff + 2) );
			break;
		    case ALWAYS_UPDATE:
			write_to_status_line( (out_buff + 2) );
			break;
		    case WRITE_TO_SMALL_STATUS_LINE:
			write_to_status_line( (out_buff) );
			break;
		    case ONE_WAY_LOG_DATA:
			save_for_logging( (out_buff + 2), ONE_WAY_LOG_DATA, file_desc);
			break;
		    case TWO_WAY_LOG_DATA:
			save_for_logging( (out_buff + 2), TWO_WAY_LOG_DATA, file_desc);
			break;
		    case TWO_WAY_REPLY_DATA:
			save_for_logging( (out_buff + 2), TWO_WAY_REPLY_DATA, file_desc);
			break;
		    case CONNECTION_ERROR:
			save_for_logging( (out_buff + 2), CONNECTION_ERROR, file_desc);
			break;
		    case INVALID_MODE:
			break;
		    default:
			break;
		    }
		   
		    g_free(out_buff);
		}
	    }
	}
    }	 
}



static gint get_log_mode(gchar *buffer)
{
    if (buffer[0] == '1')
    {
	if (buffer[1] == '1')
	    return (ONE_WAY_LOG_DATA);

	if (buffer[1] == '2')
	    return (UPDATE_WHEN_NO_LIGHT_MODE);

	if (buffer[1] == '3')
	    return(ALWAYS_UPDATE);

	if (buffer[1] == '5')
	    return(ALWAYS_UPDATE);

	if (buffer[1] == '6')
	    return(WRITE_TO_SMALL_STATUS_LINE);

	else if (buffer[1] == '7')
	    return (TWO_WAY_LOG_DATA);

	else if (buffer[1] == '9')
	    return (TWO_WAY_REPLY_DATA);

	else
	    return (INVALID_MODE);
    }

    else if (buffer[0] == '2')
    {
	if (buffer[1] == '0')
	    return(CONNECTION_ERROR);
	else
	    return (INVALID_MODE);
    }
    else
	return (INVALID_MODE);
}

void  save_for_logging(char *buffer, int flag, gint file_desc)
{
    extern pthread_mutex_t   update_reply_log_mutex;

    FILE       *fp = NULL;
    time_t     timeval;
    struct tm  *tm_data;
    gchar      *time_buffer;
    gchar      *epoch;
    gint       mutex_status;


    debug_data("I got into the save_for_logging routine\n");
    time(&timeval);
    tm_data = localtime(&timeval);
    epoch = g_strdup_printf("%u,", (unsigned int)timeval);
    time_buffer = g_malloc(121);
    strftime(time_buffer, 120, "%b %d %Y %X", tm_data); 

    if ( (mutex_status = pthread_mutex_lock(&update_reply_log_mutex)) != 0 )
	debug_data("Error locking mutex - possible data corruption");
	//perror("Error locking mutex - possible data corruption");
    debug_data("I got into mutex for save_for_logging routine\n");

    switch (flag) {

    case TWO_WAY_LOG_DATA:
	if ( ( fp = get_file_pointer("2way_log.gpage", "a+")) == NULL )
	{
	    debug_data("Error opening log file");
	    //perror("Error opening log file");
	    clean_up(epoch, time_buffer);
	    return;
	}
	strcat(time_buffer, ",");
	fputs(time_buffer, fp);
	fputs(buffer, fp);
	fputs("\n", fp);
	fclose(fp);
	clean_up(epoch, time_buffer);
	return;

    case ONE_WAY_LOG_DATA:
	if ( ( fp = get_file_pointer("log.gpage", "a+")) == NULL )
	{
	    debug_data("Error opening log file");
	    //perror("Error opening log file");
	    clean_up(epoch, time_buffer);
	    return;
	}
	break;

    case CONNECTION_ERROR:

	write_num(file_desc, "Error\n", strlen("Error\n"));
	clean_up(epoch, time_buffer);
	return;

    case TWO_WAY_REPLY_DATA:
	write_num(file_desc, buffer, strlen(buffer));
	write_num(file_desc, "\n", strlen("\n"));
	if ( ( fp = get_file_pointer("reply.gpage", "a+")) == NULL )
	{
	    debug_data("Error opening log file");
	    //perror("Error opening log file");
	    clean_up(epoch, time_buffer);
	    return;
	}
	if ( !strcmp(buffer, "DELIVERED") || (strlen(buffer) == 0) )
	{
	    fclose(fp);
	    clean_up(epoch, time_buffer);
	    return;
	}
	strcat(time_buffer, ",");
	fputs(time_buffer, fp);
	fputs(buffer, fp);
	fputs("\n", fp);
	fclose(fp);

	clean_up(epoch, time_buffer);
	return;

    default:
	break;
    }

    strcat(buffer, "\n");
    fputs(epoch, fp);
    fputs(buffer, fp);
    fclose(fp);
    clean_up(epoch, time_buffer);
}

static void clean_up(gchar *one, gchar *two)
{
    extern pthread_mutex_t   update_reply_log_mutex;
    gint  mutex_status;

    g_free(one);
    g_free(two);

    if ( (mutex_status = pthread_mutex_unlock(&update_reply_log_mutex)) != 0 )
	debug_data("Error unlocking mutex - possible data corruption");
	//perror("Error unlocking mutex - possible data corruption");

    debug_data("I got past mutex for save_for_logging routine\n");
}

void log_all(char *data_buffer)
{
    FILE     *fp;
    time_t   timeval;
    gchar    buffer[MAX_LOG_LENGTH];

    memset(buffer, '\0', sizeof(buffer));
    time(&timeval);

    strcpy(buffer, ctime(&timeval));
    buffer[strlen(buffer) -1] = ' '; /* Kill newline character */
    strcat(buffer, " -- ");    
    strcat(buffer, data_buffer);

    if ( ( fp = get_file_pointer("g-page.log", "a+")) == NULL)
	return;

    fputs(buffer, fp);
    fclose(fp);

}

void remove_carrige_return(gchar *buffer)
{

    while ( *buffer )
    {
	if ( *buffer == '\r' )
	    *buffer = ' ';
	buffer++;
    }

}
