/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"
#include "g.xpm"
#include "dot.xpm"

void format_time(gchar *mtime, gchar *recur);

extern GtkWidget *receiver_clist;
extern GtkWidget *text_entry;
extern GtkWidget *service_clist;


/*
 *    This function loads the receiver list box for the main GUI.
 */

void load_generic_list(gchar *filename, GtkCList *generic_clist)
{
  FILE   *f_pointer;

  gint   x;
  gchar  *data[2];
  gchar  *id, *name;
  gchar  *line;

  g_assert(GTK_IS_WIDGET(generic_clist));

  if ( (f_pointer = get_file_pointer(filename, "r")) == NULL)
      return;

  id = g_malloc(MAX_FIELD_LENGTH);
  name = g_malloc(MAX_FIELD_LENGTH);

  while ( (line = (read_file_by_line(f_pointer))) != NULL)
  {
      get_name_id(id, name, line);
      g_free(line);

      data[0] = g_strdup(id);
      data[1] = g_strdup(name);

      gtk_clist_append( (GtkCList *)generic_clist, data);
    
      for (x = 0; x < 2; x++)
	  g_free(data[x]);
  }

  g_free(id);
  g_free(name);

  fclose(f_pointer);

  return;                               

}

void load_receiver_list(gchar *filename, GtkCList *generic_clist)
{
  FILE  *f_pointer;

  gint  x;
  gchar *data[3];
  gchar *id, *name;
  gchar *line;

  g_assert(GTK_IS_WIDGET(generic_clist));

  if ( (f_pointer = get_file_pointer(filename, "r")) == NULL)
      return;

  id = g_malloc(MAX_FIELD_LENGTH);
  name = g_malloc(MAX_FIELD_LENGTH);

  while ( (line = (read_file_by_line(f_pointer))) != NULL)
  {
      get_name_id(id, name, line);
      g_free(line);

      data[0] = g_strdup(id);
      data[2] = g_strdup(name);
      data[1] = g_strdup(" ");

      gtk_clist_append( (GtkCList *)generic_clist, data);
    
      for (x = 0; x < 3; x++)
	  g_free(data[x]);

  }

  g_free(id);
  g_free(name);

  fclose(f_pointer);

  return;                               

}

void load_pending_clist(GtkWidget *list)
{
  FILE       *f_pointer;

  gint       x, check;
  gchar      *data[7];
  gchar      *id, *name, *protocol, *tracking, *msg, *date, *status;
  gchar      *line;

  g_assert(GTK_IS_WIDGET(list));

  if ( (f_pointer = get_file_pointer("2way_log.gpage", "r")) == NULL)
      return;

  gtk_clist_freeze(GTK_CLIST(list));
  gtk_clist_clear(GTK_CLIST(list));

  id = g_malloc(MAX_FIELD_LENGTH);
  name = g_malloc(MAX_FIELD_LENGTH);
  protocol = g_malloc(MAX_FIELD_LENGTH);
  tracking = g_malloc(MAX_FIELD_LENGTH);
  msg = g_malloc(MAX_LENGTH);
  date = g_malloc(MAX_FIELD_LENGTH);
  status = g_malloc(MAX_FIELD_LENGTH);

  while ( (line = (read_file_by_line(f_pointer))) != NULL)
  {
      check = get_data(line, date, MAX_FIELD_LENGTH, id, MAX_FIELD_LENGTH, status, MAX_FIELD_LENGTH, name, MAX_FIELD_LENGTH, 
		       protocol, MAX_FIELD_LENGTH, tracking, MAX_FIELD_LENGTH, msg, MAX_LENGTH);
      
      g_free(line);

      if ( check == -1 )
      {
	  generic_error("\n  Unable to load data  \n  Corrupt file : 2way_log.gpage  \n", "G-Page Error");
	  break;
      }

      if ( protocol[0] == 'w' )
	  strcpy(protocol, "  wctp");
      else if ( protocol[0] == 's' )
	  strcpy(protocol, "  snpp");

      data[0] = g_strdup(date);
      data[1] = g_strdup(name);
      data[2] = g_strdup(status);
      data[3] = g_strdup(tracking);
      data[4] = g_strdup(msg);
      data[5] = g_strdup(protocol);
      data[6] = g_strdup(id);

      gtk_clist_append( (GtkCList *)list, data);
    
      for (x = 0; x < 7; x++)
	  g_free(data[x]);
      
  }

  g_free(id);
  g_free(name);
  g_free(tracking);
  g_free(protocol);
  g_free(msg);
  g_free(date);
  g_free(status);

  fclose(f_pointer);

  gtk_clist_thaw(GTK_CLIST(list));
  return;                               
}

void load_old_clist(GtkWidget *list)
{
  FILE  *f_pointer;

  gint  x, check;
  gchar *data[7];
  gchar *id, *name, *msg, *tracking, *reply, *date, *status;
  gchar *line;

  g_assert(GTK_IS_WIDGET(list));

  if ( (f_pointer = get_file_pointer("reply.gpage", "r")) == NULL)
      return;

  gtk_clist_freeze(GTK_CLIST(list));
  gtk_clist_clear(GTK_CLIST(list));

  id = g_malloc(MAX_FIELD_LENGTH);
  name = g_malloc(MAX_FIELD_LENGTH);
  tracking = g_malloc(MAX_FIELD_LENGTH);
  msg = g_malloc(MAX_LENGTH);
  reply = g_malloc(MAX_LENGTH);
  date = g_malloc(MAX_FIELD_LENGTH);
  status = g_malloc(MAX_FIELD_LENGTH);

  while ( (line = (read_file_by_line(f_pointer))) != NULL)
  {
      check = get_data(line, date, MAX_FIELD_LENGTH, id, MAX_FIELD_LENGTH, status, MAX_FIELD_LENGTH, name, MAX_FIELD_LENGTH, 
		       reply, MAX_LENGTH, tracking, MAX_FIELD_LENGTH, msg, MAX_LENGTH);
      g_free(line);

      if ( check == -1 )
      {
	  generic_error("\n  Unable to load data  \n  Corrupt file : reply.gpage  \n", "G-Page Error");
	  break;
      }

      data[0] = g_strdup(date);
      data[1] = g_strdup(name);
      data[2] = g_strdup(msg);
      data[3] = g_strdup(reply);
      data[4] = g_strdup(tracking);
      data[5] = g_strdup(status);
      data[6] = g_strdup(status);
      
      gtk_clist_append( (GtkCList *)list, data);
    
      for (x = 0; x < 7; x++)
	  g_free(data[x]);
      
  }
  g_free(id);
  g_free(name);
  g_free(tracking);
  g_free(msg);
  g_free(reply);
  g_free(date);
  g_free(status);

  fclose(f_pointer);

  gtk_clist_thaw(GTK_CLIST(list));
  return;                               

}

gint get_data(gchar *line, gchar *date, gint date_length, gchar *id, gint id_length, gchar *status, gint status_length, 
	      gchar *name, gint name_length, gchar *protocol, gint protocol_length, gchar *tracking, gint tracking_length, gchar *msg, gint msg_length)
{
    gint i, x, count;
    const gint min_number_commas = 6;
    
    i = x = count = 0;

    while( *(line + i) != '\n' )
    {
	if ( *(line + i ) == ',' )
	    count++;
	i++;
    }

    if (count < min_number_commas )		/* We need to make sure we have at least the min amount of commas in the line */
	return (-1);

    i = x = 0;

    while ( ( *(line + i) != ',' ) && ( x < date_length -1 ) )
	*(date + x++) = *(line + i++);
    *(date + x) = '\0';
    i++;
    x = 0;

    while ( ( *(line + i) != ',' )  && ( x < id_length - 1 ) )
	*(id + x++) = *(line + i++);
    *(id + x) = '\0';
    i++;
    x = 0;

    while ( ( *(line + i) != ',' )  && ( x < status_length - 1 ) )
	*(status + x++) = *(line + i++);
    *(status + x) = '\0';
    i++;
    x = 0;

    while ( ( *(line + i) != ',' )  && ( x < protocol_length - 1 ) )
	*(protocol + x++) = *(line + i++);
    *(protocol + x) = '\0';
    i++;
    x = 0;

    while ( ( *(line + i) != ',' )  && ( x < tracking_length - 1 ) )
	*(tracking + x++) = *(line + i++);
    *(tracking + x) = '\0';
    i++;
    x = 0;

    while ( ( *(line + i) != ',' )  && ( x < name_length - 1 ) )
	*(name + x++) = *(line + i++);
    *(name + x) = '\0';
    i++;
    x = 0;

    while ( ( *(line + i) != '\n' )  && ( x < msg_length - 1 ) )
	*(msg + x++) = *(line + i++);
    *(msg + x) = '\0';

    return (1);
}

gint load_scheduled_pages(GtkWidget *schedule_clist)
{
    FILE   *fp;

    gchar  *id;
    gchar  *s_time;
    gchar  *sch_id;
    gchar  *msg;
    gchar  *line;
    gchar  *temp_line;
    gchar  *name = "";
    gchar  *recur;
    gchar  *data[7];
    gchar  recur_string[32];
    gchar  *count;
    gint   i;
    gint   bad_data;

    g_assert(GTK_IS_WIDGET(schedule_clist));
    bad_data = 0;

    id = g_malloc(MAX_LENGTH);
    s_time = g_malloc(MAX_LENGTH);
    sch_id = g_malloc(MAX_LENGTH);
    msg = g_malloc(MAX_LENGTH);
    line = g_malloc(MAX_LENGTH + 256);
    recur = g_malloc(32);
    count = g_malloc(32);

    if ( (fp = get_file_pointer("SCHED_DB.dat", "r")) == NULL)
    {
	generic_error("\n Error opening SCHED_DB.dat \n", "G-Page Error");
	return(1);
    }
    
    while (fgets(line, (MAX_LENGTH + 256), fp))
    {
	parse_line(line, id, sch_id, s_time, msg, recur, count, 1);
	temp_line = get_uid_line("RECEIVER.dat", id, RECEIVER_FIELDS, 0, (MAX_LENGTH + 256) );

	if ( temp_line != NULL )
	    name = get_field(temp_line, RECEIVER_FIELDS, 2);
	else
	{
	    generic_error("\n Error loading scheduled paging data: \n Are you using the wrong database? \n", "G-Page Error");
	    bad_data = 1;
	    break;		/* We have a corrupt database here, no way to recover */
	}

	if ( !strcmp(recur, "0"))
	    strcpy(recur_string, "NO");
	else if( !strcmp(recur, "1"))
	    strcpy(recur_string, "HOURLY");
	else if( !strcmp(recur, "2"))
	    strcpy(recur_string, "DAILY");
	else if( !strcmp(recur, "3"))
	    strcpy(recur_string, "WEEKLY");

	data[0] = g_strdup(name);
	data[1] = g_strdup(s_time);
	data[2] = g_strdup(recur_string);
	data[3] = g_strdup(msg);
	data[4] = g_strdup(sch_id);
	data[5] = g_strdup(count);
	data[6] = g_strdup(" ");

	gtk_clist_append( (GtkCList *)schedule_clist, data);
	
	for ( i = 0; i < 6; i++ )
	    g_free(data[i]);

	g_free(name);
	g_free(temp_line);
    }
    fclose(fp);

    g_free(recur);
    g_free(id);
    g_free(s_time);
    g_free(sch_id);
    g_free(msg);
    g_free(line);
    g_free(count);

    return(bad_data);
}

void send_text()
{
    get_text(NULL, NULL);
}

void get_text(GtkWidget *w, gpointer *text_data)
{
    extern GtkWidget *receiver_clist;
    extern GtkWidget *text_entry;

    gchar     *entry_text;
    gchar     *msg;
    gint      text_length;
    gchar     *temp;
    uid_list  *active_list;

/*
 *   Make sure they have selected at least one receiver
 */

    if (GTK_CLIST(receiver_clist)->selection == NULL)
    {
	generic_error("\n  You must select a receiver  \n", "G-Page Error");
	return;
    }

/*
 *   This should never happen but I'm checking because if it does happen it
 *   could be problems
 */

    active_list = load_members();

    text_length = gtk_text_get_length( GTK_TEXT(text_entry) );
    if ( text_length > MAX_PAGE_LENGTH )
    {
	generic_error("\n Maximum number of characers exceeded   \n", "G-Page Error");
	return;
    }
    if ( text_length == 0 )
    {
	generic_error("\n   You have not entered a message   \n", "G-Page Error");
	return;
    }

    entry_text = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1));
    msg = g_malloc(MAX_LENGTH);
    entry_text  = clean_text(entry_text);
    strcpy(msg, entry_text);

    temp = g_strdup_printf("last=%s\n", entry_text);

    edit_item("last", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    g_free(temp);
    g_free(entry_text);

    connect_to_server(active_list, msg, "none", NULL, NORMAL, NULL, -1);

}

uid_list *load_members(void)
{
    extern GtkWidget *receiver_clist;
    GList            *selected;
    gchar            *user_id;
    gint             group_num;
    uid_list         *current_uid;
    uid_list         *head;
    uid_list         *temp;
    
    current_uid = get_uid_node();
    head = current_uid;

    selected = GTK_CLIST(receiver_clist)->selection;
    while (selected != NULL)
    {
	gtk_clist_get_text(GTK_CLIST(receiver_clist), GPOINTER_TO_INT(selected->data), 0, &user_id);
	if ( user_id[0] ==  'G' )
	{
	    group_num = parse_group(user_id, current_uid);
	    selected = selected->next;
	    while(current_uid->next)
		current_uid = current_uid->next;
	    continue;
	}
	selected = selected->next;
	strcpy(current_uid->uid, user_id);
	temp = get_uid_node();
	current_uid->next = temp;
	current_uid = temp;
    }
    gtk_clist_unselect_all(GTK_CLIST(receiver_clist));

/*
    current_uid = head;
    for( ; current_uid->next; current_uid = current_uid->next)
	fprintf(stderr, "UID: %s\n", current_uid->uid);
*/
    return (head);
}

uid_list *get_uid_node()
{
    uid_list  *node;

    node = g_malloc(sizeof(*node));
    node->uid = g_malloc(MAX_GROUP_UID_CHAR);
    node->next = NULL;
    
    return (node);
}

char *clean_text(char *buffer)
{
    gchar   *temp;
    
    temp = buffer;

    while ( *buffer )
    {
        if ( *buffer == '\n' )
            *buffer = ' ';

	buffer++;
    }

    return temp;
}


/*
 *     This function is used to load the clist box in the List Receivers dialog box
 */

void load_list_clist(GtkCList *list)
{

  gint    x;
  gchar   *data[5];
  gchar   *clist_data[RECEIVER_FIELDS];
  gchar   *service_data[SERVICE_FIELDS];
  FILE    *f_pointer, *sf_pointer;
  gint    index, field, ret_value;
  gchar   *line, *service_line;


  if ( (f_pointer = get_file_pointer("RECEIVER.dat", "r")) == NULL)
      return;

  for (field = 0; field < RECEIVER_FIELDS; field++)
      clist_data[field] = g_malloc(MAX_FIELD_LENGTH);

  for (field = 0; field < SERVICE_FIELDS; field++)
      service_data[field] = g_malloc(MAX_FIELD_LENGTH);


  while ( (line = (read_file_by_line(f_pointer))) != NULL)
  {
      ret_value = get_fields_from_string(clist_data, line, RECEIVER_FIELDS, ',');
      g_free(line);

      if (ret_value == 1)
      {
	  update_status_line("Error corrupt database: RECEIVER.dat");
	  for (field = 0; field < RECEIVER_FIELDS; field++)
	      g_free(clist_data[field]);

	  for (field = 0; field < SERVICE_FIELDS; field++)
	      g_free(service_data[field]);
	  fclose(f_pointer);

	  return;
      }
      
      data[0] = g_strdup(clist_data[REC_NAME]);
      data[1] = g_strdup(clist_data[REC_NUMBER]);
      data[2] = g_strdup(clist_data[REC_EMAIL_ADR]);
 /*
  *
  * Lookup the service name, it's stored in the 4th field of the receiver data
  *
  */

      if ( (sf_pointer = get_file_pointer("SERVICE.dat", "r")) == NULL)
	  return;

      while ( (service_line = (read_file_by_line(sf_pointer))) != NULL)
      {

	  ret_value = get_fields_from_string(service_data, service_line, SERVICE_FIELDS, ',');
	  g_free(service_line);

	  if (ret_value == 1)
	  {
	      for (field = 0; field < RECEIVER_FIELDS; field++)
		  g_free(clist_data[field]);

	      for (field = 0; field < SERVICE_FIELDS; field++)
		  g_free(service_data[field]);
	      update_status_line("Error corrupt database: SERVICE.dat");
	      fclose(sf_pointer);
	      return;
	  }
	  if ( !strcmp(service_data[0], clist_data[REC_SERVICE_ID]) ) /* Does the service id match? */
	  {
	      data[3] = g_strdup(service_data[SERVICE_NAME]); /* Copy the name of the service from the 2nd field */
	      fclose(sf_pointer);
	      break;
	  }

      }

      data[4] = g_strdup(clist_data[0]);
      gtk_clist_append( (GtkCList *)list, data);
    
      for (x = 0; x < 5; x++)
      {
	  g_free(data[x]);
      }
  }

  fclose(f_pointer);

  for (index = 0; index < RECEIVER_FIELDS; index++)
      g_free(clist_data[index]);
  
  for (index = 0; index < SERVICE_FIELDS; index++)
      g_free(service_data[index]);

  return;                               


}


void load_list_service(GtkCList *list)
{

  gint    x;
  gchar   *data[6];
  gchar   *clist_data[SERVICE_FIELDS];
  FILE    *f_pointer;
  gint    index, field, ret_value;
  gchar   *line;

  g_assert(GTK_IS_WIDGET(list));

  if ( (f_pointer = get_file_pointer("SERVICE.dat", "r")) == NULL)
      return;

  while ( (line = (read_file_by_line(f_pointer))) != NULL)
  {
      for (field = 0; field < SERVICE_FIELDS; field++)
	  clist_data[field] = g_malloc(MAX_FIELD_LENGTH);

      ret_value = get_fields_from_string(clist_data, line, SERVICE_FIELDS, ',');
      g_free(line);

      if (ret_value == 1)
      {
	  update_status_line("Error corrupt database: SERVICE.dat");
	  for (field = 0; field < SERVICE_FIELDS; field++)
	      g_free(clist_data[field]);
	  fclose(f_pointer);
	  return;
      }

      data[0] = g_strdup(clist_data[SERVICE_NAME]);
      data[1] = g_strdup(clist_data[SERVICE_WCTP_URL]);
      data[2] = g_strdup(clist_data[SERVICE_SNPP_URL]);
      data[3] = g_strdup(clist_data[SERVICE_MAX_MSG]);
      data[4] = g_strdup(clist_data[3]);	               /* This is currently not used */
      data[5] = g_strdup(clist_data[SERVICE_ID]);
      
      gtk_clist_append( (GtkCList *)list, data);
    
      for (x = 0; x < 6; x++)
	  g_free(data[x]);

      for (index = 0; index < SERVICE_FIELDS; index++)
	  g_free(clist_data[index]);
  }
  fclose(f_pointer);

  return;                               

}


int get_id(char *group)
{
    FILE    *fp, *wfp;
    gchar   *filename = "IDS.dat";
    gint    id, position;
    gchar   tmp[32];
    gchar   ch;
    gint    index;
    gchar   *temp_line;

    if ( (fp = get_file_pointer(filename, "r")) == NULL)
	return(-1);

    wfp = get_temp_file();
    if ( wfp == NULL )
	return (-1);

    if( !strcmp(group,"RECEIVER") )
	position = 0;
    else if ( !strcmp(group,"TERMINAL") )
	position = 2;
    else if ( !strcmp(group,"GROUP") )
	position = 4;
    else
    {
	fprintf(stderr, "Error parsing IDS.dat: Corrupt database\n");
	return(-1);
    }

    for (index = 0; index < position; index++)
    {
	while( (ch = fgetc(fp)) != '\n' )
	    fputc(ch, wfp);
	fputc('\n', wfp);
    }
	    
    ch = '\0';
    memset(tmp, '\0', sizeof(tmp));
    index = 0;
    while( (ch != '\n') && ch != ',')
    {
	ch = fgetc(fp);
	tmp[index++] = ch;
    }
    tmp[(strlen(tmp) - 1)] = '\0';
    if (ch == '\n')
    {
	fputs(tmp, wfp);
	fputc('\n', wfp);
	memset(tmp, '\0', sizeof(tmp));
	while ( (ch = fgetc(fp)) != ',' )
	    fputc(ch, wfp);
	fputc(ch, wfp);
	index = 0;
	while ( (ch = fgetc(fp)) != '\n')
	    tmp[index++] = ch;
	id = atoi(tmp);
	memset(tmp, '\0', sizeof(tmp));
	g_snprintf(tmp, 32, "%d\n", (id + 1) );
	fputs(tmp, wfp);
    }
    else
    {
	id = atoi(tmp);
    }

    while( (ch = fgetc(fp)) != EOF ) 
	fputc(ch, wfp);

    fclose(fp);
    rewind(wfp);

    temp_line = g_malloc(MAX_LENGTH);
    
    if ( (fp = get_file_pointer("IDS.dat", "w")) == NULL)
    {
	fclose(wfp);
	return(-1);
    }
    while(fgets(temp_line, MAX_LENGTH, wfp))
	fputs(temp_line, fp);

    fclose(wfp);
    fclose(fp);

    g_free(temp_line);

    return (id);

}

int recycle_id(char *group, char *id)
{
    FILE   *rfp, *wfp;
    gint   ch;
    gint   position, index;
    gchar  *temp_line;

    if ( (rfp = get_file_pointer("IDS.dat", "r")) == NULL)
	return(-1);

    wfp = get_temp_file();
    if ( wfp == NULL )
	return(-1);

    if( !strcmp(group,"RECEIVER") )
	position = 0;
    else if ( !strcmp(group,"TERMINAL") )
	position = 2;
    else if ( !strcmp(group,"GROUP") )
	position = 4;
    else
    {
	fprintf(stderr, "Error parsing IDS.dat: Corrupt database\n");
	return(-1);
    }
    for (index = 0; index < position; index++)
    {
	while( (ch = fgetc(rfp)) != '\n' )
	    fputc(ch, wfp);
	fputc('\n', wfp);
    }

    fputs(id, wfp);
    fputc(',', wfp);

    while( (ch = fgetc(rfp)) != EOF ) 
	fputc(ch, wfp);
    fclose(rfp);

    rewind(wfp);

    temp_line = g_malloc(MAX_LENGTH);
    if ( (rfp = get_file_pointer("IDS.dat", "w")) == NULL)
    {
	fclose(wfp);
	g_free(temp_line);
	return(0);
    }
    while(fgets(temp_line, MAX_LENGTH, wfp))
	fputs(temp_line, rfp);
    fclose(rfp);
    fclose(wfp);
    g_free(temp_line);
	  
    return(0);
}

void update_status_line(char *data)
{
    extern GtkWidget *statusbar1;
    extern int       daemon_flag;
    extern gint      command_line_flag;

    gchar            *padded_buffer;


    if ( daemon_flag || command_line_flag )
	return;

    //padded_buffer = g_strdup_printf("      %s", data);
    padded_buffer = g_strdup_printf("%s", data);
    gtk_statusbar_push( GTK_STATUSBAR(statusbar1), 1, padded_buffer); 
    g_free(padded_buffer);
}

void update_status_line_small(char *data)
{
    extern GtkWidget *statusbar_small;
    extern int       daemon_flag;
    extern gint      command_line_flag;

    gchar            *padded_buffer;


    if ( daemon_flag || command_line_flag )
	return;

    //padded_buffer = g_strdup_printf("      %s", data);
    padded_buffer = g_strdup_printf("%s", data);
    gtk_statusbar_push( GTK_STATUSBAR(statusbar_small), 1, padded_buffer); 
    g_free(padded_buffer);
}


void load_group_list(gint rec_id, GtkCList *generic_clist)
{

  gint x;
  gchar *data[2];
  gchar *id, *line;
  gchar *uid, *name;

  id = g_strdup_printf("%d", rec_id);

  line = get_uid_line("RECEIVER.dat", id, RECEIVER_FIELDS, 0, MAX_LENGTH);

  uid = g_malloc(MAX_FIELD_LENGTH);
  name = g_malloc(MAX_FIELD_LENGTH);
  
  get_name_id(uid, name, line);
      
  data[0] = g_strdup(uid);
  data[1] = g_strdup(name);

  gtk_clist_append( (GtkCList *)generic_clist, data);
    
  for (x = 0; x < 2; x++)
      g_free(data[x]);


  g_free(uid);
  g_free(name);

  g_free(id);
  g_free(line);

  return;                               

}
gint parse_group(gchar *id, uid_list *current_uid)
{
    gchar     *temp, *t0;
    gchar     *data_line, *d;
    gint      i = 0;
    gint      x = 0;
    uid_list  *temp_node;
    uid_list  *head;

    head = current_uid;
    temp = g_malloc(MAX_FIELD_LENGTH);
    t0 = temp;


    data_line = get_uid_line("GROUP.dat", id, 2, 1, MAX_LENGTH);  
    d = data_line;
    
    for ( i = 0; i < 2; i++ )
	while (*data_line++ != ',');

    while (1)
    {
	if ( (*data_line == ',') || (*data_line == '\n') )
	{
	    *temp = '\0';
	    strcpy(current_uid->uid, t0);
	    temp_node = get_uid_node();
	    current_uid->next = temp_node;
	    current_uid = temp_node;
	    x++;
	    temp = t0;
	    if (*data_line == '\n')
		break;
	    data_line++;
	}
	else
	{
	    *temp++ = *data_line++;
	}
    }
    g_free(t0);
    g_free(d);
    current_uid = head;

    return x;
}

GtkCTreeNode *make_node(GtkWidget *tree_widget, GtkCTreeNode *parent, GtkCTreeNode *sibling, gchar *id, gchar *name)
{
    gchar             *data[2];
    GtkCTreeNode      *node;
    GdkPixmap         *pixmap;  
    GdkBitmap         *mask;
    GdkColormap       *colormap;

    data[1] = id;
    data[0] = name;
    
    if ( !strcmp(id, "") )
    {
	colormap = gtk_widget_get_colormap(GTK_WIDGET(tree_widget));
	gtk_widget_realize ( tree_widget );
	
	pixmap = gdk_pixmap_create_from_xpm_d ( tree_widget->window, &mask,
						&tree_widget->style->white, dot_xpm );

	node = gtk_ctree_insert_node(GTK_CTREE(tree_widget), parent, sibling, data, 0, pixmap, 
				     mask, NULL, NULL, FALSE, FALSE);
    }
    else
    {
	node = gtk_ctree_insert_node(GTK_CTREE(tree_widget), parent, sibling, data, 0, NULL, 
				     NULL, NULL, NULL, FALSE, FALSE);
    }

    return(node);

}
void  load_group_tree(GtkWidget *tree)
{
    GtkCTreeNode *node[MAX_NUM_GROUPS];
    gint         node_index;
    FILE         *fp;     
    gchar        *line, *rec_line;
    gchar        *id,  *name;
    gchar        *rid, *rname;
    gint         field, i, members;
    uid_list     *group_members;
    uid_list     *head;


    group_members = get_uid_node();
    head = group_members;

    field = i = members = 0;

    if ( (fp = get_file_pointer("GROUP.dat", "r")) == NULL)
	return;

    id = g_malloc(16);
    name = g_malloc(MAX_FIELD_LENGTH);
    rid = g_malloc(16);
    rname = g_malloc(MAX_FIELD_LENGTH);

    node_index = 0;

    while ( ( line = read_big_file_by_line(fp, MAX_LENGTH) ) != NULL )
    {
	get_name_id(id, name, line);

	node[node_index] = make_node(tree, NULL, NULL, id, name); 
	group_members = head;
	members = parse_group(id, group_members);
	group_members = head;
	for ( i = 0; i < members; i++)
	{
	    rec_line = get_uid_line("RECEIVER.dat", group_members->uid, RECEIVER_FIELDS, 0, MAX_LENGTH);
	    get_name_id(rid, rname, rec_line);
	    g_free(rec_line);
	    make_node(tree, node[node_index], NULL, "", rname);
	    group_members = group_members->next;
	}
	node_index++;
	members = 0;
	g_free(line);
    }

    g_free(rid);
    g_free(rname);
    g_free(id);
    g_free(name);

    fclose(fp);

}
gint  parse_line(gchar *line, gchar *id, gchar *sch_id, gchar *s_time, gchar *msg, gchar *recur, gchar *count, gint flag)
{
    gint   i = 0;
    gint   x = 0;
    gint   test_count = 0;
    const gint min_number_spaces = 3;
    
    while( *(line + i) != '\n' )
    {
	if ( *(line + i ) == ' ' )
	    test_count++;
	i++;
    }

    if (test_count < min_number_spaces )		/* We need to make sure we have at least the min amount of spaces in the line */
	return (-1);

    i = 0;

    while ( *(line + i) != ' ')
	*(id + x++) = *(line + i++);
    *(id + x) = '\0';
    i++;
    x = 0;

    while ( *(line + i) != ' ')
	*(sch_id + x++) = *(line + i++);
    *(sch_id + x) = '\0';
    i++;
    x = 0;

    while ( *(line + i) != ' ')
	*(s_time + x++) = *(line + i++);
    *(s_time + x) = '\0';
    i++;
    x = 0;
    if ( flag )
	format_time(s_time, recur);

    while ( (*(line + i) != ' ') && (*(line + i) != '\n') )
	*(count + x++) = *(line + i++);
    *(count + x) = '\0';

    if ( *(line + i) == '\n' )	/* We have an old or corrupt database */
    {
	strcpy(msg, count);
	strcpy(count, "0");
	return (1);
    }
    
    i++;
    x = 0;

    while ( *(line + i) != '\n')
	*(msg + x++) = *(line + i++);
    *(msg + x) = '\0';
    i++;
    x = 0;

    return (1);
}

void format_time(gchar *mtime, gchar *recur)
{
    gint  i,x;
    gint  temp;

    gchar *yr;
    gchar *month;
    gchar *day;
    gchar *hour;
    gchar *min;
    gchar *am_pm;

    yr = g_malloc(32);
    month = g_malloc(32);
    day = g_malloc(32);
    hour = g_malloc(32);
    min = g_malloc(32);
    am_pm = g_malloc(32);


    i = x = 0;

    while ( *(mtime + i) != '-')
	*(yr + x++) = *(mtime + i++);
    *(yr + x) = '\0';
    i++;
    x = 0;

    while ( *(mtime + i) != '-')
	*(month + x++) = *(mtime + i++);
    *(month + x) = '\0';
    i++;
    x = 0;
    temp = atoi(month);

    while ( *(mtime + i) != '-')
	*(day + x++) = *(mtime + i++);
    *(day + x) = '\0';
    i++;
    x = 0;

    while ( *(mtime + i) != '-')
	*(hour + x++) = *(mtime + i++);
    *(hour + x) = '\0';
    i++;
    x = 0;

    while ( *(mtime + i) != '-')
	*(min + x++) = *(mtime + i++);
    *(min + x) = '\0';
    i++;
    x = 0;

    while ( *(mtime + i) != '-')
	*(am_pm + x++) = *(mtime + i++);
    *(am_pm + x) = '\0';
    i++;
    x = 0;

    while ( *(mtime + i) != '\0')
	*(recur + x++) = *(mtime + i++);
    *(recur + x) = '\0';
    i++;
    x = 0;
    
    if ( !strcmp(am_pm, "1") )
	strcpy(am_pm, "AM");
    else
	strcpy(am_pm, "PM");

    g_snprintf(mtime, MAX_LENGTH, "%s:%s %s %d/%s/%s", hour, min, am_pm, ++temp, day, yr);

    g_free(yr);
    g_free(month);
    g_free(day);
    g_free(hour);
    g_free(min);
    g_free(am_pm);

}
    
void  get_name_id(gchar *id, gchar *name, gchar *line)
{
    gint i = 0;
    gint x = 0;

    while ( (*(line + i) != ',') && (*(line + i)) && (x < MAX_FIELD_LENGTH) )
	*(id + x++) = *(line + i++);
    *(id + x) = '\0';

    if ( (x == MAX_FIELD_LENGTH) || (*(line + i) == '\0') )
    {
	strcpy(name, "error loading field");
	return;
    }

    i++;			/* Skip the comma */
    x = 0;
    while ( (*(line + i) != ',') && (*(line+i)) && (x < MAX_FIELD_LENGTH) )
	*(name + x++) = *(line + i++);
    *(name + x) = '\0';
}
	
gchar  *clean_string(gchar *buffer)
{
    gint i = 0;

    while ( ( *(buffer + i) != '\n') && ( *(buffer + i) != '\0') )
    {
	if ( *(buffer + i) == ',' )
	    *(buffer + i) = ' ';
	i++;
    }
    return buffer;
}


gint handle_auto_log_delete()
{
    extern   gint log_delete_active; 

    gchar    *resource;
    time_t   time_val;
    gchar    *current;
    gchar    *mark;
    gchar    *num_days;

    if ( (resource = get_resource_data("CONFIG.dat", "auto_delete", MAX_FIELD_LENGTH)) )
    {
	if (!strcmp(resource, "yes") )
	{
	    log_delete_active = 1; 
	    if ( (num_days = get_resource_data("CONFIG.dat", "wait_days", MAX_FIELD_LENGTH)) )
	    {
		time(&time_val);
		current = g_strdup_printf("%s", ctime(&time_val));
		current[strlen(current) - 1] = '\0';
		time_val -= (86400 * atoi(num_days));

                /* anything less than this value is deleted */
		mark = g_strdup_printf("%u", (unsigned int)time_val);
		delete_logs(mark);
		g_free(current);
		g_free(mark);

		g_free(num_days);
	    }
	}	    
	else
	    log_delete_active = 0; 

	g_free(resource);
    }
    else
	log_delete_active = 0; 

    return(log_delete_active);
}

void  delete_logs(gchar *mark)
{
    FILE      *fp, *fpt;
    gchar     *line;
    gchar     *epoch;
    gint      i = 0;
    gchar     *full_path;
    gchar     *temp_line;
    gint      value;
    register  int del_value;

    del_value = atoi(mark);

    if  ( ( fp = get_file_pointer("log.gpage", "r") ) == NULL )
    {
	perror("Error opening log.gpage - my_functions.c: ");
	return;
    }

    fpt = get_temp_file();
    if ( fpt == NULL )
	return;

    epoch = g_malloc(MAX_LENGTH); /* way too big */

    while ( ( line = read_big_file_by_line(fp, MAX_LENGTH) ) != NULL )
    {
	i = 0;
	while ( line[i] != ',' )
	{
	    epoch[i] = line[i];
	    i++;
	}
	epoch[i] = '\0';
	
	value = atoi(epoch);
	if ( del_value < value )
	    fputs(line, fpt);

	g_free(line);
    }
    
    rewind(fpt);
    fclose(fp);

    full_path = get_working_dir();
    strcat(full_path, "log.gpage");
    unlink(full_path);
    temp_line = g_malloc(MAX_LENGTH);

    if ( ( fp = get_file_pointer("log.gpage", "w")) == NULL)
    {
	fclose(fpt);
	g_free(full_path);
	g_free(temp_line);
	return;
    }

    while (fgets(temp_line, MAX_LENGTH, fpt))
      fputs(temp_line, fp);
    fclose(fp);
    fclose(fpt);
    g_free(temp_line);

    g_free(line);
    g_free(epoch);
    g_free(full_path);
}

gchar *handle_mem(gchar *function, gint size, gchar *action, gchar *id, gchar *var)
{
    gchar   *tmp;

    //fprintf(stderr, "%s : %s %s %d \n", function, action, var, size);
#ifdef MEM_DEBUG

    FILE    *fp;
    gchar   *temp;


    if ( ( fp = fopen("mem.dump", "a+")) == NULL)
    {
	perror("Error opening mem.dump: ");
	exit(EXIT_FAILURE);
    }
    else
    {
	temp = g_strdup_printf("%s : %s %s %d \n", function, action, var, size);
	fputs(temp, fp);
	fclose(fp);
    }
#endif

    if ( !strcmp(action, "malloc") )
    {
	tmp = g_malloc(size);
	return tmp;
    }
    else if (!strcmp(action, "free") )
    {
	g_free(id);
	return (NULL);
    }
    
    return (NULL);		/* Never get here */
}

void load_main_list_with_pixmaps()
{
    extern  GtkWidget *receiver_clist;
    GdkPixmap         *pixmap;  
    GdkBitmap         *mask;
    GdkColormap       *colormap;
    gint              row = 0;
    gchar             *id;

    colormap = gtk_widget_get_colormap(GTK_WIDGET(receiver_clist));
    gtk_widget_realize ( receiver_clist );
    pixmap = gdk_pixmap_create_from_xpm_d ( receiver_clist->window, &mask,
					&receiver_clist->style->white, g_xpm );

    if ( pixmap == NULL )
    {
	fprintf(stderr, "Error creating pixmap\n");
	exit(1);
    }

    while (gtk_clist_get_selectable(GTK_CLIST(receiver_clist), row))
    {
	gtk_clist_get_text(GTK_CLIST(receiver_clist), row, 0, &id); 
	if ( id[0] == 'G' )
	    gtk_clist_set_pixmap(GTK_CLIST(receiver_clist) ,row, 1, pixmap, mask);
	row++;
    }

    gdk_pixmap_unref(pixmap);
    gdk_bitmap_unref(mask);
    
}

void  delete_text_in_text_box(GtkWidget *text_box)
{
    gint   length;

    length = gtk_text_get_length( GTK_TEXT(text_box) );
    gtk_text_set_point( GTK_TEXT(text_box) , length );
    gtk_text_backward_delete ( GTK_TEXT(text_box), length );

}

