/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"


void
on_add_pre_clicked                     (GtkButton       *button,
                                        stored_msg      *user_data)
{
    gint    text_length;
    gchar   *pre_def;
    gchar   *list_data[2];

    text_length = gtk_text_get_length( GTK_TEXT(user_data->msg_text_box) );
    if ( text_length > MAX_PAGE_LENGTH )
    {
	generic_error("\n Maximum number of characers exceeded   \n", "G-Page Error");
	return;
    }
    if ( text_length == 0 )
    {
	generic_error("\n   You have not entered a message   \n", "G-Page Error");
	return;
    }

    pre_def = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(user_data->msg_text_box), 0, -1));
    pre_def = clean_text(pre_def);

    if ( add_data(pre_def, "MSG.dat") )
	perror("Error saving predefined message");

    list_data[0] = g_strdup(" ");
    list_data[1] = g_strdup(pre_def);

    gtk_clist_append(GTK_CLIST(user_data->msg_list), list_data);

    g_free(list_data[0]);
    g_free(list_data[1]);
    g_free(pre_def);
    
    gtk_text_set_point( GTK_TEXT( user_data->msg_text_box) , 0 );
    gtk_text_forward_delete ( GTK_TEXT(user_data->msg_text_box), text_length );

}


void
on_update_pre_clicked                  (GtkButton       *button,
                                        stored_msg      *user_data)
{
    gchar     *msg, *tmp_msg;
    GdkFont   *current_font;

    if (GTK_CLIST(user_data->msg_list)->selection == NULL)
    {
	generic_error("\n You have not selected a message to edit \n", "G-Page Error");
	return;
    }

    tmp_msg = g_malloc(MAX_LENGTH);

    gtk_clist_get_text( (GtkCList *)user_data->msg_list, GPOINTER_TO_INT(GTK_CLIST(user_data->msg_list)->selection->data), 1, &msg);
    strcpy(tmp_msg, msg);
    
    gtk_text_set_point(GTK_TEXT(user_data->msg_text_box), 0);
    current_font = gdk_font_load(tmp_msg);
    gtk_text_insert(GTK_TEXT(user_data->msg_text_box), current_font, NULL, NULL, msg, strlen(tmp_msg));

    del_pre_def(tmp_msg, "MSG.dat");
    gtk_clist_remove( (GtkCList *)user_data->msg_list, GPOINTER_TO_INT(GTK_CLIST(user_data->msg_list)->selection->data));

    g_free(tmp_msg);
}

void
on_cancel_pre_clicked                  (GtkButton       *button,
                                        stored_msg      *user_data)
{

    gtk_widget_destroy(user_data->main_window);
    g_free(user_data);

}

void
on_delete_pre_clicked                  (GtkButton       *button,
                                        stored_msg      *user_data)
{
    gchar   *msg;
    gchar   *dead_msg;

    if (GTK_CLIST(user_data->msg_list)->selection == NULL)
    {
	generic_error("\n You have not selected a message \n", "G-Page Error");
	return;
    }
    gtk_clist_get_text( (GtkCList *)user_data->msg_list, GPOINTER_TO_INT(GTK_CLIST(user_data->msg_list)->selection->data), 1, &msg);
    dead_msg = g_strdup(msg);

    del_pre_def(dead_msg, "MSG.dat");

    g_free(dead_msg);
    gtk_clist_remove( (GtkCList *)user_data->msg_list, GPOINTER_TO_INT(GTK_CLIST(user_data->msg_list)->selection->data));

}


void
on_ok_pref_clicked                     (GtkWidget        *w,
                                        stored_msg      *user_data)
{
    extern  GtkWidget *text_entry;
    extern  gint      sig_length;
    
    GdkFont  *current_font;
    gchar    *buff;
    gint     text_length; 
    gchar    *msg;


    if (GTK_CLIST(user_data->msg_list)->selection == NULL)
    {
	generic_error("\n You have not selected a message \n", "G-Page Error");
	return;
    }
    gtk_clist_get_text( (GtkCList *)user_data->msg_list, GPOINTER_TO_INT(GTK_CLIST(user_data->msg_list)->selection->data), 1, &msg);

    if ( strlen(msg) + sig_length + gtk_text_get_length(GTK_TEXT(text_entry)) > MAX_PAGE_LENGTH )
    {
	    update_status_line("Unable to insert message: Maximum page length exceeded");
	    return;
    }

    gtk_text_set_point(GTK_TEXT(text_entry), 0);
    current_font = gdk_font_load(msg);
    gtk_text_insert(GTK_TEXT(text_entry), current_font, NULL, NULL, msg, strlen(msg));

    text_length = gtk_text_get_length( GTK_TEXT(text_entry) );
    text_length += sig_length;
    buff = g_strdup_printf("Characters to send: %d",text_length);
    update_status_line(buff);
    g_free(buff);

    gtk_widget_destroy(user_data->main_window);
    g_free(user_data);
}

void del_pre_def(gchar  *msg, gchar *file_name)
{
    FILE   *fp, *wfp;
    gchar  *line = NULL;
    gchar  *temp_line;
    

    if ( ( fp = get_file_pointer(file_name, "r")) == NULL)
    {
	generic_error("\n Error opening dat file for deletion \n", "G-Page Error");
	g_free(line);
	return;
    }

    if( (wfp = get_temp_file()) == NULL) 
    {
	perror("Error creating temp file ");
	generic_error("\n Error opening creating temp file \n", "G-Page Error");
	g_free(line);
	return;
    }

    while ( (line = (read_file_by_line(fp))) != NULL)
    {
	line[strlen(line) - 1] = '\0';
	if (strcmp(line, msg) )
	{
	    strcat(line, "\n");
	    fputs(line, wfp);
	}
	g_free(line);
    }

    rewind(wfp);
    fclose(fp);



    if ( ( fp = get_file_pointer(file_name, "w")) == NULL )
	fclose(wfp);

    temp_line = g_malloc(MAX_LENGTH);

    while( fgets(temp_line, MAX_LENGTH, wfp))
      fputs(temp_line, fp);
    
    fclose(fp);
    fclose(wfp);
    
    g_free(temp_line);

}

