/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

typedef struct {
    GtkWidget  *dir_label;
    GtkWidget  *file_selection;
}new_dir;

void file_ok_sel( GtkWidget *w, new_dir *data );
static void cleanup(GtkWidget *w, new_dir *data);
static int get_protocol_order(gchar *resource, pref_widgets *prefs);
static void clear_memory_prefs (pref_data  *user_data);

/* Global variable that's only used in this file */
gint    error_flag;

void
on_change_dir_clicked                   (GtkWidget       *w,
                                         pref_widgets    *user_data)
{
    new_dir      *dir_data;
    GtkWidget    *filew;

    dir_data = g_malloc(sizeof(*dir_data)); /* This is never freed if they select the ok button*/

    filew = gtk_file_selection_new ("Select new directory");
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filew));
    
    dir_data->dir_label = user_data->dir_label;
    dir_data->file_selection = filew;

    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),"clicked", 
			GTK_SIGNAL_FUNC(file_ok_sel), dir_data);

    gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),"clicked",
			       GTK_SIGNAL_FUNC(cleanup), dir_data);

    gtk_signal_connect_object( GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),"clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(filew));

    gtk_widget_show(filew);      
}

static void cleanup(GtkWidget *w, new_dir *data)
{
    g_free(data);
}

void
on_pref_save_button_clicked                 (GtkWidget    *w,
					     pref_data    *user_data)
{
    extern   GtkWidget *receiver_clist;
    extern   gint      sig_length;
    extern   gint      network_db;
    extern   guint     timeout_id;
    extern   gint      log_delete_active;
    extern   guint     log_poll_timer_id;
    extern   gint      error_flag;

    gchar    *sig;
    gchar    *snpp_adr;
    gchar    *snpp_port;
    gchar    *resource;
    gchar    *temp;
    gchar    *db_location;
    gchar    *db_port;
    //gchar    *dirl[MAX_LENGTH];
    gchar    *dirl[2];
    gchar    order[16];
    gchar    *resource_field;
    gint     spin_value;
    gint     database_changed;

    resource = g_malloc(MAX_LENGTH);
    temp = g_malloc(MAX_LENGTH);
    database_changed = 0;
    error_flag = 0;

    if ( (resource_field = get_resource_data("CONFIG.dat", "db_dir", MAX_FIELD_LENGTH)) == NULL)
	return;
    else
    {
	gtk_label_get(GTK_LABEL(user_data->prefs->dir_label), dirl);
	if ( strcmp(resource_field, *dirl) )
	{
	    strcpy(temp, "db_dir=");
	    strcat(temp, *dirl);
	    strcat(temp, "\n");
	    edit_item("db_dir", "CONFIG.dat", 2, 1, temp, EDIT, '=');
	    database_changed = 1;
	}
	
	g_free(resource_field);
    }

    sig = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->prefs->signature)));
    sig_length = strlen(sig);
    if ( strlen(sig) )
    {
	strcpy(temp, "sig=");
	strcat(temp, sig);
	strcat(temp, "\n");
    }
    else
	strcpy(temp, "sig=none\n");
    if ( strcmp(user_data->signature, sig ) )
    {
	edit_item("sig", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }

    spin_value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->prefs->spin_num_days));

    if ( atoi(user_data->days_before_delete) != spin_value )
    {
	g_snprintf(temp, MAX_LENGTH, "wait_days=%d\n", spin_value);
	edit_item("wait_days", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }

    //g_snprintf(temp, MAX_LENGTH, "wait_days=%d\n", spin_value);
    //edit_item("wait_days", "CONFIG.dat", 2, 1, temp, EDIT, '=');


    spin_value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->prefs->log_delete_poll));

    if ( atoi(user_data->log_delete_poll) != spin_value )
    {
	g_snprintf(temp, MAX_LENGTH, "log_delete_poll=%d\n", spin_value);
	edit_item("log_delete_poll", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }


    if (GTK_TOGGLE_BUTTON (user_data->prefs->radio_auto_yes)->active)
    {
	strcpy(resource, "auto_delete=yes\n");
	strcpy(temp, "yes");
	if ( strcmp(temp, user_data->log_delete) )
	{
	    edit_item("auto_delete", "CONFIG.dat", 2, 1, resource, EDIT, '=');
	}

	if ( log_delete_active == 0 ) /*  If auto delete is already active we don't want to restart the timer  */
	{
	    if ( log_poll_timer_id == -1 ) /* The timer is not active */
	    {		
		automated_check_for_log_delete();
		if ( spin_value != 0 )
		    log_poll_timer_id = gtk_timeout_add( ( 60 * ONE_SECOND  * 60 * spin_value ), (GtkFunction)automated_check_for_log_delete, NULL); 
	    }
	    else		/* The timer is active - we must remove it then set it again */
	    {
		gtk_timeout_remove(log_poll_timer_id);
		automated_check_for_log_delete();
		if ( spin_value != 0 )
		    log_poll_timer_id = gtk_timeout_add( ( 60 * ONE_SECOND  * 60 * spin_value ), (GtkFunction)automated_check_for_log_delete, NULL); 
	    }
	}
	else			/* autodelete is already active - we check to see if they are turning on the timer */
	{
	    if ( log_poll_timer_id != -1 )
		gtk_timeout_remove(log_poll_timer_id);

	    if ( spin_value != 0 )
		log_poll_timer_id = gtk_timeout_add( ( 60 * ONE_SECOND  * 60 * spin_value ), (GtkFunction)automated_check_for_log_delete, NULL); 
	}
    }
    else
    {
	strcpy(resource, "auto_delete=no\n");
	strcpy(temp, "no");
	if ( strcmp(temp, user_data->log_delete) )
	{
	    edit_item("auto_delete", "CONFIG.dat", 2, 1, resource, EDIT, '=');
	}

	if ( log_poll_timer_id != -1 )
	{
	    gtk_timeout_remove(log_poll_timer_id);
	    log_poll_timer_id = -1;
	}

	automated_check_for_log_delete();	/* make sure we turn off autodelete */	
    }


    if (GTK_TOGGLE_BUTTON (user_data->prefs->radio_sig_yes)->active)
    {
	strcpy(temp, "yes");
	strcpy(resource, "msg_sig=yes\n");
    }
    else
    {
	strcpy(temp, "no");
	strcpy(resource, "msg_sig=no\n");
	sig_length = 0;
    }
    if ( strcmp(user_data->signature_yes_no, temp) )
    {
	edit_item("msg_sig", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }


    if (GTK_TOGGLE_BUTTON (user_data->prefs->radio_sig_before)->active)
    {
	strcpy(resource, "sig_placement=before\n");
	strcpy(temp, "before");
    }
    else
    {
	strcpy(resource, "sig_placement=after\n");
	strcpy(temp, "after");
    }
    if ( strcmp(user_data->signature_before, temp) )
    {
	edit_item("sig_placement", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }

    if (GTK_TOGGLE_BUTTON (user_data->prefs->cc_email_yes)->active)
    {
	strcpy(temp, "yes");
	strcpy(resource, "cc_email=yes\n");
    }
    else
    {
	strcpy(temp, "no");
	strcpy(resource, "cc_email=no\n");
    }
    if ( strcmp(user_data->cc_email, temp) )
    {
	edit_item("cc_email", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }

    if (GTK_TOGGLE_BUTTON (user_data->prefs->set_protocols_yes)->active)
    {
	strcpy(temp, "yes");
	strcpy(resource, "set_protocols=yes\n");
    }
    else
    {
	strcpy(temp, "no");
	strcpy(resource, "set_protocols=no\n");
    }
    if ( strcmp(user_data->set_protocols, temp) )
    {
	edit_item("set_protocols", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }

    if (GTK_TOGGLE_BUTTON (user_data->prefs->set_protocols_yes)->active)
    {
	if ( (get_protocol_order(order, user_data->prefs)) )
	{
	    generic_error("\n Each protocol must have a unique precedence number \n", "G-Page Error");
	    error_flag = 1;
	    g_free(resource);
	    g_free(temp);
	    g_free(sig);
	    return;
	}
	else 
	    g_snprintf(resource, MAX_LENGTH, "order_of_protocols=%s", order);
    }
    else
	strcpy(resource, "order_of_protocols=123\n");
    edit_item("order_of_protocols", "CONFIG.dat", 2, 1, resource, EDIT, '=');



    if (GTK_TOGGLE_BUTTON (user_data->prefs->show_options_yes)->active)
    {
	strcpy(temp, "yes");
	strcpy(resource, "show_advanced_options=yes\n");
    }
    else
    {
	strcpy(temp, "no");
	strcpy(resource, "show_advanced_options=no\n");
    }
    if ( strcmp(user_data->show_options, temp) )    
    {
	edit_item("show_advanced_options", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }



    snpp_adr = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->prefs->qpage_adr)));
    if ( strlen(snpp_adr) )
    {
	strcpy(temp, "default_address=");
	strcat(temp, snpp_adr);
	strcat(temp, "\n");
    }
    else
	strcpy(temp, "default_address=localhost\n");
    if (  strcmp(user_data->qpage_adr, snpp_adr) )
    {
	edit_item("default_address", "CONFIG.dat", 2, 1, temp, EDIT, '=');

    }
    g_free(snpp_adr);



    snpp_port = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->prefs->qpage_port)));
    if ( strlen(snpp_port) )
    {
	strcpy(temp, "default_port=");
	strcat(temp, snpp_port);
	strcat(temp, "\n");
    }
    else
	strcpy(temp, "default_port=444\n");
    if (  strcmp(user_data->qpage_port, snpp_port) )
    {
	edit_item("default_port", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }
    g_free(snpp_port);



    if (GTK_TOGGLE_BUTTON (user_data->prefs->status_yes)->active)
    {
	strcpy(temp, "yes");
	strcpy(resource, "status_mode=yes\n");
    }
    else
    {
	strcpy(temp, "no");
	strcpy(resource, "status_mode=no\n");
    }
    if ( strcmp(user_data->show_status, temp) )
    {
	edit_item("status_mode", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }



    if (GTK_TOGGLE_BUTTON (user_data->prefs->serial_mode)->active)
    {
	strcpy(temp, "yes");
	strcpy(resource, "serial_mode=yes\n");
    }
    else
    {
	strcpy(temp, "no");
	strcpy(resource, "serial_mode=no\n");
    }
    if ( strcmp(user_data->serial_mode, temp) )
    {
	edit_item("serial_mode", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }



    if (GTK_TOGGLE_BUTTON (user_data->prefs->time_stamp_yes)->active)
    {
	strcpy(temp, "yes");
	strcpy(resource, "time_stamp=yes\n");
	sig_length += 6;

    }
    else
    {
	strcpy(temp, "no");
	strcpy(resource, "time_stamp=no\n");
    }
    if ( strcmp(user_data->time_stamp, temp) )
    {
	edit_item("time_stamp", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }

				/* Auto page prefs here */

    spin_value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->prefs->default_polling));

    if ( atoi(user_data->default_polling) != spin_value )
    {
	g_snprintf(temp, MAX_LENGTH, "polling_interval=%d\n", spin_value);
	edit_item("polling_interval", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }



    spin_value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->prefs->script_delay));

    if ( atoi(user_data->script_delay) != spin_value )
    {
	g_snprintf(temp, MAX_LENGTH, "script_delay=%d\n", spin_value);
	edit_item("script_delay", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }


    spin_value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->prefs->retries));
    
    if ( atoi(user_data->retries) != spin_value )
    {
	g_snprintf(temp, MAX_LENGTH, "locate_retries=%d\n", spin_value);
	edit_item("locate_retries", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }



    if (GTK_TOGGLE_BUTTON (user_data->prefs->radio_local)->active)
    {
	strcpy(temp, "local");
	strcpy(resource, "database=local\n");
	network_db = 0;
    }
    else
    {
	strcpy(temp, "network");
	strcpy(resource, "database=network\n");
	network_db = 1;
    }
    if ( strcmp(user_data->database_location, temp) )
    {
	edit_item("database", "CONFIG.dat", 2, 1, resource, EDIT, '=');
	database_changed = 1;	/* Set the flag so we can reload the receiver list */
    }


    db_location = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->prefs->database_adr)));
    if ( strlen(db_location) )
    {
	strcpy(temp, "database_adr=");
	strcat(temp, db_location);
	strcat(temp, "\n");
    }
    else
	strcpy(temp, "database_adr=localhost\n");
    if (  strcmp(user_data->database_adr, db_location) )
    {
	edit_item("database_adr", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }
    g_free(db_location);

    spin_value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->prefs->update_interval));

    if ( atoi(user_data->update_interval) != spin_value )
    {
	g_snprintf(temp, MAX_LENGTH, "update_interval=%d\n", spin_value);
	edit_item("update_interval", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }

    //g_snprintf(temp, MAX_LENGTH, "update_interval=%d\n", spin_value);
    //edit_item("update_interval", "CONFIG.dat", 2, 1, temp, EDIT, '=');

    if ( spin_value == 0 )
	if ( timeout_id != 0 )
	{
	    gtk_timeout_remove(timeout_id);
	    timeout_id = 0;
	}
	
    if ( network_db )
    {
	if ( spin_value != 0 )
	{
	    if ( timeout_id != 0 )
		gtk_timeout_remove(timeout_id);
	    timeout_id = gtk_timeout_add( ((spin_value * 60) * ONE_SECOND), (GtkFunction)network_db_refresh, NULL);
	}
    }
    else 
	if ( timeout_id != 0 )
	{
	    gtk_timeout_remove(timeout_id);
	    timeout_id = 0;
	}
		    
    db_port = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->prefs->db_port)));
    if ( strlen(db_port) )
    {
	strcpy(temp, "database_port=");
	strcat(temp, db_port);
	strcat(temp, "\n");
    }
    else
	strcpy(temp, "database_port=1301\n");
    if (  strcmp(user_data->db_port, db_port) )
    {
	edit_item("database_port", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    }
    g_free(db_port);


    if (GTK_TOGGLE_BUTTON (user_data->prefs->msg_entry_options_always)->active)
    {
	strcpy(resource, "clear_msg_box=always\n");
	strcpy(temp, "always");
    }
    else if  (GTK_TOGGLE_BUTTON (user_data->prefs->msg_entry_options_success)->active)
    {
	strcpy(resource, "clear_msg_box=success\n");
	strcpy(temp, "success");
    }
    else
    {
	strcpy(resource, "clear_msg_box=never\n");
	strcat(temp, "never");
    }
    if ( strcmp(user_data->msg_entry_options, temp) )
    {
	edit_item("clear_msg_box", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }

    
    if (GTK_TOGGLE_BUTTON (user_data->prefs->rollover_yes)->active)
    {
	strcpy(resource, "rollover_no_connect=yes\n");
	strcpy(temp, "yes");
    }
    else
    {
	strcpy(resource, "rollover_no_connect=no\n");
	strcpy(temp, "no");
    }
    if ( strcmp(temp, user_data->rollover) )
    {
	edit_item("rollover_no_connect", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }


    if (GTK_TOGGLE_BUTTON (user_data->prefs->connect_yes)->active)
    {
	strcpy(resource, "dial_no_connect=yes\n");
	strcpy(temp, "yes");
    }
    else
    {
	strcpy(resource, "dial_no_connect=no\n");
	strcpy(temp, "no");
    }
    if ( strcmp(user_data->connect_yes, temp) )
    {
	edit_item("dial_no_connect", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }


    if (GTK_TOGGLE_BUTTON (user_data->prefs->create_error_status_yes)->active)
    {
	strcpy(resource, "create_error_status_file=yes\n");
	strcpy(temp, "yes");
    }
    else
    {
	strcpy(resource, "create_error_status_file=no\n");
	strcpy(temp, "no");
    }
    if ( strcmp(user_data->create_error_status_file, temp) )
    {
	edit_item("create_error_status_file", "CONFIG.dat", 2, 1, resource, EDIT, '=');
    }

    g_free(resource);
    g_free(temp);
    g_free(sig);

    if ( network_db )
    {
	if ( ! (get_network_database()) )
	{
	    if ( database_changed )
	    {
		initialize_main();
		database_changed = 0;
	    }
	}
	else
	{
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (user_data->prefs->radio_local), TRUE);
	    edit_item("database", "CONFIG.dat", 2, 1, "database=local\n", EDIT, '=');	    
	    network_db = 0;

	    gtk_clist_freeze(GTK_CLIST(receiver_clist));
	    gtk_clist_clear(GTK_CLIST(receiver_clist));
	    load_receiver_list("RECEIVER.dat", GTK_CLIST(receiver_clist));
	    load_receiver_list("GROUP.dat", GTK_CLIST(receiver_clist));
	    load_main_list_with_pixmaps();
	    gtk_clist_thaw(GTK_CLIST(receiver_clist));
	}
    }
    else
    {
	if ( database_changed ) 
	{
	    gtk_clist_freeze(GTK_CLIST(receiver_clist));
	    gtk_clist_clear(GTK_CLIST(receiver_clist));
	    load_receiver_list("RECEIVER.dat", GTK_CLIST(receiver_clist));
	    load_receiver_list("GROUP.dat", GTK_CLIST(receiver_clist));
	    load_main_list_with_pixmaps();
	    gtk_clist_thaw(GTK_CLIST(receiver_clist));
	    database_changed = 0;
	}
    }


}


static int get_protocol_order(gchar *resource, pref_widgets *prefs)
{

    int   one = 1;
    int   two = 1;
    int   three = 1;

    if (GTK_TOGGLE_BUTTON (prefs->wctp_one)->active)
    {
	resource[0] = '1';
	one--;
    }
    else if (GTK_TOGGLE_BUTTON (prefs->wctp_two)->active)
    {
	resource[1] = '1';
	two--;
    }
    else
    {
	resource[2] = '1';
	three--;
    }

    if (GTK_TOGGLE_BUTTON (prefs->snpp_one)->active)
    {
	resource[0] = '2';
	one--;
    }
    else if (GTK_TOGGLE_BUTTON (prefs->snpp_two)->active)
    {
	resource[1] = '2';
	two--;
    }
    else
    {
	resource[2] = '2';
	three--;
    }

    if (GTK_TOGGLE_BUTTON (prefs->smtp_one)->active)
    {
	resource[0] = '3';
	one--;
    }
    else if (GTK_TOGGLE_BUTTON (prefs->smtp_two)->active)
    {
	resource[1] = '3';
	two--;
    }
    else
    {
	resource[2] = '3';
	three--;
    }

    resource[3] = '\n';
    resource[4] = '\0';
    if ( one || two || three )
	return (1);
    else
	return (0);

}

void
on_set_precedence_toggled                (GtkToggleButton *togglebutton,
                                          gpointer         user_data)
{
    gtk_widget_show(user_data);
}

void
on_set_no_precedence_toggled                (GtkToggleButton *togglebutton,
					     gpointer         user_data)
{
    gtk_widget_hide(user_data);
}

void
on_show_options_toggled                   (GtkToggleButton *togglebutton,
                                          gpointer         user_data)
{
    gtk_widget_show(GTK_WIDGET(user_data));
}

void
on_hide_options_toggled                   (GtkToggleButton *togglebutton,
                                          gpointer         user_data)
{
    gtk_widget_hide(GTK_WIDGET(user_data));
}


void
on_pref_apply_button_clicked                 (GtkWidget       *w,
					     pref_data    *user_data)
{
    clear_memory_prefs (user_data);
    user_data = get_pref_mem(user_data);
    load_prefs(user_data);    
}

void on_pref_cancel_button_clicked               (GtkWidget       *w,
	       				          pref_data       *user_data)
{
    extern gint    error_flag;

/*
 *   We have created an error dialog box telling the user
 *   about an error that occured while setting their prefs
 *   We don't destroy the main dialog box until they fix the error
 */

    if ( error_flag )
	return;

    clear_memory_prefs (user_data);

    gtk_widget_destroy(user_data->main);

    g_free(user_data);

}

pref_data  *get_pref_mem(pref_data *prefs)
{

    prefs->current_dir = g_malloc(MAX_LENGTH);	/* Insanely long */
    prefs->log_delete = g_malloc(MAX_FIELD_LENGTH);
    prefs->days_before_delete = g_malloc(MAX_FIELD_LENGTH);
    prefs->signature_yes_no = g_malloc(MAX_FIELD_LENGTH);
    prefs->signature_before = g_malloc(MAX_FIELD_LENGTH);
    prefs->signature = g_malloc(MAX_FIELD_LENGTH);
    prefs->cc_email = g_malloc(MAX_LENGTH);
    prefs->set_protocols = g_malloc(MAX_FIELD_LENGTH);
    prefs->order_of_protocols = g_malloc(MAX_FIELD_LENGTH);
    prefs->show_options = g_malloc(MAX_FIELD_LENGTH);
    prefs->qpage_adr = g_malloc(MAX_FIELD_LENGTH);
    prefs->qpage_port = g_malloc(MAX_FIELD_LENGTH);
    prefs->show_status = g_malloc(MAX_FIELD_LENGTH);
    prefs->serial_mode = g_malloc(MAX_FIELD_LENGTH);
    prefs->time_stamp = g_malloc(MAX_FIELD_LENGTH);
    prefs->script_delay = g_malloc(MAX_FIELD_LENGTH);
    prefs->default_polling = g_malloc(MAX_FIELD_LENGTH);
    prefs->retries = g_malloc(MAX_FIELD_LENGTH);
    prefs->database_location = g_malloc(MAX_FIELD_LENGTH);
    prefs->database_adr = g_malloc(MAX_FIELD_LENGTH);
    prefs->update_interval = g_malloc(MAX_FIELD_LENGTH);
    prefs->db_port = g_malloc(MAX_FIELD_LENGTH);
    prefs->msg_entry_options = g_malloc(MAX_FIELD_LENGTH);
    prefs->log_delete_poll = g_malloc(MAX_FIELD_LENGTH);
    prefs->rollover = g_malloc(MAX_FIELD_LENGTH);
    prefs->connect_yes = g_malloc(MAX_FIELD_LENGTH);
    prefs->create_error_status_file = g_malloc(MAX_FIELD_LENGTH);
    return (prefs);

}

void load_prefs(pref_data *user_pref)
{
    gchar  *resource_field;


    if ( ( resource_field = get_resource_data("CONFIG.dat", "db_dir", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->current_dir, "~/");
    else
    {
	strcpy(user_pref->current_dir, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "auto_delete", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->log_delete, "0");
    else
    {
	strcpy(user_pref->log_delete, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "wait_days", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->days_before_delete, "1");
    else
    {
	strcpy(user_pref->days_before_delete, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "msg_sig", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->signature_yes_no, "no");
    else
    {
	strcpy(user_pref->signature_yes_no, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "sig_placement", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->signature_before, "after");
    else
    {
	strcpy(user_pref->signature_before, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "sig", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->signature, "none");
    else
    {
	strcpy(user_pref->signature, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "cc_email", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->cc_email, "none");
    else
    {
	strcpy(user_pref->cc_email, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "set_protocols", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->set_protocols, "none");
    else
    {
	strcpy(user_pref->set_protocols, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "order_of_protocols", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->order_of_protocols, "123");
    else
    {
	strcpy(user_pref->order_of_protocols, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "show_advanced_options", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->show_options, "no");
    else
    {
	strcpy(user_pref->show_options, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "default_address", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->qpage_adr, "localhost");
    else
    {
	strcpy(user_pref->qpage_adr, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "default_port", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->qpage_port, "444");
    else
    {
	strcpy(user_pref->qpage_port, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "status_mode", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->show_status, "yes");
    else
    {
	strcpy(user_pref->show_status, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "serial_mode", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->serial_mode, "yes");
    else
    {
	strcpy(user_pref->serial_mode, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "time_stamp", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->time_stamp, "no");
    else
    {
	strcpy(user_pref->time_stamp, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "polling_interval", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->default_polling, "10");
    else
    {
	strcpy(user_pref->default_polling, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "script_delay", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->script_delay, "2");
    else
    {
	strcpy(user_pref->script_delay, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "locate_retries", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->retries, "10");
    else
    {
	strcpy(user_pref->retries, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "database", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->database_location, "local");
    else
    {
	strcpy(user_pref->database_location, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "database_adr", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->database_adr, "localhost");
    else
    {
	strcpy(user_pref->database_adr, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "update_interval", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->update_interval, "0");
    else
    {
	strcpy(user_pref->update_interval, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "database_port", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->db_port, "1301");
    else
    {
	strcpy(user_pref->db_port, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "clear_msg_box", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->msg_entry_options, "always");
    else
    {
	strcpy(user_pref->msg_entry_options, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "log_delete_poll", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->log_delete_poll, "0");
    else
    {
	strcpy(user_pref->log_delete_poll, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "rollover_no_connect", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->rollover, "no");
    else
    {
	strcpy(user_pref->rollover, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "dial_no_connect", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->connect_yes, "no");
    else
    {
	strcpy(user_pref->connect_yes, resource_field);
	g_free(resource_field);
    }

    if ( (resource_field = get_resource_data("CONFIG.dat", "create_error_status_file", MAX_FIELD_LENGTH)) == NULL)
	strcpy(user_pref->create_error_status_file, "no");
    else
    {
	strcpy(user_pref->create_error_status_file, resource_field);
	g_free(resource_field);
    }

}

void file_ok_sel( GtkWidget        *w,
                  new_dir          *data)
{
    char      *file_name;
    struct    stat   statbuf;

    file_name = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION (data->file_selection)));
    stat(file_name, &statbuf);


/*  We got a directory -good */

    if (S_ISDIR(statbuf.st_mode))
    {
	if ( strlen(file_name) > MAX_LENGTH - 10 )
	{
	    generic_error("\n  You have selected a directory path that \n exceeds the maximum length supported by g-page \n", "G-Page Error");
	    g_free(file_name);
	    return;
	}
	gtk_label_set_text(GTK_LABEL(data->dir_label), file_name);
	gtk_widget_show(data->dir_label);
	gtk_widget_destroy(data->file_selection);
    }

/* We have a file  - bad */

    if (S_ISREG(statbuf.st_mode))
    {
	generic_error("\n  Please select a directory  \n", "G-Page Error");
	g_free(file_name);
	return;
    }
    g_free(file_name);
}     

void reset_interface(GtkWidget *w, pref_data *user_data)
{
    //extern        gint network_db;

    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (user_data->prefs->radio_local), TRUE);		
}

void convert_database(GtkWidget *w, pref_data *user_data)
{
    extern        gint network_db;
    GtkWidget          *dlg_box;

    if ( !network_db )
    {
	generic_error("\n  You do not have a network database to convert  \n  You must be reading from a network database \n", "G-Page Error");
	return;
    }

    dlg_box = confirm_convert();
    gtk_widget_show(dlg_box);

}

void convert_database_okay(GtkWidget *w, gchar *user_data)
{
    extern   GtkWidget  *receiver_clist;
    extern   gint       network_db;
    FILE                *pp, *fp;
    gint                x;
    gint                flag;
    gchar               *resource;
    gchar               *old = NULL;
    gchar               *new = NULL;
    gchar               *command = NULL;
    gchar               *net = NULL;
    gchar               *dat = NULL;
    gchar               *bak = NULL;

    resource = get_working_dir();

/* First we make sure all the proper files exist before we trash them */

    flag = 0;

    for ( x = 0; x < 4; x++ )
    {
	switch (x) {
	case 0:
	    net = g_strdup("RECEIVER.net");
	    dat = g_strdup("RECEIVER.dat");
	    bak = g_strdup("RECEIVER.bak");
	    break;
	case 1:
	    net = g_strdup("SERVICE.net");
	    dat = g_strdup("SERVICE.dat");
	    bak = g_strdup("SERVICE.bak");
	    break;
	case 2:
	    net = g_strdup("GROUP.net");
	    dat = g_strdup("GROUP.dat");
	    bak = g_strdup("GROUP.bak");
	    break;
	case 3:
	    net = g_strdup("IDS.net");
	    dat = g_strdup("IDS.dat");
	    bak = g_strdup("IDS.bak");
	    break;
	default:
	    break;	    
	}

	if ( user_data == NULL )
	    old = g_strdup_printf("%s%s", resource, net);
	else if ( !strcmp(user_data, "backup") )
	    old = g_strdup_printf("%s%s", resource, dat);
	else if ( !strcmp(user_data, "restore") )
	    old = g_strdup_printf("%s%s", resource, bak);

	if ( ( fp = fopen(old, "r")) == NULL ) /* oops - file doesn't exist */
	{
	    flag = 1;
	    g_free(net);
	    g_free(dat);
	    g_free(bak);
	    g_free(old);
	    break;
	}
	fclose(fp);
	g_free(net);
	g_free(dat);
	g_free(bak);
	g_free(old);
    }


    if ( flag == 1 )
    {
	generic_error("\n Unable to perform database operation: missing or corrupt file \n", "G-Page Error");
	return;
    }

/*  Now we do the backup, restore or convert   */

    for ( x = 0; x < 4; x++ )
    {
	switch (x) {
	case 0:
	    net = g_strdup("RECEIVER.net");
	    dat = g_strdup("RECEIVER.dat");
	    bak = g_strdup("RECEIVER.bak");
	    break;
	case 1:
	    net = g_strdup("SERVICE.net");
	    dat = g_strdup("SERVICE.dat");
	    bak = g_strdup("SERVICE.bak");
	    break;
	case 2:
	    net = g_strdup("GROUP.net");
	    dat = g_strdup("GROUP.dat");
	    bak = g_strdup("GROUP.bak");
	    break;
	case 3:
	    net = g_strdup("IDS.net");
	    dat = g_strdup("IDS.dat");
	    bak = g_strdup("IDS.bak");
	    break;
	default:
	    break;	    
	}

	if ( user_data == NULL )
	{
	    old = g_strdup_printf("%s%s", resource, net);
	    new = g_strdup_printf("%s%s", resource, dat);
	    command = g_strdup_printf("mv %s %s", old, new);
	}
	else if ( !strcmp(user_data, "backup") )
	{
	    old = g_strdup_printf("%s%s", resource, dat);
	    new = g_strdup_printf("%s%s", resource, bak);
	    command = g_strdup_printf("cp %s %s", old, new);
	}
	else if ( !strcmp(user_data, "restore") )
	{
	    old = g_strdup_printf("%s%s", resource, dat);
	    new = g_strdup_printf("%s%s", resource, bak);
	    command = g_strdup_printf("cp %s %s", new, old);
	}
	pp = popen(command, "r");
	pclose(pp);
	g_free(net);
	g_free(dat);
	g_free(bak);
	g_free(old);
	g_free(new);
	g_free(command);
	
    }

    g_free(resource);

    if ( user_data == NULL )
    {
	network_db = 0;
	edit_item("database", "CONFIG.dat", 2, 1, "database=local\n", EDIT, '=');
    }
    
    gtk_clist_freeze(GTK_CLIST(receiver_clist));
    gtk_clist_clear(GTK_CLIST(receiver_clist));
    load_receiver_list("RECEIVER.dat", GTK_CLIST(receiver_clist));
    load_receiver_list("GROUP.dat", GTK_CLIST(receiver_clist));
    load_main_list_with_pixmaps();
    gtk_clist_thaw(GTK_CLIST(receiver_clist));
    
    return;
}

void selection_made(GtkWidget *w, GtkCTreeNode *node, gint row, dynamic_box_data *vbox_data)
{
    gchar  *data;


    if ( vbox_data->active_box == NULL )
	return;

    gtk_widget_hide(vbox_data->active_box);
    if ( GTK_CLIST(w)->selection == NULL)
    {
	fprintf(stderr, "Error no selection\n");
    }
    else
    {
	gtk_widget_hide(vbox_data->active_box);
	gtk_ctree_node_get_text(GTK_CTREE(w), GTK_CLIST(w)->selection->data , 1, &data);
	if ( !strcmp(data, "10") )
	{
	    gtk_widget_show(vbox_data->main_db_box);
	    vbox_data->active_box = vbox_data->main_db_box;
	}
	else if ( !strcmp(data, "11") )
	{
	    gtk_widget_show(vbox_data->vbox_network_settings);
	    vbox_data->active_box = vbox_data->vbox_network_settings;
	}
	else if ( !strcmp(data, "20") )
	{
	    gtk_widget_show(vbox_data->auto_delete_logs_box);
	    vbox_data->active_box = vbox_data->auto_delete_logs_box;
	}	       
	else if ( !strcmp(data, "30") )
	{
	    gtk_widget_show(vbox_data->signature_box);
	    vbox_data->active_box = vbox_data->signature_box;
	}	       
	else if ( !strcmp(data, "40") )
	{
	    gtk_widget_show(vbox_data->protocols_box);
	    vbox_data->active_box = vbox_data->protocols_box;
	}	       
	else if ( !strcmp(data, "50") )
	{
	    gtk_widget_show(vbox_data->auto_page_box);
	    vbox_data->active_box = vbox_data->auto_page_box;
	}	       
	else if ( !strcmp(data, "60") )
	{
	    gtk_widget_show(vbox_data->msg_entry_box);
	    vbox_data->active_box = vbox_data->msg_entry_box;
	}	       
	else if ( !strcmp(data, "70") )
	{
	    gtk_widget_show(vbox_data->advanced_box);
	    vbox_data->active_box = vbox_data->advanced_box;
	}	       
	else if ( !strcmp(data, "71") )
	{
	    gtk_widget_show(vbox_data->advanced_box_sending);
	    vbox_data->active_box = vbox_data->advanced_box_sending;
	}	       
	else if ( !strcmp(data, "72") )
	{
	    gtk_widget_show(vbox_data->advanced_box_qpage);
	    vbox_data->active_box = vbox_data->advanced_box_qpage;
	}	       

    }

}

static void clear_memory_prefs (pref_data  *user_data)
{
    g_free(user_data->current_dir);
    g_free(user_data->log_delete);
    g_free(user_data->days_before_delete);
    g_free(user_data->signature_yes_no);
    g_free(user_data->signature_before);
    g_free(user_data->signature);
    g_free(user_data->cc_email);
    g_free(user_data->set_protocols);
    g_free(user_data->order_of_protocols);
    g_free(user_data->show_options);
    g_free(user_data->qpage_adr);
    g_free(user_data->qpage_port);
    g_free(user_data->show_status);
    g_free(user_data->serial_mode);
    g_free(user_data->time_stamp);
    g_free(user_data->script_delay);
    g_free(user_data->default_polling);
    g_free(user_data->retries);
    g_free(user_data->database_location);
    g_free(user_data->database_adr);
    g_free(user_data->update_interval);
    g_free(user_data->db_port);
    g_free(user_data->msg_entry_options);
    g_free(user_data->create_error_status_file);
    g_free(user_data->log_delete_poll);
    g_free(user_data->rollover);
    g_free(user_data->connect_yes);
}
