/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

#define TWO_MINUTES 120
#define ONE_MINUTE   60

static gint look_for_reply(reply_info *data);
static gint clean_log_file(gchar *filename, gchar *tracking_number);
static void change_log_file_status(gchar *track);
static void free_the_reply_mem(gchar *one, gchar *two, gchar *three, gchar *four, gchar *five, gchar *six,
			 gchar *seven, gchar *eight);

void on_click_column_reply (GtkCList *list, gint row, gint col, GdkEventButton *event, update_text *user_data)
{
    GdkFont   *current_font;
    gchar     *msg, *display_msg, *reply;
    gint      load_reply;

    load_reply = 0;

    if (GTK_CLIST(user_data->old_list)->selection != NULL)
    {
	gtk_clist_get_text( (GtkCList *)user_data->old_list, GPOINTER_TO_INT(GTK_CLIST(user_data->old_list)->selection->data), 2, &msg);
	gtk_clist_get_text( (GtkCList *)user_data->old_list, GPOINTER_TO_INT(GTK_CLIST(user_data->old_list)->selection->data), 3, &reply);
	load_reply = 1;
    }
    else
    {
	gtk_clist_get_text( (GtkCList *)user_data->new_list, GPOINTER_TO_INT(GTK_CLIST(user_data->new_list)->selection->data), 4, &msg);
    }
    
    display_msg = g_strdup(msg);

    current_font = gdk_font_load(display_msg);

    delete_text_in_text_box(user_data->text_box);

    gtk_text_insert(GTK_TEXT(user_data->text_box), current_font, NULL, NULL, display_msg, strlen(display_msg));
    gtk_text_set_point( GTK_TEXT(user_data->text_box) , 0 );
    
    if (load_reply)
    {
	delete_text_in_text_box(user_data->reply_box);

	gtk_text_insert(GTK_TEXT(user_data->reply_box), current_font, NULL, NULL, reply, strlen(reply));
	gtk_text_set_point( GTK_TEXT(user_data->reply_box) , 0 );
    }
    else
    {
	delete_text_in_text_box(user_data->reply_box);
    }

    g_free(display_msg);

}


void
on_reply_refresh_button_clicked         (GtkButton       *button,
                                        reply_info      *user_data)
{

  load_pending_clist(user_data->pending_clist);
  load_old_clist(user_data->old_clist);

}

void
on_reply_status_button_clicked         (GtkButton       *button,
                                        reply_info      *user_data)
{
    gchar            *uid, *tracking, *msg, *track;
    uid_list         *current_uid;
    uid_list         *head;
    uid_list         *temp;
    gint             file_desc[2];


    if (GTK_CLIST(user_data->pending_clist)->selection == NULL)
    {
	generic_error("\n  You must select a message from the list  \n  of outstanding messages to query  \n", "G-Page Error");
	return;
    }
    
    gtk_clist_get_text( (GtkCList *)user_data->pending_clist, GPOINTER_TO_INT(GTK_CLIST(user_data->pending_clist)->selection->data), 6, &uid);
    gtk_clist_get_text( (GtkCList *)user_data->pending_clist, GPOINTER_TO_INT(GTK_CLIST(user_data->pending_clist)->selection->data), 3, &tracking);
    gtk_clist_get_text( (GtkCList *)user_data->pending_clist, GPOINTER_TO_INT(GTK_CLIST(user_data->pending_clist)->selection->data), 4, &msg);

    track = g_strdup(tracking);

    current_uid = get_uid_node();
    head = current_uid;
    strcpy(current_uid->uid, uid);
    temp = get_uid_node();
    current_uid->next = temp;
    current_uid = temp;
    user_data->msg = g_strdup(msg);
    user_data->track = g_strdup(tracking);

/*
 *   Here we create a pipe so the funtion receiving data from the server can tell us the status 
 *   of our query.
 */

    if ( (pipe(file_desc)) == -1 )
    {
	gtk_label_set_text(GTK_LABEL(user_data->status_label), "Internal error: unable to continue");
	return;
    }
    user_data->file_desc = file_desc[0];
    user_data->write_file_desc = file_desc[1];
    gtk_label_set_text(GTK_LABEL(user_data->status_label), "Connecting with wireless service - please wait...");
    connect_to_server(head, user_data->msg, "none", NULL, TRACK_WCTP, track, file_desc[1]);


    gtk_timeout_add(ONE_SECOND, (GtkFunction)look_for_reply, (gpointer) user_data);

}


void
on_reply_delete_button_clicked         (GtkButton       *button,
                                        reply_info      *user_data)
{
    gchar   *tracking, *track;

    if ( (GTK_CLIST(user_data->pending_clist)->selection == NULL) && (GTK_CLIST(user_data->old_clist)->selection == NULL) )
    {
	generic_error("\n  You must select a message to delete  \n", "G-Page Error");
	return;
    }
    else if ( (GTK_CLIST(user_data->pending_clist)->selection != NULL) && (GTK_CLIST(user_data->old_clist)->selection != NULL) )
    {
	generic_error("\n  You can only delete from one list at a time  \n", "G-Page Error");
	return;
    }
	
    if ( GTK_CLIST(user_data->pending_clist)->selection != NULL )
    {
	gtk_clist_get_text( (GtkCList *)user_data->pending_clist, GPOINTER_TO_INT(GTK_CLIST(user_data->pending_clist)->selection->data), 3, &tracking);
	track = g_strdup(tracking);
	clean_log_file("2way_log.gpage", track);
	load_pending_clist(user_data->pending_clist);
    }
    else
    {
	gtk_clist_get_text( (GtkCList *)user_data->old_clist, GPOINTER_TO_INT(GTK_CLIST(user_data->old_clist)->selection->data), 4, &tracking);
	track = g_strdup(tracking);
	clean_log_file("reply.gpage", track);
	load_old_clist(user_data->old_clist);
    }
    g_free(track);
}


/*
 *  Right now I'm leaving these structures as not freed. A problem happens if the user closes the dialog box
 *  before the transaction is completed. I might end up kludging this with a global variable
 *
 */

void
on_reply_okay_button_clicked           (GtkButton       *button,
                                        window_info     *user_data)
{
    //g_free(user_data->reply_data);
    //user_data->reply_data = NULL;
    g_free(user_data->text_info);

    gtk_widget_destroy(user_data->main_win);
    //g_free(user_data);
}


static gint look_for_reply(reply_info *data)
{
    static gint              counter = 0;
    gchar                    *buffer, *track_copy;
    gint                     maxfdp1;
    fd_set                   rset;
    gint                     select_result;
    struct timeval           tv;


    if ( !(GTK_IS_WIDGET(data->status_label)) )
    {
	close(data->file_desc);
	close(data->write_file_desc);
	g_free(data->track);
	return(FALSE);
    }

    counter++;
    if ( counter > TWO_MINUTES )
    {
	if ( GTK_IS_WIDGET(data->status_label) )
	    gtk_label_set_text(GTK_LABEL(data->status_label), "Timeout connecting with messaging service");

	close(data->file_desc);
	close(data->write_file_desc);
	g_free(data->track);
	counter = 0;
	return(FALSE);
    }

    buffer = g_malloc(MAX_LENGTH);

    FD_ZERO(&rset);
    FD_SET(data->file_desc, &rset);

    maxfdp1 = data->file_desc + 1;
    tv.tv_sec = 0;		/* Set select timeout here  */
    tv.tv_usec = 0;

    select_result = select(maxfdp1, &rset, NULL, NULL, &tv);

    if ( select_result == -1 )
    {
	counter = 0;
	if ( GTK_IS_WIDGET(data->status_label) )
	    gtk_label_set_text(GTK_LABEL(data->status_label), "Error connecting with messaging service");

	close(data->file_desc);
	close(data->write_file_desc);
	g_free(data->track);
	g_free(buffer);
	return(FALSE);
    }
    else if ( select_result == 0 )
    {
	g_free(buffer);
	return(TRUE);
    }
    else
    {
	read_a_line(data->file_desc, buffer, MAX_LENGTH);

	if ( strlen(buffer) == 1 )
	{
	    if ( GTK_IS_WIDGET(data->status_label) )
		gtk_label_set_text(GTK_LABEL(data->status_label), "No data available from messaging service");
	}
	else if ( !strcmp(buffer, "Error\n") )
	{
	    if ( GTK_IS_WIDGET(data->status_label) )
		gtk_label_set_text(GTK_LABEL(data->status_label), "Unable to connect with messaging service");
	}
	else if ( !strcmp(buffer, "DELIVERED\n") )
	{
	    track_copy = g_strdup(data->track);
	    change_log_file_status(track_copy);
	    g_free(track_copy);

	    if ( GTK_IS_WIDGET(data->status_label) )
		gtk_label_set_text(GTK_LABEL(data->status_label), "Message was delivered - no reply");

	    if ( GTK_IS_WIDGET(data->pending_clist) )
	    {
		load_pending_clist(data->pending_clist);
	    }
	}
	else
	{
	    if ( GTK_IS_WIDGET(data->status_label) )
		gtk_label_set_text(GTK_LABEL(data->status_label), "Message reply has been logged");

	    track_copy = g_strdup(data->track);
	    change_log_file_status(track_copy);
	    clean_log_file("2way_log.gpage", track_copy);    
	    g_free(track_copy);

	    if ( GTK_IS_WIDGET(data->pending_clist) )
	    {
		load_pending_clist(data->pending_clist);
	    }
    
	    if ( GTK_IS_WIDGET(data->old_clist) )
	    {
		load_old_clist(data->old_clist);
	    }	    
	}
    }

    close(data->file_desc);
    close(data->write_file_desc);
    g_free(data->track);
    g_free(buffer);
    counter = 0;

    if ( ! (GTK_IS_WIDGET(data->status_label)) )
	g_free(data);

    return(FALSE);
}

static gint clean_log_file(gchar *filename, gchar *tracking_number)
{
    extern pthread_mutex_t   update_reply_log_mutex;

    FILE  *fp, *tfp;
    gint  check;
    gint  mutex_status;
    gchar *id, *name, *protocol, *tracking, *msg, *date, *temp, *status;
    gchar *line;

    if ( (fp = get_file_pointer(filename, "r")) == NULL)
	return(-1);

    tfp = get_temp_file();
    if ( tfp == NULL )
	return(-1);

    if ( (mutex_status = pthread_mutex_lock(&update_reply_log_mutex)) != 0 )
	perror("Error locking mutex - possible data corruption");
    
    id = g_malloc(MAX_FIELD_LENGTH);
    name = g_malloc(MAX_FIELD_LENGTH);
    protocol = g_malloc(MAX_FIELD_LENGTH);
    tracking = g_malloc(MAX_FIELD_LENGTH);
    msg = g_malloc(MAX_LENGTH);
    date = g_malloc(MAX_FIELD_LENGTH);
    status = g_malloc(MAX_FIELD_LENGTH);
    
    while ( (line = (read_file_by_line(fp))) != NULL)
    {
      check = get_data(line, date, MAX_FIELD_LENGTH, id, MAX_FIELD_LENGTH, status, MAX_FIELD_LENGTH, name, MAX_FIELD_LENGTH, 
		       protocol, MAX_FIELD_LENGTH, tracking, MAX_FIELD_LENGTH, msg, MAX_LENGTH);
      //check = get_data(line, date, id, status, name, protocol, tracking, msg);
	if ( strcmp(tracking, tracking_number) )
	    fputs(line, tfp);
	g_free(line);
    }

    fclose(fp);
    rewind(tfp);
    
    if ( (fp = get_file_pointer(filename, "w")) == NULL)
    {
	if ( (mutex_status = pthread_mutex_unlock(&update_reply_log_mutex)) != 0 )
	    perror("Error unlocking mutex - possible data corruption");
	return(-1);
    }

    temp = g_malloc(MAX_LENGTH);
    while(fgets(temp, MAX_LENGTH, tfp))
	fputs(temp, fp);

    fclose(fp);
    fclose(tfp);

    if ( (mutex_status = pthread_mutex_unlock(&update_reply_log_mutex)) != 0 )
	perror("Error unlocking mutex - possible data corruption");

    free_the_reply_mem(temp, id, name, protocol, msg, date, tracking, status);

    return(0);
}

static void change_log_file_status(gchar *track)
{
    extern pthread_mutex_t   update_reply_log_mutex;

    FILE  *fp, *tfp;
    gint  check;
    gint  mutex_status;
    gchar *id, *name, *protocol, *tracking, *msg, *date, *temp, *status, *buffer;
    gchar *line;

    if ( (fp = get_file_pointer("2way_log.gpage", "r")) == NULL)
	return;

    tfp = get_temp_file();
    if ( tfp == NULL )
	return;

    if ( (mutex_status = pthread_mutex_lock(&update_reply_log_mutex)) != 0 )
	perror("Error locking mutex - possible data corruption");
    
    id = g_malloc(MAX_FIELD_LENGTH);
    name = g_malloc(MAX_FIELD_LENGTH);
    protocol = g_malloc(MAX_FIELD_LENGTH);
    tracking = g_malloc(MAX_FIELD_LENGTH);
    msg = g_malloc(MAX_LENGTH);
    date = g_malloc(MAX_FIELD_LENGTH);
    status = g_malloc(MAX_FIELD_LENGTH);
    
    while ( (line = (read_file_by_line(fp))) != NULL)
    {
      check = get_data(line, date, MAX_FIELD_LENGTH, id, MAX_FIELD_LENGTH, status, MAX_FIELD_LENGTH, name, MAX_FIELD_LENGTH, 
		       protocol, MAX_FIELD_LENGTH, tracking, MAX_FIELD_LENGTH, msg, MAX_LENGTH);
      //check = get_data(line, date, id, status, name, protocol, tracking, msg);
	if ( strcmp(tracking, track) )
	    fputs(line, tfp);
	else
	{
	    buffer = g_strdup_printf("%s,%s,D,%s,%s,%s,%s\n", date, id,protocol, tracking, name, msg);
	    fputs(buffer, tfp);
	    g_free(buffer);
	}
	g_free(line);
    }

    fclose(fp);
    rewind(tfp);
    
    if ( (fp = get_file_pointer("2way_log.gpage", "w")) == NULL)
    {
	if ( (mutex_status = pthread_mutex_unlock(&update_reply_log_mutex)) != 0 )
	    perror("Error unlocking mutex - possible data corruption");
	return;
    }

    temp = g_malloc(MAX_LENGTH);
    while(fgets(temp, MAX_LENGTH, tfp))
	fputs(temp, fp);

    fclose(fp);
    fclose(tfp);

    if ( (mutex_status = pthread_mutex_unlock(&update_reply_log_mutex)) != 0 )
	perror("Error unlocking mutex - possible data corruption");
    
    free_the_reply_mem(temp, id, name, protocol, msg, date, tracking, status);

    return;
}

static void free_the_reply_mem(gchar *one, gchar *two, gchar *three, gchar *four, gchar *five, gchar *six,
			       gchar *seven, gchar *eight)
{
    g_free(one);
    g_free(two);
    g_free(three);
    g_free(four);
    g_free(five);
    g_free(six);
    g_free(seven);
    g_free(eight);
}
