/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

void schedule_page()
{

    GtkWidget  *sch_win;

    sch_win = create_schedule_window();
    gtk_widget_show(sch_win);

}


GtkWidget*
create_schedule_window (void)
{
  GtkWidget *schedule_window;
  GtkWidget *vbox3;
  GtkWidget *calendar;
  GtkWidget *hbox3;
  GtkWidget *hour_label;
  GtkObject *spin_hour_adj;
  GtkWidget *spin_hour;
  GtkWidget *minute_label;
  GtkObject *spin_minute_adj;
  GtkWidget *spin_minute;
  GSList    *sched_group = NULL;
  GtkWidget *am_sched;
  GtkWidget *pm_sched;
  GtkWidget *hseparator1;
  GtkWidget *hbuttonbox2;
  GtkWidget *sched_ok;
  GtkWidget *sched_cancel;

  GtkWidget *frame_recurring;
  GtkWidget *hbox1;
  GSList *recurring__group = NULL;
  GtkWidget *radio_no;
  GtkWidget *radio_hourly;
  GtkWidget *radio_daily;
  GtkWidget *radio_weekly;

  time_t     time_val;
  struct tm  *time_str;
  int        hour = 0;
  int        am = 0;
  sched_msg_info *sch_data;

  sch_data = g_malloc(sizeof(*sch_data));


/*
 *    Get the current time so we can load it as default values
 */

  time(&time_val);
  time_str = localtime(&time_val);
  
  schedule_window = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_object_set_data (GTK_OBJECT (schedule_window), "schedule_window", schedule_window);
  gtk_window_set_title (GTK_WINDOW (schedule_window), "Schedule Page");

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox3);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "vbox3", vbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (schedule_window), vbox3);

  calendar = gtk_calendar_new ();
  gtk_widget_ref (calendar);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "calendar", calendar,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (calendar);
  gtk_box_pack_start (GTK_BOX (vbox3), calendar, TRUE, TRUE, 0);
  gtk_calendar_display_options (GTK_CALENDAR (calendar),
                                GTK_CALENDAR_SHOW_HEADING
                                | GTK_CALENDAR_SHOW_DAY_NAMES);
  gtk_calendar_select_day( GTK_CALENDAR(calendar), (time_str->tm_mday));

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox3, TRUE, FALSE, 5);

/*
 *   Here are the spin buttons for the hour and the minute
 */

  hour_label = gtk_label_new (" Hour: ");
  gtk_widget_ref (hour_label);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "hour_label", hour_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hour_label);
  gtk_box_pack_start (GTK_BOX (hbox3), hour_label, FALSE, FALSE, 1);

  if ( time_str->tm_hour <= 11 )
  {
      am = 1;
      hour = time_str->tm_hour;
      if ( hour == 0 )
	  hour = 12;
  }
  else
  {
      am = 0;
      hour = time_str->tm_hour -12;
      if ( hour == 0 )
	  hour = 12;
  }
      
  spin_hour_adj = gtk_adjustment_new (hour, 1, 12, 1, 10, 10);
  spin_hour = gtk_spin_button_new (GTK_ADJUSTMENT (spin_hour_adj), 1, 0);
  gtk_widget_ref (spin_hour);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "spin_hour", spin_hour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spin_hour);
  gtk_box_pack_start (GTK_BOX (hbox3), spin_hour, FALSE, TRUE, 1);

  minute_label = gtk_label_new (" Minute: ");
  gtk_widget_ref (minute_label);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "minute_label", minute_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (minute_label);
  gtk_box_pack_start (GTK_BOX (hbox3), minute_label, FALSE, FALSE, 1);

  spin_minute_adj = gtk_adjustment_new ( (time_str->tm_min), 0, 59, 1, 10, 10);
  spin_minute = gtk_spin_button_new (GTK_ADJUSTMENT (spin_minute_adj), 1, 0);
  gtk_widget_ref (spin_minute);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "spin_minute", spin_minute,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spin_minute);
  gtk_box_pack_start (GTK_BOX (hbox3), spin_minute, FALSE, TRUE, 1);

/*
 *   Here are the radio buttons for  am pm 
 */

  am_sched = gtk_radio_button_new_with_label (sched_group, " AM ");
  sched_group = gtk_radio_button_group (GTK_RADIO_BUTTON (am_sched));
  gtk_widget_ref (am_sched);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "am_sched", am_sched,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (am_sched);
  gtk_box_pack_start (GTK_BOX (hbox3), am_sched, FALSE, FALSE, 1);

  pm_sched = gtk_radio_button_new_with_label (sched_group, " PM ");
  sched_group = gtk_radio_button_group (GTK_RADIO_BUTTON (pm_sched));
  gtk_widget_ref (pm_sched);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "pm_sched", pm_sched,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pm_sched);
  gtk_box_pack_start (GTK_BOX (hbox3), pm_sched, FALSE, FALSE, 1);

  if ( am )
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (am_sched), TRUE);
  else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pm_sched), TRUE);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator1);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "hseparator1", hseparator1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox3), hseparator1, FALSE, TRUE, 0);

/*
 *    Here is the frame and the radio buttons for sending a recurring page
 */

  frame_recurring = gtk_frame_new (" Recurring page: ");
  gtk_widget_ref (frame_recurring);
  gtk_object_set_data_full (GTK_OBJECT (vbox3), "frame_recurring", frame_recurring,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame_recurring);
  gtk_container_add (GTK_CONTAINER (vbox3), frame_recurring);
  gtk_container_set_border_width (GTK_CONTAINER (frame_recurring), 3);

  hbox1 = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (vbox3), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_container_add (GTK_CONTAINER (frame_recurring), hbox1);

  radio_no = gtk_radio_button_new_with_label (recurring__group, " No");
  recurring__group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_no));
  gtk_widget_ref (radio_no);
  gtk_object_set_data_full (GTK_OBJECT (vbox3), "radio_no", radio_no,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radio_no);
  gtk_box_pack_start (GTK_BOX (hbox1), radio_no, FALSE, FALSE, 2);

  radio_hourly = gtk_radio_button_new_with_label (recurring__group, "Hourly");
  recurring__group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_hourly));
  gtk_widget_ref (radio_hourly);
  gtk_object_set_data_full (GTK_OBJECT (vbox3), "radio_hourly", radio_hourly,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radio_hourly);
  gtk_box_pack_start (GTK_BOX (hbox1), radio_hourly, FALSE, FALSE, 2);

  radio_daily = gtk_radio_button_new_with_label (recurring__group, " Daily");
  recurring__group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_daily));
  gtk_widget_ref (radio_daily);
  gtk_object_set_data_full (GTK_OBJECT (vbox3), "radio_daily", radio_daily,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radio_daily);
  gtk_box_pack_start (GTK_BOX (hbox1), radio_daily, FALSE, FALSE, 2);

  radio_weekly = gtk_radio_button_new_with_label (recurring__group, "Weekly");
  recurring__group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_weekly));
  gtk_widget_ref (radio_weekly);
  gtk_object_set_data_full (GTK_OBJECT (vbox3), "radio_weekly", radio_weekly,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radio_weekly);
  gtk_box_pack_start (GTK_BOX (hbox1), radio_weekly, FALSE, FALSE, 2);

/*
 *   Here are the send and cancel buttons
 */

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator1);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "hseparator1", hseparator1,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox3), hseparator1, FALSE, TRUE, 0);

  hbuttonbox2 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox2);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "hbuttonbox2", hbuttonbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox2);
  gtk_box_pack_start (GTK_BOX (vbox3), hbuttonbox2, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox2), GTK_BUTTONBOX_SPREAD);

  sched_ok = gtk_button_new_with_label ("Send");
  gtk_widget_ref (sched_ok);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "sched_ok", sched_ok,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (sched_ok);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), sched_ok);
  GTK_WIDGET_SET_FLAGS (sched_ok, GTK_CAN_DEFAULT);

  sched_cancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_ref (sched_cancel);
  gtk_object_set_data_full (GTK_OBJECT (schedule_window), "sched_cancel", sched_cancel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (sched_cancel);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), sched_cancel);
  GTK_WIDGET_SET_FLAGS (sched_cancel, GTK_CAN_DEFAULT);

  sch_data->main_win = schedule_window;
  sch_data->cal = calendar;
  sch_data->am_sch = am_sched;
  sch_data->hour = spin_hour;
  sch_data->minutes = spin_minute;
  sch_data->radio_no = radio_no;
  sch_data->radio_hourly = radio_hourly;
  sch_data->radio_daily = radio_daily;
  sch_data->radio_weekly = radio_weekly;

  gtk_signal_connect (GTK_OBJECT (sched_ok), "clicked",
                      GTK_SIGNAL_FUNC (on_sched_ok_clicked),
                      sch_data);

  gtk_signal_connect (GTK_OBJECT (sched_cancel), "clicked",
                      GTK_SIGNAL_FUNC (on_sched_cancel_clicked),
                      sch_data);

  gtk_widget_grab_focus(sched_cancel);
  return schedule_window;
}

