/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"


void
on_sched_ok_clicked                    (GtkButton         *button,
                                        sched_msg_info    *user_data)
{
    extern GtkWidget *text_entry;

    gint  text_length = 0;
    gint  year = 0;
    gint  month = 0;
    gint  day = 0;
    gchar *send_hour;
    gchar *send_minute;
    gchar *temp;
    gchar *entry_text;
    gchar *sched_pg_info;
    gchar recurring_page[2];
    gint  am;
    sched_data  *sched_obj;
    uid_list  *active_list;

    sched_obj = g_malloc(sizeof(*sched_obj));
    sched_obj->sch_time = g_malloc(MAX_LENGTH);
    sched_obj->sch_msg = g_malloc(MAX_LENGTH);

    if (GTK_CLIST(receiver_clist)->selection == NULL)
    {
	generic_error("\n      You must select a receiver      \n", "G-Page Error");
	g_free(sched_obj->sch_msg);
	g_free(sched_obj->sch_time);
	g_free(sched_obj);
	return;
    }

    active_list = load_members();

    text_length = gtk_text_get_length( GTK_TEXT(text_entry) );
    if ( text_length > MAX_PAGE_LENGTH )
    {
	generic_error("\n Maximum number of characers exceeded   \n", "G-Page Error");
	g_free(sched_obj->sch_msg);
	g_free(sched_obj->sch_time);
	g_free(sched_obj);
	return;
    }
    if ( text_length == 0 )
    {
	generic_error("\n   You have not entered a message   \n", "G-Page Error");
	g_free(sched_obj->sch_msg);
	g_free(sched_obj->sch_time);
	g_free(sched_obj);
	return;
    }

    if (GTK_TOGGLE_BUTTON (user_data->am_sch)->active)
	am = 1;
    else
	am = 0;

    recurring_page[0] = '0';	/* This is the default behaviour. Do this just in case something evil happens */

    if (GTK_TOGGLE_BUTTON (user_data->radio_no)->active)
	strcpy(recurring_page, "0"); /* This is a zero */
    else if (GTK_TOGGLE_BUTTON (user_data->radio_hourly)->active)
	strcpy(recurring_page, "1"); 
    else if (GTK_TOGGLE_BUTTON (user_data->radio_daily)->active)
	strcpy(recurring_page, "2"); 
    if (GTK_TOGGLE_BUTTON (user_data->radio_weekly)->active)
	strcpy(recurring_page, "3"); 

    gtk_calendar_get_date( GTK_CALENDAR(user_data->cal ), &year, &month, &day);

    send_hour = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->hour)));
    send_minute = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->minutes)));

    entry_text = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1));
    entry_text = clean_text(entry_text);

    gtk_text_freeze( GTK_TEXT(text_entry) );
    gtk_text_set_point(GTK_TEXT(text_entry), 0);
    gtk_text_forward_delete(GTK_TEXT(text_entry), text_length);
    gtk_text_thaw( GTK_TEXT(text_entry) );

    temp = g_strdup_printf("last=%s\n", entry_text);

    edit_item("last", "CONFIG.dat", 2, 1, temp, EDIT, '=');
    g_free(temp);
    
    if (  (atoi(send_minute)) < 10 )
	sched_pg_info = g_strdup_printf("%d-%d-%d-%s-0%s-%d-%s", year,month,day,send_hour,send_minute,am,recurring_page);
    else
	sched_pg_info = g_strdup_printf("%d-%d-%d-%s-%s-%d-%s", year,month,day,send_hour,send_minute,am,recurring_page);

    strcpy(sched_obj->sch_time, sched_pg_info);
    strcpy(sched_obj->sch_msg, entry_text);

    connect_to_server(active_list, entry_text, sched_pg_info, sched_obj, NORMAL, NULL, -1);

    g_free(send_hour);
    g_free(send_minute);

    g_free(sched_pg_info);

    g_free(sched_obj->sch_time);
    g_free(sched_obj->sch_msg);
    g_free(sched_obj);

    gtk_widget_destroy(user_data->main_win);
    g_free(user_data);
}


void
on_sched_cancel_clicked                (GtkButton        *button,
                                        sched_msg_info    *user_data)

{
    gtk_widget_destroy(user_data->main_win);
    g_free(user_data);
}
