/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

typedef struct {
    gchar *name;
    gchar *message;
    gint  error;
} manual_page_data;


typedef struct auto_object {    
    gint       reg_number;
    gint       select_receiver_method;
    gint       file_exists;
    gint       create_error_file;
    GSList     *list_of_files;
    GSList     *list_of_raw_data;
    gchar      *message;
    gchar      *name;
    gchar      *file_cleanup_method;
} active_data;


enum {
    MANUAL,
    FROM_LIST,
};


static gchar *parse_buffer(gchar  *buffer);
static void process_target_file(gchar *file_name, active_data *data_object);
static void handle_page(active_data *page_data);
static void my_sleep(gint length);
static gint send_now(gint interval);
static void get_list_of_targets(gchar  *filename, active_data *data);
static void delete_target_files(GSList *targets);
static void rename_target_files(GSList *targets);
static void read_and_load_data_from_file(GSList *targets, active_data *data);
static void free_the_memory(gchar *four, gchar *five, manual_page_data *data);
static void on_profile_save_button_clicked(GtkWidget *w, script_data *data);
static void on_profile_cancel_button_clicked(GtkWidget *w, GtkWidget *dialog_box);
static gint get_delay();
static gint get_interval(gchar *manual_time_string);
static gint get_retries();
static gint get_mode(gchar *user_defined_script);
static void handle_file_cleanup(gchar *action, GSList *targets);
static manual_page_data *parse_data_for_manual_page(gchar *raw_data);
static void store_profile_and_save(script_text *text, script_data *data, FILE *fp);
void  register_autopage_thread(pthread_t thread_id);
static active_data *init_data();
static void free_object_memory(active_data *data);
static GSList *get_parsed_data(GSList *data);
static gint  get_error_file_status(void);
static void  write_error_file(active_data *page);
static gchar *get_temp_path(gchar *full_path);


#define DATE_LENGTH 16

enum {
    HAVE_SCRIPT,
    NO_SCRIPT,
    GO,
    WAIT
};

void on_browse_script_button_clicked        (GtkButton       *button,
					     script_data     *user_data)
{
    script_struct       *s_data;

    s_data = g_malloc(sizeof(*s_data));
    s_data->entry_box = user_data->script_entry;

    s_data->sel_box = gtk_file_selection_new ("Select file to execute");
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(s_data->sel_box));
    
    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (s_data->sel_box)->ok_button),"clicked", 
			GTK_SIGNAL_FUNC(file_has_been_selected), s_data);

    gtk_signal_connect_object( GTK_OBJECT(GTK_FILE_SELECTION(s_data->sel_box)->cancel_button),"clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(s_data->sel_box));

    gtk_widget_show(s_data->sel_box);      
    
}

void on_browse_target_button_clicked        (GtkButton       *button,
					     script_data     *user_data)
{
    script_struct       *s_data;

    s_data = g_malloc(sizeof(*s_data));
    s_data->entry_box = user_data->target_entry;

    s_data->sel_box = gtk_file_selection_new ("Select file location");
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(s_data->sel_box));
    
    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (s_data->sel_box)->ok_button),"clicked", 
			GTK_SIGNAL_FUNC(target_has_been_selected), s_data);

    gtk_signal_connect_object( GTK_OBJECT(GTK_FILE_SELECTION(s_data->sel_box)->cancel_button),"clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(s_data->sel_box));

    gtk_widget_show(s_data->sel_box);      
    
}

void file_has_been_selected( GtkWidget        *w,
			     script_struct    *data)
{
    struct    stat   statbuf;

    data->script_name_sel = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION (data->sel_box)));
    stat(data->script_name_sel, &statbuf);

/*  We got a directory -bad */

    if (S_ISDIR(statbuf.st_mode))
    {
	gtk_widget_destroy(GTK_WIDGET(data->sel_box));
	strcpy(data->script_name_sel, "");                 /* I'm going to ignore the request to load a directory */
	gtk_entry_set_text(GTK_ENTRY(data->entry_box), data->script_name_sel);
	g_free(data->script_name_sel);
    }

/* We have a file  - good */

    if (S_ISREG(statbuf.st_mode))
    {
	gtk_entry_set_text(GTK_ENTRY(data->entry_box), data->script_name_sel);
	gtk_widget_destroy(GTK_WIDGET(data->sel_box));
	g_free(data->script_name_sel);
    }
}     

void target_has_been_selected( GtkWidget        *w,
			     script_struct    *data)
{
    data->script_name_sel = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION (data->sel_box)));
    gtk_entry_set_text(GTK_ENTRY(data->entry_box), data->script_name_sel);
    g_free(data->script_name_sel);
    gtk_widget_destroy(GTK_WIDGET(data->sel_box));
}     

void on_auto_save_button_clicked            (GtkButton       *button,
					     script_data     *user_data)
{
    extern GtkWidget *receiver_clist;

    gint          hour_data;
    gint          minute_data;
    gint          year;
    gint          month;
    gint          day;
    gint          num_fields;
    gint          one_start;
    gint          one_end;
    gint          two_start;
    gint          two_end;
    gint          three_start;
    gint          three_end;
    gint          four_start;
    gint          four_end;
    gint          num_selected;
    gint          custom_interval;
    gchar         *script_name;
    gchar         *target_name;
    gchar         how_often[20];
    gchar         after_sent[10];
    gchar         defined_fields[250];
    gchar         *user_id;
    GList         *selected;
    script_text   *active_page;

    
    if ( get_status_script_page() == 1)
    {
	generic_error("\n  You already have a script defined  \n", "G-Page Error");
	return;
    }

    user_data->run_flag = 0;

    if (GTK_TOGGLE_BUTTON (user_data->select_user_radio)->active)
    {
	if (GTK_CLIST(receiver_clist)->selection == NULL)
	{
	    generic_error("\n  You must choose a reciever or group from the main list \n If you wish to define the receiver in the file you must select that under the file options tab  \n", "G-Page Error");
	    return;
	}
	num_selected = 0;
	selected = GTK_CLIST(receiver_clist)->selection;
	while (selected != NULL)
	{
	    gtk_clist_get_text(GTK_CLIST(receiver_clist), GPOINTER_TO_INT(selected->data), 0, &user_id);
	    selected = selected->next;
	    num_selected++;
	}
	if ( num_selected != 1) 
	{
	    generic_error("\n  You must select only one receiver or group  \n", "G-Page Error");
	    return;
	}
    }
    else
    {
	user_id = g_malloc(8);
	strcpy(user_id, "0");	/* That's a zero */
    }
    
    active_page = get_script_memory(); /* We initialize all the fields here too */

    strcpy(active_page->send_to, user_id);

    gtk_calendar_get_date( GTK_CALENDAR(user_data->auto_calender ), &year, &month, &day);

    hour_data =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->hour_spin));
    minute_data = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->minute_spin));

    month++;

    if (GTK_TOGGLE_BUTTON (user_data->am_radio)->active)
    {
	strcpy(active_page->am_pm_radio, "AM");
	if ( hour_data == 12 )
	    hour_data = 0;
    }
    else
    {
	strcpy(active_page->am_pm_radio, "PM");
	if ( hour_data != 12 )
	    hour_data += 12;
    }

    g_snprintf(active_page->year, DATE_LENGTH, "%d", year);
    g_snprintf(active_page->month, DATE_LENGTH, "%d",month);
    g_snprintf(active_page->date, DATE_LENGTH, "%d", day );
    g_snprintf(active_page->hour_spin, DATE_LENGTH, "%d", hour_data);
    g_snprintf(active_page->minute_spin, DATE_LENGTH, "%d", minute_data);

    
    if (GTK_TOGGLE_BUTTON (user_data->once_radio)->active)
	strcpy(active_page->manual_time, "-1");

    else if (GTK_TOGGLE_BUTTON (user_data->hourly_radio)->active)
	strcpy(active_page->manual_time, "60");

    else if (GTK_TOGGLE_BUTTON (user_data->daily_radio)->active)
	strcpy(active_page->manual_time, "1440");

    else 
    {
	custom_interval =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->interval_spin));
	g_snprintf(active_page->manual_time, 128, "%d", custom_interval);
    }
    strcpy(active_page->how_often, how_often);

    script_name = g_strdup(gtk_entry_get_text( (GtkEntry *) user_data->script_entry) );
    if ( strlen(script_name) == 0  )
	 strcat(script_name, "null");

    strcpy(active_page->script_entry, script_name);

    target_name = g_strdup(gtk_entry_get_text( (GtkEntry *) user_data->target_entry) );
    if ( strlen(target_name) == 0 )
    {
	generic_error("\n  You have not selected a file to send  \n", "G-Page Error");
	free_script_memory(active_page);
	return;
    }

    strcpy(active_page->target_entry, target_name);

    if (GTK_TOGGLE_BUTTON (user_data->rename_radio)->active)
	strcpy(after_sent, "rename");
    if (GTK_TOGGLE_BUTTON (user_data->delete_radio)->active)
	strcpy(after_sent, "delete");
    if (GTK_TOGGLE_BUTTON (user_data->ignore_radio)->active)
	strcpy(after_sent, "ignore");

    strcpy(active_page->rename_delete_ignore, after_sent);

    if (GTK_TOGGLE_BUTTON (user_data->select_user_radio)->active)
	strcpy(active_page->manual_user, "select");
    else
	strcpy(active_page->manual_user, "manual");

    if (GTK_TOGGLE_BUTTON (user_data->send_all_radio)->active)
    {
	g_snprintf(defined_fields, 250, " send_all 0\n");
	strcpy(active_page->all_or_parse, "all");
	strcpy(active_page->number_of_fields, "0");
    }
    else
    {
	num_fields =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->fields_spin));
	g_snprintf(defined_fields, 250, " define %d", num_fields);
	strcpy(active_page->all_or_parse, "parse");
	g_snprintf(active_page->number_of_fields, 16,  "%d", num_fields);
	one_start =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->one_start));
	one_end =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->one_end));
	if ( one_end <= one_start )
	{
	    generic_error("\n Illegal field length - Field 1 \n End field must be larger than start field \n", "G-Page Error");
	    free_script_memory(active_page);
	    return;
	}

	g_snprintf(active_page->one_start, 16, "%d", one_start);
	g_snprintf(active_page->one_end, 16, "%d", one_end);

	if ( num_fields > 1 )
	{
	    two_start =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->two_start));
	    two_end =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->two_end));
	    if ( two_end <= two_start )
	    {
		generic_error("\n Illegal field length - Field 2 \n End field must be larger than start field \n", "G-Page Error");
		free_script_memory(active_page);
		return;
	    }
	    
	    g_snprintf(active_page->two_start, 16, "%d", two_start);
	    g_snprintf(active_page->two_end, 16, "%d", two_end);
	}
	if ( num_fields > 2 )
	{
	    three_start =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->three_start));
	    three_end =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->three_end));
	    if ( three_end <= three_start )
	    {
		generic_error("\n Illegal field length - Field 3 \n End field must be larger than start field \n", "G-Page Error");
		free_script_memory(active_page);
		return;
	    }
	    g_snprintf(active_page->three_start, 16, "%d", three_start);
	    g_snprintf(active_page->three_end, 16, "%d", three_end);
	}
	if ( num_fields > 3)
	{
	    four_start =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->four_start));
	    four_end =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data->four_end));
	    if ( four_end <= four_start )
	    {
		generic_error("\n Illegal field length - Field 4 \n End field must be larger than start field \n", "G-Page Error");
		free_script_memory(active_page);
		return;
	    }
	    g_snprintf(active_page->four_start, 16, "%d", four_start);
	    g_snprintf(active_page->four_end, 16, "%d", four_end);
	}
    }
    user_data->run_flag = 1;
    user_data->active_script = active_page;
    gtk_clist_unselect_all( (GtkCList *)receiver_clist);
}


void on_auto_cancel_button_clicked          (GtkButton       *button,
					     script_data     *user_data)
{
    gtk_widget_destroy(user_data->script_page);
}

script_text  *get_script_memory()
{
    script_text    *data;

    data = g_malloc(sizeof(*data));

    data->year = g_malloc(DATE_LENGTH);
    data->month = g_malloc(DATE_LENGTH);
    data->date = g_malloc(DATE_LENGTH);
    data->hour_spin = g_malloc(DATE_LENGTH);
    data->minute_spin = g_malloc(DATE_LENGTH);
    data->am_pm_radio = g_malloc(DATE_LENGTH);
    data->how_often = g_malloc(128);
    data->manual_time = g_malloc(128);
    data->script_entry = g_malloc(MAX_LENGTH);
    data->target_entry = g_malloc(MAX_LENGTH);
    data->rename_delete_ignore = g_malloc(128);
    data->manual_user = g_malloc(128);
    data->all_or_parse = g_malloc(128);

    data->number_of_fields = g_malloc(16);
    data->one_start = g_malloc(16);
    strcpy(data->one_start, "0");
    data->one_end = g_malloc(16);
    strcpy(data->one_end, "0");
    data->two_start = g_malloc(16);
    strcpy(data->two_start, "0");
    data->two_end = g_malloc(16);
    strcpy(data->two_end, "0");
    data->three_start = g_malloc(16);
    strcpy(data->three_start, "0");
    data->three_end = g_malloc(16);
    strcpy(data->three_end, "0");
    data->four_start = g_malloc(16);
    strcpy(data->four_start, "0");
    data->four_end = g_malloc(16);
    strcpy(data->four_end, "0");
    data->send_to = g_malloc(128);
    data->fix_time_flag = 0;

    return (data);

}

void  free_script_memory(script_text *data)
{

    g_free(data->year);
    g_free(data->month);
    g_free(data->date);
    g_free(data->hour_spin);
    g_free(data->minute_spin);
    g_free(data->am_pm_radio);
    g_free(data->how_often);
    g_free(data->manual_time);
    g_free(data->script_entry);
    g_free(data->target_entry);
    g_free(data->rename_delete_ignore);
    g_free(data->manual_user);
    g_free(data->all_or_parse);
    g_free(data->number_of_fields);
    g_free(data->one_start);
    g_free(data->one_end);
    g_free(data->two_start);
    g_free(data->two_end);
    g_free(data->three_start);
    g_free(data->three_end);
    g_free(data->four_start);
    g_free(data->four_end);
    g_free(data->send_to);
    g_free(data);

}

void start_script_page(GtkButton *button, script_data *user_data)
{
    gint               status;

    //fprintf(stderr, "Here's the name: %s\n", user_data->profile_name);

    if ( user_data->run_flag )
    {
	status = get_status_script_page();

	if ( status != 1 )
	{
	    gtk_widget_destroy(user_data->script_page);    
	    activate_script_page(user_data, NEW_SCRIPT, NULL);
	}

    }

}

void reload_previous_script(GtkWidget *w, reload_data *user_data)
{
    extern  int  daemon_flag;    
    gchar        *string;

    if ( get_status_script_page() )
    {
	if ( daemon_flag)
	    log_daemon_data("Error - Autopage is alread active, unable to load another profile\n");
	else
	    generic_error("\n Autopage is already active \n", "G-page Error");
	return;
    }

    string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(user_data->combo_box)->entry));

    activate_script_page(NULL, RELOAD_SCRIPT, string);
    gtk_clist_freeze( GTK_CLIST(user_data->list) ); 
    gtk_clist_clear(GTK_CLIST(user_data->list));
    load_script_list(GTK_CLIST(user_data->list));
    gtk_clist_thaw( GTK_CLIST(user_data->list) ); 
    gtk_widget_hide(user_data->reload_button);
    gtk_widget_show(user_data->deactivate_button);
}

void view_script(GtkWidget *w, reload_data *user_data)
{
    extern pthread_t   autopage_thread;
    gchar              *string;

    string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(user_data->combo_box)->entry));

    activate_script_page(NULL, VIEW_SCRIPT, string);

    gtk_clist_freeze( GTK_CLIST(user_data->list) ); 
    gtk_clist_clear(GTK_CLIST(user_data->list));
    load_script_list(GTK_CLIST(user_data->list));
    gtk_clist_thaw( GTK_CLIST(user_data->list) ); 

    autopage_thread = -1;	/* kludge - we have no autopage thread running  */
    deactivate_script_page();


}

void reload_previous_script_daemon(gchar  *file_name)
{
    activate_script_page(NULL, RELOAD_SCRIPT, file_name);
}


gint activate_script_page(script_data *user_data, gint mode, gchar *profile)
{
    extern script_text *active_script_data;    
    extern int         daemon_flag;

    FILE               *fp;
    pthread_t          thread_id;
    gint               thread_status;
    gchar              *line;
    gchar              *file_name;


    if ( mode == NEW_SCRIPT)
    {			
	if ( strlen(user_data->profile_name) == 0 )
	{
	    if ( (fp = get_file_pointer("profiles/AUTO_SCRIPT.dat", "w")) == NULL)
	    {
		info_msg("Unable to open AUTO_SCRIPT.dat");
		return 0;
	    }
	}
	else
	{
	    file_name = g_strdup_printf("profiles/%s", user_data->profile_name);
	    if ( (fp = get_file_pointer(file_name, "w")) == NULL)
	    {
		info_msg("Unable to open AUTO_SCRIPT.dat");
		return 0;
	    }
	    g_free(file_name);
	    strcpy(user_data->profile_name, "");
	}

	store_profile_and_save(active_script_data, user_data, fp);

	fclose(fp);

	free_script_memory(user_data->active_script);
	g_free(user_data);
    }
    else			/* We are loading a profile from a saved file */
    {

	file_name = g_strdup_printf("profiles/%s", profile);

	if ( (fp = get_file_pointer(file_name, "r")) == NULL)
	{
	    info_msg("Unable to open defined autoscript profile: read");
	    g_free(file_name);
	    return 0;
	}

	line = g_malloc(MAX_LENGTH);
	if ( (fgets(line, MAX_LENGTH, fp)) == NULL )
	{
	    if (daemon_flag)
	    {
		log_daemon_data("Error - %s is corrupt or not a valid profile\n", file_name);
		info_msg("You do not have a valid script to load");
	    }
	    else
		generic_error("  You do not have a valid profile to load  ", "G-Page Error");

	    g_free(file_name);
	    g_free(line);
	    return 0;
	}


	sscanf(line, "%s %s %s %s %s %s %s", active_script_data->year, active_script_data->month,
	       active_script_data->date, active_script_data->hour_spin,active_script_data->minute_spin,
	       active_script_data->am_pm_radio,active_script_data->manual_time);

	//active_script_data->fix_time_flag = 1;
	active_script_data->fix_time_flag = 0;  /* leave this at zero for now, it's not very functional */
/*
	fprintf(stderr, "YEAR: %s\n", active_script_data->year);
	fprintf(stderr, "month: %s\n", active_script_data->month);
	fprintf(stderr, "date: %s\n", active_script_data->date);
	fprintf(stderr, "man time: %s\n", active_script_data->manual_time);
*/
	if ( (fgets(active_script_data->script_entry, MAX_LENGTH, fp)) == NULL )
	{
	    if (daemon_flag)
	    {
		log_daemon_data("Error - %s is corrupt or not a valid profile\n", file_name);
		info_msg("You do not have a valid script to load");
	    }
	    else
		generic_error("  You do not have a valid profile to load  ", "G-Page Error");

	    g_free(file_name);
	    g_free(line);
	    return 0;
	}

	active_script_data->script_entry[strlen(active_script_data->script_entry) -1] = '\0';

	if ( (fgets(active_script_data->target_entry, MAX_LENGTH, fp)) == NULL )
	{
	    if (daemon_flag)
	    {
		log_daemon_data("Error - %s is corrupt or not a valid profile\n", file_name);
		info_msg("You do not have a valid script to load");
	    }
	    else
		generic_error("  You do not have a valid profile to load  ", "G-Page Error");

	    g_free(file_name);
	    g_free(line);
	    return 0;
	}

	active_script_data->target_entry[strlen(active_script_data->target_entry) -1] = '\0';

	if ( (fgets(line, MAX_LENGTH, fp)) == NULL )
	{
	    if (daemon_flag)
	    {
		log_daemon_data("Error - %s is corrupt or not a valid profile\n", file_name);
		info_msg("You do not have a valid script to load");
	    }
	    else
		generic_error("  You do not have a valid profile to load  ", "G-Page Error");

	    g_free(file_name);
	    g_free(line);
	    return 0;
	}

	sscanf(line, "%s %s %s %s %s %s %s %s %s %s %s %s %s", active_script_data->rename_delete_ignore,active_script_data->manual_user,
	       active_script_data->all_or_parse,
	       active_script_data->number_of_fields, active_script_data->one_start, active_script_data->one_end,
	       active_script_data->two_start, active_script_data->two_end, active_script_data->three_start,
	       active_script_data->three_end, active_script_data->four_start, active_script_data->four_end,
	       active_script_data->send_to);
	fclose(fp);
	g_free(line);
	g_free(file_name);
    } /* End of the main if-then-else */


    active_script_data->run_status = 1;

    if ( mode == VIEW_SCRIPT )
	return 1;

    thread_status = pthread_create(&thread_id, NULL, run_auto_page, NULL);

    register_autopage_thread(thread_id);
    //thread_status = pthread_create(&thread_id, NULL, run_auto_page, NULL);

    if ( daemon_flag )
	pthread_join(thread_id, NULL);

    return 0;
}

void deactivate_script(GtkWidget *w, reload_data *user_data)
{
    if ( get_status_script_page() )
    {
	deactivate_script_page();
	gtk_clist_clear(GTK_CLIST(user_data->list));
	load_script_list(GTK_CLIST(user_data->list));
	gtk_widget_hide(user_data->deactivate_button);
	gtk_widget_show(user_data->reload_button);
    }

}


gint deactivate_script_page(void)
{
    extern script_text *active_script_data;
    extern pthread_t   autopage_thread;

    active_script_data->run_status = 0;

    if ( autopage_thread != -1 )
	kill_autopage_thread(autopage_thread);

    return 0;
}

gint get_status_script_page(void)
{
    extern script_text *active_script_data;

    return (active_script_data->run_status);
}

void init_script_page_data(void)
{
    extern script_text *active_script_data;

    active_script_data = get_script_memory();
    active_script_data->run_status = 0;

}


void *run_auto_page(void *arg)
{
    extern script_text *active_script_data;
    extern int         daemon_flag;

    GSList        *head = NULL;
    char          *my_argv[2];
    gint          mode;
    gint          ready;
    gint          status;
    gint          interval;
    gint          polling_interval;
    gint          loop_status;
    gint          delay;
    active_data   *current_data;
    time_t        time_start, time_end;

    current_data = init_data();

    delay = get_delay();	/* This is how long we wait after executing a script before we look for a file */

    if ( daemon_flag )
    {
	if ( current_data->select_receiver_method == FROM_LIST )
	{
	    log_daemon_data("Fatal error: You must select the send-user-in-file mode when running as a daemon\n");
	    return (NULL);
	}
    }
    else
	status = pthread_detach( pthread_self() );	

/*
 *   interval is the length of time between file locate attempts, or -1 if
 *   this is a one time event 
 */

    interval = get_interval(active_script_data->manual_time);    

/*
 *  This is a temporary kludge. If you set this to 0 gpage will try to sync the time
 *  with the same minutes after the hour that a profile was originally launched Leave
 *  this set to one for now
 */

    polling_interval = 1;
				

    mode = get_mode(active_script_data->script_entry); /* mode is either HAVE_SCRIPT or NO_SCRIPT */


    while (1)
    {
	time_start = time(&time_start);
	loop_status = get_status_script_page();
	if ( loop_status == 0 )	/* Someone has deactivated the current script */
	{
	    debug_data("Someone has deactivated the script\n");
	    break;
	}

	ready = send_now(polling_interval); /* ready is either WAIT or GO  */
	if ( ready == GO )
	{
	    if ( mode == HAVE_SCRIPT ) 
	    {
		if ( fork() == 0 )	/* Child process */
		{		    
		    my_argv[0]= active_script_data->script_entry;
		    my_argv[1]= 0;
		    if ( (execv(active_script_data->script_entry, my_argv)) == -1 )
		    {
			debug_data("Error launching script: %s", active_script_data->script_entry);
		    }
		}
		else			/* Parent process */
		{
		    my_sleep(delay);
		}
	    }
	    
	    process_target_file(active_script_data->target_entry, current_data);
	    debug_data("I got back from process target file\n");
	    
	    if ( current_data->list_of_raw_data == NULL )
	    {
		debug_data("Couldn't find a file to send\n");
	    }
	    else
	    {
		debug_data("We found something to send\n");
		head = current_data->list_of_files; /* We need to save our place  */
		while ( current_data->list_of_raw_data )
		{
		    handle_page(current_data); 
		    g_free( (gchar *) current_data->list_of_raw_data->data );
		    current_data->list_of_raw_data = g_slist_next(current_data->list_of_raw_data);
		    current_data->list_of_files = g_slist_next(current_data->list_of_files);
		}
		g_slist_free(current_data->list_of_raw_data);
		current_data->list_of_raw_data = NULL;

		/* Perform file cleanup here */
		handle_file_cleanup(active_script_data->rename_delete_ignore, head);
	    }

	    /* We're done sending pages - check to see if we are done or if we sleep and continue looping */
	    if ( interval == -1 ) 
	    {
		break;
	    }
	    else
	    {
		loop_status = get_status_script_page();
		if ( loop_status == 0 )
		    return(NULL);
		
		time_end = time(&time_end);
		
		if ( active_script_data->fix_time_flag )
		    my_sleep(30);
		else
		    my_sleep(interval - ( (int)(time_end - time_start) ));
		continue;
	    }

	}
	else			/*  go == WAIT  */
	    my_sleep(10);
    }
    deactivate_script_page();
    free_object_memory(current_data);
    return(NULL);
}


static void process_target_file(gchar *file_name, active_data *data_object)
{
    enum {
	MAX_RETRIES = 59
    };

    extern script_text *active_script_data;
    struct stat   statbuf;
    
    gchar         *filename;
    gint          status;
    gint          no_file;
    gint          attempts;
    gint          retries;
    gint          loop_status;
    gint          glob;
    gint          emergency_exit;

    debug_data("Looking for a file in process_target_file\n");
    no_file = 1;
    glob = 0;
    attempts = 0;
    filename = g_strdup(file_name);

    retries = get_retries();
    
    emergency_exit = MAX_RETRIES; /* This should not be needed but it will stop us from looping forver */

    while ( no_file  && emergency_exit )
    {
	emergency_exit--;

	debug_data("At the top of the no_file while loop\n");
	loop_status = get_status_script_page();
	if ( loop_status == 0 )	/* Someone has deactivated the current script */
	{
	    debug_data("someone deactivated the loop\n");
	    g_free(filename);
	    return;
	}

	status = stat(filename, &statbuf);
	if ( !(S_ISREG(statbuf.st_mode)) ) /* The file doesn't exist - try to glob */
	{
	    debug_data("No file, I'm globbing\n");
	    if ( attempts >= retries )
	    {
		debug_data("I'm out of retries\n");
		g_free(filename);
		return;
	    }
	    else
	    {
		debug_data("I still have a retry - heading into get list of targets\n");
		get_list_of_targets(filename, data_object);
		    
		debug_data("I got back from get list of targets\n");
		if ( data_object->list_of_files == NULL )
		    sleep(1);
		else
		{
		    no_file = 0;
		    glob = 1;
		    g_free(filename); /* We can only free it if we are globbing, otherwise the pointer gets
				         copied into the linked list and we'll free it later */
		}
	    }
	    attempts++;
	}
	else
	{
	    debug_data("We found a file, adding to the list\n");
	    data_object->list_of_files = g_slist_append(data_object->list_of_files, (gpointer) filename);
	    no_file = 0;
	}
    } /* This is the end of the while loop */
    debug_data("I'm out of the while loop - time to process files, if we found some\n");

/*  Run through  all the globed files */
    if ( data_object->list_of_files == NULL )	/* We did not find a file */
    {
	debug_data("Targets = null - no file\n");
	if ( !glob )
	    g_free(filename);
	return;
    }

    read_and_load_data_from_file(data_object->list_of_files, data_object);

    if ( data_object->list_of_raw_data == NULL )
    {
	debug_data("I got a NULL after loading the data from the target file\n");
	if ( !glob )
	    g_free(filename);
	return;
    }

    if ( !strcmp(active_script_data->all_or_parse, "parse") )
	data_object->list_of_raw_data = get_parsed_data(data_object->list_of_raw_data);
	
    debug_data("I'm done returning from process target file\n");
    return;
}

static gchar *parse_buffer(gchar  *buffer)
{
    extern script_text *active_script_data;
    gchar              *final_buffer;
    gint               fields;
    gint               x;
    gint               offset;
    gint               start, end, index, length;

    start = end = 0;
    final_buffer = g_malloc(MAX_LENGTH + MAX_FIELD_LENGTH);

    offset = 0;
    index = 0;

    if ( ! strcmp(active_script_data->manual_user, "manual") )
    {
	while ( *(buffer + offset) != '\n' )
	{
	    *(final_buffer + index++) = *(buffer + offset++);
	}
	*(final_buffer + index++) = *(buffer + offset++);
    }
	    
    length = strlen(buffer);

    fields = atoi(active_script_data->number_of_fields);

    for( x = 0; x < fields; x++)
    {
	switch (x) {
	case 0:
	    start = atoi(active_script_data->one_start) + offset;
	    end = atoi(active_script_data->one_end) + offset;
	    break;
	case 1:
	    start = atoi(active_script_data->two_start) + offset;
	    end = atoi(active_script_data->two_end) + offset;
	    break;
	case 2:
	    start = atoi(active_script_data->three_start) + offset;
	    end = atoi(active_script_data->three_end) + offset;
	    break;
	case 3:
	    start = atoi(active_script_data->four_start) + offset;
	    end = atoi(active_script_data->four_end) + offset;
	    break;
	}
	
	if ( end > length )
	{
	    debug_data("Error maximun characters exceeded in file - truncating message\n");	    
	    end = length;
	}
	while( start <= end )
	    *(final_buffer + index++) = *(buffer + start++);
	
    }
    *(final_buffer + index) = '\0';

    return (final_buffer);
	    
}

static void handle_page(active_data *page_data)
{
    extern script_text *active_script_data;

    manual_page_data   *manual_data;
    uid_list           *list;
    gchar              *send_data;
    gchar              *t_uid;
    gchar              *message;
    gchar              *temp_buffer;
    gint               good_id;


/*
    If it's a manual page we are going to parse the receiver/group name from the 
    file. It will be the first line in the file treminated by a newline. The actual
    page message is everything else until EOF
*/
    good_id = 0;

    if ( page_data->select_receiver_method == MANUAL )
    {
	list = get_uid_node();

	manual_data = parse_data_for_manual_page(page_data->list_of_raw_data->data);
	if (manual_data->error)
	{
	    g_free(manual_data); /* manual_data->name and manual_data->message don't get allocated
				    if we have an error */
	    return;
	}

	t_uid = get_id_from_receiver_name(manual_data->name);

	if ( strlen(t_uid) )	/* This is a empty string if we didn't match the name to an ID */
	{
	    strcpy(list->uid, t_uid);
	    list->next = get_uid_node();
	    good_id = 1;
	    g_free(t_uid);
	}
	else   /* no match in the receiver database - look in the group database */
	{
	    g_free(t_uid);
	    t_uid = get_id_from_group_name(manual_data->name);
	    if ( strlen(t_uid) )
	    {
		parse_group(t_uid, list);
		good_id = 1;
	    }
	    g_free(t_uid);
	}

	if (good_id)
	{
	    message = g_strdup(manual_data->message);
	    connect_to_server(list, message, "none", NULL, AUTO, NULL, -1);
	}
	else
	{
	    
	    if ( page_data->create_error_file )
		write_error_file(page_data);
	    
	    g_free(list->uid);
	    g_free(list->next);
	    g_free(list);
	    debug_data("Error matching receiver from file with database - unable to send auto page\n");
	}
	free_the_memory(manual_data->name, manual_data->message, manual_data);	    
    }
    else     	/* This else matches the if (manual) statement. If we get here the user has selected a receiver from the list*/
    {
	send_data = g_strdup((gchar *) page_data->list_of_raw_data->data);

	if ( strlen (send_data) > MAX_LENGTH )
	{
	    temp_buffer = g_strndup(send_data, MAX_PAGE_LENGTH);
	    g_free(send_data);
	    send_data = g_strdup(temp_buffer);
	    g_free(temp_buffer);
	}

	if ( active_script_data->send_to[0] == 'G' )
	{
	    list = get_uid_node();
	    parse_group(active_script_data->send_to, list);
	    send_data = clean_text(send_data);

	    connect_to_server(list, send_data, "none", NULL, AUTO, NULL, -1);
	}
	else
	{
	    list = get_uid_node();    
	    strcpy(list->uid, active_script_data->send_to);
	    list->next = get_uid_node();
	    send_data = clean_text(send_data);

	    connect_to_server(list, send_data, "none", NULL, AUTO, NULL, -1);
	}
    }
}



void on_show_minutes_toggled(GtkWidget *w, manual_widgets *user_data)
{
    gtk_widget_show(user_data->label);
    gtk_widget_show(user_data->spinbox);
}

void on_hide_minutes_toggled(GtkWidget *w, manual_widgets *user_data)
{
    gtk_widget_hide(user_data->label);
    gtk_widget_hide(user_data->spinbox);
}


static void free_the_memory(gchar *four, gchar *five, manual_page_data *data)
{
    g_free(four);
    g_free(five);
    g_free(data);
}

static gint send_now(gint interval)
{

/*
 *   GO and WAIT are enums defined at the top of this file
 */

    extern script_text *active_script_data;
    gint               ready;
    time_t             time_val;
    struct  tm         *t_struct;

    time_val = time(&time_val);
    t_struct = localtime(&time_val);
    ready = WAIT;

/*
 *  If interval is 1 then we have a custom polling interval and we will execute the script
 *  right away, otherwise we will wait until the correct hour/minute to start looking 
 */

    if ( interval )
	active_script_data->fix_time_flag = 0;

    //fprintf(stderr, "The fix time flag is: %d\n",active_script_data->fix_time_flag); 
    if ( active_script_data->fix_time_flag != 0 )
    {
	if ( (t_struct->tm_min) == (atoi(active_script_data->minute_spin)) )
	    active_script_data->fix_time_flag = 0;

	else if ( (t_struct->tm_min) == ( (atoi(active_script_data->minute_spin)) + 1) )
	    active_script_data->fix_time_flag = 0;

	else
	    return(WAIT);
    }

    if ( (t_struct->tm_year + 1900) < (atoi(active_script_data->year)) )
	ready = WAIT;
    else if ( (t_struct->tm_year + 1900) > (atoi(active_script_data->year)) )
	ready = GO;
    
    else if ( (t_struct->tm_mon + 1) < (atoi(active_script_data->month)) )
	ready = WAIT;
    else if ( (t_struct->tm_mon + 1) > (atoi(active_script_data->month)) )
	ready = GO;
    
    else if ( (t_struct->tm_mday) < (atoi(active_script_data->date)) )
	ready = WAIT;
    else if ( (t_struct->tm_mday) > (atoi(active_script_data->date)) )
	ready = GO;
    
    else if ( (t_struct->tm_hour) < (atoi(active_script_data->hour_spin)) )
	ready = WAIT;
    else if ( (t_struct->tm_hour) > (atoi(active_script_data->hour_spin)) )
	ready = GO;
    
    else if ( (t_struct->tm_min) < (atoi(active_script_data->minute_spin)) )
	ready = WAIT;
    else
	ready = GO;
    

    return (ready);
}

static void get_list_of_targets(gchar  *filename, active_data *data)
{
    FILE          *pp;
    struct stat   statbuf;

    gint          x;
    gint          status;
    gchar         *goodname, *buffer;
    gchar         *save_filename;
    gchar         temp;
    gchar         *command = "ls ";

    debug_data("I'm in get list of targets\n");

    if ( ( strlen(command) + strlen(filename) ) >= MAX_LENGTH )
    {
	debug_data("Illegal filename length - quiting\n");
	return;
    }

    buffer = g_strdup_printf("%s%s", command, filename);
    debug_data("I'm going to do the popen command\n");
    pp = popen(buffer, "r");
    debug_data("I just did the popen command\n");
    if ( pp == NULL )
    {
	debug_data("Error searching for files - pp = NULL: %s\n", buffer);
	g_free(buffer);
	return;
    }	
    g_free(buffer);

    debug_data("I'm going to read the data from the pipe\n");    
    goodname = g_malloc(MAX_LENGTH);
    x = 0;
    while ( (temp = fgetc(pp)) != EOF )
    {
	putchar(temp);
	if (temp == '\n')
	{
	    *(goodname + x++) = '\0';
				/* Lets see if we actually got a file here */
	    if ( strlen(goodname) == 0 )
	    {
		g_free(goodname);
		return;
	    }
	    else
	    {		/* We have a filename, lets make sure it's a regular file */
		debug_data("I have a filename - just read from the pipe\n");
		memset(&statbuf, '\0', sizeof(statbuf));
		status = stat(goodname, &statbuf);
		if ( (int)statbuf.st_size == 0 )
		{
		    debug_data("Got a zero file length\n");
		    sleep(1);
		    g_free(goodname);
		    return;
		}
		if ( (S_ISREG(statbuf.st_mode)) ) /* If we get here we have a valid filename - load it into the linked list */
		{
		    debug_data("I'm going to load the file I found into the linked list\n");
		    save_filename = g_strdup(goodname);
		    data->list_of_files = g_slist_append(data->list_of_files, (gpointer) save_filename);
		    memset(goodname, '\0', MAX_LENGTH);
		    x = 0;
		    continue;
		}
		else
		{
		    x = 0;
		}
	    }	    
	}
	else
	    *(goodname + x++) = temp;
    debug_data("I'm at the end of the while loop after reading from the pipe\n");
    } /* This is the end of the while loop which reads from the pipe */
    pclose(pp);

    g_free(goodname);
    debug_data("I'm returning from get list of targets\n");
    return;
}


static void delete_target_files(GSList *targets)
{
    while ( targets )
    {
	if ( (unlink((gchar *)targets->data)) == -1 )
	    debug_data("Error deleting file\n");
	g_free((gchar *)targets->data);
	targets = g_slist_next(targets);
    }
}

static void rename_target_files(GSList *targets)
{
    gchar         *tfile;
    time_t        time_val;
    gchar         *temp, *file_name, *temp_path;
    gint          len,x,y;

    while ( targets )
    {
	temp = g_strdup((gchar *)targets->data);
	file_name = g_malloc(strlen(temp) + 2);
	temp_path = g_malloc(strlen(temp));

	len = strlen(temp);
	x = 0;
	y = 0;

	while( len > 0 )
	{
	    if (temp[--len] != '/')
		file_name[x++] = temp[len];
	    else
	    {
		while( len >= 0 )
		    temp_path[y++] = temp[len--];
		break;
	    }
	}

	file_name[x] = '\0';
	temp_path[y] = '\0';

	g_strreverse(file_name);
	g_strreverse(temp_path);

	time_val = time(&time_val);

	tfile = g_strdup_printf("%sr.%s.%d", temp_path, file_name, (int)time_val);

	if ( (rename((gchar *)targets->data, tfile)) == -1)
	    info_msg("Error renaming file: %s", (gchar *)targets->data);
	//perror("error renaming file ");

	g_free(tfile);
	g_free(temp);
	g_free(file_name);
	g_free(temp_path);
	g_free((gchar *)targets->data);
	targets = g_slist_next(targets);	    
    }
}



/*
 *   If we found a file(s) we call this function. Here we simply
 *   open the file(s) we found and read the data, storing it into
 *   linked list of strings containing the data. We return the linked list
 */

static void read_and_load_data_from_file(GSList *targets, active_data *data)
{
    FILE     *fp;
    GSList   *head = NULL;
    gchar    *target;
    gint     x;
    gint     max_character_count = 0;
    gchar    temp;

    head = targets;
    debug_data("I'm in read and load data from file\n");
    while ( targets )
    {
	debug_data("Top of the while loop to process the targets\n");
	if ( (gchar *) targets->data == NULL ) /* This should not happen */
	{
	    targets = g_slist_next(targets);
	    continue;
	}

	if ( (fp = fopen((gchar *)targets->data, "r")) == NULL)
	{
	    info_msg("Error opening target file: %s", (gchar *)targets->data);
	    targets = head;
	    //perror("Error opening target file ");
	    return;
	}	

	if ( data->select_receiver_method == FROM_LIST )
	    max_character_count = MAX_PAGE_LENGTH;
	else
	    max_character_count = MAX_PAGE_LENGTH + MAX_FIELD_LENGTH;

	target = g_malloc(max_character_count);
	x = 0;
	
	while ( (temp = fgetc(fp)) != EOF )
	{
	    *(target + x++) = temp;
	    if ( x == ( max_character_count -1 ) ) /* Max character count met */
		break;
	}	
	fclose(fp);
	*(target + x++) = '\0';
	data->list_of_raw_data = g_slist_append(data->list_of_raw_data, (gpointer)target);
	targets = g_slist_next(targets);
    }
    debug_data("I'm returning after processing routine\n");
    targets = head;
    return;
}


static void my_sleep(gint length)
{
    time_t        start, end;
    time_t        timeval;
    gint          diff;

    time(&timeval);
    
    debug_data("I got into mysleep\n");
    start = time(&start);
    while (1)
    {
	sleep(length);
	if ( errno == EINTR )
	{
	    end = time(&end);
	    diff = (int)end - (int)start;
	    if ( length > diff )
		continue;
	    else
	    {
		time(&timeval);
		return;
	    }
	}
	else
	{
	    time(&timeval);
	    return;
	}
    }
}

static gint get_delay()
{
    gchar *resource_field;
    gint  delay;

    if ( (resource_field = get_resource_data("CONFIG.dat", "script_delay", MAX_FIELD_LENGTH)) )
    {
	delay = atoi(resource_field);
	g_free(resource_field);
    }
    else			/* Error getting data - load a safe default */
    {
	info_msg("Error reading script_delay in CONFIG.dat- using 10\n");
	delay = 10;
    }
    
    return (delay);
}


void show_profile_save_dialog(GtkWidget *w, script_data *data)
{
    GtkWidget  *profile_dialog;
    
    if ( data->run_flag )
    {
	profile_dialog = create_save_profile_dialog(data);
	gtk_widget_show(profile_dialog);
    }

}

GtkWidget *create_save_profile_dialog (script_data *data)
{
  GtkWidget *save_profile_dialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox7;
  GtkWidget *file_save_label;
  GtkWidget *profile_save_entry;
  GtkWidget *dialog_action_area1;
  GtkWidget *hbuttonbox4;
  GtkWidget *profile_save_button;
  GtkWidget *profile_cancel_button;

  save_profile_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (save_profile_dialog), "save_profile_dialog", save_profile_dialog);
  gtk_window_set_title (GTK_WINDOW (save_profile_dialog), "Save Profile Dialog");
  gtk_window_set_policy (GTK_WINDOW (save_profile_dialog), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (save_profile_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (save_profile_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox7);
  gtk_object_set_data_full (GTK_OBJECT (save_profile_dialog), "vbox7", vbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox7);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox7, TRUE, TRUE, 0);

  file_save_label = gtk_label_new ("Save Profile as:");
  gtk_widget_ref (file_save_label);
  gtk_object_set_data_full (GTK_OBJECT (save_profile_dialog), "file_save_label", file_save_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (file_save_label);
  gtk_box_pack_start (GTK_BOX (vbox7), file_save_label, TRUE, TRUE, 10);

  profile_save_entry = gtk_entry_new_with_max_length (32);
  gtk_widget_ref (profile_save_entry);
  gtk_object_set_data_full (GTK_OBJECT (save_profile_dialog), "profile_save_entry", profile_save_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (profile_save_entry);
  gtk_box_pack_end (GTK_BOX (vbox7), profile_save_entry, FALSE, FALSE, 6);

  dialog_action_area1 = GTK_DIALOG (save_profile_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (save_profile_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbuttonbox4 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox4);
  gtk_object_set_data_full (GTK_OBJECT (save_profile_dialog), "hbuttonbox4", hbuttonbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox4);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox4, TRUE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox4), GTK_BUTTONBOX_SPREAD);

  profile_save_button = gtk_button_new_with_label ("Save");
  gtk_widget_ref (profile_save_button);
  gtk_object_set_data_full (GTK_OBJECT (save_profile_dialog), "profile_save_button", profile_save_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (profile_save_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox4), profile_save_button);
  GTK_WIDGET_SET_FLAGS (profile_save_button, GTK_CAN_DEFAULT);

  profile_cancel_button = gtk_button_new_with_label ("Cancel");
  gtk_widget_ref (profile_cancel_button);
  gtk_object_set_data_full (GTK_OBJECT (save_profile_dialog), "profile_cancel_button", profile_cancel_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (profile_cancel_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox4), profile_cancel_button);
  GTK_WIDGET_SET_FLAGS (profile_cancel_button, GTK_CAN_DEFAULT);

  data->profile_save_widget = profile_save_entry;
  data->profile_save_dialog = save_profile_dialog;

  gtk_signal_connect (GTK_OBJECT (profile_save_button), "clicked",
                      GTK_SIGNAL_FUNC (on_profile_save_button_clicked),
                      data);

  gtk_signal_connect (GTK_OBJECT (profile_cancel_button), "clicked",
                      GTK_SIGNAL_FUNC (on_profile_cancel_button_clicked),
                      save_profile_dialog);


  return save_profile_dialog;
}

static void on_profile_save_button_clicked(GtkWidget *w, script_data *data)
{
    gchar  *name;

    name = g_strdup(gtk_entry_get_text(GTK_ENTRY(data->profile_save_widget)));
    strcpy(data->profile_name, name);
    g_free(name);

    start_script_page(GTK_BUTTON(w), data);
    gtk_widget_destroy(data->profile_save_dialog);
    
}

static void on_profile_cancel_button_clicked(GtkWidget *w, GtkWidget *dialog_box)
{
    gtk_widget_destroy(dialog_box);
}

static gint get_interval(gchar *manual_time_string)
{
    gchar *resource_field;
    gint  interval;

    interval = atoi(manual_time_string);

/*
 *   If interval is zero we are polling for files in a user defined interval
 *   If interval is -1 we are only executing once
 */

    if ( interval == 0 )
    {
	if ( (resource_field = get_resource_data("CONFIG.dat", "polling_interval", MAX_FIELD_LENGTH)) )
	{
	    interval = atoi(resource_field);
	    g_free(resource_field);
	}
	else			/* Error getting data from file- load a safe default */
	{
	    info_msg("Error reading polling_interval in CONFIG.dat- using 10\n");
	    interval = 10;
	}    	
    }
    else if ( interval != -1 )
	interval = interval * 60;


    return (interval);
}

static gint get_retries()
{
    enum {
	MAX_RETRIES = 59
    };

    gchar *resource_field;
    gint  retries;


    if ( (resource_field = get_resource_data("CONFIG.dat", "locate_retries", MAX_FIELD_LENGTH)) )
    {
	retries = atoi(resource_field);
	g_free(resource_field);
    }
    else			/* Error getting data - load a safe default */
    {
	info_msg("Error reading locate_retries in CONFIG.dat- using 1\n");
	retries = 1;
    }
    

    if ( ( retries < 1 ) || ( retries > MAX_RETRIES ) )
	retries = 1;

    return (retries);
}

static gint get_mode(gchar *user_defined_script)
{
    struct stat   statbuf;
    gint          status;
    gint          mode;

    status = stat(user_defined_script, &statbuf);
    if ( (S_ISREG(statbuf.st_mode)) )
    {
	mode = HAVE_SCRIPT;
	if ( access(user_defined_script, X_OK) < 0 )
	{
	    mode = NO_SCRIPT;	/* We don't have permission to run this script */
	    debug_data("Error: We don't have permission to execute the script\n");	    
	}
    }
    else
	mode = NO_SCRIPT;

    return(mode);
}

/*
static void send_a_page(GSList *page_list, uid_list *active_list, gint manual)
{
    extern script_text *active_script_data;

    if ( manual == 0 )	       
    {
	active_list = get_uid_node();    
	strcpy(active_list->uid, active_script_data->send_to);
	active_list->next = get_uid_node();
    }
    
    handle_page(page_list, active_list, manual);
    g_free( (gchar *) page_list->data );

    return;
}
*/

static void handle_file_cleanup(gchar *action, GSList *targets)
{

    if ( !strcmp(action, "delete") )
	delete_target_files(targets);

    else if ( !strcmp(action, "rename") )
	rename_target_files(targets);

    else			/* The user chose the ignore option */
    {
	debug_data("I'm going to free the linked list of filenames\n");
	while ( targets )
	{
	    g_free((gchar *)targets->data);
	    targets = g_slist_next(targets);	    
	}	    
    }

    g_slist_free(targets);
    targets = NULL;
}

manual_page_data *parse_data_for_manual_page(gchar *raw_data)
{
    manual_page_data   *manual_data;
    gint               x, y, length;
    gchar              *name, *data;
    
    manual_data = g_malloc(sizeof(*manual_data));
    manual_data->error = 0;
    name = g_malloc(MAX_LENGTH);
    data = g_malloc(MAX_LENGTH);

    length = strlen(raw_data);
    x = 0;
    y = 0;
    while ( *( (gchar *) raw_data  + x) != '\n' )
    {
	if ( x >= (MAX_FIELD_LENGTH - 1) )
	{
	    debug_data("Error parsing receiver name from file in auto-page\n");
	    manual_data->error = 1;
	    g_free(name);
	    g_free(data);
	    return (manual_data);
	}
	*(name + x) = *( raw_data + x);
	x++;
    }
    *(name + x) = '\0';

    x++;			/* Skip the newline */
    while ( x < length )
    {
	*(data + y) = *( raw_data + x);
	x++;
	y++;
	if ( y == MAX_PAGE_LENGTH )
	    break;
    }
    *(data + y) = '\0';
    data = clean_text(data);

    manual_data->name = g_strdup(name);
    manual_data->message = g_strdup(data);
    g_free(name);
    g_free(data);
    return (manual_data);
}


/*
 *   This function returns the receiver ID if it exists
 *   otherwise it returns a zero length string
 */

gchar *get_id_from_receiver_name(gchar *name)
{
    FILE    *fp;
    gchar   *line, *temp_name, *id;
    gint    good;

    temp_name = g_malloc(MAX_FIELD_LENGTH);
    id = g_malloc(MAX_FIELD_LENGTH);
    good = 0;

    if ( (fp = get_file_pointer("RECEIVER.dat", "r")) == NULL)
    {
	debug_data("Error opening RECEIVER.dat - unable to lookup user in auto-page\n");
	good = 0;
    }
    else
    {
	while ( (line = read_file_by_line(fp)) != NULL)
	{
	    get_name_id(id, temp_name, line);

	    if ( !strcmp(temp_name, name) )
	    {
		good = 1;
		g_free(line);
		break;
	    }
	    g_free(line);
	}
	fclose(fp);
    }
	
    g_free(temp_name);

    if ( good )
	return(id);
    else
    {
	strcpy(id, "");
	return(id);
    }
}

/*
 *   This function returns the group ID if it exists
 *   otherwise it returns a zero length string
 */

gchar *get_id_from_group_name(gchar *name)
{
    FILE    *fp;
    gchar   *line, *temp_name, *id;
    gint    good;

    temp_name = g_malloc(MAX_FIELD_LENGTH);
    id = g_malloc(MAX_FIELD_LENGTH);
    good = 0;

    if ( (fp = get_file_pointer("GROUP.dat", "r")) == NULL)
    {
	debug_data("Error opening GROUP.dat - unable to lookup group in auto-page\n");
	good = 0;
    }
    else
    {
	while ( (line = read_big_file_by_line(fp, MAX_LENGTH)) != NULL)
	{
	    get_name_id(id, temp_name, line);
	    if ( !strcmp(temp_name, name) )
	    {
		good = 1;
		g_free(line);
		break;
	    }
	    g_free(line);
	}
	fclose(fp);
    }
	
    g_free(temp_name);

    if ( good )
	return(id);
    else
    {
	strcpy(id, "");
	return(id);
    }

}


/*
 *  We are doing two things here:
 *  1) we need to save the data entered by the user into a structure that will 
 *     be used by all the other autopage routines for processing. (hence the strcpy)
 *  2) We need to save the data to disk.
 */

static void store_profile_and_save(script_text *text, script_data *data, FILE *fp)
{

    strcpy(text->year,  data->active_script->year);
    fputs(text->year, fp);
    fputs(" ", fp);

    strcpy(text->month, data->active_script->month);
    fputs(text->month, fp);
    fputs(" ", fp);
    
    strcpy(text->date,  data->active_script->date);
    fputs(text->date, fp);
    fputs(" ", fp);
    
    strcpy(text->hour_spin, data->active_script->hour_spin);
    fputs(text->hour_spin, fp);
    fputs(" ", fp);
    
    strcpy(text->minute_spin,  data->active_script->minute_spin);
    fputs(text->minute_spin, fp);
    fputs(" ", fp);
    
    strcpy(text->am_pm_radio,  data->active_script->am_pm_radio);
    fputs(text->am_pm_radio, fp);
    fputs(" ", fp);
    
    strcpy(text->manual_time,  data->active_script->manual_time);
    fputs(text->manual_time, fp);
    fputs("\n", fp);
    
    strcpy(text->script_entry,  data->active_script->script_entry);
    fputs(text->script_entry, fp);
    fputs("\n", fp);
    
    strcpy(text->target_entry,  data->active_script->target_entry);
    fputs(text->target_entry, fp);
    fputs("\n", fp);
    
    strcpy(text->rename_delete_ignore,  data->active_script->rename_delete_ignore);
    fputs(text->rename_delete_ignore, fp);
    fputs(" ", fp);
    
    strcpy(text->manual_user, data->active_script->manual_user);
    fputs(text->manual_user, fp);
    fputs(" ", fp);
    
    strcpy(text->all_or_parse, data->active_script->all_or_parse);
    fputs(text->all_or_parse, fp);
    fputs(" ", fp);
    
    strcpy(text->number_of_fields, data->active_script->number_of_fields);
    fputs(text->number_of_fields, fp);
    fputs(" ", fp);
    
    strcpy(text->one_start,    data->active_script->one_start);
    fputs(text->one_start, fp);
    fputs(" ", fp);
    
    strcpy(text->one_end,      data->active_script->one_end);
    fputs(text->one_end, fp);
    fputs(" ", fp);
    
    strcpy(text->two_start,    data->active_script->two_start);
    fputs(text->two_start, fp);
    fputs(" ", fp);
    
    strcpy(text->two_end,      data->active_script->two_end);
    fputs(text->two_end, fp);
    fputs(" ", fp);
    
    strcpy(text->three_start,  data->active_script->three_start);
    fputs(text->three_start, fp);
    fputs(" ", fp);
    
    strcpy(text->three_end,    data->active_script->three_end);
    fputs(text->three_end, fp);
    fputs(" ", fp);
    
    strcpy(text->four_start,   data->active_script->four_start);
    fputs(text->four_start, fp);
    fputs(" ", fp);
    
    strcpy(text->four_end,     data->active_script->four_end);
    fputs(text->four_end, fp);
    fputs(" ", fp);
    
    strcpy(text->send_to,     data->active_script->send_to);
    fputs(text->send_to, fp);
    fputs("\n", fp);

}

void  register_autopage_thread(pthread_t thread_id)
{
    extern  pthread_t   autopage_thread;
    //fprintf(stderr, "The  thread id is: %d\n", thread_id);

    if ( pthread_equal(autopage_thread, thread_id) )
	return;
    else
	autopage_thread = thread_id;

}

void kill_autopage_thread(pthread_t thread_id)
{
    if ( (pthread_cancel(thread_id)) != 0 )
	generic_error("\n An error has occured while shutting \n down the current autopage profile \n", "G-page error");
}

/*
 *  New functions
 */

static active_data *init_data()
{
    extern script_text  *active_script_data;
    active_data         *data_object;

    data_object = g_malloc(sizeof(*data_object));
    data_object->file_exists = 0;
    data_object->create_error_file = get_error_file_status(); /* Type gint */
    data_object->list_of_files = NULL; /* Type: GSList */
    data_object->list_of_raw_data = NULL; /* Type: GSList */
    data_object->message = g_malloc(MAX_LENGTH);
    data_object->name = g_malloc(MAX_LENGTH);
    data_object->file_cleanup_method = g_strdup(active_script_data->rename_delete_ignore);

    if ( !strcmp(active_script_data->manual_user, "select") )
	data_object->select_receiver_method = FROM_LIST;		/* They have selected a user from the g-page receiver-list */
    else
	data_object->select_receiver_method = MANUAL;		/* They must include the receiver name in the file */

    
    return(data_object);
}

static void free_object_memory(active_data *data)
{
    g_free(data->message);
    g_free(data->name);
    g_free(data->file_cleanup_method);
    g_free(data);
    
}

static GSList *get_parsed_data(GSList *data)
{
    gchar    *parsed_buffer;
    GSList   *parsed_list = NULL;

    while (data)
    {
	parsed_buffer = parse_buffer((gchar *)data->data);
	parsed_list = g_slist_append(parsed_list, (gpointer)parsed_buffer);
	g_free((gchar *)data->data);
	data = g_slist_next(data);
    }
    debug_data("I'm done returning from process target file\n");
    g_slist_free(data);
    data = NULL;
    return (parsed_list);
}

static gint  get_error_file_status(void)
{

    gchar *resource_field;
    gint  return_status = 0;


    if ( (resource_field = get_resource_data("CONFIG.dat", "create_error_status_file", MAX_FIELD_LENGTH)) )
    {
	if ( !strcmp(resource_field, "yes") )
	    return_status = 1;
        else
	    return_status = 0;

	g_free(resource_field);
    }
    else
	return_status = 0;
    
    return(return_status);
}

static void  write_error_file(active_data *page)
{
    FILE    *fp;
    time_t  timeval;
    gchar   *temp_path = NULL;
    gchar   *dash = "--------------------\n";
    gchar   *stars = "\n********************\n";
    gchar   *full_path = NULL;
    gchar   *time_buff = NULL;

    temp_path = get_temp_path( (gchar *) page->list_of_files->data );
    if ( (temp_path == NULL) || ( strlen(temp_path) == 0 ) )
	return;

    full_path = g_strdup_printf("%s%s", temp_path, "GPAGE.PARSE.ERROR");

    if ( ( fp = fopen(full_path, "a") ) == NULL )
    {
	debug_data("error opening error file: %s\n", full_path);
	g_free(full_path);
	g_free(temp_path);
	return;
    }

    time(&timeval);
    time_buff = g_strdup(ctime(&timeval));

    fputs(dash, fp);
    fputs(dash, fp);
    fputs(time_buff, fp);
    fputs("Error: unable to parse file: no match found in receiver list\n\n", fp);
    fputs( (gchar *) page->list_of_raw_data->data, fp);
    fputs(stars, fp);
    fputs("\n\n", fp);

    fclose(fp);
    g_free(time_buff);
    g_free(full_path);
    g_free(temp_path);
}

static gchar *get_temp_path(gchar *full_path)
{
    gchar  *file_name;
    gchar  *temp_path;
    gint   len, x, y;
    
    file_name = g_malloc(strlen(full_path) + 2);
    temp_path = g_malloc(strlen(full_path));
    
    len = strlen(full_path);
    x = 0;
    y = 0;
    
    while( len > 0 )
    {
	if (full_path[--len] != '/')
	    file_name[x++] = full_path[len];
	else
	{
	    while( len >= 0 )
		temp_path[y++] = full_path[len--];
	    break;
	}
    }
    
    file_name[x] = '\0';
    temp_path[y] = '\0';
    
    g_strreverse(file_name);
    g_strreverse(temp_path);

    g_free(file_name);
    return(temp_path);
}
