/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */


#include "constants.h"
#include "callbacks.h"

void
on_two_way_close_button_clicked        (GtkButton       *button,
                                        response_info   *user_data)
{
    extern gchar *response_one;
    extern gchar *response_two;
    extern gchar *response_three;

    g_free(response_one);
    g_free(response_two);
    g_free(response_three);

    response_one = response_two = response_three = NULL;

    gtk_widget_destroy(user_data->main_win);
    g_free(user_data);
}


void
on_two_way_edit_button_clicked         (GtkButton       *button,
                                        response_info   *user_data)
{
    gchar  *msg, *tmp_msg;

    if ( GTK_CLIST(user_data->list)->selection == NULL )
    {
	generic_error("\n You must make a selection to edit \n", "G-Page Error");
	return;
    }

    gtk_clist_get_text( (GtkCList *)user_data->list, GPOINTER_TO_INT(GTK_CLIST(user_data->list)->selection->data), 1, &msg);
    tmp_msg = g_strdup(msg);

    user_data->edit_flag = 1;
    user_data->edit_string = g_strdup(tmp_msg);

    gtk_clist_remove( (GtkCList *)user_data->list, GPOINTER_TO_INT(GTK_CLIST(user_data->list)->selection->data));
    del_pre_def(tmp_msg, "TWO_WAY_RESPONSE.dat");
    g_free(tmp_msg);

    show_edit_box(user_data);
}


void
on_two_way_add_button_clicked          (GtkButton       *button,
                                        response_info   *user_data)
{
    show_edit_box(user_data);
}


void
on_two_way_delete_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{

    gchar  *msg, *mesg;

    if (GTK_CLIST(user_data)->selection == NULL)
    {
	generic_error("\n You have not selected a message \n", "G-Page Error");
	return;
    }

    gtk_clist_get_text( (GtkCList *)user_data, GPOINTER_TO_INT(GTK_CLIST(user_data)->selection->data), 1, &msg);

    mesg = g_strdup(msg);

    del_pre_def(mesg, "TWO_WAY_RESPONSE.dat");
    g_free(mesg);

    gtk_clist_remove( (GtkCList *)user_data, GPOINTER_TO_INT(GTK_CLIST(user_data)->selection->data));
}


void
on_edit_text_box_changed               (GtkEditable     *editable,
                                        response_info   *user_data)
{
}


void
on_edit_cancel_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{

    gtk_widget_destroy(user_data);

}


void
on_edit_save_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
    gint    text_length;
    gchar   *response;
    gchar   *list_data[2];
    response_info *data;

    data = (response_info *) user_data;

    text_length = gtk_text_get_length( GTK_TEXT(data->box) );
    if ( text_length > 200 )
    {
	generic_error("\n Maximum number of characers (200) exceeded   \n", "G-Page Error");
	return;
    }
    if ( text_length == 0 )
    {
	generic_error("\n   You have not entered a message   \n", "G-Page Error");
	return;
    }

    response = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(data->box), 0, -1));
    response = clean_text(response);

    if ( add_data(response, "TWO_WAY_RESPONSE.dat") )
	perror("Error saving predefined message");

    list_data[0] = g_strdup(" ");
    list_data[1] = g_strdup(response);

    gtk_clist_append(GTK_CLIST(data->list), list_data);

    g_free(list_data[0]);
    g_free(list_data[1]);
    g_free(response);
    
    gtk_text_set_point( GTK_TEXT( data->box) , 0 );
    gtk_text_forward_delete ( GTK_TEXT(data->box), text_length );

}

void on_two_way_apply_button_clicked(GtkWidget *w, response_info *user_data)
{
    extern gchar *response_one;
    extern gchar *response_two;
    extern gchar *response_three;

    /*  Free these here in case the user is pounding on the apply button */
    /*  They might not have been allocated but g_free should handle it */
    
    gtk_widget_hide(user_data->red_button);
    gtk_widget_show(user_data->green_button);

    g_free(response_one);
    g_free(response_two);
    g_free(response_three);

    response_one = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->entry1)));
    if ( strlen(response_one) < 1 )
	response_one = NULL;

    response_two = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->entry2)));
    if ( strlen(response_two) < 1 )
	response_two = NULL;

    response_three = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data->entry3)));
    if ( strlen(response_three) < 1 )
	response_three = NULL;

    if ( !response_one && !response_two && !response_three )
	generic_error(" \n  You have not entered any response options \n Please deactivate, enter some responses then activate \n", "G-Page Error");
}

void on_two_way_deactivate_button_clicked(GtkWidget *w, response_info *user_data)
{
    extern gchar *response_one;
    extern gchar *response_two;
    extern gchar *response_three;

    /*  Free these here in case the user is pounding on the apply button */
    /*  They might not have been allocated but g_free should handle it */
    
    gtk_widget_hide(user_data->green_button);
    gtk_widget_show(user_data->red_button);

    g_free(response_one);
    g_free(response_two);
    g_free(response_three);

    response_one = NULL;

    response_two = NULL;

    response_three = NULL;
}

void  load_two_way_response(GtkWidget *selection_clist)
{


}
