/*
 * g-page Copyright (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */        

#include "constants.h"
#include "callbacks.h"

void reload_msg_popup(GtkWidget *w, gpointer data);
void print_detail_error(gchar *msg_code, GdkEventButton *event);
void error_code_dialog(GtkWidget *W, gchar *error_code);

void
on_close_button_clicked         (GtkButton              *button,
                                 log_status_bar_data    *user_data)
{
    gtk_widget_destroy(user_data->main_window);
    g_free(user_data);
}

void select_row_callback_log(GtkCList *list, gint row, gint col, GdkEventButton *event, log_status_bar_data *data)
{
    extern GtkWidget  *log_clist;

    gchar     *msg;
    gchar     *log_msg;
    gchar     *code;
    gchar     *err_msg;
    GdkFont   *current_font;
    gint      length;

    if ( data->s_options == NAME )
	gtk_clist_get_text(GTK_CLIST(log_clist), row, 3, &msg);
    else
	gtk_clist_get_text(GTK_CLIST(log_clist), row, 4, &msg);

    log_msg = g_strdup(msg);

    gtk_text_set_point(GTK_TEXT(data->text_box), 0);
    current_font = gdk_font_load(msg);

    gtk_clist_get_text(GTK_CLIST(list), row, 5, &code);
    err_msg = get_error_string(code);

    gtk_statusbar_pop( GTK_STATUSBAR(data->st_bar), 1 );
    gtk_statusbar_push( GTK_STATUSBAR(data->st_bar), 1, err_msg);
    g_free(err_msg);

    length = gtk_text_get_length( GTK_TEXT(data->text_box) );
    gtk_text_set_point( GTK_TEXT(data->text_box) , length );
    gtk_text_backward_delete ( GTK_TEXT(data->text_box), length );

    gtk_text_insert(GTK_TEXT(data->text_box), current_font, NULL, NULL, log_msg, strlen(log_msg));
    gtk_text_set_point( GTK_TEXT(data->text_box) , 0 );

    g_free(log_msg);

}             

void select_row_popup(GtkCList *list, GdkEventButton *event, gpointer data)
{
  gint    row, col;
  gchar   *msg;
  gchar   *temp;
  gchar   *code;

  gtk_clist_get_selection_info(list, event->x, event->y, &row, &col);
  temp = g_malloc(MAX_LENGTH);

  gtk_clist_get_text(GTK_CLIST(list), row, 3, &msg);
  gtk_clist_get_text(GTK_CLIST(list), row, 5, &code);
  strcpy(temp, msg);

 if (event->button == 3 && col != 0)        /* We have a right mouse click here */
    reload_from_log(temp, event);
 else if(event->button == 3 && col == 0)
     print_detail_error(code, event);
 g_free(temp);
 return;

}     


void select_row_popup_date(GtkCList *list, GdkEventButton *event, gpointer data)
{
  gint    row, col;
  gchar   *msg;
  gchar   *temp;
  gchar   *code;

  gtk_clist_get_selection_info(list, event->x, event->y, &row, &col);

  gtk_clist_get_text(GTK_CLIST(list), row, 4, &msg);
  gtk_clist_get_text(GTK_CLIST(list), row, 5, &code);
  temp = g_strdup(msg);

 if (event->button == 3 && col != 0)        /* We have a right mouse click here */
    reload_from_log(temp, event);
 else if (event->button == 3 && col == 0)
     print_detail_error(code, event);
 g_free(temp);
 return;

}   

void reload_from_log(gchar *msg, GdkEventButton *event)
{

  GtkWidget   *pop_menu;
  GtkWidget   *insert_item;
  gchar       *tmp;

  tmp = g_strdup(msg);
  pop_menu = gtk_menu_new();

  insert_item = gtk_menu_item_new_with_label("Resend message");
  gtk_signal_connect( GTK_OBJECT(insert_item), "activate",
                      GTK_SIGNAL_FUNC( reload_msg_popup ), tmp);

  gtk_menu_append( GTK_MENU (pop_menu), insert_item);

  gtk_widget_show(insert_item);

  gtk_menu_popup(GTK_MENU(pop_menu), NULL, NULL, NULL, NULL, event->button, event->time);

  return;

}             

void reload_msg_popup(GtkWidget *w, gpointer data)
{
    extern GtkWidget *text_entry;
    
    GdkFont  *current_font;
    gchar    *temp;

    temp = g_strdup(data);
    g_free(data);

    gtk_text_set_point(GTK_TEXT(text_entry), 0);
    current_font = gdk_font_load(temp);
    gtk_text_insert(GTK_TEXT(text_entry), current_font, NULL, NULL, temp, strlen(temp));

    text_entry_changed(NULL, NULL);

    g_free(temp);
}



void on_click_column_log(GtkWidget *w, gint col, gpointer data)
{
    extern GtkWidget *log_clist;
    gtk_clist_set_sort_column (GTK_CLIST(log_clist), col);
    gtk_clist_set_auto_sort(GTK_CLIST(log_clist), TRUE);
    gtk_clist_clear(GTK_CLIST(log_clist));
    gtk_clist_freeze(GTK_CLIST(log_clist));
    load_log_list(log_clist, data , DATE);
    gtk_clist_thaw(GTK_CLIST(log_clist));

}


void on_click_column_log_name(GtkWidget *w, gint col, gpointer data)
{
    extern GtkWidget *log_clist;
    gtk_clist_set_sort_column (GTK_CLIST(log_clist), col);
    gtk_clist_set_auto_sort(GTK_CLIST(log_clist), TRUE);
    gtk_clist_clear(GTK_CLIST(log_clist));
    gtk_clist_freeze(GTK_CLIST(log_clist));
    load_log_list(log_clist, data , NAME);
    gtk_clist_thaw(GTK_CLIST(log_clist));

}

void print_detail_error(gchar *msg_code, GdkEventButton *event)
{

  GtkWidget   *pop_menu;
  GtkWidget   *pop_msg;
  gchar       *tmp;

  tmp = g_strdup(msg_code);
  pop_menu = gtk_menu_new();

  pop_msg = gtk_menu_item_new_with_label("View detail");
  gtk_signal_connect( GTK_OBJECT(pop_msg), "activate",
                      GTK_SIGNAL_FUNC( error_code_dialog ), tmp);

  gtk_menu_append( GTK_MENU (pop_menu), pop_msg);

  gtk_widget_show(pop_msg);

  gtk_menu_popup(GTK_MENU(pop_menu), NULL, NULL, NULL, NULL, event->button, event->time);

  return;

}          

void error_code_dialog(GtkWidget *W, gchar *error_code)
{
  GtkWidget    *window;
  GtkWidget    *ok_button;
  GtkWidget    *label;
  gchar        *data = NULL;

  data = get_error_string(error_code);

  window = gtk_dialog_new();
  gtk_window_set_title (GTK_WINDOW (window), "G-Page Info");
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
  label = gtk_label_new(data);
  gtk_box_pack_start( GTK_BOX(GTK_DIALOG (window)->vbox), label, TRUE, TRUE, 0);
  ok_button = gtk_button_new_with_label("   OK   ");
  gtk_signal_connect_object( GTK_OBJECT(ok_button), "clicked",
		      GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(window));
  gtk_box_pack_start( GTK_BOX(GTK_DIALOG (window)->action_area), ok_button, TRUE, FALSE, 0);
  gtk_widget_show(ok_button);
  gtk_widget_show(label);
  gtk_widget_grab_focus(ok_button);
  gtk_window_set_modal(GTK_WINDOW(window), TRUE);
  gtk_widget_show(window);
  
  g_free(data);
}

/*
 *   We are passed an error number, we return the error message associated with that 
 *   number or an error message if we can't match the error number
 */

gchar *get_error_string(gchar  *error_code)
{
    gchar  *data;
    gint   i = 0;

    gchar  error_list [][2][256] = {
	{"020", "Message was sent via SNPP"}, 
	{"021", "Message was sent to local SNPP server for TAP dispatch"}, 
	{"010", "Message was sent via EMAIL"}, 
	{"030", "Message was sent via WCTP"}, 

	{"111", "SMTP Error: Pager email address was rejected"}, 
	{"112", "SMTP Error: Message was rejected by remote server"}, 
	{"113", "SMTP Error: Unable to connect with remote email server"}, 
	{"114", "SMTP Error: Return email address was rejected by remote email server"}, 
	{"115", "SMTP Error: Remote email server unable to send message"}, 
	{"116", "SMTP Error: Lost connection with remote server"}, 

	{"120", "SNPP Error: Unable to connect to SNPP server"}, 
	{"121", "SNPP Error: Invalid pager number"}, 
	{"122", "SNPP Error: Message rejected by remote server"}, 
	{"123", "SNPP Error: Remote server unable to send message"}, 
	{"125", "SNPP Error: Unable to connect with remote snpp server"}, 
	{"126", "SNPP Error: Connection lost with remote server while sending page"}, 
	{"220", "SNPP Msg: 2 way message was queued for delivery"}, 
	{"920", "SNPP Error: Timeout receiving data from snpp server"}, 

        {"13300", "WCTP Error: Operation not supported"}, 
        {"13301", "WCTP Error: xml parsing error"}, 
        {"13302", "WCTP Error: xml valiadtion error"}, 
        {"13303", "WCTP Error: version error "}, 
        {"13304", "WCTP Error: remote server cannot accept operation"}, 
        {"13400", "WCTP Error: function not supported"},
        {"13401", "WCTP Error: Invalid sender ID"},
        {"13402", "WCTP Error: Invalid security code"},
        {"13403", "WCTP Error: Invalid pager number"},
        {"13404", "WCTP Error: Invalid authorization code"},
        {"13405", "WCTP Error: Invalid date/time format"},
        {"13406", "WCTP Error: Date/Time range error"},
        {"13407", "WCTP Error: Invalid Date/Time period"},
        {"13408", "WCTP Error: Date/Time has already expired"},
        {"13409", "WCTP Error: Domain alias is not unique"},
        {"13410", "WCTP Error: Domain alias denied"},

	{"13500", "WCTP Error Message has timed out"},
	{"13501", "WCTP Error Message has expired"},
	{"13502", "WCTP Error No further responses possible"},
	{"13503", "WCTP Error Sender permanantly disabled due to excessive polling"},
	{"13504", "WCTP Error Unknown messge reference"},

	{"13600", "WCTP Error Poll rate exceeded"},
	{"13601", "WCTP Error Excessive polling"},
	{"13602", "WCTP Error Traffic rate exceeded"},
	{"13603", "WCTP Error Traffic rate excessive"},

        {"13900", "WCTP error: Unable to connect with remote server"}, 
        {"13901", "WCTP Error: Unable to connect - check wctp server address"}, 
        {"13902", "Connect error: Unable to establish Internet connection"}, 
        {"13905", "WCTP Error: Invalid URL - check wctp server address"}, 
        {"13906", "WCTP Error: remote server has returned an invalid response"}, 
        {"13991", "WCTP Error: Mailbox full"}, 
	{ "NULL", "NULL"},
    };

    i = 0;
    while ( strcmp(error_list[i][0], "NULL") )
    {
	if ( g_str_equal(error_code, error_list[i][0]) )
	{
	    data = g_strdup(error_list[i][1]);
	    return (data);
	}
	i++;
    }
    data = g_strdup("Error looking up page code");
    return (data);
}


