/*
 * g-page Copyright (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"


int connect_to_remote_server(char *ip_address, int port, int socket_fd, int time_out)
{
    extern int              debug_flag;

    struct    sockaddr_in   address, s_adr;
    struct    in_addr       inaddr;
    struct    hostent       *host;
    int                     sadr_len, sock;
    char                    *error_msg = "";
    char                    *connect_message = "";
    Sigfunc                 *sigfunc;

    error_msg = get_memory(MAX_LENGTH, "connect_remote - error_msg");

    connect_message = get_memory(MAX_LENGTH, "connect_remote - connect_message");

/*
 *   Lookup the ip_address
 */  
    if (inet_aton(ip_address, &inaddr))
	host = gethostbyaddr((char *) &inaddr, sizeof(inaddr), AF_INET);
    else
	host = gethostbyname(ip_address);
    if (!host)
    {
	free(error_msg);
	free(connect_message);
	return (-1);		/* Unable to lookup server */
    }
    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
    {
	free(error_msg);
	free(connect_message);
	return(-1);		/* Unable to create socket */
    }
    
    address.sin_family = AF_INET;
    address.sin_port = htons(port);
    
    memcpy(&address.sin_addr, host->h_addr_list[0], sizeof(address.sin_addr));
    /* Setup the timeout for connect */

    sigfunc = Signal(SIGALRM, connect_alarm );

    if ( alarm(time_out) != 0 )
	info_msg("Error - Alarm was already set - connect_remote.c ");

    if (connect(sock, (struct sockaddr *) &address, sizeof(address)))
    {
	if ( errno == EINTR )
	    strcpy(error_msg, "Timeout connecting to remote server\n");
	else
	    sprintf(error_msg, "Unable to connect with %s\n", ip_address);

	write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);

	alarm(0);
	Signal(SIGALRM, sigfunc);
	free(error_msg);
	free(connect_message);
	return (-1);
    }
    else
    {
	    
/*
 *   We have a connection !
 */

	/* Get the address of the server for debugging / logging purposes */
	alarm(0);
	sadr_len = sizeof(s_adr);
	getpeername(sock, (struct sockaddr *)&s_adr, &sadr_len);
	if (debug_flag)
	{
	    /* sprintf(connect_message, "Connect with %s on port %d\n", inet_ntoa(s_adr.sin_addr), ntohs(s_adr.sin_port)); */
	    sprintf(connect_message, "Connect with %s on port %d\n", ip_address, ntohs(s_adr.sin_port));
	    info_msg("Connect with %s on port %d", inet_ntoa(s_adr.sin_addr), ntohs(s_adr.sin_port));
	    log_transaction(connect_message);
	    write_to_status_line(socket_fd, connect_message, strlen(connect_message), STATUS_UPDATE);
	}
	free(error_msg);
	free(connect_message);
	return (sock);
    }
    return (-1);		/* Something evil has happened if we get here */
}


Sigfunc *Signal(int signo, Sigfunc *func)
{

    struct sigaction  act, oact;

    act.sa_handler = func;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    if (  signo == SIGALRM )
    {
#ifdef SA_INTERRUPT
	act.sa_flags |= SA_INTERRUPT;
#endif
    }
    else
    {
#ifdef SA_RESTART
	act.sa_flags |= SA_RESTART;
#endif
    }
    if ( sigaction(signo, &act, &oact) < 0 )
	return(SIG_ERR);
    return (oact.sa_handler);

}

void connect_alarm(int signo)
{
    info_msg("Got an alarm\n");
    return;
}
