/*
 * g-page Copyright (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#define TRUE    1
#define FALSE   0

typedef int BOOL;
typedef void Sigfunc (int);	

#define CONNECT_SLEEP_VALUE 40

#define IP_LENGTH 20

#define MAX_LENGTH       2500
#define MAX_FIELD        500
#define DEFAULT_PORT     1299
#define MAX_TIMEOUT      600
#define MAX_NUM_THREADS  100

#define STANDARD   0
#define DIRECT     1
#define STATUS     3
#define DELETE     4
#define TELNET     5
#define CHECK_SNPP 6
#define CHECK_WCTP 7

#define MAX_NAME_LENGTH   128
#define MAX_NUMBER_LENGTH 64

#define SCHEDULE  1
#define SEND_NOW  2

#define WCTP   0
#define SNPP   1
#define SMTP   2

#define SERIAL    0
#define ASYNCH    1
#define ROLLOVER  2

#define HOURLY   1
#define DAILY    2
#define WEEKLY   3

#define INVALID_PAGE    0
#define SEND_SMTP       1
#define SEND_SNPP       2
#define SEND_WCTP       3
#define SENT_SMTP       4
#define SENT_SNPP       5
#define SENT_WCTP       6
#define ERROR_SMTP      7
#define ERROR_SNPP      8
#define ERROR_WCTP      9
#define PAGE_INIT       10
#define PAGE_ERROR      11
#define ERROR_ROLLOVER  12

#define IGNORE                10
#define LOG_DATA              11
#define STATUS_UPDATE         12
#define ALWAYS_UPDATE         13
#define DENIED                14
#define SPECIAL               15
#define UPDATE_SMALL          16
#define TWO_WAY_LOG_DATA      17
#define TWO_WAY_REPLY_DATA    19
#define CONNECTION_ERROR      20

#define SCHEDULED_FILE_ENTRIES 18

/***********************************************************
 ***********************************************************

 Data structure for the page queue. This will be a series 
 of pointers that point to all of the pages being sent
 to the paging server. A double linked list is used to 
 make it easy to manipulate (sorting etc..)

************************************************************
************************************************************/

typedef struct active_page_queue
{
    char                   *user_id;
    char                   *msg_id;
    char                   *receiver_number;
    char                   *wctp_url;
    char                   *receiver_name;
    char                   *smtp;
    char                   *snpp_url;
    char                   *snpp_port;
    char                   *snpp_password;
    char                   *max_msg_length;
    char                   *message;
    char                   *sig;
    char                   *sig_yes_no;
    char                   *sig_snpp;
    char                   *sig_email;
    char                   *sig_place;
    char                   *sched_data;
    char                   *protocols;
    char                   *protocols_order;
    char                   *cc_email;
    char                   *response_one;
    char                   *response_two;
    char                   *response_three;
    char                   *tracking_number;
    char                   *alternative_snpp;
    char                   *alternative_port;
    char                   *dial_on_no_connect;
    int                    connection_fail_mode;
    int                    pwd_yes_no;
    int                    auto_split;
    int                    msg_was_split;
    int                    split_nodes;
    int                    page_status;
    int                    page_result;
    int                    asynch_page;
    int                    time_stamp;
    char                   *error_code;
    struct active_page_queue           *next;
    struct active_page_queue           *previous;
} page_profile;

/************************************************************
************************************************************/

typedef  struct {
    char *isp;
    char *email_domain;
    char *email_address;
}email_data;

typedef struct {
    int  sent_wctp;
    int  error_wctp;
    int  sent_snpp;
    int  error_snpp;
    int  sent_smtp;
    int  error_smtp;
    int  sent_total;
    int  error_total;
    int  queued;
    int  no_connect;
}message_data;

typedef struct {
    char id[MAX_LENGTH];
    char year[32];
    char month[16];
    char day[16];
    char hour[16];
    char min[16];
    char am_pm[16];
    char recurring[2];
}sched_info;

typedef struct generic_list{
    char                 data[MAX_LENGTH];
    int                  flag;
    struct generic_list  *next;
}link_list;

typedef struct blocked_ips{
    char                 *bad_ip;
    struct blocked_ips   *next;
}banned_ips;

typedef struct thread_data_tag{
    page_profile       *data;
    int                socket_fd;
    email_data         *email_info;
    message_data       *totals;
    int                connection_timeout;
    int                server_read_timeout;
    int                send_flag;
    int                send_mode;
    int                which_function;
    char               *client_ip;
}thread_data;
