/*
 * g-page Copyright (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

/*
 *    server_main.c
 */

void sig_chld(int signo);
void init_gserver();
void verify_exit();

/*
 *    server_utilities.c
 */

void gui_page(int socket_fd, char *client_adr);
page_profile *new_sort(page_profile *data, int pages);
page_profile *get_memory_struct(page_profile *node);
void print_list(page_profile *list);
page_profile  *get_page_list_memory(page_profile *data);
void free_page_mem(page_profile *data);
void print_debug_data(char *string);

/*
 *    io_utilities.c
 */

ssize_t read_num(int file_desc, char *strptr, size_t num);
ssize_t write_num(int file_desc, char *strptr, size_t num);
ssize_t read_a_line(int file_desc, char return_buff[MAX_LENGTH], size_t maxlen, int timeout);
int lock_file(int fd, int cmd, int type, off_t offset, int whence, off_t len);
FILE *file_locking(char *filename, char *mode, char *temp_lock_file, int action);

/*
 *    data_routines.c
 */

char *get_memory(int size, char *function);
int get_resource_data(char *filename, char *match_field, char *return_value);
char *get_id(int  function);
FILE *get_temp_file();
void write_to_status_line(int sock, char *buffer, int amount, int opcode);

/*
 *    auto_page.c
 */

void auto_page(int socket_fd, char *client_adr);
void telnet_page(int socket_fd, char *client_adr);

/*
 *    connect_remote.c
 */

int connect_to_remote_server(char *ip_address, int port, int socket_fd, int time_out);
Sigfunc *Signal(int signo, Sigfunc *func);
void connect_alarm(int signo);

/*
 *    logging.c
 */

void debug_data(const char *fmt,...);
void debug_log(const char *fmt,...);
void error_ret(const char *fmt,...);
void info_msg(const char *fmt,...);
void error_sys(const char *fmt,...);
void error_core(const char *fmt,...);
void error_quit(const char *fmt,...);

/*
 *    msg_edit_delete.c
 */

void delete_msg(int sock);
void clean_up_data(char *id);
char *delete_id(char *id);

/*
 *    msg_status.c
 */

void  get_msg_status(int conn_fd);
int update_status(char *buffer, int sock, int delete_flag);
char *get_msg_id(char *line, char *status);
int recycle_id(char *group, char *id);

/*
 *    schedule_page.c
 */

void fix_scheduled_time(sched_info *obj, struct tm *tdata,int when);
int check_for_pages(FILE *fp);
void send_scheduled(link_list *page_list, int npages);
void handle_scheduled_pages();
sched_info *parse_sched_line(char  *data);
int my_rename (char *destination, char *new_file);

/*
 *    send_messages.c
 */

page_profile *send_messages(page_profile *head, int socket_fd, email_data *email_info, int send_flag, char *client_adr);
void print_debug(page_profile *stuff);

/*
 *    send_smtp.c
 */

page_profile  *send_smtp(page_profile *data, int socket_fd, email_data *email_info, message_data  *totals, int server_timeout, 
			 int server_read_timeout, int send_flag, int send_mode, char *client_adr);
char *mread(int fd, int timeout);
char *parse_codes(char *buffer, int num_char);
void print_list_debug1(page_profile *data);
void log_it(char *first, char *second);
void log_transaction(char *data_buffer);
void send_log_data(char *name, char *message, char *code, int socket_fd, int send_flag, char *msg_id, char *client_adr, char *tracking, char *uid);
void clean( char *buffer);

/*
 *    send_snpp.c
 */

page_profile  *send_snpp(page_profile *data, int socket_fd, email_data *email_info, message_data  *totals, int connection_timeout, 
			 int server_read_timeout, int send_flag, int send_mode, char *client_adr);
int new_url(char *url, char *new_url, char *new_port, int socket_fd, int current_socket, char *new_connection, int timeout, int server_timeout);
char *fix_string(char *buffer);
int are_we_done(page_profile *node, int mode, int pages);

/*
 *    send_wctp.c
 */

page_profile  *send_wctp(page_profile *data, int socket_fd, email_data *email_info, message_data  *totals, int server_timeout, 
			 int server_read_timeout, int send_flag, int send_mode, char *client_adr, int query_mode);
int expat(char *buf);
void startElement(void *userData, const char *name, const char **atts);
void endElement(void *userData, const char *name);
void strip_crap(char *buffer);
char *get_current_time();
char *clean_xml(char *original_message);
char *build_query(char *rec_num, char *send_time, char *msg, char *email_address, int query_flag, char *one, char *two, char *three);
void check_for_wctp_response(thread_data *t_data);
void charHandler(void *userData, const char *s, int len);
