/*
 * g-page (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"


ssize_t read_num(int file_desc, char *strptr, size_t num)
{

  size_t      num_left;
  ssize_t     num_read;
  char        *ptr;
  
  ptr = strptr;
  num_left = num;
  while ( num_left > 0 )
    {
      if ( (num_read = read( file_desc, ptr, num_left ) ) < 0 )
	{	
	  if ( errno == EINTR )
	    {
	      num_read = 0;
	      continue;
	    }
	  else
	    {
	      debug_log("This is the errno: %d\n", errno);
	      return (-1);
	    }
	}
      else
	if ( num_read == 0)
	  break;     /* We have reached the end of the file  */
      num_left -= num_read;
      ptr += num_read;
    }
  return ( num - num_left );
}


ssize_t write_num(int file_desc, char *strptr, size_t num)
{

  size_t             num_left;
  ssize_t            num_written;
  char               *ptr;
  int                maxfd;
  fd_set             wset;
  int                result = 0;

/*
 *   I'm going to ignore the sigpipe here. The sigpipe is generated when we try to 
 *   write to a closed file descriptor on a socket. I'll let the calling function
 *   handle it by returning a -1. This way we can continue on sending a page even it 
 *   the client has crashed or has been closed. Note that we plug the default signal 
 *   handler back in before we return
 */

  if ( file_desc < 0 )		/* In case we get sent a closed file descriptor */
      return(-1);
  Signal(SIGPIPE, SIG_IGN);

  ptr = strptr;
  num_left = num;
  FD_ZERO(&wset);

  while ( num_left > 0 )
  {
      maxfd = file_desc +1;
      FD_SET(file_desc, &wset);

/*
 *   We use select to try to catch any closed file descriptors
 */

      result = select(maxfd, NULL, &wset, NULL, NULL);
      
      if ( result == -1 )
      {
	  if ( errno == EINTR )
	  {
	      num_written = 0;
	      continue;
	  }
	  else
	  {
	      Signal(SIGPIPE, SIG_DFL);
	      return (-1);
	  }	      
      }
      else if (result)
      {
	  num_written = write(file_desc, ptr, num_left );
	  if ( num_written  <= 0 )
	  {
	      if ( errno == EINTR )
	      {
		  num_written = 0;
		  continue;
	      }
	      else
	      {
		  Signal(SIGPIPE, SIG_DFL);
		  return (-1);
	      }
	  }
	  else
	  {
	      num_left -= num_written;
	      ptr += num_written;
	  }
	  
      }
  }
  Signal(SIGPIPE, SIG_DFL);
  return (num);
}


ssize_t read_a_line(int file_desc, char return_buff[MAX_LENGTH], size_t maxlen, int timeout)
{

    int         index;
    int         select_value;
    int         read_count;
    char        ch;
    fd_set      rset;
    struct      timeval tv;


    memset(return_buff, '\0', MAX_LENGTH);
    index = 0;
    
    if ( file_desc < 0 )
	return 0;

    while (index < (MAX_LENGTH -1) )
    {
	FD_ZERO(&rset);
	FD_SET(file_desc, &rset);

	tv.tv_sec = timeout;
	tv.tv_usec = 0;

	 select_value = select(file_desc + 1, &rset, NULL, NULL, &tv);
	 
	 if ( select_value > 0 )
	 {
	      if ( FD_ISSET(file_desc, &rset) )
	      {
		   read_count = read(file_desc, &ch, 1);

		   if ( read_count == 1 )
		   {
			return_buff[index++] = ch;
			if ( (ch == '\n') )
			{
			     return index;
			}
		   }
		   else if ( read_count == 0 ) /* the socket we're talking to has closed */
		   {
			if ( index == 0 )
			    return 0;	      
			else
			{
			     return_buff[index++] = '\n';
			     return_buff[index++] = '\0';
			     return (index);
			}
		   }
		   else if ( read_count == -1 )	/* Error on read  */
		   {
		       if (errno == EINTR)
			   continue;
		       else
			   return 0;
		   }
	      }
	      else
	      {
		   continue;
	      }
	 }
	 else if ( select_value == 0 ) /* timeout in select */
	 {
	      sprintf(return_buff, "920 remote server stopped sending data\n");
	      return 0;	     
	      //return( strlen(return_buff));	  
	 }
	 else			/* Error in select */
	 {		
	      if (errno == EINTR)
		   continue;
	      else
	      {
		   return 0;	  	  
	      }
	 }
    }
    return_buff[index -2] = '\n';
    return_buff[index -1] = '\0';
    return index;
} 

/*
ssize_t read_a_line(int file_desc, char return_buff[MAX_LENGTH], size_t maxlen, int timeout)
{

    int                   index;
    char                  ch;
    int                   read_count;
    Sigfunc               *sigfunc;

    memset(return_buff, (char) '\0', MAX_LENGTH);
    index = 0;
    
    sigfunc = Signal(SIGALRM, connect_alarm );

    if ( alarm(timeout) != 0 )
    {
	 return(0);
    }

    while (index < (MAX_LENGTH -1) )
    {
	read_count = read(file_desc, &ch, 1);
	
	if ( errno == EINTR )
	{
	    errno = ETIMEDOUT;
	    alarm(0);
	    Signal(SIGALRM, sigfunc);
	    sprintf(return_buff, "920 remote server stopped sending data\n");
	    return 0;
	    return( strlen(return_buff));
	}
	
	if ( read_count == 1 )
	{
	    return_buff[index++] = ch;
	    if ( (ch == '\n') )
	    {
		alarm(0);
		Signal(SIGALRM, sigfunc);
		return index;
	    }
	}
	else if ( read_count == 0 )
	{
	    if ( index == 0 )
	    {
		alarm(0);
		Signal(SIGALRM, sigfunc);
		return 0;	      
	    }
	    else
	    {
		alarm(0);
		Signal(SIGALRM, sigfunc);
		return_buff[index++] = '\n';
		return_buff[index++] = '\0';
		return (index);
	    }
	}
    }

    alarm(0);
    Signal(SIGALRM, sigfunc);
    return_buff[index -2] = '\n';
    return_buff[index -1] = '\0';
    return index;
} 
*/

int lock_file(int fd, int cmd, int type, off_t offset, int whence, off_t len)
{
    struct  flock lock;

    lock.l_type = type;
    lock.l_start = offset;
    lock.l_whence = whence;
    lock.l_len = len;
    
    return( fcntl(fd, cmd, &lock) );
}

/*************************************************************/
/*                                                           */
/*  Return NULL on a system error or if the file is in use   */
/*          *fp if we have rights to the file                */
/*                                                           */
/*************************************************************/

FILE *file_locking(char *filename, char *mode, char *temp_lock_file, int action)
{
    enum
    {
	lock = 0,
	unlock = 1
    };

    char            tlock_file[MAX_LENGTH];
    FILE            *fpl, *fp;
    int             fd, lockval, smode;
    char            pid_str[128];
    char            tmp_pid_str[128];
    int             lock_pid;
    int             status;

    fd = lockval = smode = 0;
    strcpy(tlock_file, "/tmp/");
    strcat(tlock_file, temp_lock_file);

    switch (action)
    {
    case lock:	      
	sprintf(pid_str, "%d", (int )getpid() );
	status = open(tlock_file, O_RDWR | O_CREAT | O_EXCL, 0444);
	if (status == -1)	/* The lock file exsits - check to see if it's stale */
	{
	    if ( ( fpl = fopen(tlock_file, "r")) == NULL)
	    {
		error_ret("Error opening lock_file: %s", tlock_file);
		return (NULL);
	    }
	    fgets(tmp_pid_str, 128, fpl);
	    lock_pid = atoi(tmp_pid_str);
	    if( (kill(lock_pid, 0) == -1) && (errno == ESRCH) ) 	
	    {         /* Lock file is stale, remove it and use the port */
		fclose(fpl);
		unlink(tlock_file);
	    }		    
	    else
	    {		/* The process using the file is still alive */
		fclose(fpl);
		return(NULL);
	    }
	}
	else			/* The file didn't exist */
	{
	    close(status);
	    unlink(tlock_file);
	}

	/* The lockfile doesn't exist or it was stale. */

	if ( ( fp = fopen(filename, mode)) == NULL)
	{
	    error_ret("Error opening %s in file_locking io_utilities.c", filename);
	    return (NULL);
	}
	
	fd = fileno(fp);
	
	/* lets try to lock the file */
	lockval = lock_file(fd, F_SETLK, F_WRLCK, 0, SEEK_SET, 0);
	if ( lockval == -1 )
	{
	    if ( errno == EAGAIN )
	    {
		info_msg("Error: %s is locked", filename);
		fclose(fp);
		return(NULL);
	    }   
	    else
	    {
		fclose(fp);
		return(NULL);
	    }
	}
	else
	{
	    if ( ( fpl = fopen(tlock_file, "w")) == NULL)
	    {
		fclose(fp);
		return(NULL);
	    }
	    else
	    { /* SUCCESS Store the pid and move on */
		fputs(pid_str, fpl);
		fclose(fpl);
		return(fp);
	    }
	}
	break;
	
    case unlock:
	unlink(tlock_file);
	return(NULL);
	break;
    }


    return(NULL);			/* Uh oh...We should never get here */
}


