/*
 * g-page (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"

static void handle_error(int errno_flag, int level, const char *fmt, va_list ap);
static void log_debug_data(char *data_buffer);

extern int daemon_flag;		/*  one if we are a daemon, zero otherwise  */

/*
 *   This is for the debug logging
 */

void debug_data(const char *fmt,...)
{

#ifdef DEBUG_LOG
    char buf[MAX_LENGTH];

    va_list ap;
    
    va_start(ap, fmt);
    vsprintf(buf, fmt, ap);
    log_debug_data(buf);
    va_end(ap);
#endif
    return;
}

static void log_debug_data(char *data_buffer)
{
    FILE    *fp;
    time_t  timeval;
    char    buffer[MAX_LENGTH];
    char    *home;
    char    *logfile = "/.gpage/debug_logging";
    int     path_length = 0;
    
    path_length = strlen(getenv("HOME")) + strlen(logfile) + 10;
    home = get_memory(path_length, "logging - path_length");

    strcpy(home, getenv("HOME"));
    strcat(home, logfile);

    memset(buffer, '\0', sizeof(buffer));
    time(&timeval);
    strcpy(buffer, ctime(&timeval));
    buffer[strlen(buffer) -1] = ' '; /* Kill newline character */
    strcat(buffer, " -- ");
    strcat(buffer, data_buffer);

    if ( ( fp = fopen(home, "a+")) == NULL)
	error_ret("Error opening debug_logging");

    fputs(buffer, fp);
    fclose(fp);
    free(home);
}         

void debug_log(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(0, LOG_INFO, fmt, ap);
    va_end(ap);
    return;
}

/*
 *   Non fatal error - system call
 */

void error_ret(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(1, LOG_INFO, fmt, ap);
    va_end(ap);
    return;
}

/*
 *   Informational logging - no error
 */

void info_msg(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(0, LOG_INFO, fmt, ap);
    va_end(ap);
    return;
}

/*
 *   Fatal error - system call
 */

void error_sys(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(1, LOG_ERR, fmt, ap);
    va_end(ap);
    exit(EXIT_FAILURE);
}

/*
 *   Fatal error - system call
 */

void error_core(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(1, LOG_ERR, fmt, ap);
    va_end(ap);
    abort();			/* Dump core */
    exit(EXIT_FAILURE);
}

/*
 *   Fatal error - non system call
 */

void error_quit(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(0, LOG_ERR, fmt, ap);
    va_end(ap);
    exit(EXIT_FAILURE);
}

static void handle_error(int errno_flag, int level, const char *fmt, va_list ap)
{
    int   errno_save = 0;
    int   n;
    char  buf[MAX_LENGTH];

    if (errno_flag)
	errno_save = errno;

#ifdef HAVE_VSNPRINTF
    vsnprintf(buf, sizeof(buf), fmt, ap);
#else
    vsprintf(buf, fmt, ap);
#endif

    n = strlen(buf);
    if (errno_flag)
	snprintf(buf + n, sizeof(buf) - n, ": %s", strerror(errno_save));
    strcat(buf, "\n");

    if (daemon_flag)
	syslog(level, buf);
    else
    {
	fflush(stdout);
	fputs(buf, stderr);
	fflush(stderr);
    }
    
    return;
}
