/*
 * g-page (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"

/*
 *    get_msg_status: We are passed a scheduled message ID from the client and we are going 
 *                    to go through the scheduled log file to see if it has been sent yet. 
 *                    If it was sent we send back the log info to the client.
 *         
 *                    If it was a recurring page we will send back the date of the next attempt to 
 *                    send the page. If it was not a recurring page we recycle the msg ID.
 *
 */

void  get_msg_status(int conn_fd)
{
    FILE        *fp;
    int         num_read = 0;
    int         okay = 0;
    int         match = 0;
    int         path_length = 0;
    char        buffer[MAX_LENGTH];
    char        *msg_status;
    char        *id = "";
    sched_info  *obj;
    char        *id_file = "";
    char        *line = "";
    char        *new_time = "";
    char        *logfile = "/.gpage/gsched_id.dat";
    
    line = get_memory(MAX_LENGTH, "msg_status - line");
    new_time = get_memory(MAX_LENGTH, "msg_status - new_time");

    strcpy(new_time, " \n");

    path_length = strlen(getenv("HOME")) + strlen(logfile) + 10;
    id_file = get_memory(path_length, "msg_status - id_file");

    strcpy(id_file, getenv("HOME"));
    strcat(id_file, logfile);


    num_read =  read_a_line(conn_fd, buffer, MAX_LENGTH, 60);
    write_to_status_line(conn_fd, buffer, strlen(buffer), IGNORE);

    id = get_memory(MAX_LENGTH, "msg_status - id");
    strcpy(id, buffer);
    id[strlen(id) - 1] = '\0';
    
    msg_status = get_memory(MAX_LENGTH, "msg_status - msg_status");
    
    okay = 0;
    okay = update_status(buffer, conn_fd, 0);

    //write_to_status_line(conn_fd, "time\n", 5, IGNORE);
    write_num(conn_fd, "time\n", 5);

    num_read =  read_a_line(conn_fd, buffer, MAX_LENGTH, 60);

    match = 0;
    if ( ( fp = fopen(id_file, "r")) == NULL)
    {
	if ( !okay )
	    	strcpy(new_time, "In Progress\n");
	else
	    strcpy(new_time, "No further attempts will be made\n");
    }
    else
    {
	while ( fgets(line, MAX_LENGTH, fp) )
	{
	    
	    obj = parse_sched_line(line);
	    if ( !strcmp(id, obj->id) )
	    {
		if ( !strcmp(obj->hour, "0" ) )
		     strcpy(obj->hour, "12");

		match = 1;
		if ( *(obj->am_pm) == '1' )
		    sprintf(new_time, "%s:%s %s %d/%s/%s\n", obj->hour, obj->min, "AM", (atoi(obj->month) + 1), obj->day, obj->year);
		else
		    sprintf(new_time, "%s:%s %s %d/%s/%s\n", obj->hour, obj->min, "PM", (atoi(obj->month) + 1), obj->day, obj->year);
		fclose(fp);
		break;
	    }
	}
	if ( !match )
	{
	    if ( !okay )
	    	strcpy(new_time, "In Progress\n");
	    else
		strcpy(new_time, "No further attempts will be made\n");
	}
    }
    
    //write_to_status_line(conn_fd, new_time, strlen(new_time), IGNORE);
    write_num(conn_fd, new_time, strlen(new_time));
    num_read =  read_a_line(conn_fd, buffer, MAX_LENGTH, 60);

    //write_to_status_line(conn_fd, "done\n", 6, IGNORE);
    write_num(conn_fd, "done\n", 6);
    num_read =  read_a_line(conn_fd, buffer, MAX_LENGTH, 60);

    close(conn_fd);
    free(msg_status);

    return;
}

/*
 *    update_status: We go through the log file for scheduled pages and try to match the ID
 *                   for any logged messages with the ID we were sent from the client. If we match then we
 *                   will do one of two things based upon the delete_flag:
 *                   0 - We are just getting the status so we send back any logs we find to the socket 
 *                       we were passed and delete the log from the file
 *                   1 - We are deleting the ID and we are cleaning up any old log files, so we don't write 
 *                       the log back to the socket, we just delete it
 */

int update_status(char *buffer, int sock, int delete_flag)
{
    enum
    {
	lock = 0,
	unlock = 1
    };

    FILE           *fp, *fpt;
    int            flag;
    char           *log_file = NULL;
    char           *line = NULL;
    char           *id = NULL;
    char           *prev_id = NULL;
    char           *status = NULL;
    char           *logfile = "/.gpage/gpage_sched_log.dat";
    char           *temp_line;
    int            wait;
    int            file_locked = 1;
    struct stat    file_info;
    int            id_len = 0;
    int            path_length = 0;

    line = get_memory(MAX_LENGTH, "msg_status - line msg_status");
    status = get_memory(MAX_LENGTH, "msg_status - status msg_status");
    prev_id = get_memory(MAX_LENGTH, "msg_status - prev_id msg_status");
    flag = 0;

    path_length = strlen(getenv("HOME")) + strlen(logfile) + 10;
    log_file = get_memory(path_length, "msg_status - log_file");

    strcpy(log_file, getenv("HOME"));
    strcat(log_file, logfile);

    if ( (stat(log_file, &file_info)) == -1 ) 
	return (flag);		/* The file doesn't exist so there is no need to go any further */

    wait = 0;
    while ( file_locked )
    {
	if ( wait++ == 300 )	/* If the file is locked for 5 minutes something evil has happened */
	    return (flag);

	fp = file_locking(log_file, "r+", "gpage_sched_log.dat", lock);

	if (fp)
	    file_locked = 0;
	else
	    sleep(1);
    }

    if ( (fpt = get_temp_file()) == NULL)
    {
	error_ret("Error opening temp_file update_status msg_status.c");
	return (flag);
    }

    buffer[strlen(buffer) -1] = '\0';
    strcpy(prev_id, "null");
    while ( fgets(line, MAX_LENGTH, fp) )
    {
	id = get_msg_id(line, status);
	if ( !strcmp(id, buffer) )
	{
	    if ( !delete_flag )
	    {
		flag = 1;
		strcat(status, "\n");
		//write_to_status_line(sock, status, strlen(status), IGNORE);
		write_num(sock, status, strlen(status));
		read_a_line(sock, status, MAX_LENGTH, 60);

		/* If a page has autosplit into multiple pages we only want to recycle the id once */
		
		if (strcmp(id, prev_id) )
		{
		    id_len = strlen(id);
		    id_len--;
		    if ( *(id + id_len) != 'r' ) /* If this is a recurring page we don't recycle the id */
			recycle_id("SCHEDULE", buffer);
		}
		strcpy(prev_id, id);
	    }
	}
	else
	    fputs(line, fpt);

	free(id);
    }

    free(status);
    free(line);

    fclose(fp);
    rewind(fpt);

    temp_line = get_memory(MAX_FIELD, "msg_status - temp_line");	
    
    if ( (fp = fopen(log_file, "w")) == NULL)
    {
	fclose(fpt);
	free(temp_line);
	return (flag);
    }
    while(fgets(temp_line, MAX_LENGTH, fpt))
	fputs(temp_line, fp);
    fclose(fpt);
    fclose(fp);
    free(temp_line);

    file_locking(log_file, "r+", "gpage_sched_log.dat",unlock);

    return ( flag );

}

char *get_msg_id(char *line, char *status)
{
    char   *id = NULL;
    int    x, i;
    
    id = get_memory(MAX_LENGTH, "msg_status - id");

    x = 0;
    while ( *(line + x) != ' ')
    {
	*(id + x) = *(line + x);
	x++;
    }
    *(id + x) = '\0';

    i = 0;
    while ( *(line + x) != '\n')
	*(status + i++) = *(line + x++);

    *(status + i) = '\0';
    
    return (id);
}

int recycle_id(char *group, char *id)
{
    FILE  *rfp, *wfp;
    char  *full_path = NULL;
    char  *logfile = "/.gpage/SCH_ID.dat";
    char  *temp_line;
    int   position, index;
    int   path_length = 0;
    int   ch;
    
    path_length = strlen(getenv("HOME")) + strlen(logfile) + 10;
    full_path = get_memory(path_length, "msg_status - full_path");

    strcpy(full_path, getenv("HOME"));
    strcat(full_path, logfile);

    if ( (rfp = fopen(full_path, "r")) == NULL)
    {
	error_ret("Error opening SCH_ID.dat recycle_id msg_status.c");
	return(-1);
    }

    if ( (wfp = get_temp_file()) == NULL)
    {
	error_ret("Error opening temp_file recycle_id msg_status.c");
	return(-1);
    }

    if( !strcmp(group,"SCHEDULE") )
	position = 0;
    else
    {
	debug_log("Error parsing SCH_ID.dat: Corrupt database");
	return(-1);
    }

    for (index = 0; index < position; index++)
    {
	while( (ch = fgetc(rfp)) != '\n' )
	    fputc(ch, wfp);
	fputc('\n', wfp);
    }

    fputs(id, wfp);
    fputc(',', wfp);

    while( (ch = fgetc(rfp)) != EOF ) 
	fputc(ch, wfp);

    fclose(rfp);
    rewind(wfp);

    temp_line = get_memory(MAX_FIELD, "msg_status - temp_line");	
    
    if ( (rfp = fopen(full_path, "w")) == NULL)
    {
	fclose(wfp);
	free(temp_line);
	return (-1);
    }
    while(fgets(temp_line, MAX_LENGTH, wfp))
	fputs(temp_line, rfp);
    fclose(rfp);
    fclose(wfp);

    free(temp_line);
    free(full_path); 
    return(0);
}
