/*
 * g-page (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"

#define CLOSED_SOCKET -1

void handle_scheduled_pages()
{
    enum
    {
	file_exists = 60,
	no_file_exists = 90,
	file_was_locked = 2,
	lock = 0,
	unlock = 1
    };

    FILE             *fp;
    int              delay;
    struct timeval   tv;
    struct stat      file_info;
    int              retval, statval;
    int              num_sent = 0;
    int              num_left = 0;
    int              path_length = 0;
    char             *id_file = NULL;
    char             *logfile = "/.gpage/gsched_id.dat";

    delay = file_exists;

    path_length = strlen(getenv("HOME")) + strlen(logfile) + 10;
    id_file = get_memory(path_length, "schedule_page - id_file");

    strcpy(id_file, getenv("HOME"));
    strcat(id_file, logfile);

    for(;;)
    {
	tv.tv_sec = delay;
	tv.tv_usec = 0;

	retval = select(0, NULL, NULL, NULL, &tv); /* This is used as a timer only */
	switch (retval) 
	{
	case 0:
	    if ( (statval = stat(id_file, &file_info)) == -1) /* The file does not exist */
	    {
		delay = no_file_exists;
		break;
	    }
	    else if (statval == 0) /* The file exists */
	    {
		delay = file_exists;
		fp = file_locking(id_file, "r+", "gsched_id.dat", lock);
		if (!fp)	/* If fp is NULL then the file is alraedy locked */
		{
		    delay = file_was_locked;
		    break;
		}
		else
		{
		    num_sent = check_for_pages(fp);
		    fclose(fp);
		    if ( num_sent )
		    {
			num_left = my_rename(id_file, "/tmp/gpage_sched_tmp.tmp");

			if ( num_left == 0 )
			    unlink(id_file);
			unlink("/tmp/gpage_sched_tmp.tmp");
			file_locking(id_file, "r+", "gsched_id.dat", unlock);
		    }
		    else
		    {
			unlink("/tmp/gpage_sched_tmp.tmp");
			file_locking(id_file, "r+", "gsched_id.dat", unlock);
		    }
		}
	    }
	    continue;
	    break;
	case -1:
	    if ( errno == EINTR )
		continue;
	    else
		info_msg("Select error: handle_schedule_page - schedule_page.c");
	    break;
	}		
    }
}

int check_for_pages(FILE *fp)
{
    FILE        *fp_temp;
    char        line[MAX_LENGTH];
    char        *temp_file = "/tmp/gpage_sched_tmp.tmp";
    int         comp;
    int         val, send_now, good;
    int         recur = 0;
    time_t      t_val;
    struct tm   *tdata;
    sched_info  *obj;

    link_list   *page_list, *head, *temp;

    page_list = head = temp = NULL;

    if ( ( page_list = malloc(sizeof(*page_list))) == NULL)
	 error_sys("Error allcating memory: page_list schedule_page.c");
    
    head = page_list;

    good = 0;

    if ( (fp_temp = fopen(temp_file, "w+")) == NULL)
	error_ret("Error: unable to open gpage_sched_tmp.tmp");


    while ( fgets(line, MAX_LENGTH, fp) )
    {

	time(&t_val);
	tdata = localtime(&t_val);
	tdata->tm_year += 1900;

	obj = parse_sched_line(line);
	
	if ( !strcmp(obj->am_pm, "0") )	/* 0 stands for pm it's a 1 if it's am */
	    comp = atoi(obj->hour) + 12;
	else
	    comp = atoi(obj->hour);
	send_now = 0;
				/* YEAR */
	if ( (val = atoi(obj->year)) < tdata->tm_year) 
	    send_now = 1;
	else if ( val > tdata->tm_year )
	    send_now = 0;
				/* MONTH */
	else if ( (val = atoi(obj->month)) < tdata->tm_mon )
	    send_now = 1;
	else if ( val > tdata->tm_mon )
	    send_now = 0;
				/* DAY */
	else if ( (val = atoi(obj->day)) < tdata->tm_mday )
	    send_now = 1;
	else if ( val > tdata->tm_mday )
	    send_now = 0;
				/* HOUR */
	else if ( comp < tdata->tm_hour )
	    send_now = 1;
	else if ( comp > tdata->tm_hour )
	    send_now = 0;
				/* MINUTE */
	else if ( (val = atoi(obj->min)) <= tdata->tm_min )
	    send_now = 1;
	else if ( val > tdata->tm_min )
	    send_now = 0;

	if ( !send_now )
	{
	    fputs(line, fp_temp);
	    free(obj);
	}
	else
	{
	    recur = atoi(obj->recurring);
	    switch (recur) {
	    case HOURLY:      

		fix_scheduled_time(obj, tdata,  HOURLY);
		sprintf(line, "ID %s %s-%s-%s-%s-%s-%s-%s\n", obj->id, obj->year, obj->month, obj->day, obj->hour, obj->min, obj->am_pm, obj->recurring); 
		fputs(line, fp_temp);
		break;

	    case DAILY:	 

		fix_scheduled_time(obj, tdata, DAILY);
		sprintf(line, "ID %s %s-%s-%s-%s-%s-%s-%s\n", obj->id, obj->year, obj->month, obj->day, obj->hour, obj->min, obj->am_pm, obj->recurring); 
		fputs(line, fp_temp);
		break;

	    case WEEKLY: 

		fix_scheduled_time(obj, tdata, WEEKLY);
		sprintf(line, "ID %s %s-%s-%s-%s-%s-%s-%s\n", obj->id, obj->year, obj->month, obj->day, obj->hour, obj->min, obj->am_pm, obj->recurring); 
		fputs(line, fp_temp);
		break;
	    }

	    if ( recur == 0 )
		page_list->flag = 0;
	    else
		page_list->flag = 1;

	    strcpy(page_list->data, obj->id);	    
	    free(obj);

	    if ( ( temp = malloc(sizeof(*temp))) == NULL)
		error_sys("Error allcating memory: temp schedule_page.c");
	    page_list->next = temp;
	    temp->next = NULL;
	    page_list = temp;

	    good++;
	}
    }

    fclose(fp_temp);

    Signal(SIGCHLD, sig_chld);

    if ( good )
    {
	if ( fork() == 0 )
	{
	    send_scheduled(head,good);
	    exit(EXIT_SUCCESS);
	}
	else
	{
	    while (head)
	    {
		temp = head->next;
		free(head);
		head = temp;
	    }	    
	    free(head);
	    return(good);
	}
    }
    else
    {
	free (head);
	return ( good );
    }

    /* Should not get here */

    return(good);
    
}

sched_info *parse_sched_line(char  *data)
{
    sched_info  *obj;
    int         i,x;

    if ( (obj = malloc(sizeof(*obj))) == NULL)
      error_quit("Error allcoating memory - obj - schedule_page.c ");

    
    i = 3;
    x = 0;

    while ( *(data + i) != ' ' )
	obj->id[x++] = *(data + i++);
    obj->id[x++] = '\0';
    i++;
    x = 0;

    while ( *(data + i) != '-' )
	obj->year[x++] = *(data + i++);
    obj->year[x++] = '\0';
    i++;
    x = 0;

    while ( *(data + i) != '-' )
	obj->month[x++] = *(data + i++);
    obj->month[x++] = '\0';
    i++;
    x = 0;

    while ( *(data + i) != '-' )
	obj->day[x++] = *(data + i++);
    obj->day[x++] = '\0';
    i++;
    x = 0;

    while ( *(data + i) != '-' )
	obj->hour[x++] = *(data + i++);
    obj->hour[x++] = '\0';
    i++;
    x = 0;

    while ( *(data + i) != '-' )
	obj->min[x++] = *(data + i++);
    obj->min[x++] = '\0';
    i++;
    x = 0;

    while ( *(data + i) != '-' )
	obj->am_pm[x++] = *(data + i++);
    obj->am_pm[x++] = '\0';
    i++;
    x = 0;

    while ( *(data + i) != '\n' )
	obj->recurring[x++] = *(data + i++);
    obj->recurring[x++] = '\0';


    if ( !strcmp(obj->hour, "12") )
	strcpy(obj->hour, "0");
	
    return obj;
    
}

int my_rename (char *destination, char *new_file)
{
    char  line[MAX_LENGTH];
    FILE  *fp, *fp1;

    int   num_left = 0;


    if ( (fp = fopen(destination, "w+")) == NULL)
    {
	error_ret("Error opening destination file - my_rename - schedule_page.c");
	return (1);
    }

    if ( (fp1 = fopen(new_file, "r")) == NULL)
    {
	error_ret("Error opening new_file file - my_rename - schedule_page.c");
	fclose(fp);
	return (1);
    }

    while ( fgets(line, MAX_LENGTH, fp1) )
    {
	num_left++;
	fputs(line, fp);
	memset(line, '\0', sizeof(line));
    }
    fclose(fp);
    fclose(fp1);
    unlink(new_file);

    return ( num_left );
}

void send_scheduled(link_list *page_list, int npages)
{
    enum
    {
	lock = 0, 
	unlock = 1
    };

    FILE           *fp, *fpt;
    char           *sched_data = NULL;
    char           *temp_line;
    char           *line = NULL;
    char           *logfile = "/.gpage/gsched.dat";
    char           temp_as[128];
    char           *client_adr;
    int            flag;
    int            i;
    int            num_pages;
    int            path_length = 0;
    page_profile   *send_list, *head, *temp;
    email_data     *email_info = NULL;

    line = get_memory(MAX_LENGTH, "schedule_page - line");
    
    num_pages = npages;

    path_length = strlen(getenv("HOME")) + strlen(logfile) + 10;
    sched_data = get_memory(path_length, "schedule_page - sched_data");
    client_adr = get_memory(MAX_LENGTH, "schedule_page, client_adr");

    strcpy(sched_data, getenv("HOME"));
    strcat(sched_data, logfile);

    if ( ( email_info = malloc(sizeof(*email_info))) == NULL)
	error_quit("Error allcoating memory - email_info - server_utilities.c ");

    email_info->isp = get_memory(MAX_LENGTH, "schedule_page - email_info->isp - schedule_page");
    email_info->email_domain = get_memory(MAX_LENGTH, "schedule_page - email_info->email_domain - schedule_page");
    email_info->email_address = get_memory(MAX_LENGTH, "schedule_page - email_info->email_address - schedule_page");

    fp = file_locking(sched_data, "r+", "gsched.dat", lock);
    if (fp == NULL)
    {
	error_ret("Error: unable to open gsched.dat - send_schedule - schedule_page.c");
	free(line);
	return;
    }

    if ( (fpt = get_temp_file()) == NULL)
    {
	error_ret("Error: unable to open gpage_sch_data_tmp.tmp - send_schedule = schedule_page.c");
	free(line);
	return;
    }

    send_list = head = temp = NULL;
    send_list = get_page_list_memory(send_list);  
    head = send_list;
    head->previous = NULL;
    head->next = NULL;

    flag = 1;

    while (flag)
    {
	memset(line, MAX_LENGTH, '\0');
	if ( (fgets(line, MAX_LENGTH, fp)) == NULL )
	{
	    flag = 0;
	    break;
	}
	if (npages)
	{
	    line[strlen(line) -1] = '\0';
	    if ( !strcmp((line + 3), page_list->data) )
	    {
		if (page_list->flag)
		{
		    fputs(line, fpt); /* Write the id number */
		    fputs("\n", fpt);
		}
		npages--;
		strcpy(send_list->msg_id, page_list->data);
		fgets(email_info->isp, MAX_LENGTH, fp);

		fgets(email_info->email_domain, MAX_LENGTH, fp);

		fgets(email_info->email_address, MAX_LENGTH, fp);

		fgets(send_list->user_id, MAX_LENGTH, fp);

		fgets(send_list->receiver_number, MAX_LENGTH, fp);

		fgets(send_list->wctp_url, MAX_LENGTH, fp);

		fgets(send_list->smtp, MAX_LENGTH, fp);

		fgets(send_list->snpp_url, MAX_LENGTH, fp);

		fgets(send_list->snpp_port, MAX_LENGTH, fp);

		fgets(send_list->snpp_password, MAX_LENGTH, fp);

		fgets(send_list->protocols, MAX_LENGTH, fp);

		fgets(send_list->max_msg_length, MAX_LENGTH, fp);

		fgets(line, MAX_LENGTH, fp);
		send_list->auto_split = atoi(line);

		fgets(send_list->receiver_name, MAX_LENGTH, fp);

		fgets(send_list->message, MAX_LENGTH, fp);

		fgets(client_adr, MAX_LENGTH, fp);

		fgets(line, MAX_LENGTH, fp);
		send_list->time_stamp = atoi(line);

		/* For not we will not support rollover on no connect with a scheduled page */
		strcpy(send_list->alternative_snpp, "NULL");		
		strcpy(send_list->alternative_port, "NULL");


		fgets(line, MAX_LENGTH, fp);

/*
 *   If the page list flag is set here then we are loading a recurring page 
 *   so we need to save the data back to disk.
 */

		if (page_list->flag)
		{
		    /* We already wrote the id number */
		    fputs(email_info->isp, fpt);
		    fputs(email_info->email_domain, fpt);
		    fputs(email_info->email_address, fpt);

		    fputs(send_list->user_id, fpt);
		    fputs(send_list->receiver_number,fpt);
		    fputs(send_list->wctp_url, fpt);
		    fputs(send_list->smtp, fpt);
		    fputs(send_list->snpp_url, fpt);
		    fputs(send_list->snpp_port, fpt);
		    fputs(send_list->snpp_password, fpt);
		    fputs(send_list->protocols, fpt);
		    fputs(send_list->max_msg_length, fpt);
		    sprintf(temp_as, "%d\n", send_list->auto_split);
		    fputs(temp_as, fpt);
		    fputs(send_list->receiver_name, fpt);
		    fputs(send_list->message, fpt);
		    fputs(client_adr, fpt);
		    sprintf(temp_as, "%d\n", send_list->time_stamp);
		    fputs(temp_as, fpt);
  
		    fputs(line, fpt);
		}

		send_list->pwd_yes_no = atoi(line);
		temp = get_page_list_memory(temp);
		temp->previous = send_list;
		send_list->next = temp;
		send_list = temp;
		send_list->next = NULL;
		
		page_list = page_list->next;
		strcpy(send_list->receiver_number, "none\n");

	    }
	    else
	    {
		fputs(line, fpt);
		fputs("\n", fpt);
		for ( i = 0; i < SCHEDULED_FILE_ENTRIES; i++)
		{
		    if( fgets(line, MAX_LENGTH, fp) )
			fputs(line, fpt);
		    else
			break;
		}
	    }
	}
	else
	{
	    fputs(line, fpt);
	    while (fgets(line, MAX_LENGTH, fp))
		fputs(line, fpt);
	    flag = 0;
	}
    }
    free(line);

    rewind(fpt);
    fclose(fp);

    temp_line = get_memory(MAX_FIELD, "schedule_page - temp_line");	
    
    if ( (fp = fopen(sched_data, "w")) == NULL)
    {
	fclose(fpt);
	free(temp_line);
	return;
    }
    while(fgets(temp_line, MAX_LENGTH, fpt))
	fputs(temp_line, fp);
    fclose(fpt);
    fclose(fp);
    free(temp_line);

    file_locking(sched_data, "r+", "gsched.dat", unlock);

    free(sched_data);

    if ( num_pages > 2 )
	head = new_sort(head, num_pages);    

    Signal(SIGCHLD, sig_chld);
    temp = head;
    client_adr[strlen(client_adr) -1] = '\0';

    send_messages(head, CLOSED_SOCKET, email_info, SCHEDULE, client_adr);

    return;

}

void fix_scheduled_time(sched_info *obj, struct tm *tdata,int when)
{
    
    time_t  new_time;
    int     hour = 0;

    switch (when) {
	
    case HOURLY:		     
	tdata->tm_hour++;
	tdata->tm_year -= 1900;
	new_time = mktime(tdata);
	tdata = localtime(&new_time);

	tdata->tm_year += 1900;
	sprintf(obj->year, "%d", tdata->tm_year);
	sprintf(obj->month, "%d", tdata->tm_mon);
	sprintf(obj->day, "%d", tdata->tm_mday);

	if ( tdata->tm_hour <= 11 )
	{
	    strcpy(obj->am_pm, "1");
	    hour = tdata->tm_hour;
	    if ( hour == 0 )
		strcpy(obj->hour, "12");
	    else
		sprintf(obj->hour, "%d", hour);
	}
	else
	{
	    strcpy(obj->am_pm, "0");
	    hour = tdata->tm_hour -12;
	    if ( hour == 0 )
		strcpy(obj->hour, "12");
	    else
		sprintf(obj->hour, "%d", hour);
	}
	break;

    case DAILY:
	tdata->tm_mday++;

	tdata->tm_year -= 1900;
	new_time = mktime(tdata);
	tdata = localtime(&new_time);

	tdata->tm_year += 1900;
	sprintf(obj->year, "%d", tdata->tm_year);
	sprintf(obj->month, "%d", tdata->tm_mon);
	sprintf(obj->day, "%d", tdata->tm_mday);
	break;

    case WEEKLY:
	tdata->tm_mday += 7;

	tdata->tm_year -= 1900;
	new_time = mktime(tdata);
	tdata = localtime(&new_time);

	tdata->tm_year += 1900;
	sprintf(obj->year, "%d", tdata->tm_year);
	sprintf(obj->month, "%d", tdata->tm_mon);
	sprintf(obj->day, "%d", tdata->tm_mday);
	break;
    }

}




