/*
 * g-page Copyright (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"
#include <time.h>

static page_profile *prepare_for_transmission(page_profile *data, int protocol, int action, int cc_email_flag);
static page_profile *initialize_queue(page_profile *data, email_data *email_info, message_data  *totals);
static int   check_for_autosplit(page_profile *data, email_data *email_info);
static void  insert_node(page_profile *previous, page_profile *new);
static void  dupe_node(page_profile *node, page_profile  *data);
static void  send_results(int socket_fd, message_data  *totals, int tot_time);
static void  load_resources(int *connection_timeout, int *read_timeout);
static void  load_args(thread_data *function_args, page_profile *data, int socket_fd, email_data *email_info, message_data  *totals, int connection_timeout, 
		int server_read_timeout, int send_flag, int send_mode, int which_function, char *client_adr );
static void  send_it(thread_data *function_args);
static void  wait_for_threads(pthread_t threads_used[MAX_NUM_THREADS], int number_used);
static pthread_t  send_node(page_profile *temp_node, int socket_fd, email_data *email_info, message_data  *totals, int connection_timeout, 
			    int read_timeout, int send_flag, char *client_adr);
static int check_for_rollover(page_profile *list);

page_profile *send_messages(page_profile *head, int socket_fd, email_data *email_info, int send_flag, char *client_adr)
{
    
    message_data  *totals;
    char          page_order[MAX_FIELD];
    char          *tname;
    int           connection_timeout;
    int           read_timeout;
    int           i;
    int           x, y;
    int           cc = 0;
    int           number_of_threads;
    int           split_flag;
    int           split_nodes;
    int           rollover = 0;
    time_t        t1, t2;
    int           total_time;
    pthread_t     threads[MAX_NUM_THREADS];
    const int     PROTOCOLS = 3;
    page_profile  *temp_page_list, *temp_head, *temp_node = NULL;

    load_resources(&connection_timeout, &read_timeout);

    total_time = 0;
    
    if ( ( totals = malloc(sizeof(*totals))) == NULL )
	error_quit("Error allcoating memory - totals - send_messages.c ");

    temp_page_list = head;
    temp_head = head;

    head = initialize_queue(temp_page_list, email_info, totals);

    cc = !strcmp(head->cc_email, "yes\n");

/*
 *  If we are sending a scheduled page the protocol order is after the protocol field
 *  The data looks something like this: 1101A123\n - yeah, it's not very clean
 *  The first three fields are a 1 or a zero depending on what protocols are 
 *  supported by that pager, the next field is for wctp and tells us to send 1.0
 *  or 1.5, the 'A' is used to seperate the data, the last fields are for the 
 *  order that the protocols are used. Note this will not work corrctly for group
 *  pages that have different protocol orders. 
 */

    if ( send_flag == SCHEDULE )
    {
	head->asynch_page = 0; 
	x = 0;
	while( (*(head->protocols + x) != 'A') && (*(head->protocols + x) != '\n') )
	    x++;
	if ( *(head->protocols + x) == '\n' ) /* Error - load default */
	{
	    strcpy(page_order, "123");
	}
	else
	{
	    y = 0;
	    x++;
	    while( (*(head->protocols + x) != '\n') )
		*(page_order + y++) = *(head->protocols + x++);
	    *(page_order + y) = '\0';
	}
    }
    else
	strncpy(page_order, head->protocols_order, PROTOCOLS + 2);

    temp_page_list = head;

/*
 *     Here is a kludge - if we have a tracking number we need to make sure to force serial mode
 */

    if ( strlen(temp_page_list->tracking_number) != 0 )
	temp_page_list->asynch_page = 0;

/*
 *   When we are running in the multi thread mode we need to make sure any message which has
 *   been split gets sent to the same thread. Otherwise we risk sending the messages in the 
 *   wrong order
 */

    if ( temp_page_list->asynch_page )		/* Multi threaded mode */
    {
	t1 = time(&t1);
	number_of_threads = 0;
	split_flag = 0;
	split_nodes = 0;
	tname = get_memory(MAX_LENGTH, "tname - send_messages");
	strcpy(tname, "  ");
	temp_page_list = head;
	for (; temp_page_list->next; temp_page_list = temp_page_list->next)
	{
	    if ( temp_page_list->msg_was_split)
	    {
		if ( strcmp(tname, temp_page_list->user_id) )
		{
		    /*
		     * This is a new message that was split - this is a new thread - mark it and don't 
		     * create a thread yet. Count the number of pages with this node
		     */
		    temp_node = temp_page_list;
		    strcpy(tname, temp_page_list->user_id);
		    split_flag = 1;
		    split_nodes = 1;
		}
                /*
		 * This is message that was split - we don't need to create a new thread. Increment
		 * the number of nodes variable.
		 */
		else
		    split_nodes++;
	    }
            /* 
	     *  This is a new thread. Check to see if the split counter was set. If so, send the saved node
	     *  then send this node
	     */
	    else
	    {
		if ( split_flag )
		{
		    temp_node->split_nodes = split_nodes;
		    
		    threads[number_of_threads] = send_node(temp_node,socket_fd,email_info, totals, connection_timeout, read_timeout, send_flag, client_adr);
		    number_of_threads++;
		    
		} /* We did not split this message */
		temp_page_list->split_nodes = 1;
		
		threads[number_of_threads] = send_node(temp_page_list,socket_fd,email_info, totals, connection_timeout, read_timeout, send_flag, client_adr);
		number_of_threads++;
		split_flag = 0;

	    }

	    if ( number_of_threads == MAX_NUM_THREADS )
	    {
		wait_for_threads(threads, number_of_threads);
		number_of_threads = 0;
	    }

	}
	/*
	 *  If there is only one page or if the last page was split we catch it here
	 */
	if ( split_flag )
	{
	    temp_node->split_nodes = split_nodes;

	    threads[number_of_threads] = send_node(temp_node,socket_fd,email_info, totals, connection_timeout, read_timeout, send_flag, client_adr);
	    number_of_threads++;
	}

	free(tname);
	temp_page_list = head;

	wait_for_threads(threads, number_of_threads);

	t2 = time(&t2);
	total_time = (int )(t2 - t1);

    }
    else			/* serial mode */
    {
	t1 = time(&t1);
	for (i = 0; i < PROTOCOLS; i++)
	{

	    switch ( page_order[i] ) {
	    case '1':
		temp_page_list = prepare_for_transmission(temp_page_list, WCTP, SEND_WCTP, 0);
		if ( strlen(temp_page_list->tracking_number) == 0 )
		    head = send_wctp(temp_page_list, socket_fd, email_info, totals, connection_timeout, read_timeout, send_flag, SERIAL, client_adr, 0);
		else
		    head = send_wctp(temp_page_list, socket_fd, email_info, totals, connection_timeout, read_timeout, send_flag, SERIAL, client_adr, 1);
		break;
	    case '2':
		temp_page_list = prepare_for_transmission(temp_page_list, SNPP, SEND_SNPP, 0);
		head = send_snpp(temp_page_list, socket_fd, email_info, totals, connection_timeout, read_timeout, send_flag, SERIAL, client_adr);
		break;
	    case '3':
		temp_page_list = prepare_for_transmission(temp_page_list, SMTP, SEND_SMTP, cc);
		head = send_smtp(temp_page_list, socket_fd, email_info, totals, connection_timeout, read_timeout, send_flag, SERIAL, client_adr);
		break;
	    }
	}
	t2 = time(&t2);
	total_time = (int )(t2 - t1);
    }
    
    rollover = 0;
    temp_page_list = temp_head;
    rollover = check_for_rollover(temp_page_list);
    if ( rollover )
    {
	head = send_snpp(temp_head, socket_fd, email_info, totals, connection_timeout, read_timeout, send_flag, ROLLOVER, client_adr);
    }

    send_results(socket_fd, totals, total_time);

    free(totals);


return (head);
}

static int check_for_rollover(page_profile *list)
{
    int rollover_status;

    rollover_status = 0;
    while(list->next != NULL)    
    {
	if ( list->page_result == ERROR_ROLLOVER )
	{
	        rollover_status = 1;
		list->page_status = SEND_SNPP;
		list = list->next;
	}
	else
	    list = list->next;
    }
    
    return(rollover_status);

}
static page_profile *initialize_queue(page_profile *data, email_data *email_info, message_data  *totals)
{
    page_profile   *head;
    int            num_pages = 0;
    int            i = 0;
    char           buff[8];
    time_t         time_val;
    struct  tm     *t_struct;

    totals->sent_wctp = 0;
    totals->sent_snpp = 0;
    totals->sent_smtp = 0;
    totals->sent_total = 0;
    totals->error_wctp = 0;
    totals->error_snpp = 0;
    totals->error_smtp = 0;
    totals->error_total = 0;
    totals->no_connect = 0;
    totals->queued = 0;

    time_val = time(&time_val);
    t_struct = localtime(&time_val);
    strftime(buff, 8, " %H:%M",t_struct);

    head = data;

    while(data->next != NULL)
    {
	if ( data->time_stamp )
	{
	    if ( data->message[strlen(data->message) -1] == '\n' )
	    {
		data->message[strlen(data->message) -1] = '\0';
		strcat(data->message, buff);
		strcat(data->message, "\n");
	    }
	}
	    
	data->page_result = PAGE_INIT;

	if ( data->protocols[WCTP] == '1' )
	    data->page_status = SEND_WCTP;
	else if ( data->protocols[SNPP] == '1' )
	    data->page_status = SEND_SNPP;
	else if ( data->protocols[SMTP] == '1' )
	    data->page_status = SEND_SMTP;
	else
	{
	    data->page_status = INVALID_PAGE;
	    totals->error_total++;
	    data = data->next;
	    continue;
	}
	
	num_pages = check_for_autosplit(data, email_info);
	for ( i = 0; i < num_pages; i++)
	    data = data->next;

    }

    return head;
}

static page_profile *prepare_for_transmission(page_profile *data, int protocol, int action, int cc_email_flag)
{
    page_profile *head;

    head = data;

    while ( data->next != NULL )
    {
	if ( cc_email_flag )
	{
	    if (  ( (data->page_result == SENT_SNPP) || (data->page_result == SENT_WCTP) )
		  &&  (data->protocols[protocol] == '1') )
		data->page_status = action;
	    data = data->next;
	}
	else
	{
	    if ( ( (data->page_result != SENT_SMTP) &&
		   (data->page_result != SENT_SNPP) &&
		   (data->page_result != SENT_WCTP) ) && ( data->protocols[protocol] == '1' ) )
	    {
		data->page_status = action;
	    }
	    data = data->next;
	}
    }
    return head;
}

void print_debug(page_profile *stuff)
{
    while(stuff->next != NULL)
    {
	fprintf(stderr, "MSG: %s", stuff->message);
	stuff = stuff->next;
    }
return;
}

/*
 *  This is a bloody mess but it seems to work so I'm afraid to touch it...
 */

static int  check_for_autosplit(page_profile *data, email_data *email_info)
{
    int            pages = 1;
    int            size = 0;
    int            max_msg = 0;
    int            i = 0;
    int            count = 0;
    int            tag_size = 4;
    page_profile   *node = NULL;
    char           *msg_string, *msg_string_ptr;
    
    msg_string = get_memory(MAX_LENGTH, "send_messages - msg_string");
    
    strcpy(msg_string, data->message);
    msg_string_ptr = msg_string;

    size = ( strlen(data->message) -1);
    max_msg = atoi(data->max_msg_length); 

    if ( data->auto_split )
    {
	if ( size > max_msg )
	{
	    while ( ( size - max_msg ) >= 0 )
	    {
		pages++;
		size -= (max_msg - tag_size);
		if ( size <= max_msg )
		    break;
	    }
	    count = 0;

	    memset(data->message, '\0', (strlen(data->message) + 1));
	    while ( count < (max_msg - tag_size) )
		*(data->message + count++) = *msg_string++;
	    strcat(data->message, " -->\n");
	    count = 0;
	    for ( i = 1; i < pages; i++)
	    {
		data->msg_was_split = 1;
		node = get_page_list_memory(node);
		dupe_node(node, data);

		while ( count < (max_msg - tag_size) )
		    *(node->message + count++) = *msg_string++;

		if ( i == (pages - 1) )
		    while ( ( *(node->message + count++) = *msg_string++) );
		else
		    strcat(node->message, " -->\n");

		insert_node(data, node);
		data = node;
		count = 0;
	    }
	    free(msg_string_ptr);
	    return ( pages );
	}
	else
	{
	    data->msg_was_split = 0;
	    free(msg_string);
	    return 1;
	}
    }
    else
    {
	data->msg_was_split = 0;
	free(msg_string);
	return 1;
    }
}

static void  insert_node(page_profile *previous, page_profile *new)
{
    page_profile *temp = NULL;
    temp = get_page_list_memory(temp);
    
    if ( previous->next != NULL )
    {
	temp = previous->next;
	previous->next = new;
	new->previous = previous;
	new->next = temp;
    }
    else
    {
	new->next = NULL;
	new->previous = previous;
	previous->next = new;
    }
}

static void  dupe_node(page_profile *node, page_profile  *data)
{
    strcpy(node->user_id, data->user_id);
    strcpy(node->msg_id, data->msg_id);
    strcpy(node->receiver_number, data->receiver_number);
    strcpy(node->receiver_name, data->receiver_name);
    strcpy(node->wctp_url, data->wctp_url);
    strcpy(node->smtp, data->smtp);
    strcpy(node->snpp_url, data->snpp_url);
    strcpy(node->snpp_port, data->snpp_port);
    strcpy(node->snpp_password, data->snpp_password);
    strcpy(node->max_msg_length, data->max_msg_length);
    strcpy(node->protocols, data->protocols);
    strcpy(node->alternative_snpp, data->alternative_snpp);
    strcpy(node->alternative_port, data->alternative_port);
    node->connection_fail_mode = data->connection_fail_mode;
    node->auto_split = data->auto_split;
    node->page_status = data->page_status;
    node->page_result = data->page_result;
    node->msg_was_split = data->msg_was_split;
}

/* 
 *  Send the final results back to the client when we are all done
 */
static void  send_results(int socket_fd, message_data  *totals, int tot_time)
{
    char     *buffer;
    char     *tmp_str;
    char     *time_string;

    if ( socket_fd < 0 )
	return;

    buffer = get_memory(MAX_LENGTH, "send_messages - buffer");

    tmp_str = get_memory(MAX_LENGTH, "send_messages - tmp_string");

    time_string = get_memory(MAX_LENGTH, "send_messages - time_string");

    if (tot_time == 1)
	sprintf(time_string, "Elapsed time: %d second", tot_time);
    else
	sprintf(time_string, "Elapsed time: %d seconds", tot_time);

/*
 *   We tell the client if a page was sent or failed to let it perform
 *   some special functions based upon that data
 */

    if ( totals->sent_total > 0 )
	write_to_status_line(socket_fd, "sent\n", strlen("sent\n"), SPECIAL);	
    else
	write_to_status_line(socket_fd, "failed\n", strlen("failed\n"), SPECIAL);	


    sprintf(buffer, "Done: %d", totals->sent_total);
    if (totals->sent_total == 1)
	strcat(buffer, " message sent  ");
    else
	strcat(buffer, " messages sent  ");

    if ( totals->queued > 0 )
    {
	sprintf(tmp_str, "%d", totals->queued);
	strcat(buffer, tmp_str);
	
	if ( totals->queued == 1 )
	    strcat(buffer, " message queued  ");
	else
	    strcat(buffer, " messages queued  ");

    }

    sprintf(tmp_str, "%d", totals->error_total);
    strcat(buffer, tmp_str);

    if ( totals->error_total == 1 )
	strcat(buffer, " message failed\n");
    else
	strcat(buffer, " messages failed\n");

    write_to_status_line(socket_fd, buffer, strlen(buffer), ALWAYS_UPDATE);	
    write_to_status_line(socket_fd, time_string, strlen(time_string), UPDATE_SMALL);	
    

    free(buffer);
    free(tmp_str);
    free(time_string);

    return;
}

static void  load_resources(int *connection_timeout, int *read_timeout)
{
    char          *resource_data;
    int           resource;

    resource_data = get_memory(64, "send_messages - resource_data");

    if ( (resource = get_resource_data("/etc/gpage.conf", "connect_timeout", resource_data)) != 1)
    {
	info_msg("Error reading gpage.conf.. using defaults ");
	*connection_timeout = 60;
    }
    else
	*connection_timeout = atoi(resource_data);

    memset(resource_data, '\0', sizeof(resource_data));
    if ( (resource = get_resource_data("/etc/gpage.conf", "read_timeout", resource_data)) != 1)
    {
	info_msg("Error reading gpage.conf.. using defaults ");
	*read_timeout = 120;
    }
    else
	*read_timeout = atoi(resource_data);

    free(resource_data);

}

static void  load_args(thread_data *function_args, page_profile *data, int socket_fd, email_data *email_info, message_data  *totals, int connection_timeout, 
		int server_read_timeout, int send_flag, int send_mode, int which_function, char *client_adr )
{
    function_args->data = data;
    function_args->socket_fd = socket_fd;
    function_args->email_info = email_info;
    function_args->totals = totals;
    function_args->connection_timeout = connection_timeout;
    function_args->server_read_timeout = server_read_timeout;
    function_args->send_flag = send_flag;
    function_args->send_mode = send_mode;
    function_args->which_function = which_function;
    function_args->client_ip = client_adr;
}

static void  send_it(thread_data *function_args)
{
    switch (function_args->which_function) {
    case SEND_SNPP:
	send_snpp(function_args->data, function_args->socket_fd, function_args->email_info, function_args->totals,
		  function_args->connection_timeout, function_args->server_read_timeout, function_args->send_flag, function_args->send_mode, 
		  function_args->client_ip);
	free(function_args);
	break;
    case SEND_SMTP:
	send_smtp(function_args->data, function_args->socket_fd, function_args->email_info, function_args->totals,
		  function_args->connection_timeout, function_args->server_read_timeout, function_args->send_flag, function_args->send_mode,
		  function_args->client_ip);
	free(function_args);
	break;
    case SEND_WCTP:
	send_wctp(function_args->data, function_args->socket_fd, function_args->email_info, function_args->totals,
		  function_args->connection_timeout, function_args->server_read_timeout, function_args->send_flag, function_args->send_mode,
		  function_args->client_ip, 0);
	free(function_args);
	break;
    }

}

static void  wait_for_threads(pthread_t threads_used[MAX_NUM_THREADS], int number_used)
{
    int x;

    for ( x = 0; x < number_used; x++)
	pthread_join(threads_used[x], NULL);

}

static pthread_t  send_node(page_profile *temp_node, int socket_fd, email_data *email_info, message_data  *totals, int connection_timeout, 
			    int read_timeout, int send_flag, char *client_adr)
{
    thread_data   *function_args;
    pthread_t     new_thread;

    if ( (function_args = malloc(sizeof(*function_args))) == NULL )
	error_quit("Error allocating memory - function_args - send_messages.c");

    load_args(function_args, temp_node, socket_fd, email_info, totals, connection_timeout, read_timeout, send_flag, ASYNCH, temp_node->page_status, client_adr);

    pthread_create(&new_thread, NULL, (void *)&send_it, function_args);

    return (new_thread);
}
