/*
 * g-page Copyright (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"


#define PORT             25
#define RCPT_TO          1
#define SEND_DATA        2
#define SEND_MSG         3
#define ADDRESS_REJECTED 4
#define EMAIL_TO         5
#define DONE_ERROR       6
#define SEND_QUIT        7
#define EMAIL_FROM       8
#define MSG_DONE         9
#define MSG_REJECTED     10
#define CONNECT          11
#define FATAL_ERROR      12
#define NO_CONNECT       13
#define CONNECTION_LOST  14

extern pthread_mutex_t update_client_mutex;

page_profile  *send_smtp(page_profile *data, int socket_fd, email_data *email_info, message_data  *totals, int server_timeout, 
			 int server_read_timeout, int send_flag, int send_mode, char *client_adr)
{
    extern int   debug_flag;
    extern int   dial_script;
    extern char  *internet_script;
    static int   connect_attempts = 0;

  
    int       sock = 0;
    int       more_pages_to_send;
    int       number_of_nodes;
    int       mutex_status;
    int       num, sent, failed;
    int       state;

    char      *email_greeting = "";
    char      *email_from = "";
    char      email_to[MAX_LENGTH] = {"rcpt to:"};
    char      *data_statement = "data\n";
    char      *to_header;
    char      *from_header;
    char      *subject = "Subject: cc Alpha Message\n";
    char      *end = "\n.\n";
    char      *e_quit = "quit\n";
    char      *code = "";
    char      *buff = "";
    char      error_msg[MAX_LENGTH];


    char      log[128];

    //print_list_debug1(data);
    *(email_info->email_domain + strlen(email_info->email_domain) - 1) = '\0';
    
  /*   Initialize the email greeting, email address, and isp domain */

    email_greeting = get_memory(MAX_LENGTH, "send_smtp - email_greeting");

    to_header = get_memory(MAX_LENGTH, "send_smtp - to_header");
    from_header = get_memory(MAX_LENGTH, "send_smtp - from_header");


    sprintf(from_header, "From: %s", email_info->email_address);
    
    strcpy(email_greeting, "helo ");
    strcat(email_greeting, email_info->isp);
    email_from = get_memory(MAX_LENGTH, "send_smtp - email_from");
    
    strcpy(email_from,"mail from:");
    strcat(email_from, email_info->email_address);
    
    buff = get_memory(MAX_LENGTH, "send_smtp - buff");
    

    state = CONNECT;

    if ( send_mode == ASYNCH )
	number_of_nodes = data->split_nodes;
    else
	number_of_nodes = 0;

    more_pages_to_send = are_we_done(data, send_mode, number_of_nodes);


    /*   Begin the main while loop. Cycle through each receiver in the list  */

    sent = failed = 0;
    while ( more_pages_to_send )
    {	  	      
	if (data->page_status == SEND_SMTP)
	{
	    memset(to_header, '\0', sizeof(to_header));
	    sprintf(to_header, "To: %s", data->smtp);

	    switch (state) 
	    {
		
	    case CONNECT:
		
		if ( (sock = connect_to_remote_server(email_info->email_domain, PORT, socket_fd, server_timeout)) == -1)
		{
		    info_msg("Error connecting with mail server");
		    state = NO_CONNECT;
		}
		else
		{
		    if (debug_flag)
		    {
			sprintf(log, "Connect with %s on port: %d\n", email_info->email_domain, PORT);
			log_transaction(log);
		    }

		    /* Connect... get the initial email_greeting from the smtp port */
		    
		    code = parse_codes(mread(sock, server_read_timeout), 3);
		    if ( code == NULL )
			state = CONNECTION_LOST;
		    else if (  atoi(code) != 220 )
		    {
			strcpy(error_msg, "Error logging on to mail server\n");

			if ( send_mode == ASYNCH )
			{
			    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
				error_quit("Error locking mutex - possible data corruption");
			    write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
				error_quit("Error unlocking mutex - possible data corruption");
			}
			else
			{
			    write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			}
			state = FATAL_ERROR;
		    }	  
		    else
			state = EMAIL_FROM;
		    
		    free(code);
		    /* Send the email_greeting... */
		    
		    num = write_num(sock, email_greeting, strlen(email_greeting));
		    if (debug_flag)
			log_it("SENT-> ", email_greeting);
		    
		    code = parse_codes(mread(sock, server_read_timeout), 3);
		    
		    /*   We assume anything except a 250 is not okay */
		    if ( code == NULL )
		    {
			state = CONNECTION_LOST;
			free(code);
			break;
		    }
		    else if ( (atoi(code)) != 250)
		    {
			strcpy(error_msg, "Error logging on to mail server:\n");
			if ( send_mode == ASYNCH )
			{
			    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
				error_quit("Error locking mutex - possible data corruption");
			    write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
				error_quit("Error unlocking mutex - possible data corruption");
			}
			else
			{
			    write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			}
			state = FATAL_ERROR;
		    }
		    else
			state = EMAIL_FROM;

		    free(code);
		}
		
		break;
		    
	    case EMAIL_FROM: 
		
		num = write_num(sock, email_from, strlen(email_from));
		if (debug_flag)
		    log_it("SENT-> ", email_from);
		
		code = parse_codes(mread(sock, server_read_timeout), 3);
		if ( code == NULL )
		{
		    state = CONNECTION_LOST;
		    free(code);
		    break;
		}
		if ( atoi(code) != 250)
		{
		    strcpy(error_msg, "Email server rejected sender: No messages sent\n");
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			send_log_data(data->receiver_name, data->message, "114", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			send_log_data(data->receiver_name, data->message, "114", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    state =  DONE_ERROR;
		}
		else
		    state = RCPT_TO;

		free(code);
		break;
		
	    case RCPT_TO:
		
		sprintf(email_to, "rcpt to:%s", data->smtp);
		write_num(sock, email_to, strlen(email_to));
		if (debug_flag)
		    log_it("SENT-> ", email_to);
		
		code = parse_codes(mread(sock, server_read_timeout), 3);
		if ( code == NULL )
		{
		    state = CONNECTION_LOST;
		    free(code);
		    break;
		}
		else if ( atoi(code) != 250)
		{
		    sprintf(error_msg, "Mail server rejected %s", data->smtp);
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
		    }
		    state = ADDRESS_REJECTED;
		}
		else
		{
		    sprintf(buff, "Sending to %s", data->receiver_name);
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			write_to_status_line(socket_fd, buff, strlen(buff), STATUS_UPDATE);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			write_to_status_line(socket_fd, buff, strlen(buff), STATUS_UPDATE);
		    }
		    state = SEND_DATA;			  
		}

		free(code);
		break;
		
	    case SEND_DATA:
		write_num(sock, data_statement, strlen(data_statement));
		if (debug_flag)
		    log_it("SENT-> ", data_statement);
		
		code = parse_codes(mread(sock, server_read_timeout), 3);
		if ( code == NULL )
		{
		    state = CONNECTION_LOST;
		    free(code);
		    break;
		}
		else if ( (atoi(code) != 250) && ( atoi(code) != 354) )
		{
		    strcpy(error_msg, "Email server rejected data");
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			send_log_data(data->receiver_name, data->message, "115", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			send_log_data(data->receiver_name, data->message, "115", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    state = DONE_ERROR;
		}
		else
		    state = SEND_MSG;

		free(code);
		break;
		
	    case SEND_MSG:
		if ( !strcmp(data->cc_email, "yes\n") )
		{
		    num = write_num(sock, subject, strlen(subject));		
		    if (debug_flag)
			log_it("SENT-> ", subject);
		}

		num = write_num(sock, to_header, strlen(to_header));
		num = write_num(sock, from_header, strlen(from_header));
		num = write_num(sock, "\n", strlen("\n"));

		num = write_num(sock, data->message, strlen(data->message));
		if (debug_flag)
		{
		    log_it("SENT-> ", to_header);
		    log_it("SENT-> ", from_header);
		    log_it("SENT-> ", "\n");
		    log_it("SENT-> ", data->message);
		}
		
		/* Send the period -  we are done */
		
		write_num(sock, end, strlen(end));
		if (debug_flag)
		    log_it("SENT-> ", ".\n");
		
		code = parse_codes(mread(sock, server_read_timeout), 3);
		if ( code == NULL )
		{
		    state = CONNECTION_LOST;
		    free(code);
		    break;
		}
		else if ( atoi(code) != 250 )
		{
		    strcpy(error_msg, "Email server rejected message\n");
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			write_to_status_line(socket_fd, error_msg, strlen(error_msg), STATUS_UPDATE);
		    }
		    state = MSG_REJECTED;
		}		  
		else
		    state = MSG_DONE;

		free(code);
		break;
		
	    case DONE_ERROR:
		
		if ( strcmp(data->cc_email, "yes\n")  )
		{
		    if ( data->page_result != PAGE_ERROR )
			totals->error_total++;
		    totals->error_smtp++;
		    data->page_result = PAGE_ERROR;
		    data->page_status = ERROR_SMTP;
		}
		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		break;

	    case ADDRESS_REJECTED:
		if ( strcmp(data->cc_email, "yes\n")  )
		{
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "111", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "111", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    if ( data->page_result != PAGE_ERROR )
			totals->error_total++;
		    totals->error_smtp++;
		    data->page_result = PAGE_ERROR;
		    data->page_status = ERROR_SMTP;
		    totals->error_smtp++;
		}
		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		state = RCPT_TO;
		break;
		
	    case MSG_REJECTED:
		if ( strcmp(data->cc_email, "yes\n")  )
		{
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "112", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "112", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    if ( data->page_result != PAGE_ERROR )
			totals->error_total++;
		    data->page_result = PAGE_ERROR;
		    data->page_status = ERROR_SMTP;
		    totals->error_smtp++;
		}
		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		state = DONE_ERROR;
		break;
		
	    case MSG_DONE:
		if ( strcmp(data->cc_email, "yes\n")  )
		{
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "010", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "010", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    if ( data->page_result == PAGE_ERROR )
			totals->error_total--;
		    data->page_result = SENT_SMTP;
		    data->page_status = SENT_SMTP;
		    totals->sent_total++;
		    totals->sent_smtp++;
		}
		state = EMAIL_FROM;
		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		break;

	    case FATAL_ERROR:
		if ( strcmp(data->cc_email, "yes\n")  )
		{
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "113", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "113", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    if ( data->page_result != PAGE_ERROR )
			totals->error_total++;
		    data->page_result = PAGE_ERROR;
		    data->page_status = ERROR_SMTP;
		    totals->error_smtp++;
		}
		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		break;

	    case CONNECTION_LOST:
		if ( strcmp(data->cc_email, "yes\n")  )
		{
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "116", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "116", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    if ( data->page_result != PAGE_ERROR )
			totals->error_total++;
		    data->page_result = PAGE_ERROR;
		    data->page_status = ERROR_SMTP;
		    totals->error_smtp++;
		}
		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		break;

	    case NO_CONNECT:
		/*   Here is the new dial on no connect script code    */
		/* I need to try to figure out the difference betweeen no internet connection
		   and a remote server that is not available  */
		
		if ( !strcmp(data->dial_on_no_connect, "yes\n") ) /* The comes from the client */
		{
		    if ( (connect_attempts == 0) && (dial_script == 1) ) /* dial_script comes from a server switch */
		    {
			connect_attempts++;
			if ( fork() == 0 )	/* Child process */
			{
			    char *my_argv[2];
			    
			    my_argv[0]= internet_script;
			    my_argv[1]= 0;
			    
			    if ( (execv(internet_script, my_argv)) == -1 )
				error_quit("Error execv with this script: %s ", internet_script);
			    
			}
			else
			{	/* parent process */
			    sleep(CONNECT_SLEEP_VALUE);	/* This is defined in data_structures.h  */
				/* Here is where I try to see if an intenet connection has been made */
				/* Then we reset the state to connect and give it another try */
			    state = CONNECT;
			    break;
			}
		    }
		    else if ( (connect_attempts == 1) && (dial_script == 1) )
		    {
			sleep(15);
			connect_attempts++;
			state = CONNECT;
			break;
		    }
		}
		
		if ( strcmp(data->cc_email, "yes\n")  )
		{
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "113", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "113", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    if ( data->page_result != PAGE_ERROR )
			totals->error_total++;
		    totals->no_connect++;
		    clean(data->alternative_snpp);
		    if ( strcmp(data->alternative_snpp,"NULL") ) /* rollover on no connect is active is this is not NULL */
		    {
			data->page_result = ERROR_ROLLOVER;
		    }
		    else
			data->page_result = PAGE_ERROR;

		    data->page_status = ERROR_SMTP;			
		    totals->error_smtp++;
		}
		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		break;
		
	    }	/* End of switch statement */
		
	} /* End of if smtp block */
	else
	{
	    data = data->next;
	    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
	}
	    
    }	/* End of main while loop */
	
    /* Send the quit message to the mail server */

    if ( totals->sent_smtp > 0 )
    {
	write_num(sock, e_quit, strlen(e_quit));
	if (debug_flag)
	    log_it("SENT-> ", e_quit);
    }
    

    free(buff);
    free(email_from);
    close(sock);
    free(email_greeting);      
    free(to_header);
    free(from_header);
    return data;
}

/*
 *   mread: read a line from a remote server and pass it back to the calling function. 
 *          We can timeout while waiting for the file descriptor to become readable.
 */

char *mread(int fd, int timeout)
{
    extern int debug_flag;

    char   retbuf[MAX_LENGTH] = "";
    char   *tload = NULL;
    int    x = 0;
    int    max_fd;
    int    ret_val = 0;
    //int    val, len;
    fd_set rset;
    struct timeval  t_struct;

    if ( fd < 0 )
	return(NULL);

    FD_ZERO(&rset);
    tload = get_memory(MAX_LENGTH, "send_smtp - tload");

    while (1)
    {
	FD_SET(fd, &rset);
	max_fd = fd + 1;

	t_struct.tv_sec = timeout;
	t_struct.tv_usec = 0;
	//getsockopt(fd, SOL_SOCKET, SO_ERROR, &val, &len);
	//fprintf(stderr, "VAL is %d\n", val);

	ret_val = select(max_fd, &rset, NULL, NULL, &t_struct);

	if (errno == EINTR)
	    continue;
	
	if ( ret_val > 0 )
	{
	    while ( (x = read_a_line(fd, retbuf, sizeof(retbuf), timeout)) > 0 )
	    {
		//fprintf(stderr, "%s\n", retbuf);
		if (retbuf[strlen(retbuf)-1] == '\n')
		{
		    if (debug_flag)
			log_transaction(retbuf);
		    strcpy(tload, retbuf);
		    return tload;
		}
	    }

	    if (strlen(retbuf) == 0) /* Something bad has heppened to the server we were talking to */
		return (NULL);

	    if (debug_flag)
		log_transaction(retbuf);

	    strcpy(tload, retbuf);
	    return tload;
	}
	else if ( ret_val == 0 )
	{
	    strcpy(tload, "920");	/* Timeout reading from server */
	    return tload;
	}
	else if ( ret_val == -1 )
	{
	    strcpy(tload, "888");	/* Select error */
	    return tload;
	}
    }
    return(NULL);
}
     
char *parse_codes(char *buffer, int num_char)
{
    int i;
    int max_code_length = 10;
    char *code;

    //fprintf(stderr, "BUFFER: %s\n", buffer);
    code = get_memory(max_code_length, "send_smtp - code");

    if ( buffer == NULL )
	return(NULL);

    for (i = 0; i < num_char; i++){
	*(code + i) = *(buffer + i);}
    code[3] = '\0'; 
    //fprintf(stderr, "CODE: %s\n", code);
    return code;
}

void print_list_debug1(page_profile *data)
{
    while(data->next != NULL)
    {
	fprintf(stderr, "User MSG: %s", data->message);
	data = data->next;
    }   
}

void log_it(char *first, char *second)
{
    char buffer[2048];

    strcpy(buffer, first);
    strcat(buffer, second);
    log_transaction(buffer);
}

void log_transaction(char *data_buffer)
{
    FILE    *fp;
    time_t  timeval;
    char    buffer[MAX_LENGTH];
    char    *home;
    char    *logfile = "/.gpage/g-page-trans.log";
    int     path_length = 0;
    
    path_length = strlen(getenv("HOME")) + strlen(logfile) + 10;
    home = get_memory(path_length, "send_smtp - path_length");

    strcpy(home, getenv("HOME"));
    strcat(home, logfile);

    memset(buffer, '\0', sizeof(buffer));
    time(&timeval);
    strcpy(buffer, ctime(&timeval));
    buffer[strlen(buffer) -1] = ' '; /* Kill newline character */
    strcat(buffer, " -- ");
    strcat(buffer, data_buffer);

    if ( ( fp = fopen(home, "a+")) == NULL)
	error_ret("Error opening g-page-trans.log");

    fputs(buffer, fp);
    fclose(fp);
    free(home);
}         

void send_log_data(char *name, char *message, char *code, int socket_fd, int send_flag, char *msg_id, char *client_adr, char *tracking, char *uid)
{
    extern int audit_pages;

    enum
    {
	lock = 0,
	unlock = 1,
	MAX_RETRIES = 120
    };

    FILE    *fp;
    time_t  timeval;
    char    *buffer = NULL;
    char    log_time[32];
    char    *logname = "/.gpage/gpage_sched_log.dat";
    char    *audit_log_name = "/.gpage/server_log.dat";
    char    *log_file = NULL;
    char    *audit_file = NULL;
    int     flag;
    int     path_length = 0;

    buffer = get_memory(MAX_LENGTH + 150 + 3 + 32, "send_smtp - buffer");
    
    time(&timeval);
    
    strcpy(log_time, ctime(&timeval));
    clean(name);
    clean(log_time);

    if ( audit_pages )
    {
	path_length = strlen(getenv("HOME")) + strlen(audit_log_name) + 10;
	audit_file = get_memory(path_length, "send_smtp - audit_file");
	strcpy(audit_file, getenv("HOME"));
	strcat(audit_file, audit_log_name);
	if ( ( fp = fopen(audit_file, "a+")) == NULL )
	    error_ret("Error opening log server log file");
	else
	{
	    fputs(client_adr, fp);
	    fputs(" ", fp);
	    sprintf(buffer, "%s, %s, %s, %s", code, log_time, name, message);
	    fputs(buffer, fp);
	    fclose(fp);
	}
	free(audit_file);	    
    }
	

    switch (send_flag){

    case SEND_NOW:
        //memset(buffer, '\0', sizeof(buffer));
	sprintf(buffer, "%s, %s, %s, %s", code, log_time, name, message);
	write_to_status_line(socket_fd, buffer, strlen(buffer), LOG_DATA);

	free(buffer);
	break;

    case SCHEDULE:
	assert( tracking != NULL );

	if ( strlen(tracking) == 0 )
	    strcpy(tracking, "NULL");

	strip_crap(uid);

	path_length = strlen(getenv("HOME")) + strlen(logname) + 10;
	log_file = get_memory(path_length, "send_smtp - log_file");
	strcpy(log_file, getenv("HOME"));
	strcat(log_file, logname);

	flag = MAX_RETRIES;
        //memset(buffer, '\0', sizeof(buffer));
	sprintf(buffer, "%s %u, %s, %s, %s, %s, %s, %s", msg_id, (unsigned int)timeval, code, log_time, name, tracking, uid, message);
	while ( flag )
	{
	    fp = file_locking(log_file, "a+", "gpage_sched_log.dat", lock);
	    if (fp == NULL)
	    {
		flag--;
		sleep(1);
	    }
	    else
	    {
		fputs(buffer, fp);
		fclose(fp);
		file_locking(log_file, "a+", "gpage_sched_log.dat", unlock);
		free(buffer);
		flag = 0;
	    }
	}
	free(log_file);
	break;
    }

    strcat(name, "\n");
}

void clean( char *buffer)
{
    
    if ( *(buffer + strlen(buffer) - 1) == '\n' )
	*(buffer + strlen(buffer) - 1) = '\0';  
}


