/*
 * g-page Copyright (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"
 
#define ERROR             0
#define PAGE_ID           1  
#define MESSAGE           2
#define SEND              3
#define FATAL_ERROR       5
#define CONNECT           6
#define CONNECT_ERROR     7
#define TIMEOUT_ERROR     8
#define PASSWORD          9
#define TWO_WAY          10
#define NO_QUEUE         11
#define QUEUED           12
#define CONNECTION_LOST  13
#define NO_CONNECT       14


extern pthread_mutex_t update_client_mutex;
static void handle_debug(char *buffer);
static char *parse_two_way(char *server_response, char *msg_tag, char *pass_code);


/*
 *      We cycle through the linked list and try to send to anybody who has the snpp protocol marked as active. 
 *      I need to do a little better job of error handling with regards to the response back from the snpp server.
 */
      
page_profile  *send_snpp(page_profile *data, int socket_fd, email_data *email_info, message_data  *totals, int connection_timeout, 
			 int server_read_timeout, int send_flag, int send_mode, char *client_adr)
{
    extern int   dial_script;
    extern char  *internet_script;
    static int   connect_attempts = 0;

    enum 
    {
	SNPP_ONE_WAY = 1, 
	SNPP_ONE_POINT_FIVE = 2,
	SNPP_TWO_WAY = 3
    };

    int        state = 0;
    int        sock  = -1;
    int        more_pages_to_send;
    int        wrote = 0;
    int        number_of_nodes;
    int        mutex_status;
    int        page_method;

    char       my_pid[10];
    char       *code = "";
    char       *buffer = "";
    char       *update = "";
    char       new_connection[2];
    char       *current_url = "";
    char       *server_response;
    char       *msg_tag = NULL;
    char       *pass_code = NULL;
    char       *two_way_data;

    current_url = get_memory(MAX_LENGTH, "send_snpp - current_url");
    strcpy(current_url, data->snpp_url);
    buffer = get_memory(MAX_LENGTH, "send_snpp - buffer");
    update = get_memory(MAX_LENGTH, "send_snpp - update");

    snprintf(my_pid, 9, "%d ", getpid());

    //fprintf(stderr, "HERE IS THE SNPP PROTOCOL: %c\n", data->protocols[4]);
    state = CONNECT;

    if ( send_mode == ASYNCH )
	number_of_nodes = data->split_nodes;
    else
	number_of_nodes = 0;

    more_pages_to_send = are_we_done(data, send_mode, number_of_nodes);

    while ( more_pages_to_send )
    {
	if ( data->page_status == SEND_SNPP )
	{
	    page_method = SNPP_ONE_WAY;	/* Set this as the default incase something evil happens here */

	    if ( data->protocols[4] == '0' )
		page_method = SNPP_ONE_WAY;
	    else if ( data->protocols[4] == '1' )
		page_method = SNPP_ONE_POINT_FIVE;
	    else if ( data->protocols[4] == '2' )
		page_method = SNPP_TWO_WAY;

		switch (state) {
		    
		case CONNECT:
		    if (send_mode == ROLLOVER)
		    {
			strcpy(data->snpp_url, data->alternative_snpp);
			strcpy(data->snpp_port, data->alternative_port);
		    }
		    if ( (sock = new_url(current_url, data->snpp_url, data->snpp_port, socket_fd, sock, new_connection, 
					 server_read_timeout, connection_timeout)) != -1 )
		    {
			if ( atoi(new_connection) )
			{
			    code = parse_codes(mread(sock, server_read_timeout), 3);
			    if ( code == NULL )
				state = CONNECTION_LOST;
			    else if ( atoi(code) == 220 )
			    {
				if ( data->pwd_yes_no )
				    state = PASSWORD;
				else if ( page_method == SNPP_ONE_WAY )
				    state = PAGE_ID;
				else
				    state = TWO_WAY;
			    }
			    else if ( (atoi(code) == 920) || (atoi(code) == 888) )
			    {
				state = TIMEOUT_ERROR;
			    }
			    else
			    {
				state = FATAL_ERROR;
			    }
			    free(code);
			}
			else
			{
			    if ( data->pwd_yes_no )
				state = PASSWORD;
			    else if ( page_method == SNPP_ONE_WAY )
				state = PAGE_ID;
			    else
				state = TWO_WAY;
			}
		    }
		    else
		    {
			state = CONNECT_ERROR;
		    }

		    break;

		case PASSWORD:
		    data->snpp_password = fix_string(data->snpp_password);
		    sprintf(buffer, "LOGI %s", data->snpp_password);		  

		    wrote = write_num(sock, buffer, strlen(buffer));
		    if ( wrote == -1 )
		    {
			state = CONNECTION_LOST;
			break;
		    }

		    handle_debug(buffer);
		    code = parse_codes(mread(sock, server_read_timeout), 3);
		    if ( code == NULL )
			state = CONNECTION_LOST;
		    else if ( atoi(code) == 220 )
		    {
			if ( page_method == SNPP_ONE_WAY )
			    state = PAGE_ID;
		        else
			    state = TWO_WAY;
		    }
		    else
			state = FATAL_ERROR;

		    free(code);
		    break;

		case PAGE_ID:
		    data->receiver_number = fix_string(data->receiver_number);
		    sprintf(buffer, "PAGE %s", data->receiver_number);
		    wrote = write_num(sock, buffer, strlen(buffer));
		    if ( wrote == -1 )
		    {
			state = CONNECTION_LOST;
			break;
		    }
		    handle_debug(buffer);
		    code = parse_codes(mread(sock, server_read_timeout), 3);
		    if ( code == NULL )
		    {
			state = CONNECTION_LOST;
		    }
		    else if ( atoi(code) == 250 || atoi(code) == 214 )
		    {
			state = MESSAGE;
		    }
		    else if ( (atoi(code) == 920) || (atoi(code) == 888) )
		    {
			state = TIMEOUT_ERROR;
		    }
		    else
		    {
			memset(update, '\0', sizeof(update));
			sprintf(update, "Server rejected %s", data->receiver_name);
			if ( send_mode == ASYNCH )
			{
			    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
				error_quit("Error locking mutex - possible data corruption");
			    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
			    send_log_data(data->receiver_name, data->message, "121", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
				error_quit("Error unlocking mutex - possible data corruption");
			}
			else
			{
			    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
			    send_log_data(data->receiver_name, data->message, "121", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			}
			state = ERROR;
		    }
		    free(code);
		    break;

		case TWO_WAY:
		    strcpy(buffer, "2WAY\r\n");
		    write_num(sock, buffer, strlen(buffer));
		    handle_debug(buffer);
		    code = parse_codes(mread(sock, server_read_timeout), 3);
		    if ( code == NULL )
		    {
			state = CONNECTION_LOST;
			free(code);
			break;
		    }
		    else if ( atoi(code) == 250)
			state = PAGE_ID;
		    else if ( (atoi(code) == 920) || (atoi(code) == 888) )
			state = TIMEOUT_ERROR;
		    else
		    {
			memset(update, '\0', sizeof(update));
			strcpy(update, "Error establishing connection for 2 way\n");
			if ( send_mode == ASYNCH )
			{
			    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
				error_quit("Error locking mutex - possible data corruption");
			    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
			    send_log_data(data->receiver_name, data->message, "121", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
				error_quit("Error unlocking mutex - possible data corruption");
			}
			else
			{
			    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
			    send_log_data(data->receiver_name, data->message, "121", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			}
			state = ERROR;
		    }
		    free(code);

		    break;

		case MESSAGE:
		    data->message = fix_string(data->message);
		    sprintf(buffer, "MESS %s", data->message);
		    wrote = write_num(sock, buffer, strlen(buffer));
		    if ( wrote == -1 )
		    {
			state = CONNECTION_LOST;
			break;
		    }
		    handle_debug(buffer);

		    code = parse_codes(mread(sock, server_read_timeout), 3);
		    if ( code == NULL )
		    {
			state = CONNECTION_LOST;
		    }
		    //else if ( atoi(code) == 250)
		    else if ( code[0] == '2' )
		    {
			state = SEND;
		    }
		    else if ( (atoi(code) == 920) || (atoi(code) == 888) )
		    {
			state = TIMEOUT_ERROR;
		    }
		    else
		    {
			state = ERROR;
			if ( send_mode == ASYNCH )
			{
			    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
				error_quit("Error locking mutex - possible data corruption");
			    send_log_data(data->receiver_name, data->message, "122", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
				error_quit("Error unlocking mutex - possible data corruption");
			}
			else
			{
			    send_log_data(data->receiver_name, data->message, "122", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			}

		    }
		    free(code);
		    break;

		case SEND:
		    sprintf(buffer, "SEND\r\n");
		    wrote = write_num(sock, buffer, strlen(buffer));
		    if ( wrote == -1 )
		    {
			state = CONNECTION_LOST;
			break;
		    }

		    handle_debug(buffer);
		    server_response = mread(sock, server_read_timeout);
		    if ( (page_method = SNPP_TWO_WAY) || (page_method = SNPP_ONE_POINT_FIVE) )
		    {
			msg_tag = get_memory(MAX_LENGTH, "msg_tag, send_snpp");
			pass_code = get_memory(MAX_LENGTH, "pass_code, send_snpp");
			code = parse_two_way(server_response, msg_tag, pass_code);
		    }
		    else
			code = parse_codes(server_response, 3);
		    if ( code == NULL )
		    {
			state = CONNECTION_LOST;
		    }
		    //else if ( atoi(code) == 250 )
		    else if ( code[0] == '2' )
		    {
			if ( (data->page_result == PAGE_ERROR)  || (data->page_result == ERROR_ROLLOVER) )
			    totals->error_total--;
			sprintf(update, "Sending to: %s", data->receiver_name);

			if ( send_mode == ASYNCH )
			{
			    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
				error_quit("Error locking mutex - possible data corruption");
			    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
			    send_log_data(data->receiver_name, data->message, "020", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
				error_quit("Error unlocking mutex - possible data corruption");
			}
			else
			{
			    if (send_mode == ROLLOVER)
			    {
				write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
				send_log_data(data->receiver_name, data->message, "021", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			    }
			    else
			    {
				write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
				send_log_data(data->receiver_name, data->message, "020", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			    }
			}
			data->page_status = SENT_SNPP;
			data->page_result = SENT_SNPP;
			data = data->next;
			more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
			state = CONNECT;
			totals->sent_snpp++;
			totals->sent_total++;
		    }
		    else if ( (atoi(code) == 920) || (atoi(code) == 888) )
		    {
			state = TIMEOUT_ERROR;
		    }
		    else if ( atoi(code) == 960 ) 
			state = QUEUED;
		    else
		    {
			state = ERROR;
			if ( send_mode == ASYNCH )
			{
			    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
				error_quit("Error locking mutex - possible data corruption");
			    send_log_data(data->receiver_name, data->message, "123", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
				error_quit("Error unlocking mutex - possible data corruption");
			}
			else
			{
			    send_log_data(data->receiver_name, data->message, "123", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			}
		    }
		    free(code);
		    break;

		case NO_QUEUE:
		    strcpy(buffer, "NOQUEUE\r\n");
		    wrote = write_num(sock, buffer, strlen(buffer));
		    if ( wrote == -1 )
		    {
			state = CONNECTION_LOST;
			break;
		    }

		    handle_debug(buffer);
		    code = parse_codes(mread(sock, server_read_timeout), 3);
		    if ( code == NULL )
		    {
			state = CONNECTION_LOST;
		    }
		    else if ( atoi(code) == 250)
			state = PAGE_ID;
		    else if ( (atoi(code) == 920) || (atoi(code) == 888) )
			state = TIMEOUT_ERROR;
		    else if ( atoi(code) == 500 ) /* command not implemented.. lets try to continue anyways */
			state = PAGE_ID;
		    else
		    {
			memset(update, '\0', sizeof(update));
			strcpy(update, "Error disabling queue\n");
			if ( send_mode == ASYNCH )
			{
			    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
				error_quit("Error locking mutex - possible data corruption");
			    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
			    send_log_data(data->receiver_name, data->message, "121", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
				error_quit("Error unlocking mutex - possible data corruption");
			}
			else
			{
			    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
			    send_log_data(data->receiver_name, data->message, "121", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			}
			state = ERROR;
		    }
		    free(code);
		    break;

		case QUEUED:
		    two_way_data = get_memory( (strlen("220") + strlen(msg_tag) + strlen(pass_code) + 10), "2way_data - send_snpp");
		    sprintf(two_way_data, "220,%s,%s", msg_tag, pass_code);
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, two_way_data, socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, two_way_data, socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    free(two_way_data);
		    totals->queued++;
		    data = data->next;
		    state = CONNECT;
		    break;

		case CONNECT_ERROR:

		    /*   Here is the new dial on no connect script code    */
		    /* I need to try to figure out the difference betweeen no internet connection
		       and a remote server that is not available  */

		    if ( !strcmp(data->dial_on_no_connect, "yes\n") ) /* The comes from the client */
		    {
			if ( (connect_attempts == 0) && (dial_script == 1) ) /* dial_script comes from a server switch */
			{
			    connect_attempts++;
			    if ( fork() == 0 )	/* Child process */
			    {
				char *my_argv[2];
				
				my_argv[0]= internet_script;
				my_argv[1]= 0;
				
				if ( (execv(internet_script, my_argv)) == -1 )
				    error_quit("Error execv with this script: %s ", internet_script);
				
			    }
			    else
			    {	/* parent process */
				sleep(CONNECT_SLEEP_VALUE); /* This value is defined in data_structures.h */
				/* Here is where I try to see if an intenet connection has been made */
				/* Then we reset the state to connect and give it another try */
				state = CONNECT;
				break;
			    }
			}
			else if ( (connect_attempts == 1) && (dial_script == 1) )
			{
			    sleep(15);
			    connect_attempts++;
			    state = CONNECT;
			    break;
			}
		    }
		    /*   Here is the end of the new dial on no connect script code    */

		    if ( (data->page_result != PAGE_ERROR) && (data->page_result != ERROR_ROLLOVER) )
			totals->error_total++;
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "120", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "120", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    
		    totals->no_connect++;
		    clean(data->alternative_snpp);
		    if ( strcmp(data->alternative_snpp,"NULL") ) /* rollover on no connect if active is this is not NULL */
			data->page_result = ERROR_ROLLOVER;
		    else
			data->page_result = PAGE_ERROR;

		    data->page_status = ERROR_SNPP;
		    data = data->next;
		    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		    totals->error_snpp++;
		    state = CONNECT;
		    break;

		case CONNECTION_LOST:
		    if ( (data->page_result != PAGE_ERROR) && (data->page_result != ERROR_ROLLOVER) )
			totals->error_total++;
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "126", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "126", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }
		    data->page_status = ERROR_SNPP;
		    data->page_result = PAGE_ERROR;
		    data = data->next;
		    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		    totals->error_snpp++;
		    state = CONNECT;
		    break;


		case ERROR:
		    if ( (data->page_result != PAGE_ERROR) && (data->page_result != ERROR_ROLLOVER) )
			totals->error_total++;
		    data->page_status = ERROR_SNPP;
		    data->page_result = PAGE_ERROR;
		    data = data->next;
		    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		    state = CONNECT;
		    totals->error_snpp++;
		    break;

		case TIMEOUT_ERROR:
		    if ( (data->page_result != PAGE_ERROR) && (data->page_result != ERROR_ROLLOVER) )
			totals->error_total++;
		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "920", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "920", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			
		    }
		    data->page_status = ERROR_SNPP;
		    data->page_result = PAGE_ERROR;
		    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		    data = data->next;
		    state = CONNECT;
		    totals->error_snpp++;
		    break;


		case FATAL_ERROR:
		    if ( (data->page_result != PAGE_ERROR) && (data->page_result != ERROR_ROLLOVER) )
			totals->error_total++;

		    if ( send_mode == ASYNCH )
		    {
			if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			    error_quit("Error locking mutex - possible data corruption");
			send_log_data(data->receiver_name, data->message, "125", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
			if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			    error_quit("Error unlocking mutex - possible data corruption");
		    }
		    else
		    {
			send_log_data(data->receiver_name, data->message, "125", socket_fd, send_flag, data->msg_id, client_adr, "NULL", data->user_id);
		    }

		    data->page_status = ERROR_SNPP;
		    data->page_result = PAGE_ERROR;
		    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		    data = data->next;
		    state = CONNECT;
		    totals->error_snpp++;

		    strcpy(buffer, "QUIT\r\n");
		    write_num(sock, buffer, strlen(buffer));
		    handle_debug(buffer);
		    mread(sock, server_read_timeout);                          /* We don't care what the response is */

		    break;

		}  /* End of switch block */

	} /* End of if block */
	else
	{
	    state = CONNECT;
	    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
	    data = data->next;
	}

    } /* End of the main while block */
    

    if ( totals->sent_snpp > 0 )
    {
	strcpy(buffer, "QUIT\r\n");
	write_num(sock, buffer, strlen(buffer));
	handle_debug(buffer);
	mread(sock, server_read_timeout);                          /* We don't care what the response is */
	close(sock);
    }

    free(update);
    free(buffer);
    free(current_url);
    return data;
}

/*
 *     new_url takes the current snpp address we are connected to, if we are connnected at all, and compares it to the
 *     new address that is passed also passed as an argument. If they are the same address then we simply return the current connected
 *     socket. If they are different we close the current connection and attempt to connect with the new address. We pass a
 *     -1 if we cannot connect to the server or we pass the connected socket if we can.
 */
     
int new_url(char *url, char *new_url, char *new_port, int socket_fd, int current_socket, char *new_connection, int timeout, int server_timeout)
{
    extern int debug_flag;

    int   new_socket = -1;
    char  error_msg[1024];
    char  *buffer = "QUIT\r\n";
    char  log_buffer[1024];
    
    memset(error_msg, '\0', sizeof(error_msg) );
    memset(log_buffer, '\0', sizeof(log_buffer) );

    strcpy(error_msg, "Error connecting to server: ");

    if (   *(new_url + strlen(new_url) - 1) == '\n' )
	*(new_url + strlen(new_url) - 1) = '\0';  
    if (  *(new_port + strlen(new_port) - 1) == '\n' )
	*(new_port + strlen(new_port) - 1) = '\0'; 

    if ( (!strcmp(url, new_url)) && ( current_socket != -1 ) )
    {
	strcpy(new_connection, "0");
	return current_socket;
    }
    else
    {
	strcpy(url, new_url);
	
	if ( current_socket != -1 )
	{
	    write_num(current_socket, buffer, strlen(buffer));
	    mread(current_socket, timeout);                          /* We don't care what the response is */
	    close(current_socket);
	}
	info_msg("Attempt connect with %s on port %d", url, atoi(new_port));    
	new_socket = connect_to_remote_server(url, atoi(new_port), socket_fd, server_timeout);
	if ( new_socket == -1 )
	    return (-1);
	else
	{
	    if (debug_flag)
	    {
		sprintf(log_buffer, "Connect with: %s on port: %d\n", url, atoi(new_port));
		log_transaction(log_buffer);
	    }
	    info_msg("Connect with %s on port %d", url, atoi(new_port));    
	    strcpy(new_connection, "1");

	    return new_socket;
	}
    }
}

/*
 *   We add a \r\n to the end of the passed string
 */

char *fix_string(char *buffer)
{    
    *(buffer + strlen(buffer) - 1) = '\r';  
    strcat(buffer, "\n");
    return buffer;
}

int are_we_done(page_profile *node, int mode, int pages)
{
    int result;
    
    if ( (mode == SERIAL) || (mode == ROLLOVER) )
    {
	if ( node->next )
	    result = 1;
	else
	    result = 0;
    }
    else
	result = pages;

    return (result);
}

static void handle_debug(char *buffer)
{
    extern int debug_flag;

    char  *log;

    log = get_memory( (strlen(buffer) + 15), "send_snpp - log");
    if ( debug_flag )
    {
	sprintf(log, "SENT -> %s", buffer);
	log_transaction(log);
    }
    free(log);
}

static char *parse_two_way(char *buffer, char *msg_tag, char *pass_code)
{
    int i, x;
    int max_code_length = 10;
    int len = 0;
    char *code;

    code = get_memory(max_code_length, "send_smtp - code");
    len = strlen(buffer);

    for (i = 0; i < 3; i++)
	*(code + i) = *(buffer + i);
    code[3] = '\0'; 
    if ( code[0] == '8' || code[0] == '9' )
    {
	i = 4;
	x = 0;
	while ( ( *(buffer + i) != ' ') && (i <= len) )
	    ( *(msg_tag + x++) = *(buffer + i++) );
	*(msg_tag + x) = '\0';
	i++;
	x = 0;
	while ( ( *(buffer + i) != ' ') && (i <= len) )
	    ( *(pass_code + x++) = *(buffer + i++) );
	*(pass_code + x) = '\0';
    
	//fprintf(stderr, "HERE IS THE MESSAGE TAG: %s\n", msg_tag);
	//fprintf(stderr, "HERE IS THE PASS CODE: %s\n", pass_code);
    }
    return code;
}



