/*
 * g-page Copyright (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"
#include "xmlparse.h"
#include "ghttp.h"
 
#define ERROR          0
#define PAGE_ID        1  
#define MESSAGE        2
#define SEND           3
#define FATAL_ERROR    5
#define CONNECT        6
#define CONNECT_ERROR  7
#define TIMEOUT_ERROR  8
#define PASSWORD       9
#define READ_RESULT    10
#define HTTP_ERROR     11
#define INVALID_URL    12
#define NO_CONNECT     13
#define READ_ERROR     14
#define MAX_QUERY      4096	/* Extra big */

//#define TEST

extern pthread_mutex_t update_client_mutex;
char   wctp_tracking_number[MAX_FIELD];
char   wctp_return_code[MAX_FIELD];
char   results[1000];
char   notification[1000];

static int  handle_result(char *result_code, page_profile *data, int socket_fd, int send_flag, int send_mode, char *client_adr, char *tracking, char *uid);
void debug_dump(char *buffer);
static void strip(char *buffer);
void charHandler01(void *userData, const char *s, int len);

page_profile  *send_wctp(page_profile *data, int socket_fd, email_data *email_info, message_data  *totals, int connection_timeout, 
			 int server_read_timeout, int send_flag, int send_mode, char *client_adr, int query_mode)
{
    extern  int              debug_flag;
    extern  char             wctp_return_code[MAX_FIELD];
    extern  char             wctp_tracking_number[MAX_FIELD];
    extern  char             notification[1000];
    extern  int              dial_script;
    extern  char             *internet_script;

    enum
    {
	one_way = 1,
	one_point_five = 2,
	two_way = 3,
	check_for_response = 4
    };

    char           *msg_data          = "";
    char           *update            = "";
    char           *cur_time          = "";
    char           *log               = "";
    char           *body              = "";
    char           *tbody             = "";

    static int     connect_attempts   =  0;
    int            state              =  0;
    int            sent               =  0;
    int            status_code        =  0;
    int            http_sock          =  0;
    int            parse_status       =  0;
    int            more_pages_to_send =  0;
    int            number_of_nodes    =  0;
    int            mutex_status;

    ghttp_status   status;
    page_profile   *head;
    ghttp_request  *request =NULL;
    
    strcpy(notification, "");
    log = get_memory(MAX_LENGTH, "send_wctp - log");
    update = get_memory(MAX_LENGTH, "send_wctp - update");

    state = CONNECT;

    if ( send_mode == ASYNCH )
	number_of_nodes = data->split_nodes;
    else
	number_of_nodes = 0;

    more_pages_to_send = are_we_done(data, send_mode, number_of_nodes);
    if ( query_mode == 1 )
	more_pages_to_send = 1;

    head = data;

    while ( more_pages_to_send )
    {
	if ( data->page_status == SEND_WCTP || query_mode)
	{
	    switch (state) {
		
	    case CONNECT:

		clean(data->wctp_url);

		if ( ghttp_uri_validate(data->wctp_url) == -1)
		{
		    state = INVALID_URL;
		    break;
		}
		request = NULL;
		request = ghttp_request_new();
		
		if ( (ghttp_set_uri(request, data->wctp_url)) == -1 )
		{
		    state = CONNECT_ERROR;
		    ghttp_request_destroy(request);
		    break;
		}
		else
		    state = SEND;
		break;

	    case SEND:
		cur_time = get_current_time();

		if (query_mode)
		{

		    strcpy(wctp_tracking_number, data->tracking_number);
		    msg_data = build_query(data->receiver_number, cur_time, data->message, email_info->email_address, check_for_response, 
					   data->response_one, data->response_two, data->response_three);	

		}
		else
		{
		    if ( data->protocols[3] == '0' )
			msg_data = build_query(data->receiver_number, cur_time, data->message, email_info->email_address, one_way, NULL, NULL, NULL);
		    else if ( data->protocols[3] == '1' )
			msg_data = build_query(data->receiver_number, cur_time, data->message, email_info->email_address, one_point_five, NULL, NULL, NULL);
		    else
			msg_data = build_query(data->receiver_number, cur_time, data->message, email_info->email_address, two_way, 
					       data->response_one, data->response_two, data->response_three);	
		}

		if ( msg_data == NULL )
		{
		    state = FATAL_ERROR;
		    free(cur_time);
		    break;
		}

		free(cur_time);

		if ( (ghttp_set_type(request, ghttp_type_post)) == -1 )
		{
		    state = FATAL_ERROR;
		    free(msg_data);
		    ghttp_request_destroy(request);
		    break;
		}		

		ghttp_set_header(request, http_hdr_Accept, "text/xml");
		ghttp_set_header(request, http_hdr_Content_Type, "text/xml");

		if ( (ghttp_set_body(request, msg_data, strlen(msg_data))) == -1 )
		{
		    state = FATAL_ERROR;
		    free(msg_data);
		    ghttp_request_destroy(request);
		    break;
		}

		if ( debug_flag )
		{
		    sprintf(log, "Attempt connect -> %s\n", data->wctp_url);
		    log_transaction(log);
		}

		ghttp_prepare(request);

		status = ghttp_process(request);

		http_sock =  ghttp_get_socket(request);

		if ( debug_flag )
		{
		    status_code = ghttp_status_code(request);
		    sprintf(log, "Server returned status: -> %d\n", status_code);
		    log_transaction(log);
		}


		if ( status == ghttp_error )
		{
		    if ( http_sock == -1 )
			state = NO_CONNECT;
		    else
			state = HTTP_ERROR;
		    free(msg_data);
		    ghttp_request_destroy(request);
		    break;
		}
		
		status_code = ghttp_status_code(request);

		if ( status_code == 200 )
		    state = READ_RESULT;
		else
		{
		    state = HTTP_ERROR;
		    free(msg_data);
		    ghttp_request_destroy(request);
		    break;
		}

		if ( send_mode == ASYNCH )
		{
		    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			error_quit("Error locking mutex - possible data corruption");
		    sprintf(update, "Sending to: %s", data->receiver_name);
		    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
		    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			error_quit("Error unlocking mutex - possible data corruption");
		}
		else
		{
		    sprintf(update, "Sending to: %s", data->receiver_name);
		    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);

		}

		if ( debug_flag )
		{
		    sprintf(log, "CONNECT -> %s\n", data->wctp_url);
		    log_transaction(log);
		    if ( strlen(msg_data) < (MAX_LENGTH - 1) )
			sprintf(log, "SENT -> %s", msg_data);
		    else
		    {
			strcpy(log, "SENT ->");
			strncat(log, msg_data, (MAX_LENGTH - 8));
		    } 
		    log_transaction(log);		    
		}

		free(msg_data);
		break;

	    case READ_RESULT:

		tbody = ghttp_get_body(request);

		body = get_memory( (ghttp_get_body_len(request) + 100), "send_wctp - body");

		strcpy(body, tbody);

		/* debug_dump(body);  */

		if ( strlen(body) < 3 )
		{
		    state = READ_ERROR;
		    ghttp_clean(request);
		    ghttp_request_destroy(request);
		    free(body);
		    break;
		}
		
		*(body + (ghttp_get_body_len(request))) = '\0';

		if ( debug_flag )
		{
		    sprintf(log, "%s", body);
		    log_transaction(log);
		}

		ghttp_clean(request);
		ghttp_request_destroy(request);

		strip_crap(body);

		strcpy(wctp_tracking_number, "\0");
		parse_status = expat(body);

		if ( query_mode )
		{
		    if ( strlen(results) > 0 ) /* We have a reply */
		    {
			clean(data->receiver_name);
			clean(data->user_id);
			clean(data->tracking_number);
			clean(data->message);
			strip(results);
			sprintf(update, "%s,U,%s,%s,%s,%s\n", data->user_id, results, data->tracking_number, data->receiver_name, data->message);
			write_to_status_line(socket_fd, update, strlen(update), TWO_WAY_REPLY_DATA);
		    }
		    else
		    {
			if ( strlen(notification) > 0 )	/* We don't have a reply but the message was delivered */
			{
			    sprintf(update, "%s\n", notification);
			    write_to_status_line(socket_fd, update, strlen(update), TWO_WAY_REPLY_DATA);
			}
			else	/* we have no data to report */
			    write_to_status_line(socket_fd, "\n", strlen("\n"), TWO_WAY_REPLY_DATA);
		    }
		}
		else
		{
		    strcpy(data->tracking_number, wctp_tracking_number);
		    if ( (data->protocols[3] == '0') || (data->protocols[3] == '1') ) /* We only save the tracking # if it's a twp way pager */
			strcpy(wctp_tracking_number, "NULL");
		}


		if ( (data->protocols[3] == '2') && (query_mode != 1) ) 
		{
		    if ( strlen(data->tracking_number) > 0 )
		    {
			if ( send_mode == ASYNCH )
			{
			    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
				error_quit("Error locking mutex - possible data corruption");
			    clean(data->receiver_name);
			    clean(data->user_id);
			    sprintf(update, "%s,U,w,%s,%s,%s", data->user_id, data->tracking_number, data->receiver_name, data->message);
			    write_to_status_line(socket_fd, update, strlen(update), TWO_WAY_LOG_DATA);

			    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
				error_quit("Error unlocking mutex - possible data corruption");
			}
			else
			{
			    clean(data->receiver_name);
			    clean(data->user_id);
			    sprintf(update, "%s,U,w,%s,%s,%s", data->user_id, data->tracking_number, data->receiver_name, data->message);
			    write_to_status_line(socket_fd, update, strlen(update), TWO_WAY_LOG_DATA);   
			}			
		    }
		}

		if (parse_status == 1)
		{
		    state = READ_ERROR;
		    free(body);
		    break;
		}
		
		sent = handle_result(wctp_return_code, data, socket_fd, send_flag, send_mode, client_adr, wctp_tracking_number, data->user_id);
		strcat(data->wctp_url, "\n");
		//fprintf(stderr, "Here is the wctp return code: %s\n", wctp_return_code);

		free(body);
		if (sent)
		{
		    if ( data->page_result == PAGE_ERROR )
			totals->error_total--;
		    data->page_status = SENT_WCTP;
		    data->page_result = SENT_WCTP;
		    totals->sent_wctp++;
		    totals->sent_total++;
		    data = data->next;	    
		    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		}
		else
		{
		    if ( data->page_result != PAGE_ERROR )
			totals->error_total++;
		    data->page_status = ERROR_WCTP;
		    data->page_result = PAGE_ERROR;
		    totals->error_wctp++;
		    data = data->next;
		    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		}
		state = CONNECT;
		break;

	    case FATAL_ERROR:

		if ( query_mode )
		    write_to_status_line(socket_fd, "No connect", strlen("No connect"), CONNECTION_ERROR);

		if ( data->page_result != PAGE_ERROR )
		    totals->error_total++;
		data->page_status = ERROR_WCTP;
		data->page_result = PAGE_ERROR;
		totals->error_wctp++;

		if ( send_mode == ASYNCH )
		{
		    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			error_quit("Error locking mutex - possible data corruption");
		    send_log_data(data->receiver_name, data->message, "13900", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			error_quit("Error unlocking mutex - possible data corruption");
		}
		else
		{
		    send_log_data(data->receiver_name, data->message, "13900", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		}


		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		state = CONNECT;
		break;

	    case READ_ERROR:
		if ( data->page_result != PAGE_ERROR )
		    totals->error_total++;
		data->page_status = ERROR_WCTP;
		data->page_result = PAGE_ERROR;
		totals->error_wctp++;

		if ( send_mode == ASYNCH )
		{
		    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			error_quit("Error locking mutex - possible data corruption");
		    send_log_data(data->receiver_name, data->message, "13906", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			error_quit("Error unlocking mutex - possible data corruption");
		}
		else
		{
		    send_log_data(data->receiver_name, data->message, "13906", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		}


		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		state = CONNECT;
		break;

	    case CONNECT_ERROR:
                /* Start the dial on no connect code here */
		if ( !strcmp(data->dial_on_no_connect, "yes\n") ) /* The comes from the client */
		{
		    if ( (connect_attempts == 0) && (dial_script == 1) ) /* dial_script comes from a server switch */
		    {
			connect_attempts++;
			if ( fork() == 0 )	/* Child process */
			{
			    char *my_argv[2];
			    
			    my_argv[0]= internet_script;
			    my_argv[1]= 0;
			    
			    if ( (execv(internet_script, my_argv)) == -1 )
				error_quit("Error execv with this script: %s ", internet_script);
			    
			}
			else
			{	/* parent process */
			    sleep(CONNECT_SLEEP_VALUE); /* This value is defined in data_structures.h */
				/* Here is where I try to see if an intenet connection has been made */
				/* Then we reset the state to connect and give it another try */
			    state = CONNECT;
			    break;
			}
		    }
		    else if ( (connect_attempts == 1) && (dial_script == 1) )
		    {
			sleep(15);
			connect_attempts++;
			state = CONNECT;
			break;
		    }
		}

                /* End the dial on no connect code here */

		if ( query_mode )
		    write_to_status_line(socket_fd, "No connect", strlen("No connect"), CONNECTION_ERROR);

		sprintf(update, "Error connecting to %s\n", data->wctp_url);
		write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);

		if ( data->page_result != PAGE_ERROR )
		    totals->error_total++;

		totals->no_connect++;
		clean(data->alternative_snpp);
		if ( strcmp(data->alternative_snpp,"NULL") ) /* rollover on no connect is active is this is not NULL */
		{
		    data->page_result = ERROR_ROLLOVER;
		}
		else
		    data->page_result = PAGE_ERROR;

		data->page_status = ERROR_WCTP;
		//data->page_result = PAGE_ERROR;
		totals->error_wctp++;

		if ( send_mode == ASYNCH )
		{
		    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			error_quit("Error locking mutex - possible data corruption");
		    send_log_data(data->receiver_name, data->message, "13901", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			error_quit("Error unlocking mutex - possible data corruption");
		}
		else
		{
		    send_log_data(data->receiver_name, data->message, "13901", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		}


		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		state = CONNECT;
		break;

	    case INVALID_URL:
		if ( query_mode )
		    write_to_status_line(socket_fd, "No connect", strlen("No connect"), CONNECTION_ERROR);

		if ( data->page_result != PAGE_ERROR )
		    totals->error_total++;
		data->page_status = ERROR_WCTP;
		data->page_result = PAGE_ERROR;
		totals->error_wctp++;

		if ( send_mode == ASYNCH )
		{
		    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			error_quit("Error locking mutex - possible data corruption");
		    send_log_data(data->receiver_name, data->message, "13905", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			error_quit("Error unlocking mutex - possible data corruption");
		}
		else
		{
		    send_log_data(data->receiver_name, data->message, "13905", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		}


		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		state = CONNECT;
		break;

	    case HTTP_ERROR:
		if ( query_mode )
		    write_to_status_line(socket_fd, "No connect", strlen("No connect"), CONNECTION_ERROR);

		sprintf(update, "Fatal error connecting to wctp server: invalid response\n");
		write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);

		if ( data->page_result != PAGE_ERROR )
		    totals->error_total++;
		data->page_status = ERROR_WCTP;
		data->page_result = PAGE_ERROR;
		totals->error_wctp++;

		if ( send_mode == ASYNCH )
		{
		    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			error_quit("Error locking mutex - possible data corruption");
		    send_log_data(data->receiver_name, data->message, "13901", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			error_quit("Error unlocking mutex - possible data corruption");
		}
		else
		{
		    send_log_data(data->receiver_name, data->message, "13901", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		}


		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		state = CONNECT;
		break;

	    case NO_CONNECT:
                /* Start the dial on no connect code here */
		if ( !strcmp(data->dial_on_no_connect, "yes\n") ) /* The comes from the client */
		{
		    if ( (connect_attempts == 0) && (dial_script == 1) ) /* dial_script comes from a server switch */
		    {
			connect_attempts++;
			if ( fork() == 0 )	/* Child process */
			{
			    char *my_argv[2];
			    
			    my_argv[0]= internet_script;
			    my_argv[1]= 0;
			    
			    if ( (execv(internet_script, my_argv)) == -1 )
				error_quit("Error execv with this script: %s ", internet_script);
			    
			}
			else
			{	/* parent process */
			    sleep(40);
				/* Here is where I try to see if an intenet connection has been made */
				/* Then we reset the state to connect and give it another try */
			    state = CONNECT;
			    break;
			}
		    }
		    else if ( (connect_attempts == 1) && (dial_script == 1) )
		    {
			sleep(15);
			connect_attempts++;
			state = CONNECT;
			break;
		    }
		}

                /* End the dial on no connect code here */

		if ( query_mode )
		    write_to_status_line(socket_fd, "No connect", strlen("No connect"), CONNECTION_ERROR);

		if ( data->page_result != PAGE_ERROR )
		    totals->error_total++;
		sprintf(update, "Error: Unable to establish Internet connection\n");

		if ( send_mode == ASYNCH )
		{
		    if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
			error_quit("Error locking mutex - possible data corruption");
		    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
		    send_log_data(data->receiver_name, data->message, "13902", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		    if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
			error_quit("Error unlocking mutex - possible data corruption");
		}
		else
		{
		    write_to_status_line(socket_fd, update, strlen(update), STATUS_UPDATE);
		    send_log_data(data->receiver_name, data->message, "13902", socket_fd, send_flag, data->msg_id, client_adr, wctp_tracking_number, data->user_id);
		}

		totals->no_connect++;
		clean(data->alternative_snpp);
		if ( strcmp(data->alternative_snpp,"NULL") ) /* rollover on no connect is active is this is not NULL */
		{
		    data->page_result = ERROR_ROLLOVER;
		}
		else
		    data->page_result = PAGE_ERROR;

		data->page_status = ERROR_WCTP;
		//data->page_result = PAGE_ERROR;
		totals->error_wctp++;

		data = data->next;
		more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
		state = CONNECT;
		break;

	    } /* End of the switch statement */
	} /* Goes with the if data = send_wctp  */
	else
	{
	    data = data->next;
	    more_pages_to_send = are_we_done(data, send_mode, --number_of_nodes);
	}
    } /* End of the main while loop */

    free(log);
    free(update);


    return head;
}

/*
 *     This function replaces any "reserved" xml characters with the translation that the spec allows
 */

char *clean_xml(char *original_message)
{
    char   *message = "";
    int    i, x, y;
    char   *lt = "&lt;";
    char   *gt = "&gt;";
    char   *amp = "&amp;";
    char   *quot = "&quot;";
    char   *apos = "&apos;";

    i = x = y = 0;

    message = get_memory(MAX_LENGTH, "send_wctp - message");
    while ( ( *(original_message + i) != '\n') &&  *(original_message + i) )
    {
	switch ( *(original_message + i) ){
	    
	case '<':
	    for( y = 0; y < 4; y++)
		 *(message + x++) = *(lt + y);
	    i++;
	    break;

	case '>':
	    for( y = 0; y < 4; y++)
		 *(message + x++) = *(gt + y);
	    i++;
	    break;
	    
	case '&':
	    for( y = 0; y < 5; y++)
		 *(message + x++) = *(amp + y);
	    i++;
	    break;

	case '\'':
	    for( y = 0; y < 6; y++)
		 *(message + x++) = *(apos + y);
	    i++;
	    break;

	case '\"':
	    for( y = 0; y < 6; y++)
		 *(message + x++) = *(quot + y);
	    i++;
	    break;

	default:
	    *(message + x++) = *(original_message + i++);
	    break;
	}
    }

    *(message + x) = '\0';
    return message;
}

/*
 *     Report back to the client the g-page code that results from the actual wctp response code
 */

static int  handle_result(char *result_code, page_profile *data, int socket_fd, int send_flag, int send_mode, char *client_adr, char *tracking, char *uid)
{
    int  sent = 0;
    int  mutex_status;

    if ( send_mode == ASYNCH )
    {
	if ( (mutex_status = pthread_mutex_lock(&update_client_mutex)) != 0 )
	    error_quit("Error locking mutex - possible data corruption");
    }


    switch (atoi(result_code)) {
    case 200:			/* Sent */
	sent = 1;
	send_log_data(data->receiver_name, data->message, "030", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 300:			/* Operation not supported */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13300", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 301:			/* Input can not be parsed */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13301", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 302:			/* XML validation error */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13302", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 303:			/* Version Error */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13303", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 304:			/* Cannont accept operation */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13304", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 400:			/* Function not supported */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13400", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 401:			/* Invalid sender ID */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13401", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 402:			/* Invalid security code */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13402", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 403:			/* Invalid recipient ID */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13403", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 404:			/* Invalid authorization code */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13404", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 405:			/* Invalid date or time format */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13405", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 406:			/* Date/Time range error */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13406", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 407:			/* Invalid date/time period */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13407", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;     
    case 408:			/* Date/Time has already expired */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13408", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 409:			/* Domain alias is not uniqur */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13409", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 410:			/* Domain alias denied */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13410", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 500:			/* Message has timed out */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13500", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 501:			/* Message has expired */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13501", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 502:			/* No further responses possible */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13502", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 503:			/* Sender permanantly disabled due to excessive polling or traffic*/
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13503", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 504:			/* Unknown message reference */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13504", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 600:			/* Poll rate exceeded */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13600", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 601:			/* Excessive polling */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13601", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 602:			/* Traffic rate exceeded */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13602", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 603:			/* traffic rate excessive */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13603", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    case 901:			/* mail box full */
	sent = 0;
	send_log_data(data->receiver_name, data->message, "13991", socket_fd, send_flag, data->msg_id, client_adr, tracking, uid);
	break;
    }

    if ( send_mode == ASYNCH )
    {
	if ( (mutex_status = pthread_mutex_unlock(&update_client_mutex)) != 0 )
	    error_quit("Error unlocking mutex - possible data corruption");
    }

    return (sent);
}

/*
 *     We need to remove any extra linefeeds or newlines that might be in the message, typically from MS IIS servers
 */

void strip_crap(char *buffer)
{
    if ( strlen(buffer) < 2 )
	return;

    if ( ((buffer[strlen(buffer) -1]) == '\n' ) || ( (buffer[strlen(buffer) -1]) == '\r' ) )
	buffer[strlen(buffer) -1] = '\0';

}

/*
 *     Here we build the actual xml query. This needs to be changed to handle different queries, 
 *     1.75 or two way for example. For now we are just sending simple one way queries.
 */

char *build_query(char *rec_num, char *send_time, char *msg, char *email_address, int query_flag, char *one, char *two, char *three)
{
    extern char   wctp_tracking_number[MAX_FIELD];

    enum
    {
	one_way = 1,
	one_point_five = 2,
	two_way = 3,
	check_for_response = 4,
	two_way_no_mcr = 5
    };
				/* This is the one way data */
    char           *dtd  = "<?xml version=\"1.0\"?> ";
    char           *dtd1 = "<!DOCTYPE wctp-Operation SYSTEM \"http://www.msg-stds.org/src-dtd/wctpv1-0.dtd\"> ";
    char           *op_tag = "<wctp-Operation wctpVersion=\"1.0\"> ";
    char           *submit_tag = "<wctp-SubmitClientMessage> ";
    char           *client_header = "<wctp-SubmitClientHeader submitTimestamp=\"";
    char           *client_originator = "<wctp-ClientOriginator senderID=\"";
    char           *client_recipient = "<wctp-Recipient recipientID=\"";
    char           *client_message_control_response = "<wctp-ClientMessageControl allowResponse=\"true\" notifyWhenDelivered=\"true\"/>";
    char           *tracking_number = "trackingNumber=\"";
    char           *client_query = "<wctp-ClientQuery \n";
    char           *payload_tag = "<wctp-Payload> ";
    char           *msg_tag = "<wctp-Alphanumeric> ";
    char           *mcr_tag = "<wctp-MCR>";
    char           *text_tag = "<wctp-MessageText>";
    char           *close_op_tag = "</wctp-Operation> ";
    char           *close_submit_tag = "</wctp-SubmitClientMessage> ";
    char           *close_payload_tag = "</wctp-Payload> ";
    char           *close_client_header = "</wctp-SubmitClientHeader> ";
    char           *close_msg_tag = "</wctp-Alphanumeric> ";
    char           *end_tag = "> ";
    char           *close_mcr_tag = "</wctp-MCR>";
    char           *close_end_tag = "/> ";
    char           *close_text_tag = "</wctp-MessageText>";
    char           *mcr_msg;
				/* End of the one way data */

				/* Additional data needed for one point five */
    char           *client_message_control = "<wctp-ClientMessageControl notifyWhenQueued=\"true\" notifyWhenDelivered=\"true\" notifyWhenRead=\"true\"/>";
    
    char           *msg_data;
    char           *clean_message = "";

    msg_data = get_memory(MAX_QUERY, "send_wctp - msg_data");
    mcr_msg = get_memory(MAX_LENGTH, "send_wctp - mcr_msg");
    memset(mcr_msg, '\0', sizeof(mcr_msg));

    clean_message = clean_xml(msg);
    email_address[strlen(email_address) -1 ] = '\0';
    rec_num[strlen(rec_num) -1 ] = '\0';

    if ( query_flag == two_way )
    {
	clean(one);
	clean(two);
	clean(three);

	if ( strlen(one) == 0 )
	    query_flag = two_way_no_mcr;
	else
	{
	    strcpy(mcr_msg, "<wctp-Choice>");
	    strcat(mcr_msg, one);
	    strcat(mcr_msg, "</wctp-Choice>");
	}

	if ( strlen(two) != 0 )
	{
	    strcat(mcr_msg, "<wctp-Choice>");
	    strcat(mcr_msg, two);
	    strcat(mcr_msg, "</wctp-Choice>");
	}
	if ( strlen(three) != 0 )
	{
	    strcat(mcr_msg, "<wctp-Choice>");
	    strcat(mcr_msg, three);
	    strcat(mcr_msg, "</wctp-Choice>");
	}
    }

    if ( query_flag == one_way )
    {
				/* There are 22 %s in the sprintf statement */
	sprintf(msg_data, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\r\n", 
		dtd, dtd1, 
		op_tag, 
		   submit_tag, 
		      client_header, send_time, end_tag, 
		        client_originator, email_address, "\"", close_end_tag, 
		        client_recipient, rec_num, "\"/>", 
                      close_client_header, 
                         payload_tag, msg_tag, clean_message, close_msg_tag, 
		         close_payload_tag, 
		   close_submit_tag, 
		close_op_tag);

	strcat(email_address, "\n");
	strcat(rec_num, "\n");
    }
    else if ( query_flag == one_point_five )
    {
			/* There are 23 %s in the sprintf statement */
	sprintf(msg_data, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\r\n", 
		dtd, dtd1, 
		op_tag, 
		   submit_tag, 
		      client_header, send_time, end_tag, 
		        client_originator, email_address, "\"", close_end_tag, 
		        client_message_control,	/* This is the only addition from one way */
		        client_recipient, rec_num, "\"/>", 
                      close_client_header, 
                         payload_tag, msg_tag, clean_message, close_msg_tag, 
		         close_payload_tag, 
		   close_submit_tag, 
		close_op_tag);

	strcat(email_address, "\n");
	strcat(rec_num, "\n");
	memset(wctp_tracking_number, '\0', sizeof(wctp_tracking_number));
    }
    else if ( query_flag == two_way )
    {
	if ( strlen(mcr_msg) == 0 )
	    strcpy(mcr_msg, " ");
	    
				/* There are 25 %s in the sprintf statement */
	sprintf(msg_data, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\r\n", 
		dtd, dtd1, 
		op_tag, 
		   submit_tag, 
		      client_header, send_time, end_tag, 
		        client_originator, email_address, "\"", close_end_tag, 
		        client_message_control_response,
		        client_recipient, rec_num, "\"/>", 
                      close_client_header, 
		         payload_tag, mcr_tag, text_tag, clean_message, close_text_tag,
		         mcr_msg, /* This is the list of response choices */
		         close_mcr_tag, close_payload_tag, 
		   close_submit_tag, 
		close_op_tag);

	strcat(email_address, "\n");
	strcat(rec_num, "\n");
	memset(wctp_tracking_number, '\0', sizeof(wctp_tracking_number));
    }
    else if ( query_flag == two_way_no_mcr )
    {
	sprintf(msg_data, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\r\n", 
		dtd, dtd1, 
		op_tag, 
		   submit_tag, 
		      client_header, send_time, end_tag, 
		        client_originator, email_address, "\"", close_end_tag, 
		        client_message_control_response,
		        client_recipient, rec_num, "\"/>", 
                      close_client_header, 
                         payload_tag, msg_tag, clean_message, close_msg_tag, 
		         close_payload_tag, 
		   close_submit_tag, 
		close_op_tag);

	strcat(email_address, "\n");
	strcat(rec_num, "\n");
	memset(wctp_tracking_number, '\0', sizeof(wctp_tracking_number));
    }
    else if ( query_flag == check_for_response )
    {
				/* There are 13 %s in this statement */
	sprintf(msg_data, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\r\n", 
		dtd, dtd1,
	        op_tag,
	        client_query,
		"senderID=\"",  email_address, "\" ",
		"recipientID=\"", rec_num, "\" ", 
		tracking_number, wctp_tracking_number,  "\"", close_end_tag,
		close_op_tag );
    }

    free(clean_message);
    free(mcr_msg);
    return (msg_data);
}

/*
 *     Get the current time and store it in the wctp specified time format
 */

char *get_current_time()
{
    char           *cur_time = "";
    time_t         timeval;
    struct tm      *tm_str;

    time(&timeval);
    /* tm_str = localtime(&timeval); */
    tm_str = gmtime(&timeval);
    cur_time = get_memory(MAX_LENGTH, "send_wctp - cur_time");

    strftime(cur_time, MAX_LENGTH, "%Y-%m-%dT%X\"", tm_str);

    return ( cur_time );
}


void startElement(void *userData, const char *name, const char **atts)
{
    extern char   wctp_return_code[MAX_FIELD];
    extern char   wctp_tracking_number[MAX_FIELD];
    extern char   notification[1000];
    static int    flag = 0;

    int           *depthPtr = userData;
    int           i = 0;
    
/*
    fprintf(stderr, "%s ", name);
    i = 0;
    while ( atts[i] )
    {
	fprintf(stderr, "%d %s ", i, atts[i]);
	i++;
    }
    fprintf(stderr, "\n");
*/

    if ( flag == 1 )
    {
	if ( !strcmp(name, "wctp-Failure") || !strcmp(name, "wctp-ClientSuccess") )
	{
	    strcpy(wctp_return_code, atts[1]);
	    if (!strcmp(name, "wctp-ClientSuccess") )
	    {
		i = 0;
		while ( atts[i] )
		{
		    if ( !strcmp(atts[i], "trackingNumber") )
		    {
			i++;
			strcpy(wctp_tracking_number, atts[i]);
			flag = 0;
		    }
		    else
			i++;
		}
	    }		
	}
    }

    if ( !strcmp(name,"wctp-SubmitClientResponse") )
	flag = 1;
    if ( !strcmp(name, "wctp-Notification") )
    {
	i = 0;
	while ( atts[i] )
	{
	    if ( strcmp(atts[i], "type") )
		strcpy(notification, atts[i]);
	    i++;
	}
    }
    *depthPtr += 1;
}

void endElement(void *userData, const char *name)
{
  int *depthPtr = userData;
  *depthPtr -= 1;
}

void charHandler(void *userData, const char *s, int len)
{
    char *tag = "<wctp-Alphanumeric>";
    extern char results[1000];
    int  num;

    (char *) userData = tag;
    

    if ( len > 1 )
    {
	num = 0;;
	while(num <= len)
	{
	    results[num] = *(s + num);
	    num++;
	}
	results[len] = '\0';
	//fprintf(stderr, "Data: %s\n", results);
    }
}


int expat(char *buf)
{
  XML_Parser parser = XML_ParserCreate(NULL);
  int depth = 0;
  int done = 1;
  size_t len = strlen(buf);


  XML_SetUserData(parser, &depth);  
  XML_SetElementHandler(parser, startElement, endElement);
  XML_SetCharacterDataHandler(parser, charHandler);
  
  if (!XML_Parse(parser, buf, len, done)) 
  {
      error_ret("%s at line %d\n",
		XML_ErrorString(XML_GetErrorCode(parser)),
		XML_GetCurrentLineNumber(parser));
      return 1;
  }
      
  XML_ParserFree(parser);

  return 0;
}

void debug_dump(char *buffer)
{
    FILE   *fp;
    
    fp = fopen("gpage_dump", "a");
    fputs(buffer, fp);
    fclose(fp);
}

static void strip(char *buffer)
{
    int len;

    len = strlen(buffer);
    while ( len >= 0 )
    {
	if ( buffer[len] == ',' )
	    buffer[len] = ' ';
	len--;
    }
}
