/*
 * g-page Copyright (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"
#include "data_structures.h"
#include "function_def.h"

#define NUMBER_OF_ARGUMENTS 25

#ifndef SHUT_RD
  #define SHUT_RD   0
  #define SHUT_WR   1
  #define SHUTRDWR  2
#endif

static void save_scheduled_page(page_profile *page_data, char *isp, char *domain, char *address, int socket_fd, char *client_adr);
static void get_email_info(email_data *info, int live_sock, int max_len, int max_time);
extern int debug_flag;

void print_debug_data(char *string)
{

    if (debug_flag)
	info_msg("%s", string);

    return;
}

void gui_page(int socket_fd, char *client_adr)
{

  char           *buf = "";
  int            amount;
  int            index;
  int            number_of_pages;
  int            page_index = 0;
  int            scheduled_page = 0;
  int            len;
  page_profile   *page_list = NULL;
  page_profile   *head = NULL;
  page_profile   *temp = NULL;
  email_data     *email_info = NULL;

  if ( ( email_info = malloc(sizeof(*email_info))) == NULL)
      error_quit("Error allcoating memory - email_info - server_utilities.c ");
  
  get_email_info(email_info, socket_fd, MAX_LENGTH, MAX_TIMEOUT);

  page_list = get_page_list_memory(page_list);  
  page_list->next = NULL;
  page_list->previous = NULL;
  head = page_list;

/* Get the number of pages  */

  buf = get_memory(MAX_LENGTH, "server_utilities - buf");
  memset(buf, '\0', sizeof(buf));

  if ( ( amount = read_a_line(socket_fd, buf, MAX_LENGTH, MAX_TIMEOUT) ) <= 0) 
      return;
  write_to_status_line(socket_fd, buf, amount, IGNORE);

  number_of_pages = atoi(buf);

  for(page_index = 0; page_index < number_of_pages; page_index++)
  {
      for(index = 0; index < NUMBER_OF_ARGUMENTS; index++)
      {
	  memset(buf, '\0', MAX_LENGTH);
	  if ( ( amount = read_a_line(socket_fd, buf, MAX_LENGTH, MAX_TIMEOUT) ) <= 0) 
	      return;
	  else
	      len = strlen(buf);

	  switch (index)
	  {
	  case 0:		/* User ID */
	      strncpy(page_list->user_id, buf, MAX_FIELD - 1 );
	      break;
	  case 1:		/* Receiver number */
	      strncpy(page_list->receiver_number, buf, MAX_FIELD - 1 );
	      break;
	  case 2:		/* WCTP address */
	      strncpy(page_list->wctp_url, buf, MAX_FIELD - 1 );
	      break;
	  case 3:		/* Email address */
	      strncpy(page_list->smtp, buf, MAX_FIELD - 1 );
	      break;
	  case 4:		/* SNPP address */
	      strncpy(page_list->snpp_url, buf, MAX_FIELD - 1 );
	      break;
	  case 5:		/* SNPP port */
	      strncpy(page_list->snpp_port, buf, MAX_FIELD - 1 );
	      break;
	  case 6:		/* SNPP password */
	      strncpy(page_list->snpp_password, buf, MAX_FIELD - 1 );
	      break;
	  case 7:		/* max message length */
	      strncpy(page_list->max_msg_length, buf, MAX_FIELD - 1 );
	      break;
	  case 8:		/* autosplit */
	      page_list->auto_split = atoi(buf);
	      break;
	  case 9:		/* receiver name */
	      strncpy(page_list->receiver_name, buf, MAX_FIELD - 1 );
	      break;
	  case 10:		/* message */

/*
 *    This is the actual message to be send and needs to be of size MAX_LENGTH. Other fields can be 
 *    smaller to be more efficient unless noted
 */
	      strncpy(page_list->message, buf, MAX_LENGTH - 1 );
	      break;
	  case 11:		/* is there a password */
	      page_list->pwd_yes_no = atoi(buf);
	      break;
	  case 12:		/* shceduling data */
	      strncpy(page_list->sched_data, buf, MAX_FIELD - 1 );
	      break;
	  case 13:		/* protocols supported */
	      strncpy(page_list->protocols, buf, MAX_FIELD - 1 );
	      break;
	  case 14:		/* page order of protocols  */
	      strncpy(page_list->protocols_order, buf, MAX_FIELD - 1 );
	      break;
	  case 15:		/* cc page to email ?  */
	      strncpy(page_list->cc_email, buf, MAX_FIELD - 1 );
	      break;
	  case 16:		/* zero for normal mode and one for asynch mode */
	      page_list->asynch_page = atoi(buf);
	      break;
	  case 17:
	      page_list->time_stamp = atoi(buf);
	      break;
	  case 18:
	      strncpy(page_list->response_one, buf, MAX_FIELD - 1 );
	      if ( (page_list->response_one[0] == '\n') )
		  page_list->response_one[0] = '\0';
	      break;
	  case 19:
	      strncpy(page_list->response_two, buf, MAX_FIELD - 1 );
	      if ( (page_list->response_two[0] == '\n') )
		  page_list->response_two[0] = '\0';
	      break;
	  case 20:
	      strncpy(page_list->response_three, buf, MAX_FIELD - 1 );
	      if ( (page_list->response_three[0] == '\n') )
		  page_list->response_three[0] = '\0';
	      break;

	  case 21:
	      strncpy(page_list->tracking_number, buf, MAX_FIELD - 1 );
	      if ( (page_list->tracking_number[0] == '\n') )
		  page_list->tracking_number[0] = '\0';
	      break;

	  case 22:
	      strncpy(page_list->alternative_snpp, buf, MAX_FIELD - 1 );
	      break;

	  case 23:
	      strncpy(page_list->alternative_port, buf, MAX_FIELD - 1 );
	      break;

	  case 24:
	      strncpy(page_list->dial_on_no_connect, buf, MAX_FIELD - 1 );
	      break;
	      
	  }
	  /* print_debug_data(buf); */
	  write_to_status_line(socket_fd, buf, strlen(buf), IGNORE);
      }
  
      strcpy(page_list->msg_id, "none");
    
      if ( strcmp(page_list->sched_data, "none\n") )
      {
	  save_scheduled_page(page_list, email_info->isp, email_info->email_domain, 
			      email_info->email_address, socket_fd, client_adr);
	  scheduled_page = 1;
      }

      temp = get_page_list_memory(temp);
      temp->previous = page_list;
      page_list->next = temp;
      page_list = page_list->next;
      page_list->next = NULL;
  }
  free(buf);

/*
 *   If we are scheduling a page we skip the sorting and sending stuff
 */

  if ( scheduled_page )
  {
      if ( number_of_pages > 1 )
      {
	  write_to_status_line(socket_fd, "Messages scheduled\n", 19, ALWAYS_UPDATE);
      }
      else
      {
	  write_to_status_line(socket_fd, "Message scheduled\n", 18, ALWAYS_UPDATE);
      }
  }
  else
  {
      head->previous = NULL;
      if ( number_of_pages > 2 )
	  head = new_sort(head, number_of_pages);
      /* print_list(head); */
      send_messages(head, socket_fd, email_info, SEND_NOW, client_adr);
  }

  close(socket_fd);
  page_list = head;
  verify_exit();
  while (head->next)
  {
      free_page_mem(head);
      head = head->next;
  }

  free_page_mem(head);
  free(email_info->email_address);
  free(email_info->email_domain);
  free(email_info->isp);
  free(email_info);
  return;
}


page_profile *new_sort(page_profile *data, int pages)
{
    page_profile *tmp_head, *marker, *head;
    page_profile *save, *compare;
    page_profile *before, *after;

    int not_done = 1;

    head = data;
    head->previous = NULL;
    marker = head->next;
    tmp_head = head;

    if ( pages < 3 )
	return head;
    
    do {
	compare = marker;
	do {
	    if ( (strcmp(compare->snpp_url,tmp_head ->snpp_url)) <=  0 )
	    {
		if (tmp_head->previous != NULL)
		{
		    before = compare->previous;
		    after = compare->next;
		    marker = compare->next;
		    save = tmp_head->previous;

		    tmp_head->previous = compare;
		    compare->next = tmp_head;
		    compare->previous = save;
		    save->next = compare;
		    
		    before->next = after;
		    after->previous = before;
		}
		else
		{
		    before = compare->previous;
		    after = compare->next;
		    marker = compare->next;

		    tmp_head->previous = compare;
		    compare->next = tmp_head;
		    compare->previous = NULL;
		    head = compare;
		    before->next = after;
		    after->previous = before;
		}
		not_done = 0;
	    }
	    else
	    {
		tmp_head = tmp_head->next;
		if (tmp_head == marker)
		{
		    marker = marker->next;
		    not_done = 0;
		}
	    }
	}while (not_done);
	not_done = 1;
	tmp_head = head;
    }while (marker->next != NULL);
return head;
}

static void save_scheduled_page(page_profile *page_data, char *isp, char *domain, char *address, int socket_fd, char *client_adr)
{

    enum
    {
	lock = 0,
	unlock = 1
    };

    FILE  *fp = NULL;
    FILE  *fp1 = NULL;
    char  *sch_id = NULL;
    char  *temp = NULL;
    char  *dat_file = NULL;
    char  *id_file = NULL;
    char  recur_flag;
    char  *logfile = "/.gpage/gsched_id.dat";
    char  *logfile1 = "/.gpage/gsched.dat";
    int   locked, unlocked;
    int   tmp;
    int   number_of_retries = 30;
    int   path_length = 0;

    /* 1) Generate a unique id number for this message
     * 2) Write the data to a file
     * 3) Send the id back to the client
     */ 

    path_length = strlen(getenv("HOME")) + strlen(logfile) + 10;
    id_file = get_memory(path_length, "server_utilities - id_file");

    strcpy(id_file, getenv("HOME"));
    strcat(id_file, logfile);

    path_length = strlen(getenv("HOME")) + strlen(logfile1) + 10;
    dat_file = get_memory(path_length, "server_utilities - dat_file");

    strcpy(dat_file, getenv("HOME"));
    strcat(dat_file, logfile1);

    locked = number_of_retries;
    unlocked = 0;

    temp = get_memory(MAX_LENGTH, "server_utilities - temp");

    while( locked )
    {
	fp1 = file_locking(id_file, "a", "gsched_id.dat", lock);
	if (fp1)
	{
	    fp = file_locking(dat_file, "a", "gsched.dat", lock);
	    if (fp)
	    {
		locked = 0;
		unlocked = 1;
	    }
	    else
	    {
		file_locking(id_file, "a", "gsched_id.dat", unlock);
		locked--;
		sleep(1);
	    }
	}
	else
	{
	    locked--;
	    sleep(1);
	}	    
    }

    if ( unlocked )
    {
	sch_id = get_id(SCHEDULE);
	if ( !strcmp(sch_id, "-1") )
	{
	    free(sch_id);
	    init_gserver();
	    sch_id = get_id(SCHEDULE);
	}	

/*
 *   If the recur_flag is not a zero then we have a recurring scheduled page and we 
 *   are going to append an r to the id so we know what we have
 */

	tmp = strlen(page_data->sched_data);
	tmp--;
	if ( *(page_data->sched_data + tmp) == '\n' )
	    recur_flag = *(page_data->sched_data + --tmp);
	else
	    recur_flag = *(page_data->sched_data + tmp);
	
	if ( recur_flag != '0' )
	    strcat(sch_id, "r");

	sprintf(temp, "500 %s\n", sch_id);

	//write_to_status_line(socket_fd, temp, strlen(temp), ALWAYS_UPDATE);
	write_num(socket_fd, temp, strlen(temp));
	
	sprintf(temp, "ID %s\n", sch_id);
	fputs(temp, fp);
	fputs(isp, fp);
	fputs(domain, fp);
	fputs(address, fp);
	fputs(page_data->user_id, fp);
	fputs(page_data->receiver_number, fp);
	fputs(page_data->wctp_url, fp);
	fputs(page_data->smtp, fp);
	fputs(page_data->snpp_url, fp);
	fputs(page_data->snpp_port, fp);
	fputs(page_data->snpp_password, fp);

	page_data->protocols[strlen(page_data->protocols) -1] = '\0';
	strcat(page_data->protocols, "A");
	strcat(page_data->protocols, page_data->protocols_order);
	fputs(page_data->protocols, fp);
	fputs(page_data->max_msg_length, fp);
	sprintf(temp, "%d\n", page_data->auto_split);
	fputs(temp, fp);
	fputs(page_data->receiver_name, fp);
	fputs(page_data->message, fp);
	fputs(client_adr, fp);
	fputs("\n", fp);
	sprintf(temp, "%d\n", page_data->time_stamp);
	fputs(temp, fp);

	sprintf(temp, "%d\n", page_data->pwd_yes_no);
	fputs(temp, fp);
	
	fclose(fp);

	sprintf(temp, "ID %s %s", sch_id, page_data->sched_data);
	fputs(temp, fp1);
	fclose(fp1);
		
	free(sch_id);

	file_locking(dat_file, "a", "gsched.dat", unlock);
	file_locking(id_file, "a", "gsched_id.dat", unlock);
    }
    else
    {
	/* We need to tell the client we could not schedule the page */
	debug_log("Unable to schedule page");
    }
    
    free(temp);
    free(dat_file);
    free(id_file);
}

page_profile *get_memory_struct(page_profile *node)
{
    
    if ( (node = malloc(sizeof(*node))) == NULL)
	error_quit("Error allcoating memory - node - server_utilities.c ");
	
    return node;
}

void print_list(page_profile *list)
{
    char    temp[128];

    memset(temp, '\0', sizeof(temp));
    while(list->next != NULL)
    {
	sprintf(temp, "SNPP URL: %s", list->receiver_name);
	print_debug_data(temp);
	list = list->next;
    }

    return;
}

static void get_email_info(email_data *info, int live_sock, int max_len, int max_time)
{
    char     *buf;
    int      amount;

    info->isp = get_memory(MAX_LENGTH, "server_utilities - email_info->isp");
    info->email_domain = get_memory(MAX_LENGTH, "server_utilities - email_info->email_domain");
    info->email_address = get_memory(MAX_LENGTH, "server_utilities - email_info->email_address");
    buf = get_memory(MAX_LENGTH, "server_utilities - buf");

    memset(info->isp, '\0', sizeof(info->isp));
/*  Get the domain name of the isp  */

  if ( ( amount = read_a_line(live_sock, info->isp, max_len, max_time) ) <= 0) 
      return;

  write_to_status_line(live_sock, info->isp, amount, IGNORE);
  

  memset(buf, '\0', sizeof(buf));
  memset(info->email_domain, '\0', sizeof(info->email_domain));
/* Get the email domain */
  if ( ( amount = read_a_line(live_sock, info->email_domain, max_len, max_time) ) <= 0) 
      return;

  write_to_status_line(live_sock, info->email_domain, amount, IGNORE);

  memset(info->email_address, '\0', sizeof(info->email_address));
  memset(buf, '\0', sizeof(buf));

/* Get the email address */
  if ( ( amount = read_a_line(live_sock, info->email_address, max_len, max_time) ) <= 0) 
      return;

  write_to_status_line(live_sock, info->email_address, amount, IGNORE);

  free(buf);

}

page_profile  *get_page_list_memory(page_profile *data)
{

  if ( (data = malloc(sizeof(*data))) == NULL)
      error_quit("Error allcoating memory - data - server_utilities.c ");

  data->user_id = get_memory(MAX_FIELD, "server_utilities -data->user_id");

  data->msg_id = get_memory(MAX_FIELD, "server_utilities -data->msg_id");

  data->receiver_number = get_memory(MAX_FIELD, "server_utilities -data->receiver_number");

  data->wctp_url = get_memory(MAX_FIELD, "server_utilities -data->wctp_url");

  data->receiver_name = get_memory(MAX_FIELD, "server_utilities -data->receiver_name");

  data->smtp = get_memory(MAX_FIELD, "server_utilities -data->smtp");

  data->snpp_url = get_memory(MAX_FIELD, "server_utilities -data->snpp_url");

  data->snpp_port = get_memory(MAX_FIELD, "server_utilities -data->snpp_port");

  data->snpp_password = get_memory(MAX_FIELD, "server_utilities -data->snpp_password");

  data->max_msg_length = get_memory(MAX_FIELD, "server_utilities -data->max_msg_length");

  data->message = get_memory(MAX_LENGTH, "server_utilities - data->message");

  data->sig = get_memory(MAX_FIELD, "server_utilities -data->sig");

  data->sched_data = get_memory(MAX_FIELD, "server_utilities -data->sched_data");

  data->protocols = get_memory(MAX_FIELD, "server_utilities -data->protocols");

  data->cc_email = get_memory(MAX_FIELD, "server_utilities -data->cc_email");

  data->response_one = get_memory(MAX_FIELD, "server_utilities -response_one");

  data->response_two = get_memory(MAX_FIELD, "server_utilities -response_two");

  data->response_three = get_memory(MAX_FIELD, "server_utilities -response_three");

  data->protocols_order = get_memory(MAX_FIELD, "server_utilities -data->protocols_order");

  data->sig_yes_no = get_memory(MAX_FIELD, "server_utilities -data->sig_yes_no");

  data->error_code = get_memory(MAX_FIELD, "server_utilities -data->error_code");

  data->tracking_number = get_memory(MAX_FIELD, "server_utilities -data->tracking_number");

  data->alternative_snpp = get_memory(MAX_FIELD, "server_utilities -data->alternative_snpp");

  data->alternative_port = get_memory(MAX_FIELD, "server_utilities -data->alternative_port");

  data->dial_on_no_connect = get_memory(MAX_FIELD, "server_utilities -data->dial_on_no_connect");

  return data;
}

void free_page_mem(page_profile *data)
{

   free(data->user_id);
   free(data->msg_id);
   free(data->receiver_number);
   free(data->wctp_url);
   free(data->receiver_name);
   free(data->smtp);
   free(data->snpp_url);
   free(data->snpp_port);
   free(data->snpp_password);
   free(data->max_msg_length);
   free(data->message);
   free(data->sig);
   free(data->sched_data);
   free(data->protocols);
   free(data->protocols_order);
   free(data->cc_email);
   free(data->response_one);
   free(data->response_two);
   free(data->response_three);
   free(data->sig_yes_no);
   free(data->error_code);
   free(data->tracking_number);
   free(data->alternative_snpp);
   free(data->alternative_port);
   free(data->dial_on_no_connect);

   free(data);
   return;
}
    




