/*
 * g-page (C) 2001 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"

#define MAXFD  64
#define PORT_NUMBER 1301

static void load_file(FILE *fp, int sock);

void get_data(int connected_fd, char *client_ip)
{
    enum {
	RECEIVER = 1,
	SERVICE = 2,
	GROUP = 3,
	IDS = 4
    };

    FILE   *fp;

    char   *home, *temp;
    int    length;
    int    not_done, file;

    length = strlen(getenv("HOME"));
    length += 20;
    
    home = get_memory(length, "home - functions");
    temp = get_memory(length, "temp - functions");

    strcpy(home, getenv("HOME"));

    file = RECEIVER;
    not_done = 1;
    
    while ( not_done )
    {
	switch (file) {
	case RECEIVER:
	    strcpy(temp, home);
	    strcat(temp, "/.gpage/RECEIVER.dat");
	    if ( (fp = fopen(temp, "r")) == NULL )
		not_done = 0;
	    else
		load_file(fp, connected_fd);
	    file++;
	    break;
	case SERVICE:
	    strcpy(temp, home);
	    strcat(temp, "/.gpage/SERVICE.dat");
	    if ( (fp = fopen(temp, "r")) == NULL )
		not_done = 0;
	    else
		load_file(fp, connected_fd);
	    file++;
	    break;
	case GROUP:
	    strcpy(temp, home);
	    strcat(temp, "/.gpage/GROUP.dat");
	    if ( (fp = fopen(temp, "r")) == NULL )
		not_done = 0;
	    else
		load_file(fp, connected_fd);
	    file++;
	    break;
	case IDS:
	    strcpy(temp, home);
	    strcat(temp, "/.gpage/IDS.dat");
	    if ( (fp = fopen(temp, "r")) == NULL )
		not_done = 0;
	    else
		load_file(fp, connected_fd);
	    file++;
	    not_done = 0;
	    break;
	default:
	    not_done = 0;
	    break;
	    }
    }
    close(connected_fd);
    free(home);
    free(temp);
    return;
}

static void load_file(FILE *fp, int sock)
{
    char   *end = "**********\n";
    char   *buffer;

    buffer = get_memory(MAX_LENGTH, "buffer - functions");

    while ( fgets(buffer, MAX_LENGTH, fp) )
    {
	write_num(sock, buffer, strlen(buffer));
	read_a_line(sock, buffer, MAX_LENGTH, 1);
    }
    
    write_num(sock, end, strlen(end));
    read_a_line(sock, buffer, MAX_LENGTH, 1);

    free(buffer);
    fclose(fp);
    return;
}

ssize_t write_num(int file_desc, char *strptr, size_t num)
{

  size_t             num_left;
  ssize_t            num_written;
  char               *ptr;
  int                maxfd;
  fd_set             wset;
  int                result = 0;

/*
 *   I'm going to ignore the sigpipe here. The sigpipe is generated when we try to 
 *   write to a closed file descriptor on a socket. I'll let the calling function
 *   handle it by returning a -1. Note that we plug the default signal 
 *   handler back in before we return
 */

  if ( file_desc < 0 )		/* In case we get sent a closed file descriptor */
      return(-1);
  Signal(SIGPIPE, SIG_IGN);

  ptr = strptr;
  num_left = num;
  FD_ZERO(&wset);

  while ( num_left > 0 )
  {

      maxfd = file_desc +1;
      FD_SET(file_desc, &wset);

/*
 *   We use select to try to catch any closed file descriptors
 */

      result = select(maxfd, NULL, &wset, NULL, NULL);
      
      if ( result == -1 )
      {
	  if ( errno == EINTR )
	  {
	      num_written = 0;
	      continue;
	  }
	  else
	  {
	      Signal(SIGPIPE, SIG_DFL);
	      return (-1);
	  }	      
      }
      else if (result)
      {
	  num_written = write(file_desc, ptr, num_left );
	  if ( num_written  <= 0 )
	  {
	      if ( errno == EINTR )
	      {
		  num_written = 0;
		  continue;
	      }
	      else
	      {
		  Signal(SIGPIPE, SIG_DFL);
		  return (-1);
	      }
	  }
	  else
	  {
	      num_left -= num_written;
	      ptr += num_written;
	  }
	  
      }
  }
  Signal(SIGPIPE, SIG_DFL);
  return (num);
}


ssize_t read_a_line(int file_desc, char return_buff[MAX_LENGTH], size_t maxlen, int timeout)
{

    int         index;
    char        ch;
    int         read_count;
    fd_set      rset;
    struct      timeval tv;
    int         select_value;

    memset(return_buff, (char) '\0', MAX_LENGTH);
    index = 0;
    
    if ( file_desc < 0 )
      return 0;

    FD_ZERO(&rset);
    FD_SET(file_desc, &rset);

    tv.tv_sec = timeout;
    tv.tv_usec = 0;


    while (index < (MAX_LENGTH -1) )
    {
	 FD_ZERO(&rset);
	 FD_SET(file_desc, &rset);
	 
	 select_value = select(file_desc + 1, &rset, NULL, NULL, &tv);
	 
	 if ( select_value > 0 )
	 {
	      if ( FD_ISSET(file_desc, &rset) )
	      {
		   read_count = read(file_desc, &ch, 1);
		   if ( read_count == 1 )
		   {
			return_buff[index++] = ch;
			if ( (ch == '\n') )
			{
			     return index;
			}
		   }
		   else if ( read_count == 0 )
		   {
			if ( index == 0 )
			{
			     return 0;	      
			}
			else
			{
			     return_buff[index++] = '\n';
			     return_buff[index++] = '\0';
			     return (index);
			}
		   }
	      }
	      else
	      {
		   continue;
	      }
	 }
	 else if ( select_value == 0 )
	 {
	      sprintf(return_buff, "920 remote server stopped sending data\n");
	      return 0;
	      return( strlen(return_buff));	  
	 }
	 else
	 {		
	      if (errno == EINTR)
	      {
		   info_msg("Caught interupt in readline\n");
		   continue;
	      }
	      else
		   return 0;	  	  
	 }
    }
    return_buff[index -2] = '\n';
    return_buff[index -1] = '\0';
    return index;
} 
