/*
 * g-page (C) 2000 Hiro Protagonist
 *
 * This software is released under the GNU General Public License.
 * Please read the included LICENSE file for more information.
 * This software comes with no warranty of any kind, use at you own risk!
 */   

#include "my_header.h"

static void handle_error(int errno_flag, int level, const char *fmt, va_list ap);

extern int daemon_flag;		/*  one if we are a daemon, zero otherwise  */

/*
 *   This is for the debug logging
 */

void debug_log(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(0, LOG_INFO, fmt, ap);
    va_end(ap);
    return;
}

/*
 *   Non fatal error - system call
 */

void error_ret(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(1, LOG_INFO, fmt, ap);
    va_end(ap);
    return;
}

/*
 *   Informational logging - no error
 */

void info_msg(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(0, LOG_INFO, fmt, ap);
    va_end(ap);
    return;
}

/*
 *   Fatal error - system call
 */

void error_sys(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(1, LOG_ERR, fmt, ap);
    va_end(ap);
    exit(EXIT_FAILURE);
}

/*
 *   Fatal error - system call
 */

void error_core(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(1, LOG_ERR, fmt, ap);
    va_end(ap);
    abort();			/* Dump core */
    exit(EXIT_FAILURE);
}

/*
 *   Fatal error - non system call
 */

void error_quit(const char *fmt,...)
{

    va_list ap;
    
    va_start(ap, fmt);
    handle_error(0, LOG_ERR, fmt, ap);
    va_end(ap);
    exit(EXIT_FAILURE);
}

static void handle_error(int errno_flag, int level, const char *fmt, va_list ap)
{
    int   errno_save = 0;
    int   n;
    char  buf[MAX_LENGTH];

    if (errno_flag)
	errno_save = errno;

#ifdef HAVE_VSNPRINTF
    vsnprintf(buf, sizeof(buf), fmt, ap);
#else
    vsprintf(buf, fmt, ap);
#endif

    n = strlen(buf);
    if (errno_flag)
	snprintf(buf + n, sizeof(buf) - n, ": %s", strerror(errno_save));
    strcat(buf, "\n");

    if (daemon_flag)
	syslog(level, buf);
    else
    {
	fflush(stdout);
	fputs(buf, stderr);
	fflush(stderr);
    }
    
    return;
}
